package com.ssi.config;

import org.elasticsearch.client.RestHighLevelClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.List;

/**
 * Elasticsearch客户端配置
 * @author zhoujibo
 */
@Configuration
@ComponentScan(basePackageClasses = ElasticsearchClientFactory.class)
public class ElasticsearchClientConfig {

    @Value("${spring.data.elasticsearch.client.connect-num}")
    private Integer connectNum;

    @Value("${spring.data.elasticsearch.client.connect-per-route}")
    private Integer connectPerRoute;

    @Value("#{'${spring.data.elasticsearch.hosts}'.split(',')}")
    private List<String> hosts;

    @Value("${spring.data.elasticsearch.userName}")
    private String userName;

    @Value("${spring.data.elasticsearch.pwd}")
    private String pwd;

    @Value("${spring.data.elasticsearch.enable-security}")
    private boolean enableSecurity = false;

    @Bean(initMethod = "init", destroyMethod = "close")
    public ElasticsearchClientFactory getFactory() {
        return ElasticsearchClientFactory.build(hosts, connectNum, connectPerRoute, userName, pwd, enableSecurity);
    }

    @Bean(name = "restHighLevelClient")
    @Scope("singleton")
    public RestHighLevelClient getRestHighLevelClient() {
        return getFactory().getRestHightLevelClient();
    }

}
