package com.ssi.config;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.List;

/**
 * Elasticsearch客户端工厂配置
 * @author zhoujibo
 */
public class ElasticsearchClientFactory {

    protected Logger logger = LoggerFactory.getLogger(getClass());

    public static int CONNECT_TIMEOUT_MILLIS = 1000;
    public static int SOCKET_TIMEOUT_MILLIS = 30000;
    public static int CONNECTION_REQUEST_TIMEOUT_MILLIS = 500;
    public static int MAX_CONN_PER_ROUTE = 10;
    public static int MAX_CONN_TOTAL = 30;
    public static String userName = "";

    public static String pwd = "";

    public static boolean enableSecurity = false;

    private static HttpHost[] httpHosts;
    private RestClientBuilder builder;
    private RestHighLevelClient restHighLevelClient;

    private static ElasticsearchClientFactory elasticsearchClientFactory = new ElasticsearchClientFactory();

    private ElasticsearchClientFactory() {
    }

    public static ElasticsearchClientFactory build(List<String> hosts, Integer maxConnectNum,
                                                   Integer maxConnectPerRoute, String userName, String pwd, boolean enableSecurity) {
        HttpHost[] httpHostArray = new HttpHost[hosts.size()];
        for (int i = 0; i < hosts.size(); i++) {
            String item = hosts.get(i);
            httpHostArray[i] = new HttpHost(item.split(":")[0], Integer.parseInt(item.split(":")[1]), "http");
        }

        return ElasticsearchClientFactory.build(httpHostArray, maxConnectNum, maxConnectPerRoute, userName, pwd, enableSecurity);
    }

    public static ElasticsearchClientFactory build(HttpHost[] httpHosts, Integer maxConnectNum,
                                                   Integer maxConnectPerRoute, String userName, String pwd, boolean enableSecurity) {
        ElasticsearchClientFactory.httpHosts = httpHosts;
        MAX_CONN_TOTAL = maxConnectNum;
        MAX_CONN_PER_ROUTE = maxConnectPerRoute;
        ElasticsearchClientFactory.userName = userName;
        ElasticsearchClientFactory.pwd = pwd;
        ElasticsearchClientFactory.enableSecurity = enableSecurity;
        return elasticsearchClientFactory;
    }

    public static ElasticsearchClientFactory build(HttpHost[] httpHosts, Integer connectTimeOut,
                                                   Integer socketTimeOut, Integer connectionRequestTime,
                                                   Integer maxConnectNum, Integer maxConnectPerRoute) {
        ElasticsearchClientFactory.httpHosts = httpHosts;
        CONNECT_TIMEOUT_MILLIS = connectTimeOut;
        SOCKET_TIMEOUT_MILLIS = socketTimeOut;
        CONNECTION_REQUEST_TIMEOUT_MILLIS = connectionRequestTime;
        MAX_CONN_TOTAL = maxConnectNum;
        MAX_CONN_PER_ROUTE = maxConnectPerRoute;

        return elasticsearchClientFactory;
    }

    public void init() {
        builder = RestClient.builder(httpHosts);
        setConnectTimeOutConfig();
        setMultiConnectConfig();

        restHighLevelClient = new RestHighLevelClient(builder);
    }

    public void setConnectTimeOutConfig() {
        builder.setRequestConfigCallback(requestConfigBuilder -> {
            requestConfigBuilder.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
            requestConfigBuilder.setSocketTimeout(SOCKET_TIMEOUT_MILLIS);
            requestConfigBuilder.setConnectionRequestTimeout(CONNECTION_REQUEST_TIMEOUT_MILLIS);

            return requestConfigBuilder;
        });
    }

    public void setMultiConnectConfig() {
        final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
        credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(userName, pwd));

        builder.setHttpClientConfigCallback(httpClientBuilder -> {
            httpClientBuilder.setMaxConnTotal(MAX_CONN_TOTAL);
            httpClientBuilder.setMaxConnPerRoute(MAX_CONN_PER_ROUTE);
            if (enableSecurity) {
                httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider);
            }

            return httpClientBuilder;
        });
    }

    public RestHighLevelClient getRestHightLevelClient() {
        return restHighLevelClient;
    }

    public void close() {
        if (restHighLevelClient != null) {
            try {
                restHighLevelClient.close();
            } catch (IOException e) {
                logger.error("Fail to close elasticsearch rest high client.");
            }
        }
    }

}
