package com.ssi.constant.enums;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Getter;

/**
 * TOS任务状态枚举类
 */
@JsonFormat(shape = JsonFormat.Shape.OBJECT)
public enum VmsTosOrderStatusEnum {

    ARRIVED_PASSPOINT1(1, "到达途径点1"),

    ARRIVED_PASSPOINT2(2, "到达途径点2"),

    ARRIVED_DESTINATION(3, "到达目的地"),

    START_PRECISE_POSITION(16, "开始精准定位"),

    START_LOADING(17, "开始装箱"),

    START_UNLOADING(18, "开始卸箱"),

    START_CHARGING(19, "开始充电"),

    FINISH_LOADING(33, "完成装箱"),

    FINISH_UNLOADING(34, "完成卸箱"),

    FINISH_CHARGING(35, "完成充电"),

    FINISH_PARKING(36, "完成停车（固定停车区任务）"),

    FINISH_CONFIRM(37, "三重确认完成"),

    FINISH_DRIVER_CONFIRM(38, "港机司机确认"),

    FINISH_CRANE_CONFIRM(39, "吊具确认"),

    TASK_FAIL(41, "任务终止"),

    VMS_RECEIVED(42, "平台接受"),

    OBU_RECEIVED(43, "OBU接受"),

    VEHICLE_LOCKUP(49, "车辆停稳锁死"),

    VEHICLE_UNLOCK(50, "车辆解除锁死"),

    PATH_PLANNING(64,"路径规划"),

    START_TASK(65,"开始行驶"),

//    ADCU_RECEIVED(66,"ADCU接受"),

    ADCU_RECEIVED_OVERTIME(67,"ADCU接受超时"),

    TASK_CANCEL_SUCCESS(51,"取消任务成功"),

    TASK_CANCEL_FAIL(52,"取消任务失败"),

    FORCE_CANCEL_SUCCESS(53,"强制取消任务成功"),

    FORCE_CANCEL_FAIL(54,"强制取消任务失败"),

    WAIT_FRONT_CONTAINER(69,"等待双箱前箱指令"),

    WAIT_BACK_CONTAINER(70,"等待双箱后箱箱指令"),

    SEND_FRONT_CONTAINER(71,"双箱前箱指令发送中"),

    SEND_TASK(72,"指令发送中");



    @Getter
    private int code;
    @Getter
    private String description;

    VmsTosOrderStatusEnum(int code, String description) {
        this.code = code;
        this.description = description;
    }

    public static boolean isLoadUnloadStatus(int status){
        if(VEHICLE_LOCKUP.code == status || START_LOADING.code == status || START_UNLOADING.code == status
                || FINISH_LOADING.code == status || FINISH_UNLOADING.code == status){
            return true;
        }
        return false;
    }

    public static VmsTosOrderStatusEnum find(int code){
        VmsTosOrderStatusEnum[] values = VmsTosOrderStatusEnum.values();
        for (VmsTosOrderStatusEnum anEnum : values){
            if(anEnum.getCode() == code){
                return anEnum;
            }
        }
        return null;
    }

}
