package com.ssi.controller;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.response.SSIResponse;
import com.ssi.service.BigScreenService;
import com.ssi.service.VmsVehicleService;
import com.ssi.service.VmsHangerStatusInfoService;
import com.ssi.service.VmsTosOrdersService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Arrays;
import java.util.Date;

/**
 * 大屏相关接口控制器
 */
@Api(tags = "大屏接口控制器",description = "大屏接口")
@RestController
@RequestMapping("/bigScreen")
public class BigScreenController {

    @Autowired
    private VmsTosOrdersService vmsTosOrdersService;
    @Autowired
    private VmsVehicleService vmsVehicleService;

    @Autowired
    private BigScreenService bigScreenService;
    @Autowired
    private VmsHangerStatusInfoService vmsHangerStatusInfoService;

    /**
     * 获得各业务箱数
     */
    @LogAudit(logName = "获得各业务箱数")
    @ApiOperation(value = "获得各业务箱数", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/queryContainerNumOneDay")
    public SSIResponse queryContainerNumByDay(Date queryDate){
        if(queryDate == null){
            return vmsTosOrdersService.queryContainerNum();
        }else {
            LocalDateTime startTime = LocalDateTime.of(queryDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(queryDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MAX);
            return vmsTosOrdersService.queryContainerNumOneDay(startTime,endTime);
        }
    }

    /**
     * 获得最近n天各业务箱数
     */
    @LogAudit(logName = "获得最近n天各业务箱数，不传参数则默认查最近七天")
    @ApiOperation(value = "获得最近n天各业务箱数，不传参数则默认查最近七天", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/queryContainerNumByDay")
    public SSIResponse queryContainerNumByDay(Date queryStartDate,Date queryEndDate) throws Exception{
        LocalDateTime startTime = null;
        LocalDateTime endTime = null;
        if(queryStartDate == null && queryEndDate == null){
            startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
            endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
        }
        if(queryStartDate != null){
            startTime = LocalDateTime.of(queryStartDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MIN);
        }
        if(queryEndDate != null){
            endTime = LocalDateTime.of(queryEndDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MAX);
        }
        return vmsTosOrdersService.queryContainerNumByDay(startTime, endTime);
    }

    /**
     * 获得各业务运输效率
     */
    @LogAudit(logName = "获得各业务运输效率")
    @ApiOperation(value = "获得各业务运输效率", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/queryContainerAvgTimeOneDay")
    public SSIResponse queryContainerAvgTimeOneDay(Date queryDate){
        if(queryDate == null){
            return vmsTosOrdersService.queryContainerAvgTime();
        }else {
            LocalDateTime startTime = LocalDateTime.of(queryDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(queryDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MAX);
            return vmsTosOrdersService.queryContainerAvgTimeOneDay(startTime,endTime);
        }
    }

    /**
     * 运力分析
     */
    @LogAudit(logName = "运力分析")
    @ApiOperation(value = "运力分析", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/transportCapacity")
    public SSIResponse transportCapacity(){
        return vmsVehicleService.transportCapacity();
    }

    /**
     * 大屏获取运营数据
     */
    @LogAudit(logName = "大屏获取运营数据")
    @ApiOperation(value = "大屏获取运营数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getOperateData")
    public SSIResponse getOperateData(String vin) throws IOException {
        return bigScreenService.getOperateData(Arrays.asList(vin),null,null);
    }

    /**
     * 车辆状态分布
     */
    @LogAudit(logName = "车辆状态分布")
    @ApiOperation(value = "车辆状态分布", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getVehicleStatusDist")
    public SSIResponse getVehicleStatusDist(){
        return bigScreenService.getVehicleStatusDist();
    }

    /**
     * 大屏查询单车信息
     */
    @LogAudit(logName = "大屏查询单车信息")
    @GetMapping("/getVehicleInfo")
    @ApiOperation(value = "大屏查询单车信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getVehicleInfo(@RequestParam String vin) {
        return bigScreenService.getVehicleInfo(vin);
    }

    /**
     * 查询车辆装卸箱时吊具状态信息
     */
    @LogAudit(logName = "查询车辆装卸箱时吊具状态信息")
    @GetMapping("/getHangerStatusByVin")
    @ApiOperation(value = "查询车辆装卸箱时吊具状态信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getHangerStatusByVin(@RequestParam String vin) {
        return vmsHangerStatusInfoService.getHangerStatusByVin(vin);
    }
}
