package com.ssi.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.vo.CargoShipConfigDto;
import com.ssi.response.SSIResponse;
import com.ssi.service.CargoShipConfigInfoService;
import io.swagger.annotations.Api;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.shiro.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author SunnyHotz
 * @PackageName:com.ssi.controller
 * @ClassName:CargoShipConfigController
 * @Description:
 * @date 2022/10/19 09:59
 */
@Api
@RestController
@RequestMapping("/cargoShipConfig")
public class CargoShipConfigController {


    @Autowired
    private CargoShipConfigInfoService cargoShipConfigInfoService;


    @GetMapping("/onWorkingList")
    @LogAudit
    public SSIResponse<List<CargoShipConfigDto>> queryOnWorkingList(){
        List<CargoShipConfigDto> results = cargoShipConfigInfoService.queryOnWorkingList();
        return SSIResponse.ok(results);
    }

    @LogAudit
    @PostMapping("/save")
    public SSIResponse<Void> save(@RequestBody JSONObject obj){
        JSONArray cargoShipConfigs = obj.getJSONArray("cargoShipConfigs");
        if(CollectionUtils.isEmpty(cargoShipConfigs)){
            return SSIResponse.ok();
        }
        List<CargoShipConfigDto> cargoShipConfigDto = cargoShipConfigs.toJavaList(CargoShipConfigDto.class);
        cargoShipConfigInfoService.enableConfig(cargoShipConfigDto);
        return SSIResponse.ok();
    }

    @LogAudit
    @PostMapping("/update")
    public SSIResponse<Void> update(@RequestBody CargoShipConfigDto cargoShipConfigDto){
        cargoShipConfigInfoService.updateById(cargoShipConfigDto);
        return SSIResponse.ok();
    }

    @LogAudit
    @PostMapping("/listByCondition")
    public SSIResponse<JSONObject> listByCondition(@RequestBody CargoShipConfigDto cargoShipConfigDto){
        JSONObject results = cargoShipConfigInfoService.listByCondition(cargoShipConfigDto);
        return SSIResponse.ok(results);
    }

    @LogAudit
    @GetMapping("/qryEnableCacheArea")
    public SSIResponse<List<JSONObject>> qryEnableCacheArea(@RequestParam("beginLocation") String beginLocation,@RequestParam("endLocation") String endLocation){
        Assert.notNull(beginLocation,"坐标输入为空");
        List<JSONObject> data = cargoShipConfigInfoService.qryEnableCacheArea(beginLocation, endLocation);
        return SSIResponse.ok(data);
    }
}
