package com.ssi.controller;

import com.ssi.constant.enums.Status;
import com.ssi.response.SSIResponse;
import com.ssi.utils.FileUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ResourceLoader;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件上传控制器
 */
@Api
@RestController
@RequestMapping("/file")
public class FileUploadController {

    @Value("${LinuxFile.upload-url}")
    private String url;

    @Value("${LinuxFile.upload-path}")
    private String path;

    private final ResourceLoader resourceLoader;

    @Autowired
    public FileUploadController(ResourceLoader resourceLoader) {
        this.resourceLoader = resourceLoader;
    }

    /**
     * @param file 上传的文件
     * @return
     */
    @RequestMapping("/fileUpload")
    @ApiOperation(value = "上传文件", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse upload(@RequestParam("file") MultipartFile file) {
        long size = file.getSize();
        //获得文件名字
        String fileName = file.getOriginalFilename();
        //上传失败提示
        String newFileName = FileUtils.upload(file, path, fileName);
        if (newFileName != null) {
            Map<String, String> fileMap = new HashMap<String, String>();
            fileMap.put("fileName", newFileName);
            fileMap.put("fileUrl", url + File.separator + newFileName);
            return SSIResponse.ok(fileMap);
        } else {
            return SSIResponse.no(Status.FILEUPLOADERROR);
        }
    }

    @GetMapping(value = "/getFile")
    @ApiOperation(value = "获取文件", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse file(@RequestParam String fileName) {
        return SSIResponse.ok(url + File.separator + fileName);
    }

}
