package com.ssi.controller;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.dto.VehicleTaskOrderReq;
import com.ssi.response.SSIResponse;
import com.ssi.service.OperationAnalysisService;
import com.ssi.service.VmsKpiAnalysisService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 运行分析控制器
 */
@Api(tags = "运行分析控制器", description = "运行分析")
@RestController
@RequestMapping("/operationAnalysis")
public class OperationAnalysisController {

    @Autowired
    private OperationAnalysisService operationAnalysisService;
    @Autowired
    private VmsKpiAnalysisService vmsKpiAnalysisService;

    /**
     * 各项总体数据
     */
    @LogAudit
    @ApiOperation(value = "各项总体数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/overall")
    public SSIResponse queryOverall(@RequestParam(name = "vin",required = false)List<String> vins ,@RequestParam(name = "interval",required = false)List<Date> interval) {
        return operationAnalysisService.overallData(vins,interval);
    }

    /**
     * 运输效率趋势图
     */
    @LogAudit
    @ApiOperation(value = "运输效率趋势图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getTaskTrend")
    public SSIResponse getTaskTrend(@RequestParam("timeType") Integer timeType,@RequestParam(value = "groupType",required = false) Integer groupType) throws Exception{
        //return operationAnalysisService.getTaskTrend(timeType);
        if(Objects.isNull(groupType)){
            groupType=3;
        }
        return vmsKpiAnalysisService.getTaskTrend(timeType,groupType);
    }

    /**
     * 停车等待时长分析
     */
    @LogAudit
    @ApiOperation(value = "停车等待时长分析", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getStopWaitTimeRate")
    public SSIResponse getStopWaitTimeRate() {
        //计算一个月内执行任务的停车等待时长
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MONTH,-1);
        return operationAnalysisService.getStopWaitTimeRate(calendar.getTime().getTime());
    }

    /**
     * 运载力趋势图（空载里程、满载里程）
     */
    @LogAudit
    @ApiOperation(value = "运载力趋势图（空载里程、满载里程）", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getTaskMileTrend")
    public SSIResponse getTaskMileTrend(Integer timeType) throws Exception{
        return operationAnalysisService.getTaskMileTrend(timeType);
    }

    /**
     * 充电桩利用率、充电时长趋势图
     */
    @LogAudit
    @ApiOperation(value = "充电桩利用率、充电时长趋势图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getChargingPileUseTrend")
    public SSIResponse getChargingPileUseTrend(Integer timeType) throws Exception{
        return operationAnalysisService.getChargingPileUseTrend(timeType);
    }

    /**
     * 平均一周充电频次分析
     */
    @LogAudit
    @ApiOperation(value = "平均一周充电频次分析", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getChargingFrequencyByVehicle")
    public SSIResponse getChargingFrequencyByVehicle() {
        return operationAnalysisService.getChargingFrequencyByVehicle();
    }

    /**
     * 平均一周充电频次平均值
     */
    @LogAudit
    @ApiOperation(value = "平均一周充电频次平均值", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getOverallChargingFrequency")
    public SSIResponse getOverallChargingFrequency() {
        return operationAnalysisService.getOverallChargingFrequency();
    }

    /**
     * 车辆使用率/里程/时长/能耗分析
     */
    @LogAudit
    @ApiOperation(value = "车辆使用率/里程/时长/能耗分析", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getUseInfoByVehicle")
    public SSIResponse getUseInfoByVehicle() {
        return operationAnalysisService.getUseInfoByVehicle();
    }



    /**
     * 车辆任务分析
     */
    @LogAudit
    @ApiOperation(value = "车辆任务分析", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/getVehicleTaskOrder")
    public SSIResponse getVehicleTaskOrder(@RequestBody VehicleTaskOrderReq vehicleTaskOrderReq) {
        return operationAnalysisService.getVehicleTaskOrder(vehicleTaskOrderReq);
    }
}
