package com.ssi.controller;

import com.alibaba.fastjson.JSONObject;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.constant.URL;
import com.ssi.response.SSIResponse;
import com.ssi.service.VehicleCommandService;
import com.ssi.utils.RestTemplateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Map;
import java.util.Objects;

/**
 *
 *
 * @author yinjianggqiao
 * @email
 * @date 2022-07-04 15:38:37
 */
@Api
@RestController
@RequestMapping("/vehicleCommand")
public class VehicleCommandController {
	private static Logger log = LoggerFactory.getLogger(VehicleCommandController.class);

	@Value("${command-url}")
	private String odbuUrl;

	@Autowired
	private VehicleCommandService vehicleCommandService;

	@LogAudit
	@ApiOperation(value = "无人集卡作业明细", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@GetMapping("/vehicleTypeCommandList")
	public SSIResponse vehicleTypeCommandList(String queryStartTime,String queryEndTime,String truckNo,
											  @RequestParam  Integer pageIndex, @RequestParam Integer pageSize) {
		try{
			return SSIResponse.ok(vehicleCommandService.vehicleTypeCommandList(queryStartTime,queryEndTime,truckNo,pageIndex,pageSize));
		}catch(Exception e){
			log.error(e.getMessage(),e);
			return SSIResponse.no("系统异常");
		}
	}

	/**
	 * 导出
	 */
	@LogAudit
	@ApiOperation(value = "导出列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/export")
	public void export(@RequestBody Map<String, Object> params, HttpServletResponse response) {
		String queryStartTime = Objects.nonNull(params.get("queryStartTime"))?String.valueOf(params.get("queryStartTime")):null;
		String queryEndTime = Objects.nonNull(params.get("queryEndTime"))?String.valueOf(params.get("queryEndTime")):null;
		String truckNo = Objects.nonNull(params.get("truckNo"))?String.valueOf(params.get("truckNo")):null;
		Integer pageIndex = Objects.nonNull(params.get("pageIndex"))?Integer.valueOf(params.get("pageIndex")+""):1;
		Integer pageSize =  Objects.nonNull(params.get("pageSize"))?Integer.valueOf(params.get("pageSize")+""):1000;
		String exportType = String.valueOf(params.get("exportType"));
		vehicleCommandService.export(queryStartTime,queryEndTime,truckNo,pageIndex,pageSize,exportType,response);
	}

	@LogAudit
	@ApiOperation(value = "远程上下电触发", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/trigger")
	public SSIResponse triggerEngine(@RequestBody JSONObject jsonObject){
		Assert.notNull(jsonObject.getString("vin"),"车辆VIN不能为空");
		Assert.notNull(jsonObject.getInteger("status"),"状态不能为空");
		String resultStr = RestTemplateUtil.post(odbuUrl.concat(URL.To_ODBU_TRIGGER), jsonObject.toJSONString(), null);
		JSONObject result = JSONObject.parseObject(resultStr);
		return SSIResponse.ok(result);
	}

	@ApiOperation(value = "车辆任务取消", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/cancelTask")
	public SSIResponse cancelTask(@RequestBody JSONObject jsonObject){
		Assert.notNull(jsonObject.getString("vin"),"车辆VIN不能为空");
		String resultStr = RestTemplateUtil.post(odbuUrl.concat(URL.To_ODBU_CANCEL_TASK), jsonObject.toJSONString(), null);
		JSONObject result = JSONObject.parseObject(resultStr);
		return SSIResponse.ok(result);
	}

	@ApiOperation(value = "车辆任务恢复", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/recoverTask")
	public SSIResponse recoverTask(@RequestBody JSONObject jsonObject){
		Assert.notNull(jsonObject.getString("vin"),"车辆VIN不能为空");
		String resultStr = RestTemplateUtil.post(odbuUrl.concat(URL.To_ODBU_RECOVERY_TASK), jsonObject.toJSONString(), null);
		JSONObject result = JSONObject.parseObject(resultStr);
		return SSIResponse.ok(result);
	}

	@ApiOperation(value = "车辆变道 4是左 5是右", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/changeRoad")
	public SSIResponse changeRoad(@RequestBody JSONObject jsonObject){
		Assert.notNull(jsonObject.getString("vin"),"车辆VIN不能为空");
		Assert.notNull(jsonObject.getString("taskOperation"),"车辆VIN不能为空");
		String resultStr = RestTemplateUtil.post(odbuUrl.concat(URL.To_ODBU_CHANGE_ROAD), jsonObject.toJSONString(), null);
		JSONObject result = JSONObject.parseObject(resultStr);
		return SSIResponse.ok(result);
	}

	@ApiOperation(value = "车辆任务状态", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/changeTaskStatus")
	public SSIResponse changeTaskStatus(@RequestBody JSONObject jsonObject){
		Assert.notNull(jsonObject.getString("vin"),"车辆VIN不能为空");
		Assert.notNull(jsonObject.getInteger("type"),"状态类型不能为空");
		Integer type = jsonObject.getInteger("type");
		String resultStr;
		if(type==1){
			resultStr = RestTemplateUtil.post(odbuUrl.concat(URL.To_ODBU_RECOVERY_TASK), jsonObject.toJSONString(), null);
		}else {
			resultStr = RestTemplateUtil.post(odbuUrl.concat(URL.To_ODBU_CANCEL_TASK), jsonObject.toJSONString(), null);
		}
		JSONObject result = JSONObject.parseObject(resultStr);
		return SSIResponse.ok(result);
	}


}
