package com.ssi.controller;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.response.SSIResponse;
import com.ssi.service.OperationAnalysisService;
import com.ssi.service.VehicleTroubleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Calendar;
import java.util.Map;

/**
 * 故障分析控制器
 */
@Api(tags = "故障分析控制器", description = "运行分析")
@Slf4j
@RestController
@RequestMapping("/faultAnalysis")
public class VehicleTroubleAnalysisController {

    @Autowired
    private VehicleTroubleService vehicleTroubleService;

    /**
     * 运输效率趋势图
     */
    @LogAudit
    @ApiOperation(value = "故障趋势图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/getFaultTrend")
    public SSIResponse getFaultTrend(@RequestBody Map<String, Object> params){
        try {
            return vehicleTroubleService.getFaultTrend(params);
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 停车等待时长分析
     */
    @LogAudit
    @ApiOperation(value = "故障分布图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/getFaultDistribution")
    public SSIResponse getFaultDistribution(@RequestBody Map<String, Object> params) {
        try {
            return vehicleTroubleService.getFaultDistribution(params);
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     *故障类型分布图
     */
    @LogAudit
    @ApiOperation(value = "故障类型分布图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/getFaultTypeTrend")
    public SSIResponse getFaultTypeTrend(@RequestBody Map<String, Object> params) {
        try {
            return vehicleTroubleService.getFaultTypeTrend(params);
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 故障等级分布图
     */
    @LogAudit
    @ApiOperation(value = "故障等级分布图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/getFaultLevelTrend")
    public SSIResponse getFaultLevelTrend(@RequestBody Map<String, Object> params) throws Exception{
        try {
            return vehicleTroubleService.getFaultLevelTrend(params);
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }
}
