package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsVehicleAlertHistory;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.service.VehicleTroubleService;


/**
 * 车辆故障控制器
 * @author zhang liyao
 * @email
 * @date 2020-07-17 09:47:00
 */
@Api
@Slf4j
@RestController
@RequestMapping("/vehicletroublehistory")
public class VehicleTroubleHistoryController {
    @Autowired
    private VehicleTroubleService vehicleTroubleService;

    /**
     * 故障列表分页
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vehicleTroubleService.queryPage(params);
        return SSIResponse.ok(page);
    }

    /**
     * 故障列表分页导出
     */
    @LogAudit
    @ApiOperation(value = "导出列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/export")
    public void export(@RequestBody Map<String, Object> params, HttpServletResponse response) {
         vehicleTroubleService.export(params,response);
  
    }

    /**
     * 故障详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam String id) {
        try {
            VmsVehicleAlertHistory vehicleTroubleHistory = vehicleTroubleService.getById(id);
            return SSIResponse.ok(vehicleTroubleHistory);
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsVehicleAlertHistory vehicleTroubleHistory) {
        try {
            vehicleTroubleService.save(vehicleTroubleHistory);
            return SSIResponse.ok();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsVehicleAlertHistory vehicleTroubleHistory) {
        try {
            vehicleTroubleService.saveOrUpdate(vehicleTroubleHistory);
            return SSIResponse.ok();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        try {
            vehicleTroubleService.removeByIds(Arrays.asList(ids.split(",")));
            return SSIResponse.ok();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

}
