package com.ssi.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsAlertThresholdNew;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsAlertThreshold;
import com.ssi.service.VmsAlertThresholdService;

import java.util.Map;


/**
 * 报警阈值控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-16 09:32:59
 */
@Api
@RestController
@RequestMapping("/alertThreshold")
@Slf4j
public class VmsAlertThresholdController {
    @Autowired
    private VmsAlertThresholdService vmsAlertThresholdService;
    @Autowired
    private RedisDataModel redisDataModel;


//    /**
//     * 报警阀值信息
//     */
//    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
//    @GetMapping("/info")
//    public SSIResponse info() {
//        VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>());
//        return SSIResponse.ok(current);
//    }

//    /**
//     * 保存
//     */
//    @PostMapping("/add")
//    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
//    public SSIResponse save(@RequestBody VmsAlertThreshold vmsAlertThreshold) {
//        if (vmsAlertThreshold.getId() == null) {
//            VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>());
//            if (current != null) {
//                vmsAlertThreshold.setId(current.getId());
//            }
//        }
//        vmsAlertThresholdService.saveOrUpdate(vmsAlertThreshold);
//        //保存到redis
//        redisDataModel.set("ivccs:vms:threshold",JSONObject.toJSONString(vmsAlertThreshold));
//
//        return SSIResponse.ok();
//    }

    /**
     * 报警阀值信息列表
     */
    @LogAudit
    @ApiOperation(value = "获得信息列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsAlertThresholdService.queryPage(params);
        return SSIResponse.ok(page);
    }

    /**
     * 报警阀值信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam String id) {
        try{
            VmsAlertThreshold current = vmsAlertThresholdService.getById(id);
            return SSIResponse.ok(current);
        }catch(Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsAlertThreshold vmsAlertThresholdNew) {
        VmsAlertThreshold info=new VmsAlertThreshold();
        info.setParamKey(vmsAlertThresholdNew.getParamKey());
        VmsAlertThreshold vmsAlertThresholdNew1=vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
        if (vmsAlertThresholdNew1!=null){
            return SSIResponse.no("key已存在,请重新设置");
        }
        vmsAlertThresholdService.save(vmsAlertThresholdNew);
        //保存到redis
        //redisDataModel.set("ivccs:vms:thresholdNew", JSONObject.toJSONString(vmsAlertThresholdNew));
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsAlertThreshold vmsAlertThresholdNew) {
        try {
            vmsAlertThresholdService.saveOrUpdate(vmsAlertThresholdNew);
            //保存到redis
            // redisDataModel.get("ivccs:vms:thresholdNew");
            return SSIResponse.ok();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        try {
            VmsAlertThreshold vmsAlertThresholdNew=new VmsAlertThreshold();
            vmsAlertThresholdNew.setId(Integer.parseInt(ids));
            vmsAlertThresholdNew.setIsDel(0);
            vmsAlertThresholdService.saveOrUpdate(vmsAlertThresholdNew);
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }
    /**
     * 修改无人集卡状态
     */
    @LogAudit
    @ApiOperation(value = "修改无人集卡状态", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/vehicleTypeSwitchUpdate")
    public SSIResponse vehicleTypeSwitchUpdate() {
        try{
            VmsAlertThreshold info=new VmsAlertThreshold();
            info.setParamKey("vehicle_type_switch");
            VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
            if (current.getParamValue().equals("1")){
                current.setParamValue("0");
            }else{
                current.setParamValue("1");
            }
            vmsAlertThresholdService.saveOrUpdate(current);
            redisDataModel.set("ivccs:vms:vehicle_type_switch",JSONObject.toJSONString(current));

            return SSIResponse.ok(current);
        }catch(Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }
    /**
     * 无人集卡状态详情
     */
    @LogAudit
    @ApiOperation(value = "无人集卡状态详情", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/vehicleTypeSwitchInfo")
    public SSIResponse vehicleTypeSwitchInfo() {
        try{
            VmsAlertThreshold info=new VmsAlertThreshold();
            info.setParamKey("vehicle_type_switch");
            VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
            return SSIResponse.ok(current);
        }catch(Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }
}
