package com.ssi.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsAlertThreshold;
import com.ssi.entity.VmsAlertThresholdNew;
import com.ssi.entity.VmsVehicleAlertHistory;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsAlertThresholdNewService;
import com.ssi.service.VmsAlertThresholdService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.velocity.runtime.directive.MacroParseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Map;

/**
 * 系统配置报警阈值控制器
 *
 * @author zhou ms
 * @email
 * @date 2022-03-25
 */
@Api
@RestController
@RequestMapping("/alertThresholdNew")
@Slf4j
public class VmsAlertThresholdNewController {
    @Autowired
    private VmsAlertThresholdService vmsAlertThresholdService;
    @Autowired
    private RedisDataModel redisDataModel;

    /**
     * 报警阀值信息列表
     */
    @LogAudit
    @ApiOperation(value = "获得信息列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsAlertThresholdService.queryPage(params);
        return SSIResponse.ok(page);
    }

    /**
     * 报警阀值信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam String id) {
        try{
            VmsAlertThreshold current = vmsAlertThresholdService.getById(id);
            return SSIResponse.ok(current);
        }catch(Exception e){
            log.error(e.getMessage(),e);
        }
       return null;
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsAlertThreshold vmsAlertThresholdNew) {
        VmsAlertThreshold info=new VmsAlertThreshold();
        info.setParamKey(vmsAlertThresholdNew.getParamKey());
        info.setIsDel(1);
        VmsAlertThreshold vmsAlertThresholdNew1=vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
       if (vmsAlertThresholdNew1!=null){
           return SSIResponse.no("key已存在,请重新设置");
       }
        vmsAlertThresholdService.save(vmsAlertThresholdNew);
        //保存到redis
        //redisDataModel.set("ivccs:vms:thresholdNew", JSONObject.toJSONString(vmsAlertThresholdNew));
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsAlertThreshold vmsAlertThresholdNew) {
        try {
            vmsAlertThresholdService.saveOrUpdate(vmsAlertThresholdNew);
            //保存到redis
           // redisDataModel.get("ivccs:vms:thresholdNew");
            return SSIResponse.ok();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        try {
            VmsAlertThreshold vmsAlertThresholdNew=new VmsAlertThreshold();
            vmsAlertThresholdNew.setId(Integer.parseInt(ids));
            vmsAlertThresholdNew.setIsDel(0);
            vmsAlertThresholdService.saveOrUpdate(vmsAlertThresholdNew);
            return SSIResponse.ok();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
        return null;
    }

}
