package com.ssi.controller;

import java.util.*;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsAreaPosition;
import com.ssi.entity.VmsPositionInfo;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsAreaPositionService;
import com.ssi.service.VmsPositionInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import javax.validation.Valid;

/**
 * 龙锁装卸区控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-16 09:32:59
 */
@Api
@RestController
@RequestMapping("/areaPosition")
public class VmsAreaPositionController {
    @Autowired
    private VmsAreaPositionService vmsAreaPositionService;
    @Autowired
    private VmsPositionInfoService vmsPositionInfoService;

    /**
     * 龙锁装卸区分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        Integer areaType = (Integer) params.get("areaType");
        String areaNum = (String) params.get("areaNum");
        List<VmsAreaPosition> list = vmsAreaPositionService.list(new LambdaQueryWrapper<VmsAreaPosition>()
                .eq(areaType != null, VmsAreaPosition::getAreaType, areaType)
                .like(StringUtils.isNotBlank(areaNum), VmsAreaPosition::getAreaNum, areaNum)
        );
        return SSIResponse.ok(list);
    }


    /**
     * 龙锁装卸区详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsAreaPosition vmsAreaPosition = vmsAreaPositionService.getById(id);
        return SSIResponse.ok(vmsAreaPosition);
    }

    /**
     * 添加龙锁装卸区
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody @Valid VmsAreaPosition vmsAreaPosition) {
        String areaNum = vmsAreaPosition.getAreaNum();
        Integer count = vmsAreaPositionService.lambdaQuery().eq(VmsAreaPosition::getAreaNum, areaNum).count();
        if(count > 0){
            return SSIResponse.no("编号已经存在");
        }
        vmsAreaPositionService.save(vmsAreaPosition);
        return SSIResponse.ok();
    }

    /**
     * 修改龙锁装卸区
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody @Valid VmsAreaPosition vmsAreaPosition) {
        String areaNum = vmsAreaPosition.getAreaNum();
        Integer id = vmsAreaPosition.getId();
        Integer count = vmsAreaPositionService.lambdaQuery().eq(VmsAreaPosition::getAreaNum, areaNum).ne(VmsAreaPosition::getId, id).count();
        if(count > 0){
            return SSIResponse.no("编号已经存在");
        }
        vmsAreaPositionService.saveOrUpdate(vmsAreaPosition);
        return SSIResponse.ok();
    }

    /**
     * 删除龙锁装卸区
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsAreaPositionService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }
    /**
     * 编号校验
     */
    @LogAudit
    @GetMapping("/checkRepeat")
    @ApiOperation(value = "编号校验", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse checkRepeat(@RequestParam String areaNum,Integer areaType) {
        int count = vmsAreaPositionService.count(new LambdaQueryWrapper<VmsAreaPosition>()
                .eq(StringUtils.isNotBlank(areaNum), VmsAreaPosition::getAreaNum, areaNum)
                .eq(areaType != null, VmsAreaPosition::getAreaType, areaType));
        if(count > 0){
            return SSIResponse.no("编号重复!");
        }
        return SSIResponse.ok();
    }

    /**
     * 堆场贝位列表
     */
    @LogAudit
    @GetMapping("/yardBayList")
    @ApiOperation(value = "堆场贝位列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse yardBayList(@RequestParam(required = false,name = "yardNum") String yardNum,@RequestParam(required = false,name = "size") String size) {
        List<VmsPositionInfo> list = vmsPositionInfoService.lambdaQuery()
                .eq(StringUtils.isNotBlank(yardNum), VmsPositionInfo::getYardNum, yardNum)
                .and(i-> i.like(VmsPositionInfo::getYardNum,"F%").or().like(VmsPositionInfo::getYardNum,"E%"))
                .list();
        List<VmsPositionInfo> distincts = list.stream().filter(e->{ if(StringUtils.isBlank(size)){return true;}else if("1".equalsIgnoreCase(size)){return Integer.parseInt(e.getBayNum())%2 == 0;}else { return Integer.parseInt(e.getBayNum())%2 == 1;}}).collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(o -> o.getYardNum() + ";" + o.getBayNum()))), ArrayList::new));
        Map<String, List<VmsPositionInfo>> collect = distincts.stream().sorted(Comparator.comparing(a -> Integer.valueOf(a.getBayNum()))).collect(Collectors.groupingBy(VmsPositionInfo::getYardNum));
        return SSIResponse.ok(collect);
    }
}
