package com.ssi.controller;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsCarControlCommand;
import com.ssi.entity.dto.CarControlCommandParam;
import com.ssi.entity.vo.CarControlCommandVo;
import com.ssi.response.SSIResponse;
import com.ssi.service.VehicleEmergencyService;
import com.ssi.service.VmsCarControlCommandService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import javax.servlet.http.HttpServletResponse;


/**
 * 远程控车指令控制器
 *
 * @author liheng
 * @email
 * @date 2020-07-24 10:14:56
 */
@Api(tags = "远程指令清单控制器",description = "远程指令清单")
@RestController
@RequestMapping("/carControlCommand")
public class VmsCarControlCommandController {
    @Autowired
    private VmsCarControlCommandService carControlCommandService;
    @Autowired
    private VehicleEmergencyService vehicleEmergencyService;

    /**
     * 控车指令
     */
    @LogAudit
    @ApiOperation(value = "控车指令", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/executeCommand")
    public SSIResponse executeCommand(@RequestBody @Validated CarControlCommandParam param){
        return carControlCommandService.executeCommand(param);
    }

    /**
     * 控车指令分页列表
     */
    @LogAudit
	@ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody CarControlCommandVo carControlCommandVo){
        return SSIResponse.ok(carControlCommandService.queryPage(carControlCommandVo));
    }

    /**
     * 信息
     */
    @LogAudit
	@ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id){
        VmsCarControlCommand carControlCommand = carControlCommandService.getById(id);
        return SSIResponse.ok(carControlCommand);
    }

    /**
     * 导出控车指令历史
     */
    @LogAudit
    @ApiOperation(value = "导出列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/export")
    public void export(@RequestBody CarControlCommandVo carControlCommandVo, HttpServletResponse response) {
        carControlCommandService.export(carControlCommandVo,response);
    }

    /**
     * 步进接口
     * @param vin 车辆vin
     * @param direction 方向 1-前进 ；2-后退
     * @param distance 行驶距离(单位：CM)
     */
    @LogAudit
    @ApiOperation(value = "步进接口", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/stepping")
    public SSIResponse stepping(@RequestParam String vin, @RequestParam Integer direction,@RequestParam Integer distance){
        return carControlCommandService.stepping(vin, direction,distance);
    }

    /**
     * 紧急停车
     */
    @LogAudit
    @ApiOperation(value = "紧急停车", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/emergency")
    public SSIResponse emergency(@RequestParam String vin, @RequestParam Integer emergencyType){
        return vehicleEmergencyService.emergency(vin, emergencyType);
    }
}
