package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsChargingConnectorInfo;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsChargingConnectorInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsChargingPile;
import com.ssi.service.VmsChargingPileService;


/**
 * 充电桩控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-09 09:21:59
 */
@Api
@RestController
@RequestMapping("/chargingPile")
public class VmsChargingPileController {
    @Autowired
    private VmsChargingPileService vmsChargingPileService;
    @Autowired
    private VmsChargingConnectorInfoService chargingConnectorInfoService;
    /**
     * 分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = chargingConnectorInfoService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsChargingConnectorInfo vmsChargingPile = chargingConnectorInfoService.getById(id);
        return SSIResponse.ok(vmsChargingPile);
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsChargingConnectorInfo vmsChargingPile) {
        chargingConnectorInfoService.save(vmsChargingPile);
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsChargingConnectorInfo vmsChargingPile) {
        chargingConnectorInfoService.saveOrUpdate(vmsChargingPile);
        return SSIResponse.ok();
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        chargingConnectorInfoService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/checkIdentity")
    @ApiOperation(value = "校验唯一", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse checkIdentity(@RequestParam(name = "pileNum") String pileNum ,@RequestParam(name="gunNum",required = false) String gunNum) {
        Integer count = chargingConnectorInfoService.lambdaQuery().eq(StringUtils.isNotBlank(pileNum), VmsChargingConnectorInfo::getEquipmentId, pileNum)
                .eq(StringUtils.isNotBlank(gunNum), VmsChargingConnectorInfo::getConnectorId, gunNum).count();
        if(count>0){
            if(StringUtils.isNotBlank(gunNum)){
                return SSIResponse.no("充电枪编号已存在");
            }
            return SSIResponse.no("充电桩编号已存在");
        }
        return SSIResponse.ok();
    }
}
