package com.ssi.controller;

import java.util.*;

import com.alibaba.fastjson.JSONObject;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsCranePadBind;
import com.ssi.entity.map.MapDataDto;
import com.ssi.entity.vo.CranePadBindVo;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsCranePadBindService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 港机pad绑定控制器
 *
 * @author liheng
 * @email
 * @date 2020-08-10 16:29:29
 */
@Api
@RestController
@RequestMapping("/cranePadBind")
public class VmsCranePadBindController {
    @Autowired
    private VmsCranePadBindService vmsCranePadBindService;
    @Autowired
    private RedisDataModel redisDataModel;

    /**
     * 吊具pad绑定列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody CranePadBindVo cranePadBindVo) {
        return SSIResponse.ok(vmsCranePadBindService.queryPage(cranePadBindVo));
    }


    /**
     * 吊具pad绑定详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsCranePadBind vmsCranePadBind = vmsCranePadBindService.getById(id);
        return SSIResponse.ok(vmsCranePadBind);
    }

    /**
     * 保存吊具pad绑定关系
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse add(@RequestBody @Validated VmsCranePadBind vmsCranePadBind) {

        Integer padNoCount = vmsCranePadBindService.lambdaQuery().eq(VmsCranePadBind::getCraneNo, vmsCranePadBind.getCraneNo()).count();
        if (padNoCount > 0) {
            return SSIResponse.no("设备编号已存在");
        }
        vmsCranePadBind.setCreateTime(new Date());
        vmsCranePadBindService.save(vmsCranePadBind);
        redisDataModel.set("ivccs:vms:cranePadBind", JSONObject.toJSONString(vmsCranePadBindService.list()));
        return SSIResponse.ok();
    }

    /**
     * 修改吊具pad绑定关系
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsCranePadBind vmsCranePadBind) {
     
        Integer padNoCount = vmsCranePadBindService.lambdaQuery()
                .eq(VmsCranePadBind::getCraneNo, vmsCranePadBind.getCraneNo())
                .ne(VmsCranePadBind::getId,vmsCranePadBind.getId())
                .count();
        if (padNoCount > 0) {
            return SSIResponse.no("设备编号已存在");
        }
        vmsCranePadBindService.updateById(vmsCranePadBind);
        redisDataModel.set("ivccs:vms:cranePadBind", JSONObject.toJSONString(vmsCranePadBindService.list()));
        return SSIResponse.ok();
    }

    /**
     * 删除吊具pad绑定
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsCranePadBindService.removeByIds(Arrays.asList(ids.split(",")));
        redisDataModel.set("ivccs:vms:cranePadBind", JSONObject.toJSONString(vmsCranePadBindService.list()));//放入redis缓存
        return SSIResponse.ok();
    }

    /**
     * 根据mac获取桥吊信息
     */
    @LogAudit
    @GetMapping("/getCraneByMac")
    @ApiOperation(value = "根据mac获取桥吊信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getCraneByMac(@RequestParam String padMac) {
        return vmsCranePadBindService.getCraneByMac(padMac);
    }

    /**
     * 保存和更新扭锁区关联桥吊
     * @param mapdata
     * @return
     */
    @LogAudit
    @PostMapping("/save")
    @ApiOperation(value = "保存和更新扭锁区关联桥吊", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse saveCraneBind(@RequestBody MapDataDto mapdata) { 
        //符合扭锁区更新逻辑
        vmsCranePadBindService.updateCraneBind(mapdata.getCraneIds(),mapdata.getId(),mapdata.getNumber(),mapdata.getBridgeNo());
        return  SSIResponse.ok();
    }
}
