package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsDebugApp;
import com.ssi.service.VmsDebugAppService;

import javax.validation.Valid;


/**
 * 调试APP控制器
 *
 * @author zhang liyao
 * @email 
 * @date 2020-09-01 08:36:06
 */
@Api
@RestController
@RequestMapping("/debugApp")
public class VmsDebugAppController {
    @Autowired
    private VmsDebugAppService vmsDebugAppService;

    /**
     * 调试APP分页列表
     */
    @LogAudit
	@ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params){
        SSIPage page = vmsDebugAppService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 调试APP详细信息
     */
    @LogAudit
	@ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id){
        VmsDebugApp vmsDebugApp = vmsDebugAppService.getById(id);
        return SSIResponse.ok(vmsDebugApp);
    }

    /**
     * 保存调试APP信息
     */
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse add(@RequestBody @Valid VmsDebugApp vmsDebugApp){
        if(!checkMacRepeat(vmsDebugApp)){
            return SSIResponse.no("mac地址不能重复");
        }
        vmsDebugAppService.save(vmsDebugApp);
        return SSIResponse.ok();
    }

    /**
     * 修改调试APP信息
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody @Valid VmsDebugApp vmsDebugApp){
        if(!checkMacRepeat(vmsDebugApp)){
            return SSIResponse.no("mac地址不能重复");
        }
        vmsDebugAppService.saveOrUpdate(vmsDebugApp);
        return SSIResponse.ok();
    }


    private boolean checkMacRepeat(VmsDebugApp vmsDebugApp) {
        VmsDebugApp getExist = vmsDebugAppService.getOne(new LambdaQueryWrapper<VmsDebugApp>().eq(VmsDebugApp::getAppMac, vmsDebugApp.getAppMac()));
        if(vmsDebugApp.getId() == null && getExist != null){
            return false;
        }if(vmsDebugApp.getId() != null && getExist != null && vmsDebugApp.getId() != getExist.getId()){
            return false;
        }
        return true;
    }

    /**
     * 删除调试APP信息
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids){
        vmsDebugAppService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 获取密钥
     */
    @LogAudit
    @GetMapping("/getKey")
    @ApiOperation(value = "获取密钥", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getKey(){
        String key = vmsDebugAppService.getKey();
        return SSIResponse.ok(key);
    }

    /**
     * 修改密钥
     */
    @GetMapping("/updateKey")
    @ApiOperation(value = "修改密钥", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse updateKey(@RequestParam String key){
        vmsDebugAppService.updateKey(key);
        return SSIResponse.ok();
    }

    /**
     * 密钥校验
     */
    @LogAudit
    @GetMapping("/checkApp")
    @ApiOperation(value = "密钥校验", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse checkApp(@RequestParam String key,@RequestParam String appMac){
        return vmsDebugAppService.checkApp(key,appMac);
    }

    /**
     * 修改遥控接管最大距离
     */
    @LogAudit
    @GetMapping("/updateDiStance")
    @ApiOperation(value = "修改遥控接管最大距离", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse updateDiStance(@RequestParam Integer distance){
        vmsDebugAppService.updateDiStance(distance);
        return SSIResponse.ok();
    }

    /**
     * 获取遥控接管最大距离
     */
    @LogAudit
    @GetMapping("/getDistance")
    @ApiOperation(value = "获取遥控接管最大距离", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getDistance(){
        Integer distance = vmsDebugAppService.getDistance();
        return SSIResponse.ok(distance);
    }

}
