package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsDeviceVersion;
import com.ssi.service.VmsDeviceVersionService;


/**
 * 设备版本控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-20 14:01:40
 */
@Api
@RestController
@RequestMapping("/deviceVersion")
public class VmsDeviceVersionController {
    @Autowired
    private VmsDeviceVersionService vmsDeviceVersionService;

    /**
     * 设备版本分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsDeviceVersionService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsDeviceVersion vmsDeviceVersion = vmsDeviceVersionService.getById(id);
        return SSIResponse.ok(vmsDeviceVersion);
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsDeviceVersion vmsDeviceVersion) {
        vmsDeviceVersionService.save(vmsDeviceVersion);
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsDeviceVersion vmsDeviceVersion) {
        vmsDeviceVersionService.saveOrUpdate(vmsDeviceVersion);
        return SSIResponse.ok();
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsDeviceVersionService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

}
