package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.google.common.collect.Maps;
import com.ssi.constant.enums.EventTypeEnums;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsEvent;
import com.ssi.service.VmsEventService;


/**
 * 事件控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-20 13:12:42
 */
@Api
@RestController
@RequestMapping("/event")
public class VmsEventController {
    @Autowired
    private VmsEventService vmsEventService;

    /**
     * 事件分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsEventService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 事件详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsEvent event = vmsEventService.getById(id);
        return SSIResponse.ok(event);
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsEvent event) {
        event.setEventSource(0);
        vmsEventService.save(event);
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsEvent event) {
        event.setEventSource(0);
        vmsEventService.saveOrUpdate(event);
        return SSIResponse.ok();
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsEventService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 获取事件类型
     */
    @LogAudit
    @GetMapping("/getEventType")
    @ApiOperation(value = "获取事件类型", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getEventType(){
        Map<String,String> resultMap = Maps.newHashMap();
        EventTypeEnums[] values = EventTypeEnums.values();
        for(EventTypeEnums enums : EventTypeEnums.values()){
            resultMap.put(enums.getCode(),enums.getDescript());
        }
        return SSIResponse.ok(resultMap);
    }

}
