package com.ssi.controller;


import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Map;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.kafka.listener.FaultCodeListener;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsFaultCode;
import com.ssi.service.VmsFaultCodeService;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


/**
 * 故障码控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-16 13:36:49
 */
@Api
@RestController
@RequestMapping("/faultCode")
public class VmsFaultCodeController {
    @Autowired
    private VmsFaultCodeService vmsFaultCodeService;
    @Value("${LinuxFile.faultCodePath}")
    private String faultCodePath;

    /**
     * 故障码列表分页
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsFaultCodeService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 故障码详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsFaultCode vmsFaultCode = vmsFaultCodeService.getById(id);
        return SSIResponse.ok(vmsFaultCode);
    }

    /**
     * 添加故障码
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody @Valid VmsFaultCode vmsFaultCode) {
        Integer count = vmsFaultCodeService.lambdaQuery().eq(VmsFaultCode::getFmi, vmsFaultCode.getFmi())
                .eq(VmsFaultCode::getSpn, vmsFaultCode.getSpn())
                .eq(VmsFaultCode::getSa, vmsFaultCode.getSa())
                .count();
        if (count > 0) {
            return SSIResponse.no("相同FMI SPN SA的故障编码已经存在");
        }
        vmsFaultCodeService.save(vmsFaultCode);
        return SSIResponse.ok();
    }

    /**
     * 修改故障码
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody @Valid VmsFaultCode vmsFaultCode) {
        Integer count = vmsFaultCodeService.lambdaQuery().eq(VmsFaultCode::getFmi, vmsFaultCode.getFmi())
                .eq(VmsFaultCode::getSpn, vmsFaultCode.getSpn())
                .eq(VmsFaultCode::getSa, vmsFaultCode.getSa())
                .ne(VmsFaultCode::getId, vmsFaultCode.getId())
                .count();
        if (count > 0) {
            return SSIResponse.no("相同FMI SPN SA的故障编码已经存在");
        }
        vmsFaultCodeService.saveOrUpdate(vmsFaultCode);
        return SSIResponse.ok();
    }

    /**
     * 删除故障码
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsFaultCodeService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    @LogAudit
    @RequestMapping(value = "/template", method = RequestMethod.GET)
    @ApiOperation(value = "下载模板")
    public void downloadExcle(HttpServletResponse response) throws Exception {
//        String fileName = this.getClass().getClassLoader().getResource("crm2-template.xlsx").getPath();
//    	String fileName=System.getProperty("user.dir")+File.separator+"faultCode-template.xlsx";
        InputStream in = new FileInputStream(faultCodePath);
        vmsFaultCodeService.downloadExcel(in, response, "故障码模板.xlsx");
    }

    @Autowired
    private FaultCodeListener faultCodeListener;

    /**
     * 导入故障码
     */
    @LogAudit
    @PostMapping("/importList")
    @ApiOperation(value = "导入故障码", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse importList(MultipartFile file) {

        return vmsFaultCodeService.importList(file);
    }

    /**
     * 导入故障码
     */
    @LogAudit
    @PostMapping("/testPushFunction")
    public SSIResponse<Void> testPushFunction(@RequestBody JSONObject obj) {
//        faultCodeListener.alertFaultCode(new ConsumerRecord<String,String>("test",0,1,obj.getString("vin"),obj.toJSONString()));
        return null;
    }



}
