package com.ssi.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.vo.VmsRoadDeviceVo;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsRoadDevice;
import com.ssi.service.VmsRoadDeviceService;


/**
 * 路测设备控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-09 13:37:13
 */
@Api
@RestController
@RequestMapping("/roadDevice")
public class VmsRoadDeviceController {
    @Autowired
    private VmsRoadDeviceService vmsRoadDeviceService;

    /**
     * 列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsRoadDeviceService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsRoadDevice vmsRoadDevice = vmsRoadDeviceService.getById(id);
        return SSIResponse.ok(vmsRoadDevice);
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsRoadDevice vmsRoadDevice) {
        vmsRoadDeviceService.save(vmsRoadDevice);
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsRoadDevice vmsRoadDevice) {
        vmsRoadDeviceService.saveOrUpdate(vmsRoadDevice);
        return SSIResponse.ok();
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsRoadDeviceService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 视频播放列表
     */
    @LogAudit
    @ApiOperation(value = "视频播放列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/videoPlayUrlList")
    public SSIResponse videoPlayUrlList() {
        List<VmsRoadDeviceVo> list = vmsRoadDeviceService.videoPlayUrlList();
        return SSIResponse.ok(list);
    }

    /**
     * 路测安全消息信息
     */
    @LogAudit
    @ApiOperation(value = "路测安全消息信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getRsmInfo")
    public SSIResponse getRsmInfo(Long id) {
        Map info = vmsRoadDeviceService.getRsmInfo(id);
        return SSIResponse.ok(info);
    }
}
