package com.ssi.controller;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.DcvAllowGoShipLaneTxVo;
import com.ssi.entity.VmsShipInfo;
import com.ssi.entity.VmsShipsDrawing;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsShipsDrawingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 船图控制器
 *
 * @author lvyanfeng
 * @email
 * @date 2020-07-08 15:38:37
 */
@Api
@RestController
@RequestMapping("/shipsDrawing")
public class VmsShipsDrawingController {
	@Autowired
	private VmsShipsDrawingService vmsShipsDrawingService;

	/**
	 * 船图接口
	 */
	@LogAudit
	@PostMapping("/ShipsDrawinglist")
	@ApiOperation(value = "查询船图", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	public SSIResponse<VmsShipsDrawing> ShipsDrawinglist(@RequestBody Map<String, Object> params) {
		VmsShipsDrawing vehicleList = vmsShipsDrawingService.getVmsShipsDrawing(params);
		return SSIResponse.ok(vehicleList);
	}

	/**
	 * 船图贝位列表
	 */
	@LogAudit
	@PostMapping("/ShipsDrawingBayNolist")
	@ApiOperation(value = "查询船图贝位列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	public SSIResponse<?> ShipsDrawingBayNolist(@RequestBody Map<String, Object> params) {
//       System.out.println("----shipCod"+shipCod);
//        List<VmsShipsDrawing> vehicleList = vmsShipsDrawingService.getVmsShipsDrawingBayNo(shipCod,bayNo);
		SSIPage page = vmsShipsDrawingService.getVmsShipsDrawingBayNo(params);
		return SSIResponse.ok(page);
	}

	/**
	 * 船信息列表
	 */
	@LogAudit
	@ApiOperation(value = "获得船列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/ShipInfoList")
	public SSIResponse<?> ShipInfoList(@RequestBody Map<String, Object> params) {
		SSIPage page = vmsShipsDrawingService.queryPage(params);
		return SSIResponse.ok(page);
	}

	/**
	 * 船信息列表
	 */
	@LogAudit
	@ApiOperation(value = "获得船信息及坐标信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/ShipInfoCoor")
	public SSIResponse<?> ShipInfoCoor(@RequestBody Map<String, Object> params) {
		List<VmsShipInfo> list = vmsShipsDrawingService.getShipInfoCoor();
		return SSIResponse.ok(list);
	}

	/**
	 * 根据桥吊获取船的编号
	 */
	@LogAudit
	@ApiOperation(value = "根据桥吊获取船的编号", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/ShipInfoCod")
	public SSIResponse<?> ShipInfoCod(@RequestBody Map<String, Object> params) {
//		String BridgeNum = (String) params.get("BridgeNum");
//		Map<String, Object> map=new HashMap<String, Object>();
		VmsShipInfo shipInfo = new VmsShipInfo();
		double BridgeXpoint = (double) params.get("BridgeXpoint");
		List<VmsShipInfo> list = vmsShipsDrawingService.getShipInfoCoor();
		for (VmsShipInfo ship : list) {
			if (null != ship.getBegcoor() && null != ship.getEndcoor()) {
				String[] begCoor = ship.getBegcoor().split(",");
				String[] endCoor = ship.getEndcoor().split(",");
				if (ship.getBerthWay().equals("L")) {
					if (BridgeXpoint < Double.parseDouble(begCoor[0])
							&& BridgeXpoint > Double.parseDouble(endCoor[0])) {
						shipInfo = ship;
					}
				}
				if (ship.getBerthWay().equals("R")) {
					if (BridgeXpoint > Double.parseDouble(begCoor[0])
							&& BridgeXpoint < Double.parseDouble(endCoor[0])) {
						shipInfo = ship;
					}
				}
			}
		}

		//查询该船在指定桥吊下 正在作业的贝位, 默认 000
		String bayNo = "000";
		if(shipInfo.getVoyageNo() != null && params.get("portCode") != null){
			bayNo = vmsShipsDrawingService.getShipWorkingBay(shipInfo.getVoyageNo(), params.get("portCode").toString());
		}
		shipInfo.setWorkingBay(bayNo);

		return SSIResponse.ok(shipInfo);
	}

	/**
	 * 获取缆装
	 */
	@LogAudit
	@ApiOperation(value = "获取缆装", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/cableList")
	public SSIResponse<?> cableList() {
		return SSIResponse.ok(vmsShipsDrawingService.getCableList());
	}

	/**
	 * 保存箱体上船信息
	 */
	@LogAudit
	@ApiOperation(value = "强制上船与禁止上船", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/saveShipTx")
	public SSIResponse<?> saveShipTx(@RequestBody DcvAllowGoShipLaneTxVo shipList) {
		if (shipList.getShipList().size() == 0) {
			return SSIResponse.no("请选择箱体！");
		}

		return SSIResponse.ok(vmsShipsDrawingService.saveShipTx(shipList));
	}

}
