package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsTerminal;
import com.ssi.service.VmsTerminalService;

/**
 * 终端控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-16 09:32:59
 */
@Api
@RestController
@RequestMapping("/terminal")
public class VmsTerminalController {
    @Autowired
    private VmsTerminalService vmsTerminalService;

    /**
     * 终端信息分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsTerminalService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 终端详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsTerminal vmsTerminal = vmsTerminalService.getById(id);
        return SSIResponse.ok(vmsTerminal);
    }

    /**
     * 保存终端信息
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsTerminal vmsTerminal) {
        return vmsTerminalService.saveTerminal(vmsTerminal);
    }

    /**
     * 修改终端信息
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsTerminal vmsTerminal) {
        return vmsTerminalService.saveTerminal(vmsTerminal);
    }

    /**
     * 删除终端信息
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsTerminalService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 视频监控
     */
    @LogAudit
    @GetMapping("/videoMonitor")
    @ApiOperation(value = "视频监控", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse videoMonitor(String vin) {
        return vmsTerminalService.videoMonitor(vin);
    }


}
