package com.ssi.controller;

import java.util.Arrays;
import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsTerminalVersion;
import com.ssi.service.VmsTerminalVersionService;


/**
 * 终端版本控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-16 13:36:49
 */
@Api
@RestController
@RequestMapping("/terminalVersion")
public class VmsTerminalVersionController {
    @Autowired
    private VmsTerminalVersionService vmsTerminalVersionService;

    /**
     * 终端版本分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        SSIPage page = vmsTerminalVersionService.queryPage(params);
        return SSIResponse.ok(page);
    }


    /**
     * 终端版本信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsTerminalVersion vmsTerminalVersion = vmsTerminalVersionService.getById(id);
        return SSIResponse.ok(vmsTerminalVersion);
    }

    /**
     * 保存终端版本信息
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsTerminalVersion vmsTerminalVersion) {
        if(!checkVersionNumRepeat(vmsTerminalVersion)){
            return SSIResponse.no("版本号不能重复");
        }
        vmsTerminalVersionService.save(vmsTerminalVersion);
        return SSIResponse.ok();
    }

    /**
     * 修改终端版本信息
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsTerminalVersion vmsTerminalVersion) {
        if(!checkVersionNumRepeat(vmsTerminalVersion)){
            return SSIResponse.no("版本号不能重复");
        }
        vmsTerminalVersionService.saveOrUpdate(vmsTerminalVersion);
        return SSIResponse.ok();
    }

    private boolean checkVersionNumRepeat(VmsTerminalVersion vmsTerminalVersion) {
        VmsTerminalVersion getExist = vmsTerminalVersionService.getOne(new LambdaQueryWrapper<VmsTerminalVersion>().eq(VmsTerminalVersion::getTerminalVersionNum, vmsTerminalVersion.getTerminalVersionNum()));
        if(vmsTerminalVersion.getId() == null && getExist != null){
            return false;
        }if(vmsTerminalVersion.getId() != null && getExist != null && vmsTerminalVersion.getId() != getExist.getId()){
            return false;
        }
        return true;
    }

    /**
     * 删除终端版本信息
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsTerminalVersionService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

}
