package com.ssi.controller;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.constant.enums.VmsTosOrderStatusEnum;
import com.ssi.entity.vo.VmsTosOrdersKPIVo;
import com.ssi.entity.vo.VmsTosOrdersVo;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.*;
import com.ssi.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import com.ssi.entity.VmsTosOrders;

import javax.servlet.http.HttpServletResponse;


/**
 * TOS任务指令控制器
 * @author zhang liyao
 * @email
 * @date 2020-07-11 13:12:01
 */
@Api(tags = "任务清单控制器",description = "任务清单")
@RestController
@RequestMapping("/vmsTosOrders")
public class VmsTosOrdersController {
    @Autowired
    private VmsTosOrdersService vmsTosOrdersService;
    @Autowired
    private VmsTosOriginalOrdersService vmsTosOriginalOrdersService;
    @Autowired
    private VmsOrdersVehicleAlterService vmsOrdersVehicleAlterService;
    @Autowired
    private VmsTosOrdersOeeService vmsTosOrdersOeeService;
    @Autowired
    private VmsTosOrderDailyKpiService vmsTosOrderDailyKpiService;


    /**
     * 任务详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsTosOrders vmsTosOrders = vmsTosOrdersService.getById(id);
        return SSIResponse.ok(vmsTosOrders);
    }

    /**
     * 保存
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsTosOrders vmsTosOrders) {
        vmsTosOrdersService.save(vmsTosOrders);
        return SSIResponse.ok();
    }

    /**
     * 修改
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsTosOrders vmsTosOrders) {
        vmsTosOrdersService.saveOrUpdate(vmsTosOrders);
        return SSIResponse.ok();
    }

    /**
     * 删除
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsTosOrdersService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 获取车辆当前任务
     */
    @LogAudit
    @GetMapping("/getOrdersByVin")
    @ApiOperation(value = "获取车辆当前任务", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getOrdersByVin(@RequestParam String vin) {
        return SSIResponse.ok(vmsTosOrdersService.getOrdersByVin(vin));
    }

    /**
     * 车辆行程查询
     */
    @LogAudit
    @GetMapping("/getVehicleTrip")
    @ApiOperation(value = "车辆行程查询", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getVehicleTrip(@RequestParam String vin, Long startTime, Long stopTime, String faultGrade) {
        return vmsTosOrdersService.getVehicleTrip(vin, startTime, stopTime, faultGrade);
    }

    /**
     * 根据时间查询车辆能耗里程
     */
    @LogAudit
    @GetMapping("/getMileTimeByTime")
    @ApiOperation(value = "根据时间查询车辆能耗里程", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getMileTimeByTime(@RequestParam String vin, Long startTime, Long stopTime) {
        return vmsTosOrdersService.getMileTimeByTime(vin, startTime, stopTime);
    }

    /**
     * 根据异常报警查询行程
     */
    @LogAudit
    @GetMapping("/getTripByAlert")
    @ApiOperation(value = "根据异常报警查询行程", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getTripByAlert(@RequestParam String vin, String alertId ) {
        return vmsTosOrdersService.getTripByAlert(vin, alertId);
    }

    /**
     * 列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody VmsTosOrdersVo vmsTosOrdersVo) {
        SSIPage page = vmsTosOrdersService.queryPage(vmsTosOrdersVo);
        return SSIResponse.ok(page);
    }

    /**
     * 列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/listOriginalOrder")
    public SSIResponse listOriginalOrder(@RequestBody VmsTosOrdersVo vmsTosOrdersVo) {
        SSIPage page = vmsTosOriginalOrdersService.queryPageByCondition(vmsTosOrdersVo);
        return SSIResponse.ok(page);
    }


    /**
     * 搬运集装箱的列表
     */
    @LogAudit
    @ApiOperation(value = "搬运集装箱的列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/listForContainerTask")
    public SSIResponse listForContainerTask(@RequestBody VmsTosOrdersVo vmsTosOrdersVo) {
        vmsTosOrdersVo.setVehicleTaskLabels(Arrays.asList(1,2,3));
        SSIPage page = vmsTosOrdersService.queryPageByAllStatus(vmsTosOrdersVo);
        return SSIResponse.ok(page);
    }

    /**
     * 导出
     */
    @LogAudit
    @ApiOperation(value = "导出列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/export")
    public void export(@RequestBody VmsTosOrdersVo vmsTosOrdersVo, HttpServletResponse response) {
        vmsTosOrdersService.export(vmsTosOrdersVo,response);
    }

    /**
     * 获取当前pad对应桥吊可选车辆
     */
    @LogAudit
    @GetMapping("/getVehicleByPadMac")
    @ApiOperation(value = "获取当前pad对应桥吊可选车辆", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getVehicleByPadMac(@RequestParam String padMac,@RequestParam String taskNo) {
        return vmsTosOrdersService.getVehicleByPadMac(padMac,taskNo);
    }

    /**
     * 确认任务车辆
     */
    @LogAudit
    @GetMapping("/confirmOrdersVehicle")
    @ApiOperation(value = "确认任务车辆", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse confirmOrdersVehicle(@RequestParam String taskNo,@RequestParam String vin) {
        return vmsOrdersVehicleAlterService.changeOrdersVehicle(taskNo,vin);
    }

    /**
     * 查询任务status字段含义
     */
    @LogAudit
    @GetMapping("/getStatusConfig")
    @ApiOperation(value = "查询任务status字段含义", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getStatusConfig() {
        VmsTosOrderStatusEnum[] values = VmsTosOrderStatusEnum.values();
        return SSIResponse.ok(values);
    }

    /**
     * 查询任务状态历史
     */
    @LogAudit
    @GetMapping("/getTaskHistory")
    @ApiOperation(value = "查询任务状态历史", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getTaskHistory(String taskNo) {
        return vmsTosOrdersService.getTaskHistory(taskNo);
    }

    /**
     * 进行中的任务列表
     */
    @LogAudit
    @ApiOperation(value = "进行中的任务列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/listInProcess")
    public SSIResponse listInProcess(@RequestBody(required = false) VmsTosOrdersVo vmsTosOrdersVo) {
        SSIPage page = vmsTosOrdersService.listInProcess(vmsTosOrdersVo);
        return SSIResponse.ok(page);
    }

    /**
     * 实时任务列表
     */
    @LogAudit
    @ApiOperation(value = "实时任务列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/realTimeTasks")
    public SSIResponse realTimeTasks(@RequestBody(required = false) VmsTosOrdersVo vmsTosOrdersVo) {
        SSIPage page = vmsTosOrdersService.realTimeTasks(vmsTosOrdersVo);
        return SSIResponse.ok(page);
    }

    /**
     * 获取某一天的任务KPI
     */
    @LogAudit
    @ApiOperation(value = "获取某一天的任务KPI", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getDailyKpi")
    public SSIResponse getDailyKpi(Date queryDate,String vehicleNum) {
        if(queryDate == null){
            queryDate = new Date();
        }
        LocalDateTime startTime = LocalDateTime.of(queryDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MIN);
        LocalDateTime endTime = LocalDateTime.of(queryDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), LocalTime.MAX);
        List list = vmsTosOrdersService.getDailyKpi(startTime,endTime,vehicleNum);
        return SSIResponse.ok(list);
    }


    /**
     * 获取某一天的任务 KPI
     * @param queryDate
     * @param vehicleNum
     * @return
     */
    @LogAudit
    @ApiOperation(value = "获取某一天的任务KPI查询预处理的表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/getOrderDailyKpi")
    public SSIResponse getOrderDailyKpi(Date queryDate,String vehicleNum) {
        if(queryDate == null){
            queryDate = new Date();
        }

        List<Map<String, String>> orderDailyKpi = vmsTosOrderDailyKpiService.getOrderDailyKpi(DateUtils.format(queryDate, DateUtils.DATE_PATTERN), vehicleNum);
        return SSIResponse.ok(orderDailyKpi);
    }

    /**
     * 任务KPI-效率OEE分析
     */
    @LogAudit
    @ApiOperation(value = "任务KPI-效率OEE分析", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/taskKPI")
    public SSIResponse taskKPI(@RequestBody VmsTosOrdersKPIVo vmsTosOrders) throws Exception{
        //List list = vmsTosOrdersService.taskKPI(vmsTosOrders);
        List list = vmsTosOrdersOeeService.taskOee(vmsTosOrders);
        return SSIResponse.ok(list);
    }

    /**
     * 车管平台强制取消
     */
    @LogAudit
    @ApiOperation(value = "车管平台强制取消", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/cancel")
    public SSIResponse cancel(String taskNo) throws Exception{
        return vmsTosOrdersService.forceCancel(taskNo);
    }
}
