package com.ssi.controller;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.vo.VehicleControlStatusVo;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VehicleEmergencyService;
import com.ssi.service.VehicleRemoteInstructionService;
import com.ssi.service.VmsDebugAppService;
import com.ssi.service.VmsVehicleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * 车辆控制器
 *
 * @author zhang liyao
 * @email
 * @date 2020-07-08 15:38:37
 */
@Api
@RestController
@RequestMapping("/vehicle")
public class VmsVehicleController {
    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private VmsDebugAppService vmsDebugAppService;
    @Autowired
    private VehicleEmergencyService vehicleEmergencyService;
    @Autowired
    private VehicleRemoteInstructionService remoteInstructionService;


    /**
     * 车辆分页列表
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody Map<String, Object> params) {
        return SSIResponse.ok(vmsVehicleService.queryPage(params));
    }


    /**
     * 车辆详细信息
     */
    @LogAudit
    @ApiOperation(value = "获得单条详细信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @GetMapping("/info")
    public SSIResponse info(@RequestParam Integer id) {
        VmsVehicle vehicle = vmsVehicleService.getById(id);
        return SSIResponse.ok(vehicle);
    }

    /**
     * 保存车辆
     */
    @LogAudit
    @PostMapping("/add")
    @ApiOperation(value = "保存信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse save(@RequestBody VmsVehicle vehicle) {
        return vmsVehicleService.saveVehicle(vehicle);
    }

    /**
     * 修改车辆信息
     */
    @LogAudit
    @PostMapping("/update")
    @ApiOperation(value = "修改信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse update(@RequestBody VmsVehicle vehicle) {
        return vmsVehicleService.saveVehicle(vehicle);
    }

    /**
     * 删除车辆
     */
    @LogAudit
    @GetMapping("/delete")
    @ApiOperation(value = "删除信息", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse delete(@RequestParam String ids) {
        vmsVehicleService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }

    /**
     * 车辆启用禁用
     */
    @LogAudit
    @GetMapping("/updateVehicleStatus")
    @ApiOperation(value = "车辆启用禁用", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse updateVehicleStatus(@RequestParam(name = "vin") String vin, @RequestParam(name = "status", defaultValue = "0") Integer status) {
        vmsVehicleService.update(new LambdaUpdateWrapper<VmsVehicle>().set(VmsVehicle::getStatus, status).eq(VmsVehicle::getVin, vin));
        return SSIResponse.ok();
    }

    /**
     * 车辆监控列表接口
     */
    @LogAudit
    @GetMapping("/listForMonitor")
    @ApiOperation(value = "车辆监控列表接口", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse listForMonitor(String vehicleNum,Integer vehicleType) {
//        List<Vehicle> vehicleList = vehicleService.listAndCache();

        List<VmsVehicle> vehicleList = vmsVehicleService.getNormalVehicleByVehicleNum(vehicleNum,vehicleType);
        return vmsVehicleService.listForMonitor(vehicleList);
    }

    /**
     * 车辆监控状态统计接口
     */
    @LogAudit
    @GetMapping("/vehicleStatusForMonitor")
    @ApiOperation(value = "车辆监控状态统计接口", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse vehicleStatusForMonitor() {
//        List<Vehicle> vehicleList = vehicleService.listAndCache();
        List<VmsVehicle> vehicleList = vmsVehicleService.getNormalVehicle();
        return vmsVehicleService.vehicleStatusForMonitor(vehicleList);
    }

    /**
     * 获取当前定位周边车辆列表
     */
    @LogAudit
    @GetMapping("/getVehicleByLocation")
    @ApiOperation(value = "获取当前定位周边车辆", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    public SSIResponse getVehicleByLocation(@RequestParam String location, Integer distance) {
        if (distance == null) {
            Integer curDis = vmsDebugAppService.getDistance();
            distance = curDis == null ? 100 : curDis;
        }
        List<VehicleControlStatusVo> vehicleList = vmsVehicleService.getVehicleByLocation(location, distance);
        return SSIResponse.ok(vehicleList);
    }

    /**
     * 急停列表分页
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/emergency/list")
    public SSIResponse emergencyList(@RequestBody Map<String, Object> params) {
        SSIPage page = vehicleEmergencyService.queryPage(params);
        return SSIResponse.ok(page);
    }

    /**
     * 远程指令列表分页
     */
    @LogAudit
    @ApiOperation(value = "获得列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/remote/instruction")
    public SSIResponse remoteInstruction(@RequestBody Map<String, Object> params) {
        SSIPage page = remoteInstructionService.queryPage(params);
        return SSIResponse.ok(page);
    }
}
