package com.ssi.controller.map;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dfssi.dataplatform.service.annotation.LogAudit;
import com.ssi.constant.URL;
import com.ssi.entity.CraneInfo;
import com.ssi.entity.VmsAreaBusinessInfo;
import com.ssi.entity.map.MapDataDto;
import com.ssi.response.SSIResponse;
import com.ssi.service.CraneInfoService;
import com.ssi.service.VmsAreaBarrierService;
import com.ssi.service.VmsAreaBusinessInfoService;
import com.ssi.service.VmsCranePadBindService;
import com.ssi.utils.RestTemplateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

@Api(tags = "地图图形数据", description = "地图")
@Slf4j
@RestController
@RequestMapping("/map/data")
public class MapController {

	@Value("${map-edit-url}")
	private String mapEditUrl;
	@Autowired
	private VmsAreaBusinessInfoService vmsAreaBusinessInfoService;
	@Autowired
	private VmsAreaBarrierService vmsAreaBarrierService;
	@Autowired
	private VmsCranePadBindService vmsCranePadBindService;
	@Autowired
	private CraneInfoService craneInfoService;

	/**
	 * 保存地图图形数据
	 */
	@LogAudit
	@ApiOperation(value = "保存地图图形数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/save")
	public SSIResponse saveData(@RequestBody MapDataDto mapdata) {
		if (StringUtils.isBlank(mapdata.getPoints())) {
			return SSIResponse.no("区域绘制不规范！");
		}
		if (!mapdata.getType().equals("4")) {
			if (StringUtils.isBlank(mapdata.getNumber())) {
				return SSIResponse.no("请输入绘制区域编号");
			}
		}
		if (mapdata.getNumber().length() < 4) {
			return SSIResponse.no("请输入至少4位编号");
		}
		if (mapdata.getNumber().length() >10 ) {
			return SSIResponse.no("输入编号过长，编号长度应小于10");
		}

		if("11".equals(mapdata.getType())){
			//属于电子围栏
			SSIResponse ssiResponse = vmsAreaBarrierService.saveMapData(mapdata);
			return SSIResponse.ok(ssiResponse);
		}
		JSONObject param = new JSONObject();
		List<Double> pointList = new ArrayList<>();
		String[] points = mapdata.getPoints().split(";");
		for (String point : points) {
			String[] loglat = point.split(",");
			pointList.add(Double.valueOf(loglat[0]));
			pointList.add(Double.valueOf(loglat[1]));
		}
		if (StringUtils.isNumeric(mapdata.getNumber().substring(1))
				&& StringUtils.isAlpha(mapdata.getNumber().substring(0, 1))) {
			param.put("seq", mapdata.getNumber().substring(1));// 编号
		} else {
			return SSIResponse.no("请输入正确的编号,头一位为字母,后面为数字");
		}
		if (pointList.size() < 8) {
			return SSIResponse.no("区域绘制不规范，请至少绘制4个点");
		}

		param.put("lng1", pointList.get(0));
		param.put("lat1", pointList.get(1));
		param.put("lng2", pointList.get(2));
		param.put("lat2", pointList.get(3));
		param.put("lng3", pointList.get(4));
		param.put("lat3", pointList.get(5));
		param.put("lng4", pointList.get(6));
		param.put("lat4", pointList.get(7));

		if (mapdata.getField().equals("E") || mapdata.getField().equals("F")) {
			param.put("field", mapdata.getField());// 场区（E/F）
		} else {
			return SSIResponse.no("场区字段有误，请填写E/F");
		}
//        param.put("field",mapdata.getField());//场区（E/F）
		param.put("placeType", mapdata.getPlacetype());// 区域类型。1桥下，2桥后
		param.put("name", mapdata.getNumber());// 名称
		param.put("type", mapdata.getType());// 类型。1缓冲区；2停车区;3:充电区
		param.put("state", mapdata.getState());// 状态。0可用；1不可用（初始状态）;2已计划;3有车

		String paramData = JSONObject.toJSONString(param);
		log.info(String.format("绘制地图区域保存请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_ADD_URL), paramData, null);
		log.info(String.format("绘制地图区域保存返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		return SSIResponse.ok(jsonObject);
	}

	/**
	 * 删除地图图形数据
	 */
	@LogAudit
	@ApiOperation(value = "删除地图图形数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/delete")
	public SSIResponse deleteData(@RequestBody MapDataDto mapdata) {
		JSONObject param = new JSONObject();
		if (StringUtils.isBlank(mapdata.getId())||StringUtils.isBlank(mapdata.getType())) {
			return SSIResponse.no("请输入ID");
		}

		if("11".equals(mapdata.getType())){
			//属于电子围栏
			SSIResponse ssiResponse = vmsAreaBarrierService.deleteMapData(mapdata);
			return SSIResponse.ok(ssiResponse);
		}
		List<Integer> tempId = new ArrayList<>();

		String[] ids = mapdata.getId().split(",");
		for (String id : ids) {
			if (StringUtils.isNumeric(id)) {
				tempId.add(Integer.parseInt(id));
			} else {
				return SSIResponse.no("请输入数字ID");
			}

		}
		String paramData = JSONObject.toJSONString(tempId);
		log.info(String.format("绘制地图区域删除请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_Delete_URL), paramData, null);
		log.info(String.format("绘制地图区域保删除返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		return SSIResponse.ok(jsonObject);
	}

	/**
	 * 修改图图形数据
	 */
	@LogAudit
	@ApiOperation(value = "修改地图图形数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/updateMapData")
	public SSIResponse updateMapData(@RequestBody MapDataDto mapdata) {
		JSONObject param = new JSONObject();
		if (StringUtils.isBlank(mapdata.getId())) {
			return SSIResponse.no("请输入ID");
		}

		if("11".equals(mapdata.getType())){
			//属于电子围栏
			SSIResponse ssiResponse = vmsAreaBarrierService.updateMapData(mapdata);
			return SSIResponse.ok(ssiResponse);
		}
		if (StringUtils.isNotBlank(mapdata.getPoints())) {
			List<Double> pointList = new ArrayList<>();
			String[] points = mapdata.getPoints().split(";");
			for (String point : points) {
				String[] loglat = point.split(",");
				pointList.add(Double.valueOf(loglat[0]));
				pointList.add(Double.valueOf(loglat[1]));
			}
			if (pointList.size() < 8) {
				return SSIResponse.no("区域绘制不规范，请至少绘制4个点");
			}
			param.put("lng1", pointList.get(0));
			param.put("lat1", pointList.get(1));
			param.put("lng2", pointList.get(2));
			param.put("lat2", pointList.get(3));
			param.put("lng3", pointList.get(4));
			param.put("lat3", pointList.get(5));
			param.put("lng4", pointList.get(6));
			param.put("lat4", pointList.get(7));
		}
		if (StringUtils.isNotBlank(mapdata.getField())) {
			if (mapdata.getField().equals("E") || mapdata.getField().equals("F")) {
				param.put("field", mapdata.getField());// 场区（E/F）
			} else {
				return SSIResponse.no("场区字段有误，请填写E/F");
			}

		}
		if (StringUtils.isNotBlank(mapdata.getPlacetype())) {
			param.put("placeType", mapdata.getPlacetype());// 区域类型。1桥下，2桥后
		}
		if (StringUtils.isNotBlank(mapdata.getNumber())) {
			if (mapdata.getNumber().length() < 4) {
				return SSIResponse.no("请输入至少4位编号");
			}
			param.put("name", mapdata.getNumber());// 名称
			if (StringUtils.isNumeric(mapdata.getNumber().substring(1))
					&& StringUtils.isAlpha(mapdata.getNumber().substring(0, 1))) {
				param.put("seq", mapdata.getNumber().substring(1));// 编号
			} else {
				return SSIResponse.no("请输入正确的编号,头一位为字母,后面为数字");
			}
//             param.put("seq",Integer.valueOf(mapdata.getNumber().substring(1,4)));//编号
		}
		if (StringUtils.isNotBlank(mapdata.getType())) {
			param.put("type", mapdata.getType());// 类型。1缓冲区；2停车区;3:充电区
		}

		param.put("id", mapdata.getId());// 区域类型。1桥下，2桥后

		if (StringUtils.isNotBlank(mapdata.getState())) {
			param.put("state", mapdata.getState());// 状态。0可用；1不可用（初始状态）;2已计划;3有车
		}
		if (StringUtils.isNotBlank(mapdata.getPriority())) {
			param.put("priority", mapdata.getPriority());// 状态。0可用；1不可用（初始状态）;2已计划;3有车
		}
		param.put("bridgeNo", mapdata.getBridgeNo());
		param.put("taskType", mapdata.getTaskType());
		String paramData = JSONObject.toJSONString(param);
		log.info(String.format("绘制地图区域update请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_EDIT_URL), paramData, null);
		log.info(String.format("绘制地图区域update返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		if(Objects.nonNull(jsonObject)&&jsonObject.getInteger("code")==1){//成功则更新
			vmsAreaBusinessInfoService.updatePassArea(mapdata);
		}
		return SSIResponse.ok(jsonObject);
	}


	@LogAudit
	@ApiOperation(value = "批量修改地图图形数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/batchUpdateMapData")
	public SSIResponse batchUpdateMapData(@RequestBody String mapdatas) {
		  if(StringUtils.isBlank(mapdatas)){
			  return SSIResponse.no("数据为空");
		  }
		  List<MapDataDto>  list = JSONObject.parseArray(mapdatas,MapDataDto.class);
		  for(MapDataDto mapdata: list){
			  return this.updateMapData(mapdata);
		  }
		  return SSIResponse.no("数据为空");
	}


	/**
	 * 查询地图图形数据
	 */
	@LogAudit
	@ApiOperation(value = "查询地图图形数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/query")
	public SSIResponse queryData(@RequestBody MapDataDto mapdata) {
		JSONObject param = new JSONObject();

		if (StringUtils.isNotBlank(mapdata.getPoints())) {
			param.put("priority", mapdata.getPoints());
		}
		if (StringUtils.isNotBlank(mapdata.getNumber())) {
			param.put("name", mapdata.getNumber());// 名称
		}
		if (StringUtils.isNotBlank(mapdata.getType())) {
			param.put("type", mapdata.getType());// 类型。1缓冲区；2停车区;3:充电区
		}
		if (StringUtils.isNotBlank(mapdata.getState())) {
			param.put("state", mapdata.getState());// 状态。0可用；1不可用（初始状态）;2已计划;3有车
		}
		if (StringUtils.isNotBlank((mapdata.getPlacetype()))) {
			param.put("state", mapdata.getPlacetype());// 状态。0可用；1不可用（初始状态）;2已计划;3有车
		}

		String paramData = JSONObject.toJSONString(param);
		log.debug(String.format("绘制地图区域请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_QUERY_URL), paramData, null);
		log.debug(String.format("绘制地图区域请求返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		if(Objects.nonNull(jsonObject)&&jsonObject.getInteger("code")==1){
			vmsAreaBarrierService.addBarrierMap(mapdata,jsonObject);//添加电子围栏
			vmsCranePadBindService.addCarineInfo(jsonObject);//添加绑定桥吊信息
			componentBusinessParam(mapdata,jsonObject);//添加业务参数
		}
		return SSIResponse.ok(jsonObject);
	}

	private void componentBusinessParam(MapDataDto mapdata,JSONObject jsonObject) {
		JSONArray dataList = jsonObject.getJSONArray("data");
		Set<String> bindedAreas = qryBindedArea();
		for(int i = 0; i< dataList.size();i++){
			JSONObject object = dataList.getJSONObject(i);
			List<VmsAreaBusinessInfo> vmsAreaBusinessInfos = vmsAreaBusinessInfoService.lambdaQuery().eq(VmsAreaBusinessInfo::getAreaId, object.getString("id"))
					.eq(Objects.nonNull(mapdata.getTaskType()), VmsAreaBusinessInfo::getTaskType, mapdata.getTaskType())
					.list();
			object.put("taskType",null);
			object.put("bridge",null);
			object.put("isBinded",0);
			List<String> collect = vmsAreaBusinessInfos.stream().map(VmsAreaBusinessInfo::getBridge).collect(Collectors.toList());
			if(CollectionUtils.isNotEmpty(vmsAreaBusinessInfos)){
				object.put("taskType",vmsAreaBusinessInfos.get(0).getTaskType());
				object.put("bridge",StringUtils.join(collect,","));
			}
			//标记绑定缓冲区
			if(bindedAreas.contains(object.getString("name"))){
				object.put("isBinded",1);
			}
		}
	}

	private Set<String> qryBindedArea() {
		Set<String> result = new TreeSet<>();
		try{
			CraneInfo craneInfo = new CraneInfo();
			craneInfo.setCraneType(1);
			craneInfo.setTaskTypes(Arrays.asList(1,2));
			SSIResponse ssiResponse = craneInfoService.queryCraneList(craneInfo);
			if(ssiResponse.getCode()==1){
				List<JSONObject> data = (List<JSONObject>)ssiResponse.getData();
				data.forEach( item -> {
					JSONArray passArea = item.getJSONArray("passArea");
					result.addAll(passArea.toJavaList(String.class));
				});
			}
		}catch (Exception e){
			log.error("qryBindedArea occur error:",e);
		}
		return result;
	}

	/**
	 * 车位优先级修改
	 */
	@LogAudit
	@ApiOperation(value = "车位优先级修改（批量）", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/updatePri")
	public SSIResponse updatePri(@RequestBody MapDataDto mapdata) {
		JSONObject param = new JSONObject();
		if (StringUtils.isBlank(mapdata.getId())) {
			return SSIResponse.no("请输入ID");
		}
		if (StringUtils.isBlank(mapdata.getPriority())) {
			return SSIResponse.no("请选择优先级");
		}
		List<Integer> tempId = new ArrayList<>();

		String[] ids = mapdata.getId().split(",");
		for (String id : ids) {
			if (StringUtils.isNumeric(id)) {
				tempId.add(Integer.parseInt(id));
			} else {
				return SSIResponse.no("请输入数字ID");
			}
		}
		param.put("ids", tempId);
		param.put("priority", mapdata.getPriority());
		String paramData = JSONObject.toJSONString(param);
		log.info(String.format("绘制地图区域保存请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_priority_URL), paramData, null);
		log.info(String.format("绘制地图区域保存返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		return SSIResponse.ok(jsonObject);
	}

	/**
	 * 查询地图图形数据
	 */
	@LogAudit
	@ApiOperation(value = "车位优先级修改（批量）", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/updateSate")
	public SSIResponse updateSate(@RequestBody MapDataDto mapdata) {
		JSONObject param = new JSONObject();
		if (StringUtils.isBlank(mapdata.getId())) {
			return SSIResponse.no("请输入ID");
		}
		if (StringUtils.isBlank(mapdata.getState())) {
			return SSIResponse.no("请选择状态");
		}
		List<Integer> tempId = new ArrayList<>();

		String[] ids = mapdata.getId().split(",");
		for (String id : ids) {
			if (StringUtils.isNumeric(id)) {
				tempId.add(Integer.parseInt(id));
			} else {
				return SSIResponse.no("请输入数字ID");
			}
		}
		param.put("ids", tempId);
		param.put("state", mapdata.getState());
		String paramData = JSONObject.toJSONString(param);
		log.info(String.format("绘制地图区域保存请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_UPDATE_STATE_URL), paramData,
				null);
		log.info(String.format("绘制地图区域保存返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		return SSIResponse.ok(jsonObject);
	}
//    private SSIResponse<?> handleResult(String result) {
//        JSONObject jsonObject = JSONObject.parseObject(result);
//        for(String str:jsonObject.keySet()) {
//        	 if(jsonObject.get("code").equals("1")) {
//        		 return SSIResponse.ok();
//        	 }else {
//        		 log.error(jsonObject.get("msg").toString());
//               return SSIResponse.no(jsonObject.get("msg").toString());
//        	 }
//
//        }
// 
//        return SSIResponse.ok();
//    }

	/**
	 * 查询地图图形数据
	 */
	@LogAudit
	@ApiOperation(value = "查询地图图形数据", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
	@PostMapping("/queryCacheArea")
	public SSIResponse queryCacheArea(@RequestBody MapDataDto mapdata) {
		JSONObject param = new JSONObject();

		if (StringUtils.isNotBlank(mapdata.getPoints())) {
			param.put("priority", mapdata.getPoints());
		}
		if (StringUtils.isNotBlank(mapdata.getNumber())) {
			param.put("name", mapdata.getNumber());// 名称
		}
		if (StringUtils.isNotBlank(mapdata.getType())&&!"1".equalsIgnoreCase(mapdata.getType())) {
			param.put("type", mapdata.getType());// 类型。1缓冲区；2停车区;3:充电区
		}
		if (StringUtils.isNotBlank(mapdata.getState())) {
			param.put("state", mapdata.getState());// 状态。0可用；1不可用（初始状态）;2已计划;3有车
		}
		if (StringUtils.isNotBlank((mapdata.getPlacetype()))) {
			param.put("state", mapdata.getPlacetype());// 状态。0可用；1不可用（初始状态）;2已计划;3有车
		}

		String paramData = JSONObject.toJSONString(param);
		log.debug(String.format("绘制地图区域请求发送:----%s", paramData));
		String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_QUERY_URL), paramData, null);
		log.debug(String.format("绘制地图区域请求返回结果:----%s", result));
		JSONObject jsonObject = JSONObject.parseObject(result);
		if(Objects.nonNull(jsonObject)&&jsonObject.getInteger("code")==1){
			JSONArray data = jsonObject.getJSONArray("data");
			List<JSONObject> results = data.toJavaList(JSONObject.class).stream().filter((obj) -> {
				if(StringUtils.isNotBlank(mapdata.getType())&&!"1".equalsIgnoreCase(mapdata.getType())){
					return true;
				}
				if (!"1,5".contains(obj.getString("type"))) {
					return false;
				}
				return true;
			}).sorted((a, b) -> {
				String oa = a.getString("name").replaceAll("^\\w","");
				String ob = b.getString("name").replaceAll("^\\w","");
				return Integer.valueOf(oa).compareTo(Integer.valueOf(ob));
			}).collect(Collectors.toList());
			jsonObject.put("data",results);
		}
		return SSIResponse.ok(jsonObject);
	}
}
