package com.ssi.controller.platform;


import com.google.common.collect.Sets;
import com.ssi.service.platform.RedisConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Description:
 *
 * @author LiXiaoCong
 * @version 2019/10/17 10:29
 */
@Api(tags = "数据配置接口", description = "用于配置设置相关")
@RestController
@RequestMapping("/config")
@Slf4j
@CrossOrigin(allowCredentials = "true", exposedHeaders = "access-control-allow-headers,access-control-allow-methods,access-control-allow-origin,access-control-max-age,X-Frame-Options,userAuth")
public class RedisConfigController {

    @Autowired
    private RedisConfigService redisConfigService;

    @ApiOperation(value = "设置单个配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "group", value = "配置组", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "key", value = "key", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "value", value = "value", dataType = "String", paramType = "query", required = true),
    })
    @GetMapping(value = "setConfig", produces = {"application/json"})
    public void setConfig(String group, String key, String value) {
        redisConfigService.setConfig(group, key, value);
    }

    @ApiOperation(value = "获取单个配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "group", value = "配置组", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "key", value = "key", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "defaultValue", value = "value", dataType = "String", paramType = "query"),
    })
    @GetMapping(value = "getConfigWithDefault", produces = {"application/json"})
    public Object getConfigWithDefault(String group, String key, String defaultValue) {
        return redisConfigService.getConfigWithDefault(group, key, defaultValue);
    }


    @ApiOperation(value = "获取单个配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "group", value = "配置组", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "key", value = "key", dataType = "String", paramType = "query", required = true)
    })
    @GetMapping(value = "getConfig", produces = {"application/json"})
    public Object getConfig(String group, String key) {
        return redisConfigService.getConfig(group, key);
    }


    @ApiOperation(value = "获取多个配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "group", value = "配置组", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "keys", value = "key", dataType = "String", paramType = "query", required = true)
    })
    @GetMapping(value = "getConfigs", produces = {"application/json"})
    public Object getConfigs(String group, String keys) {
        return redisConfigService.getConfigs(group, Sets.newHashSet(keys.split(",")));
    }

    @ApiOperation(value = "删除配置")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "group", value = "配置组", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "keys", value = "key", dataType = "String", paramType = "query", required = true)
    })
    @GetMapping(value = "deteleConfigs", produces = {"application/json"})
    public void deteles(String group, String keys) {
        redisConfigService.deteles(group, keys.split(","));
    }

}
