package com.ssi.controller.platform;

import com.ssi.entity.VmsTrafficLightInfo;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsTrafficLightInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * @author SunnyHotz
 * @PackageName:com.ssi.controller.platform
 * @ClassName:TrafficLightController
 * @Description:
 * @date 2022/9/7 16:29
 */

@Api("交通灯控制器")
@RestController
@RequestMapping("/trafficLight")
public class TrafficLightController{

    @Autowired
    private VmsTrafficLightInfoService vmsTrafficLightInfoService;


    @ApiOperation(value = "获得交通灯列表", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/list")
    public SSIResponse list(@RequestBody VmsTrafficLightInfo req) {
        SSIPage page = vmsTrafficLightInfoService.queryPage(req);
        return SSIResponse.ok(page);
    }

    @ApiOperation(value = "更新交通灯", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/update")
    public SSIResponse update(@RequestBody VmsTrafficLightInfo req) {
        vmsTrafficLightInfoService.updateById(req);
        return SSIResponse.ok();
    }


    @ApiOperation(value = "控制交通灯", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/manual")
    public SSIResponse manual(@RequestBody VmsTrafficLightInfo req) {
        String manual = vmsTrafficLightInfoService.manual(req);
        if(StringUtils.isNotBlank(manual)){
            return SSIResponse.no(manual);
        }
        return SSIResponse.ok();
    }

    @ApiOperation(value = "新增", response = SSIResponse.class, produces = MimeTypeUtils.APPLICATION_JSON_VALUE)
    @PostMapping("/add")
    public SSIResponse add(@RequestBody VmsTrafficLightInfo req) {

        vmsTrafficLightInfoService.save(req);
        return SSIResponse.ok();
    }


    @ApiOperation(value = "删除", response = SSIResponse.class)
    @GetMapping("/delete")
    public SSIResponse delete(@RequestParam("ids") String ids) {
        vmsTrafficLightInfoService.removeByIds(Arrays.asList(ids.split(",")));
        return SSIResponse.ok();
    }
}
