package com.ssi.controller.platform;


import com.ssi.service.platform.VehicleLatestDataService;
import com.ssi.service.platform.VehicleMonitorService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * description:
 *
 * @author WangHD
 * @version 1.0
 * @date 2020/3/20 0020  9:53
 */
@Api(tags = "车辆监控接口", description = "实时监控")
@RestController
@RequestMapping("/vehicle/monitor/")
@Slf4j
@CrossOrigin(allowCredentials = "true", exposedHeaders = "access-control-allow-headers,access-control-allow-methods,access-control-allow-origin,access-control-max-age,X-Frame-Options,userAuth")
public class VehicleMonitorController {

    @Autowired
    private VehicleLatestDataService vehicleLatestDataService;
    @Autowired
    private VehicleMonitorService vehicleMonitorService;

    /**
     * key给车辆vin组合成的string
     */
    @ApiOperation(value = "查询在线数据")
    @ApiImplicitParams({@ApiImplicitParam(name = "key", value = "key", dataType = "String", paramType = "query", required = true)})
    @RequestMapping(value = "getOnlineData", method = {RequestMethod.GET})
    public Object getOnlineData(String key) {
        return vehicleLatestDataService.getOnlineData(key);
    }

    @ApiOperation(value = "车辆状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vin", value = "vin", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "status", value = "status:0、离线，1、在线，2、行驶中", dataType = "String", paramType = "query", required = true)
    })
    @RequestMapping(value = "getVehicleStatus", method = {RequestMethod.GET})
    public Object getVehicleStatus(String vin, Integer status) {
        return vehicleLatestDataService.getVehicleStatus(vin, status);
    }


    @ApiOperation(value = "车载视频列表(内网)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vin", value = "车辆唯一编码", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "isHistory", defaultValue = "", value = "是否历史回放", dataType = "String", paramType = "query", required = false),
    })
    @GetMapping(value = "getCarVideoList", produces = {"application/json"})
    public Object getCarVideoList(@RequestParam(value = "vin") String vin, String isHistory) {
        if ("1".equals(isHistory)) {
            //演示用
            return vehicleMonitorService.getCarHistoryVideoList(vin);
        }
        return vehicleMonitorService.getCarVideoList(vin);
        // return vehicleMonitorService.getCarVideoPlayingList(vin);
    }

    @ApiOperation(value = "车载视频播放列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vin", value = "车辆唯一编码", dataType = "String", paramType = "query", required = true)
    })
    @GetMapping(value = "getCarVideoPlayingList", produces = {"application/json"})
    public Object getCarVideoPlayingList(@RequestParam(value = "vin") String vin) {

        return vehicleMonitorService.getCarVideoPlayingList(vin);
    }

    @ApiOperation(value = "离线车辆列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vehicleCompany", defaultValue = "", value = "车企", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "vehicleType", defaultValue = "", value = "车型", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "offlineDays", defaultValue = "", value = "离线天数", dataType = "String", paramType = "query", required = false)

    })
    @GetMapping(value = "getOfflineVehicleList", produces = {"application/json"})
    public Object getOfflineVehicleList(String vehicleCompany, String vehicleType, @RequestParam("offlineDays") Integer offlineDays) {
        return vehicleMonitorService.getOfflineVehicleList(vehicleCompany, vehicleType, offlineDays, 1, 1000);
    }

}
