package com.ssi.controller.platform;

import com.ssi.entity.ScrollTrackLocationEntity;
import com.ssi.response.SSIResponse;
import com.ssi.service.platform.VehicleTrackService;
import com.ssi.websocket.ReplaySocketServer;
import com.ssi.websocket.push.task.ReplayTask;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * Description:
 * 车辆轨迹查询控制器
 *
 * @author LiXiaoCong
 * @version 2020/2/8 12:54
 */
@Api(tags = "车辆轨迹查询控制器", description = "车辆历史轨迹查询")
@RestController
@RequestMapping("/vehicle/track")
@Slf4j
@CrossOrigin(allowCredentials = "true", exposedHeaders = "access-control-allow-headers,access-control-allow-methods,access-control-allow-origin,access-control-max-age,X-Frame-Options,userAuth")
public class VehicleTrackController {

    @Autowired
    private VehicleTrackService vehicleTrackService;

    @ApiOperation(value = "车辆历史轨迹 游标分页查询 - 适用于结果数据大于1w条的轨迹查询", produces = "application/json")
    @RequestMapping(value = "scrollTrack", method = {RequestMethod.POST})
    public Object scrollTrack(@RequestBody @Valid ScrollTrackLocationEntity scrollTrackLocationEntity) {
        return vehicleTrackService.scrollTrack(scrollTrackLocationEntity);
    }

    @ApiOperation(value = "车辆历史轨迹 - 厦门港查询两小时内全部数据", produces = "application/json")
    @RequestMapping(value = "smallTrack", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vin", value = "车辆vin，仅支持单个", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "startTime", value = "轨迹开始时间", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "stopTime", value = "轨迹结束时间", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "posType", value = "目标坐标系, 高德：gcj02， 百度：bd09 ", dataType = "String", paramType = "query")
    })
    public Object smallTrack(@RequestParam String vin, @RequestParam Long startTime, @RequestParam Long stopTime,
                             String posType) {
        return vehicleTrackService.smallTrack(vin, startTime, stopTime, posType);
    }


    @ApiOperation(value = "车辆历史轨迹 - 厦门港查询两小时内全部数据", produces = "application/json")
    @RequestMapping(value = "replayVehicle", method = {RequestMethod.GET})
    public void replayVehicle(@RequestParam String vin, @RequestParam Long startTime, @RequestParam Long stopTime,
                             String posType) {
        SSIResponse response = vehicleTrackService.smallTrack(vin, startTime, stopTime, posType);
        ReplayTask.setReplayCache((List<Map<String, Object>>) response.getData());
    }


}
