package com.ssi.entity;

import com.google.common.base.Strings;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;

/**
 * Description:
 *
 * @author LiXiaoCong
 * @version 2018/11/14 19:32
 */
@Setter
@Getter
@ToString
@ApiModel(value = "ScrollTrackLocationEntity", description = "深分页查询固定时间范围里指定车辆的抽样行驶轨迹参数实体")
public class ScrollTrackLocationEntity {


    @ApiModelProperty(name = "vin", value = "车辆vin", dataType = "string", required = true)
    @NotBlank(message = "车辆vin不能为空!")
    private String vin;

    @ApiModelProperty(name = "startTime", value = "开始日期时间戳，单位：毫秒", dataType = "long", required = true, position = 1)
    @Min(value = 1514736000000L, message = "开始日期时间戳过小")
    private Long startTime;

    @ApiModelProperty(name = "stopTime", value = "结束日期时间戳，单位：毫秒", dataType = "long", required = true, position = 2)
    @Min(value = 1514736000000L, message = "结束日期时间戳过小")
    private Long stopTime;

    @ApiModelProperty(name = "scrollId", value = "深分页游标值", dataType = "string", position = 3)
    private String scrollId;

    @ApiModelProperty(name = "maxSizePartition", value = "每个分区最大返回记录数", dataType = "int", example = "500", position = 4)
    private int maxSizePartition = 500;

    @ApiModelProperty(name = "stepLength", value = "跳点步长", dataType = "int", example = "1", position = 5)
    @Min(value = 1, message = "跳点步长最小为1")
    private int stepLength = 1;

    @ApiModelProperty(name = "sortByTime", value = "是否按时间顺序排序", dataType = "boolean", example = "false", position = 6)
    private boolean sortByTime = true;

    @ApiModelProperty(name = "posType", value = "坐标转换类型", dataType = "string", allowableValues = "gcj02,bd09", position = 7)
    private String posType;

    @ApiModelProperty(name = "columns", value = "返回字段", dataType = "Array[String]", position = 8)
    private String[] columns;

    public String getScrollId() {
        return Strings.isNullOrEmpty(scrollId) ? null : scrollId;
    }
}
