package com.ssi.entity.vo;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.ssi.entity.VmsTosOrderDailyKpi;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Getter
@Setter
@ToString
public class VehicleKpiVo {

    private String vin;

    private String vehicleNum;

    private String Date;

    private int finishedNum;

    private int totalNum;

    private HourKpi[] kpiArr = new HourKpi[24];

    public void fillData(List<HourKpi> list) {
        if (CollectionUtils.isNotEmpty(list) && this.vin != null) {
            list.forEach(hourKpi -> {
                String vin = hourKpi.getVin();
                if (this.vin.equals(vin)) {
                    int hour = hourKpi.getHour();
                    kpiArr[hour] = hourKpi;
                }
            });
        }
        for (int i = 0; i < kpiArr.length; i++) {
            if (kpiArr[i] == null) {
                kpiArr[i] = new HourKpi(this.vin, i);
            }
        }
    }

    /**
     * 根据 VmsTosOrderDailyKpi 填充
     * @param list
     */
    public void fillDataForOrderDailyKpi(List<VmsTosOrderDailyKpi> list) {
        if (CollectionUtils.isNotEmpty(list)) {
            List<HourKpi> hourKpis = list.stream().map(orderDailyKpi -> {
                int containerNum = StringUtils.isNotEmpty(orderDailyKpi.getContainerIds()) ? orderDailyKpi.getContainerIds().split(",").length : 0;
                int orderNum = StringUtils.isNotEmpty(orderDailyKpi.getOrderIds()) ? orderDailyKpi.getOrderIds().split(",").length : 0;
                int finishedOrderNum = StringUtils.isNotEmpty(orderDailyKpi.getFinishedOrderIds()) ? orderDailyKpi.getFinishedOrderIds().split(",").length : 0;
                return new HourKpi(orderDailyKpi.getVin(), orderDailyKpi.getAnalysisHour(), containerNum,
                        orderNum, finishedOrderNum);
            }).collect(Collectors.toList());
            fillData(hourKpis);
        } else {
            fillData(null);
        }
    }


    public Map<String, String> toMap() throws Exception {
        Map<String, String> map = BeanUtils.describe(this);
        map.remove("kpiArr");
        map.remove("class");
        for (int i = 0; i < kpiArr.length; i++) {
            map.put("hour_" + i, kpiArr[i].finishedOrderNum + "/" + kpiArr[i].orderNum);
            totalNum += kpiArr[i].orderNum;
            finishedNum += kpiArr[i].finishedOrderNum;
        }
        map.put("finishRate",finishedNum + "/" + totalNum);

        return map;
    }

    public VehicleKpiVo(String vin, String vehicleNum) {
        this.vin = vin;
        this.vehicleNum = vehicleNum;
    }

    public VehicleKpiVo() {
    }

    @Getter
    @Setter
    @ToString
    public static class HourKpi {
        @JsonIgnore
        private String vin;

        private int hour;

        private int containerNum;

        private int orderNum;

        private int finishedOrderNum;

        public HourKpi(String vin, int hour) {
            this.vin = vin;
            this.hour = hour;
        }
        public HourKpi(String vin, int hour, int containerNum, int orderNum, int finishedOrderNum) {
            this.vin = vin;
            this.hour = hour;
            this.containerNum = containerNum;
            this.orderNum = orderNum;
            this.finishedOrderNum = finishedOrderNum;
        }
        public HourKpi() {
        }
    }
}
