package com.ssi.entity.vo;

import com.ssi.entity.VmsTosOrders;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author ZhangLiYao
 * @version 1.0
 * @date 2020/7/10 10:18
 */
@Data
@ApiModel(value = "VehicleVo")
public class VehicleVo {
    /**
     * 异常车辆数
     */
    @ApiModelProperty(value = "异常车辆数")
    private Integer abnormalNum;

    private List<vehicleTemp> vehicleList;

    public void setVehicleList(List<vehicleTemp> vehicleTempList) {
        this.vehicleList = vehicleTempList;
        AtomicInteger abnormalNum = new AtomicInteger(0);
        vehicleTempList.parallelStream().forEach(vehicleTemp -> {
            if (vehicleTemp.getFaultStatus() == 1 || vehicleTemp.getOnlineStatus() == 1
                    || vehicleTemp.getWorkStatus() == 1 || vehicleTemp.getLowEnergyStatus() == 1) {
                abnormalNum.incrementAndGet();
            }
        });
        this.abnormalNum = abnormalNum.get();
    }

    @Data
    public class vehicleTemp {
        /**
         * vin
         */
        @ApiModelProperty(value = "vin")
        private String vin;

        /**
         * 车辆编号
         */
        @ApiModelProperty(value = "车辆编号")
        private String vehicleNum;

        /**
         * 车辆类型
         */
        @ApiModelProperty(value = "车辆类型")
        private Integer vehicleType;

        /**
         * soc
         */
        @ApiModelProperty(value = "soc")
        private Integer soc;


        /**
         * 故障状态
         */
        @ApiModelProperty(value = "故障状态 1 正常 0 故障")
        private Integer faultStatus;

        /**
         * 故障级别
         */
        @ApiModelProperty(value = "故障级别")
        private String faultLevel;

        /**
         * 工作状态
         */
        @ApiModelProperty(value = "工作状态 1 运行 0 空闲")
        private Integer workStatus;

        /**
         * 上下线状态
         */
        @ApiModelProperty(value = "上下线状态 1 上线 0 下线")
        private Integer onlineStatus;

        /**
         * 是否等待超时
         */
        @ApiModelProperty(value = "是否等待超时 1 超时 0 未超时")
        private Integer waitOvertimeStatus;

        /**
         * 是否低电量
         */
        @ApiModelProperty(value = "是否低电量 1 是 0 否")
        private Integer lowEnergyStatus;

        /**
         *
         * 是否紧急停车
         */
        @ApiModelProperty(value = "是否紧急停车 1 是 2 否")
        private Integer emergencyType;

        /**
         * 任务信息
         */
        @ApiModelProperty(value = "任务信息")
        private VmsTosOrders orderData;

        private String orderDescription;

        private Map threeConfirmStatus;

        public vehicleTemp(Map<String, Object> realDataMap) {
            this.vehicleNum = realDataMap.get("vehicleNum") == null ? null : (String) realDataMap.get("vehicleNum");
            this.vehicleType = realDataMap.get("vehicleType") == null ? 0 : (Integer) realDataMap.get("vehicleType");
            this.vin = realDataMap.get("vin") == null ? null : (String) realDataMap.get("vin");
            this.faultStatus = realDataMap.get("equipStatus") == null
                    ? 1 : (Integer) realDataMap.get("equipStatus") == 1 ? 1 : 0;
            if (faultStatus == 1) {
                this.faultLevel = "一级故障";
            }
            this.workStatus = realDataMap.get("workStatus") == null ? 0 : (Integer) realDataMap.get("workStatus");
            this.onlineStatus = checkOnline(realDataMap.get("state") == null ? 0 : (Integer) realDataMap.get("state")
                    , (Long) realDataMap.get("collectTime"));

            this.waitOvertimeStatus = realDataMap.get("waitOvertimeStatus") == null ? 0 : (Integer) realDataMap.get("waitOvertimeStatus");
            this.lowEnergyStatus = realDataMap.get("lowEnergy") == null || realDataMap.get("soc") == null ? 0 : (Integer) realDataMap.get("soc") < (Integer) realDataMap.get("lowEnergy") ? 1 : 0;
        }

        public vehicleTemp() {
        }
    }

    public static Integer checkOnline(Integer state, Long collectime) {
        /** 检查终端状态*/
        boolean dataOnline = (collectime != null && System.currentTimeMillis() - collectime < 60000L);

        // FIXME: 2020/9/9 暂时不处理采集时间判断上线
//        dataOnline = true;

        boolean isOnline = dataOnline && state == 1;

        return isOnline ? 1 : 0;
    }


}
