package com.ssi.exception;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.ssi.constant.enums.Status;
import com.ssi.response.SSIResponse;
import org.apache.shiro.authz.AuthorizationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.NoHandlerFoundException;

/**
 * 异常处理器
 */
@RestControllerAdvice
public class CustomizeExceptionHandler {

	private Logger logger = LoggerFactory.getLogger(CustomizeExceptionHandler.class);

	/**
	 * 处理自定义异常
	 */
	@ExceptionHandler(CustomizeException.class)
	public SSIResponse handleCustomizeException(CustomizeException e) {
		return SSIResponse.no(Status.ERROR);
	}

	@ExceptionHandler(NoHandlerFoundException.class)
	public SSIResponse handlerNoFoundException(Exception e) {
		String idStr = IdWorker.getIdStr();
		logger.error("全局跟踪号" + idStr + "开始:");
		logger.error(e.getMessage(), e);
		logger.error("全局跟踪号" + idStr + "結束:");
		return SSIResponse.no(Status.NOTFOUND, "路径不存在，请检查路径是否正确,全局跟踪号" + idStr);
	}

	@ExceptionHandler(DuplicateKeyException.class)
	public SSIResponse handleDuplicateKeyException(DuplicateKeyException e) {
		String idStr = IdWorker.getIdStr();
		logger.error("全局跟踪号" + idStr + "开始:");
		logger.error(e.getMessage(), e);
		logger.error("全局跟踪号" + idStr + "結束:");
		return SSIResponse.no("数据库中已存在该记录,全局跟踪号" + idStr);
	}

	@ExceptionHandler(AuthorizationException.class)
	public SSIResponse handleAuthorizationException(AuthorizationException e) {
		String idStr = IdWorker.getIdStr();
		logger.error("全局跟踪号" + idStr + "开始:");
		logger.error(e.getMessage(), e);
		logger.error("全局跟踪号" + idStr + "結束:");
		return SSIResponse.no(Status.UNAUTHORIZED, "没有权限，请联系管理员授权,全局跟踪号" + idStr);
	}

	// 入参校验异常
	@ExceptionHandler(MethodArgumentNotValidException.class)
	public SSIResponse handleDuplicateKeyException(MethodArgumentNotValidException e) {
		String idStr = IdWorker.getIdStr();
		logger.error("全局跟踪号" + idStr + "开始:");
		logger.error(e.getMessage(), e);
		logger.error("全局跟踪号" + idStr + "結束:");
		return SSIResponse.no( "全局跟踪号" + idStr);
	}

	@ExceptionHandler(Exception.class)
	public SSIResponse handleException(Exception e) {
		String idStr = IdWorker.getIdStr();
		logger.error("全局跟踪号" + idStr + "开始:");
		logger.error(e.getMessage(), e);
		logger.error("全局跟踪号" + idStr + "結束:");
		return SSIResponse.no("请求异常，全局跟踪号" + idStr);
	}

}
