package com.ssi.kafka.listener;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.entity.VmsFaultCode;
import com.ssi.entity.FaultRecord;
import com.ssi.entity.VmsVehicle;
import com.ssi.mapper.VmsFaultCodeMapper;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.websocket.AbnormalWebSocketServer;
import org.apache.commons.collections.CollectionUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 车辆故障报警kafka监听器
 */
@Component
public class AbnormalListener {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass());

  @Autowired
  private VmsFaultCodeMapper vmsFaultCodeMapper;
  @Autowired
  private VmsVehicleMapper vmsVehicleMapper;

  //@KafkaListener(topics = {"HARBOR_ALERT_UI"})
  public void run(ConsumerRecord<String, String> consumerRecord, Acknowledgment acknowledgment) {
    try {
      String value = consumerRecord.value();
      logger.info("kafka异常报警消息消费：" + value);
      FaultRecord record = JSON.parseObject(value, FaultRecord.class);
      String vin = record.getVin();
      VmsVehicle vehicle = vmsVehicleMapper.selectOne(
          new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getVin, vin));
      record.setVehicleNum(vehicle.getVehicleNum());
      String fmi = record.getFmi();
      String spn = record.getSpn();
      try {
        List<VmsFaultCode> faultCodeList = vmsFaultCodeMapper.selectList(
            new LambdaQueryWrapper<VmsFaultCode>()
                .eq(VmsFaultCode::getFmi, fmi)
                .eq(VmsFaultCode::getSpn, spn));
        if (CollectionUtils.isNotEmpty(faultCodeList)) {
          VmsFaultCode vmsFaultCode = faultCodeList.get(0);
          record.setFaultName(vmsFaultCode.getFaultName());
          record.setDescription(vmsFaultCode.getDescribition());
          record.setFaultGrade(vmsFaultCode.getFaultGrade());
          record.setFaultType(vmsFaultCode.getFaultType());
          record.setFaultSolution(vmsFaultCode.getFaultSolution());
        }
      } catch (Exception e) {
        logger.error("获取faultCode异常：{}", e);
      }
      AbnormalWebSocketServer.sendFaultRecord(record);
    } catch (Exception e) {
      logger.error("kafka异常报警消息消费：" + e.getMessage());
    } finally {
      acknowledgment.acknowledge();
    }
  }

}
