package com.ssi.kafka.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ssi.constant.RedisKey;
import com.ssi.constant.enums.V2xEventTypeEnums;
import com.ssi.entity.VehicleRoad;
import com.ssi.entity.VmsFaultCode;
import com.ssi.entity.VmsVehicleAlertProcess;
import com.ssi.model.RedisDataModel;
import com.ssi.service.VmsFaultCodeService;
import com.ssi.service.VmsVehicleAlertProcessService;
import com.ssi.utils.DateUtils;
import com.ssi.websocket.AbnormalWebSocketServer;
import com.ssi.websocket.VehicleRoadCoordinationWebSocketServer;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author SunnyHotz
 * @PackageName:com.ssi.kafka.listener
 * @ClassName:FaultCodeListener
 * @Description:
 * @date 2022/10/31 14:30
 */
@Component
@Slf4j
public class FaultCodeListener {

    @Autowired
    private VmsVehicleAlertProcessService vmsVehicleAlertProcessService;

    @Autowired
    private AbnormalWebSocketServer abnormalWebSocketServer;

    @Autowired
    private RedisDataModel redisDataModel;

    @Autowired
    private VmsFaultCodeService vmsFaultCodeService;

    @KafkaListener(topics = {"HARBOR_D001_TOPIC"})
    public void alertFaultCode(ConsumerRecord<String, String> consumerRecord, Acknowledgment acknowledgment){
        try {
            String value = consumerRecord.value();
//            log.info("kafka报警信息获取：" + value);
            JSONObject jsonObject = JSON.parseObject(value);
//            if(jsonObject.getLong("collectTime")<(System.currentTimeMillis()-60000)){
//                return;
//            }
            JSONArray faultItems = jsonObject.getJSONArray("faultItems");
            VmsVehicleAlertProcess saveEntity = new VmsVehicleAlertProcess();
            saveEntity.setVin(jsonObject.getString("vin"));
            saveEntity.setAlarmPosition(jsonObject.getString("longitude")+","+jsonObject.getString("latitude"));
            saveEntity.setAlarmTime(jsonObject.getDate("collectTime"));
            if(faultItems.size()>0){
                for(int i=0;i<faultItems.size();i++){
                    JSONObject faultItem = faultItems.getJSONObject(i);
              /*  if(!Arrays.asList("522","523").contains(faultItem.getString("fmi"))){
                    return;
                }*/
                    VmsFaultCode qryEntity = vmsFaultCodeService.lambdaQuery()
                            .eq(VmsFaultCode::getFaultCode,
                                    "A1".equalsIgnoreCase(faultItem.getString("faultCode"))?
                                            "000000".concat(faultItem.getString("faultCode")):faultItem.getString("faultCode"))
                            .eq(VmsFaultCode::getSpn, faultItem.getString("spn"))
                            .eq(VmsFaultCode::getFmi, faultItem.getString("fmi"))
//                        .eq(VmsFaultCode::getFaultType, "P")
                            .one();
//                    log.info("报警信息查询对象：{}",qryEntity);
                    if(Objects.isNull(qryEntity)){
                        continue;
                    }
                    saveEntity.setFaultCode(faultItem.getString("faultCode"));
                    saveEntity.setFmi(faultItem.getString("fmi"));
                    saveEntity.setSpn(faultItem.getString("spn"));
                    saveEntity.setFaultName(qryEntity.getFaultName());
                    if("A1".equalsIgnoreCase(saveEntity.getFaultCode())){
                        vmsVehicleAlertProcessService.save(saveEntity);
                    }
                }
                //存入redis
                redisDataModel.set(RedisKey.ERROR_TOS_INFO.getKeyPrefix().concat(jsonObject.getString("vin")),JSONObject.toJSONString(saveEntity),30000l);
            }
//            abnormalWebSocketServer.sendInfo(JSON.toJSONString(saveEntity));
        } catch (Exception e) {
            log.error("kafka报警信消费：" + e.getMessage());
        } finally {
            acknowledgment.acknowledge();
        }
    }
}
