package com.ssi.kafka.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ssi.model.RedisDataModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

/**
 * @author SunnyHotz
 * @PackageName:com.ssi.kafka.listener
 * @ClassName:TrafficLightStatusListener
 * @Description:
 * @date 2022/9/8 08:35
 */
@Component
@Slf4j
public class TrafficLightStatusListener {

    @Autowired
    private RedisDataModel redisDataModel;

   // @KafkaListener(topics = {"TRAFFIC_CROSS_PHASE_LAMP_STATUS"})
    public void run(ConsumerRecord<String, String> consumerRecord, Acknowledgment acknowledgment) {
        try {
            String value = consumerRecord.value();
            log.info("kafka路灯状态消息消费：" + value);
            JSONObject jsonObject = JSON.parseObject(value);
            JSONArray crossPhaseLampStatus = jsonObject.getJSONArray("crossPhaseLampStatus");
            for (int i = 0; i < crossPhaseLampStatus.size(); i++) {
                JSONObject crossInfo = crossPhaseLampStatus.getJSONObject(i);
                String crossID = crossInfo.getString("crossID");
                String phaseLampStatusList = crossInfo.getString("phaseLampStatusList");
                redisDataModel.hset("ivccs:vms:traffic:light",crossID,phaseLampStatusList);
            }
        } catch (Exception e) {
            log.error("kafka异常报警消息消费：" + e.getMessage());
        } finally {
            acknowledgment.acknowledge();
        }
    }
}
