package com.ssi.kafka.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ssi.constant.RedisKey;
import com.ssi.constant.URL;
import com.ssi.entity.VmsFaultCode;
import com.ssi.entity.VmsVehicleAlertProcess;
import com.ssi.service.VmsFaultCodeService;
import com.ssi.service.VmsVehicleAlertProcessService;
import com.ssi.utils.RestTemplateUtil;
import com.ssi.utils.SpringUtils;
import com.ssi.utils.grid.MercatorVehicle;
import com.ssi.utils.grid.RealTimeVehicleSchedule;
import com.ssi.websocket.AbnormalWebSocketServer;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;
import org.springframework.util.StopWatch;

import java.util.Arrays;
import java.util.Map;
import java.util.Objects;
import java.util.Timer;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author SunnyHotz
 * @PackageName:com.ssi.kafka.listener
 * @ClassName:FaultCodeListener
 * @Description:
 * @date 2022/10/31 14:30
 */
@Component
@Slf4j
public class VehicleRealTimeInfoListener {

    private static RealTimeVehicleSchedule scheduleApi = RealTimeVehicleSchedule.getInstance();
    private final Map<String,MercatorVehicle> cachedMap = new ConcurrentHashMap<>();
    @Value("${command-url}")
    private String commandUrl;

    public void processRoutingConflict(Map<String, Map<String, Object>> messages){
        //实时更新数据
        scheduleApi.pushRealTimeVehicleData(messages);
    }


    public void processVehicleCloseArea(Map<String, Object> messages){
        scheduleApi.pushSingleVehicleData(messages);
        String vid = String.format("%s-%s", messages.get("vehicleNum"), messages.get("vin"));
        if(cachedMap.containsKey(vid)){
            messages.put("areaDisplay",1);
        }
    }

    public void clearOutVersionVehicle(Integer versoinNo){
        scheduleApi.clearOutVersionVehicle(versoinNo);
    }

    public void invokeForOrder(MercatorVehicle mercatorVehicle){
        if(Objects.isNull(mercatorVehicle)){
            return;
        }
        JSONObject reqEntity = new JSONObject();
        reqEntity.put("vin",mercatorVehicle.getVid().split("-")[1]);
        if(mercatorVehicle.waitGridSize()>0&&!cachedMap.containsKey(mercatorVehicle.getVid())){//当速度大于0&&阻塞队列>0
            log.info("grids ====>vin :{}下发停止指令",mercatorVehicle.getVid());
            //下发停止指令
            if(sendOperatorOrder(reqEntity, URL.To_ODBU_CANCEL_TASK, 1))
                SpringUtils.setLocalCache(RedisKey.DYNAMIC_FENCE_ALERT.getKeyPrefix(),reqEntity.getString("vin"),"封闭区触发任务暂停");
            cachedMap.put(mercatorVehicle.getVid(), mercatorVehicle);
        }else if(mercatorVehicle.waitGridSize()<1&&mercatorVehicle.getGridsSize()>0&&cachedMap.containsKey(mercatorVehicle.getVid())){
            log.info("grids ====>vin :{}下发恢复指令",mercatorVehicle.getVid());
            //下发恢复指令
            if(!sendOperatorOrder(reqEntity, URL.To_ODBU_RECOVERY_TASK, 1)) {
                SpringUtils.setLocalCache(RedisKey.DYNAMIC_FENCE_ORDER_ERROR.getKeyPrefix(), reqEntity.getString("vin"), "下发车辆恢复任务指令，执行异常");
            }
            SpringUtils.clearStatus(RedisKey.DYNAMIC_FENCE_ALERT.getKeyPrefix(),reqEntity.getString("vin"));
            cachedMap.remove(mercatorVehicle.getVid());
        }
    }

    private boolean sendOperatorOrder(JSONObject reqEntity,String url,int retryCount) {
        if(retryCount>=0){
            ResponseEntity<JSONObject> responseEntity = RestTemplateUtil.getInstance().postForEntity(commandUrl.concat(url), reqEntity, JSONObject.class);
            JSONObject resp = responseEntity.getBody();
            if(resp.getJSONObject("status")!=null&&"0000".equalsIgnoreCase(resp.getJSONObject("status").getString("code"))){
                return true;
            }
            log.warn("封闭区域触发指令下发结果：{}",resp);
           return sendOperatorOrder(reqEntity,url,--retryCount);
        }
        return false;
    }

    public boolean isContainVin(String vin){
        return  this.cachedMap.containsKey(vin);
    }

/*
    @KafkaListener(topics = {"HARBOR_D00A_TOPIC"})
    public void alertFaultCode(ConsumerRecord<String, String> consumerRecord, Acknowledgment acknowledgment){
        try {
            String value = consumerRecord.value();
          //  log.info("kafka报警信息获取：" + value);
            JSONObject jsonObject = JSON.parseObject(value);
            if(jsonObject.getLong("collectTime")<(System.currentTimeMillis()-5000)){
                return;
            }
//            scheduleApi.pushRealTimeVehicleData();
        } catch (Exception e) {
            log.error("kafka报警信消费：" + e.getMessage());
        } finally {
            acknowledgment.acknowledge();
        }
    }
*/



}
