package com.ssi.kafka.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ssi.constant.enums.V2xEventTypeEnums;
import com.ssi.entity.VehicleRoad;
import com.ssi.utils.DateUtils;
import com.ssi.utils.SpringUtils;
import com.ssi.websocket.VehicleRoadCoordinationWebSocketServer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 *  车路协调kafka监听器
 */
@Component
public class VehicleRoadCoordinationListener {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass());



  @KafkaListener(topics = {"VMS_V2X_EVENT"})
  public void run(ConsumerRecord<String, String> consumerRecord, Acknowledgment acknowledgment) {
    try {
      String value = consumerRecord.value();
//      logger.info("kafka车路协调消费：" + value);
      VehicleRoad record = JSON.parseObject(value, VehicleRoad.class);
      record.setTime(DateUtils.format(new Date(),DateUtils.DATE_PATTERN));
      record.setTypeName(V2xEventTypeEnums.find(record.getType()).getDescript());
      VehicleRoadCoordinationWebSocketServer.sendMessage(record);
    } catch (Exception e) {
      logger.error("kafka车路协调消费：" + e.getMessage());
    } finally {
      acknowledgment.acknowledge();
    }
  }

  @KafkaListener(topics = {"VMS_CHARGING_ACTION"})
  public void chargingAction(ConsumerRecord<String, String> consumerRecord, Acknowledgment acknowledgment) {
    try {
      //
      String value = consumerRecord.value();
      logger.info("车辆充电状态信息：" + value);
      JSONObject record = JSON.parseObject(value);
      record.put("reciveTime",System.currentTimeMillis());
      SpringUtils.setLocalCache("VMS_CHARGING_ACTION",record.getString("vin"),record);
    } catch (Exception e) {
      logger.error("车辆充电状态信息异常：",e);
    } finally {
      acknowledgment.acknowledge();
    }
  }

}
