package com.ssi.model;

import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Set;

/**
 * Description:
 *
 * @author LiXiaoCong
 * @version 2019/10/17 9:38
 */
@Component
@Slf4j
public class RedisConfigModel {

  @Autowired
  private StringRedisTemplate stringRedisTemplate;

  @Value("${cism.redis.config.prefix:cism:config}")
  private String configRedisPrefix;

  @Value("${cism.redis.config.group:extra}")
  private String defaultGroup;


  public void setConfig(String group, String key, String value) {
    if (StringUtils.isNotBlank(key) && StringUtils.isNotBlank(value)) {
      stringRedisTemplate.opsForHash()
          .put(String.format("%s:%s", configRedisPrefix, group), key, value);
    }
  }

  public void setConfig(String key, String value) {
    setConfig(defaultGroup, key, value);
  }

  public void setConfig(String group, Map<String, String> configMap) {
    if (configMap != null && !configMap.isEmpty()) {
      HashOperations<String, String, String> hashOperations = stringRedisTemplate.opsForHash();
      hashOperations.putAll(String.format("%s:%s", configRedisPrefix, group), configMap);
    }
  }

  public void setConfig(Map<String, String> configMap) {
    setConfig(defaultGroup, configMap);
  }


  public String getConfigWithDefault(String key, String defaultValue) {
    return getConfigWithDefault(defaultGroup, key, defaultValue);
  }

  public String getConfigWithDefault(String group, String key, String defaultValue) {
    String value = getConfig(group, key);
    if (value == null) {
      value = defaultValue;
    }
    return value;
  }

  public String getConfig(String group, String key) {
    if (StringUtils.isNotBlank(group) && StringUtils.isNotBlank(key)) {
      HashOperations<String, String, String> hashOperations = stringRedisTemplate.opsForHash();
      return hashOperations.get(String.format("%s:%s", configRedisPrefix, group), key);
    } else {
      return null;
    }
  }

  public String getConfig(String key) {
    return getConfig(defaultGroup, key);
  }

  public Map<String, String> getConfigs(Set<String> keys) {
    return getConfigs(defaultGroup, keys);
  }

  public Map<String, String> getConfigs(String group, Set<String> keys) {
    Map<String, String> configs = Maps.newHashMap();
    if (keys != null && !keys.isEmpty()) {
      keys.forEach(key -> {
        String config = getConfig(group, key);
        if (config != null) {
          configs.put(key, config);
        }
      });
    }
    return configs;
  }

  public void deteles(String group, String... keys) {
    if (StringUtils.isNotBlank(group) && keys.length > 0) {
      HashOperations<String, String, String> hashOperations = stringRedisTemplate.opsForHash();
      hashOperations.delete(String.format("%s:%s", configRedisPrefix, group), keys);
    }
  }

  public void deteles(String... keys) {
    deteles(defaultGroup, keys);
  }
}
