package com.ssi.model;


import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Map;

/**
 * Description: 股份数据增加车辆到达报警触发 arrivedStatus = 1  可能会错过
 * <p>
 * 修改逻辑查询逻辑： 存在变化再往后发 车辆下线不发
 *
 * @author LiXiaoCong
 * @version 2019/9/20 11:48
 */
@Component
@EnableScheduling
@Slf4j
public class WebSocketDataServiceModel {

  @Value("${spring.webscoket.checkRepate:false}")
  private boolean checkRepate;

  @Autowired
  private VehicleLatestDataModel vehicleLatestDataModel;


  /**
   * 停车时经纬度跳变处理
   */
  private Map<String, String> lastMap = Maps.newConcurrentMap();

  public Map<String, Map<String, Object>> realTimeData(Collection<String> vins) {
    return realTimeData(vins, checkRepate);
  }

  public Map<String, Map<String, Object>> realTimeData(Collection<String> vins,
      boolean checkRepeat) {
    Map<String, Map<String, Object>> res;
    if (vins != null && !vins.isEmpty()) {
      String latestRedisKeyPrefix = vehicleLatestDataModel.getLatestRedisKeyPrefix();
      String latestRedisKeyPostfix = vehicleLatestDataModel.getLatestRedisKeyPostfix();
      String latestRedisKeyD001Postfix = vehicleLatestDataModel.getLatestRedisKeyPrefix();
      String latestRedisKeyD002Postfix = vehicleLatestDataModel.getLatestRedisKeyPrefix();
      Map<String, Map<String, String>> keysMap = Maps.newHashMap();
      vins.stream().forEach(key -> {
        Map<String, String> keyPreMap = Maps.newHashMap();
        keyPreMap.put("latestKey",
            String.format("%s:%s-%s", latestRedisKeyPrefix, key, latestRedisKeyPostfix));
        keyPreMap.put("d001Key",
            String.format("%s:%s-%s", latestRedisKeyPrefix, key, latestRedisKeyD001Postfix));
        keyPreMap.put("d002Key",
            String.format("%s:%s-%s", latestRedisKeyPrefix, key, latestRedisKeyD002Postfix));
        keysMap.put(key, keyPreMap);
      });
      res = vehicleLatestDataModel.readLatestDataFromRedis(keysMap, checkRepeat);
    } else {
      res = Maps.newHashMap();
    }
    return res;
  }

  public Map<String, Map<String, Object>> realTimeDataAll(boolean checkRepeat) {
    return vehicleLatestDataModel.allRealTimeData(checkRepeat);
  }

  public Map<String, Map<String, Object>> getVehicleRunData() {
    return vehicleLatestDataModel.getVehicleRunData();
  }

}

