package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alicp.jetcache.anno.CacheRefresh;
import com.alicp.jetcache.anno.Cached;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ssi.constant.enums.VehicleTaskLabelEnum;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.VmsVehicleOperateTask;
import com.ssi.entity.VmsChargingTotal;
import com.ssi.entity.VmsTosOrders;
import com.ssi.entity.vo.OperateDataVo;
import com.ssi.entity.vo.VehicleStatusVo;
import com.ssi.mapper.VmsCarryCapacityAnalysisMapper;
import com.ssi.mapper.VmsVehicleOperateTaskMapper;
import com.ssi.mapper.VmsChargingTotalMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIResponse;
import com.ssi.service.BigScreenService;
import com.ssi.service.VmsVehicleService;
import com.ssi.service.VmsTosOrdersService;
import com.ssi.task.TaskOperateExecutor;
import com.ssi.utils.ToolUtils;
import com.ssi.utils.VehicleUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 大屏接口服务层实现类
 * @author ZhangLiYao
 * @version 1.0
 * @date 2020/8/10 9:51
 */
@Service("screenService")
public class BigScreenServiceImpl implements BigScreenService {

    @Autowired
    private TaskOperateExecutor taskOperateExecutor;
    @Autowired
    private VmsVehicleOperateTaskMapper vmsVehicleOperateTaskMapper;
    @Autowired
    private VmsChargingTotalMapper vmsChargingTotalMapper;
    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private VmsTosOrdersService vmsTosOrdersService;
    @Autowired
    private RedisDataModel redisDataModel;
    @Autowired
    private BigScreenService bigScreenService;
    @Autowired
    private VmsCarryCapacityAnalysisMapper vmsCarryCapacityAnalysisMapper;

    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;
    @Value("${ivccs.vehicle.latestData.redis.prefix:ivccs:vms:vehicle:latest}")
    private String latestRedisKeyPrefix;
    @Value("${vehicle.latestData.redis.postfix:harbor_D00A}")
    private String realPostfix;

    /**
     * 大屏获取运营数据
     */
    @Override
    @Cached(name = "bigScreenServiceImpl:getOperateData", expire = 60, timeUnit = TimeUnit.SECONDS)
    @CacheRefresh(refresh = 20, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.SECONDS)
    public SSIResponse getOperateData(List<String> vins, Date startDate, Date endDate) {
        /*OperateDataVo operateDataVo = new OperateDataVo(BigDecimal.valueOf(0), BigDecimal.valueOf(0));

        List<VmsVehicleOperateTask> vmsVehicleOperateTasks = vmsVehicleOperateTaskMapper.selectList(new LambdaQueryWrapper<VmsVehicleOperateTask>()
                .eq(StringUtils.isNotBlank(vin), VmsVehicleOperateTask::getVin, vin));
        vmsVehicleOperateTasks.stream().forEach(vmsVehicleOperateTask -> {
            operateDataVo.setTravelmile(operateDataVo.getTravelmile()
                    .add(vmsVehicleOperateTask.getOperateMile().divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP)));
            operateDataVo.setTravelTime(operateDataVo.getTravelTime()
                    .add(vmsVehicleOperateTask.getOperateTime().divide(new BigDecimal(60 * 60 * 1000), 2, BigDecimal.ROUND_HALF_UP)));
        });

        long[] today = getToday();
        List<String> vins = Lists.newArrayList();
        if (StringUtils.isNotBlank(vin)) {
            vins = Arrays.asList(vin.split(","));
        } else {
            vins = vmsVehicleService.list().stream().map(VmsVehicle::getVin).collect(Collectors.toList());
        }
        for (String curVin : vins) {
            addData(operateDataVo, curVin, today[0], System.currentTimeMillis());
        }*/
        OperateDataVo operateDataVo = vmsCarryCapacityAnalysisMapper.getOverallOperateData(vins,startDate,endDate);
        return SSIResponse.ok(operateDataVo);
    }

    /**
     * 车辆状态分布
     */
    @Override
    public SSIResponse getVehicleStatusDist() {
        List<VmsVehicle> vehicleList = vmsVehicleService.getNormalVehicle();
        VehicleStatusVo vehicleStatusVo = vmsVehicleService.vehicleStatusDist(vehicleList);
        return SSIResponse.ok(vehicleStatusVo);
    }

    /**
     * 大屏查询单车信息
     */
    @Override
    public SSIResponse getVehicleInfo(String vin) {
        Map<String, Object> resultMap = new HashMap<>();

        //车辆信息添加
        VmsVehicle vehicle = vmsVehicleService.getOne(new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getVin, vin));
        resultMap.put("vehicleNum", vehicle.getVehicleNum());
        resultMap.put("vin", vehicle.getVin());
        Map<String, Object> realMap = redisDataModel.getJson2Map(String.format("%s:%s-%s", latestRedisKeyPrefix, vehicle.getVin(), realPostfix));
        if (realMap == null) {
            realMap = Maps.newHashMap();
        }
        resultMap.put("soc", realMap.get("soc"));

        //运营信息添加
        SSIResponse operateResponse = bigScreenService.getOperateData(Arrays.asList(vin),null,null);
        OperateDataVo operateDataVo = (OperateDataVo) operateResponse.getData();

        resultMap.put("travelMile", operateDataVo.getTravelmile());
        resultMap.put("travelTime", operateDataVo.getTravelTime());

        //任务箱数添加
        SSIResponse ordersNumResponse = vmsTosOrdersService.getNumByVin(vin);
        Map<String, Integer> ordersNum = (Map<String, Integer>) ordersNumResponse.getData();
        resultMap.putAll(ordersNum);

        //任务状态添加
        Map<String, Object> orderMap = redisDataModel.getJson2Map(String.format("%s:%s", latestOrderKeyPrefix, vehicle.getVin()));
        VmsTosOrders vmsTosOrders = JSON.parseObject(JSON.toJSONString(orderMap), VmsTosOrders.class);
        if (vmsTosOrders != null && !VehicleUtil.isTaskOver(vmsTosOrders.getStatus())) {
            resultMap.put("taskType", vmsTosOrders.getVehicleTaskLabel());
        } else {
            resultMap.put("taskType", VehicleTaskLabelEnum.PARK.getCode());
        }

        Integer chargeNum = vmsChargingTotalMapper.selectCount(new LambdaQueryWrapper<VmsChargingTotal>().eq(VmsChargingTotal::getVin, vin));
        resultMap.put("chargeNum", chargeNum);
        resultMap.put("mileage", realMap.get("mileage"));

        return SSIResponse.ok(resultMap);
    }

    private long[] getToday() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
        long start = calendar.getTime().getTime();
        Calendar calendar2 = Calendar.getInstance();
        calendar2.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
        long end = calendar2.getTime().getTime();
        long[] result = new long[]{start, end};
        return result;
    }

    private void addData(OperateDataVo operateDataVo, String vin, long start, long end) {
        VmsVehicleOperateTask vehicleOpreateToday = taskOperateExecutor.getVehicleOpreateByTime(vin, start, end);
        if (vehicleOpreateToday != null) {
            operateDataVo.setTravelmile(operateDataVo.getTravelmile().add(vehicleOpreateToday.getOperateMile()));
            operateDataVo.setTravelTime(operateDataVo.getTravelTime().add(vehicleOpreateToday.getOperateTime()));
        }
    }
}
