package com.ssi.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.constant.URL;
import com.ssi.entity.CargoShipConfigInfo;
import com.ssi.entity.VmsAreaBusinessInfo;
import com.ssi.entity.VmsShipInfo;
import com.ssi.entity.map.MapDataDto;
import com.ssi.entity.vo.CargoShipConfigDto;
import com.ssi.entity.vo.LocationVo;
import com.ssi.response.SSIPage;
import com.ssi.service.CargoShipConfigInfoService;
import com.ssi.mapper.CargoShipConfigInfoMapper;
import com.ssi.service.VmsAreaBusinessInfoService;
import com.ssi.service.VmsCranePadBindService;
import com.ssi.service.VmsShipsDrawingService;
import com.ssi.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.common.recycler.Recycler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.mapping.PreferredConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.swing.tree.TreeNode;
import java.awt.geom.Path2D;
import java.util.*;
import java.util.stream.Collectors;

/**
*
*/
@Service
@DS("mysql")
@Slf4j
public class CargoShipConfigInfoServiceImpl extends ServiceImpl<CargoShipConfigInfoMapper, CargoShipConfigInfo>
implements CargoShipConfigInfoService{

    @Value("${map-edit-url}")
    private String mapUrl;

    @Autowired
    private VmsCranePadBindService vmsCranePadBindService;

    @Autowired
    private VmsAreaBusinessInfoService vmsAreaBusinessInfoService;
    @Autowired
    private VmsShipsDrawingService vmsShipsDrawingService;

    @Override
    public List<CargoShipConfigDto> queryOnWorkingList() {
        List<CargoShipConfigDto> dtos = this.baseMapper.queryOnWorkingList(new CargoShipConfigDto());
        dtos.parallelStream().forEach(cargoShipConfigDto -> {
            if(StringUtils.isNotBlank(cargoShipConfigDto.getPassArea())){
                cargoShipConfigDto.setPassAreaObj(JSONObject.parseObject(cargoShipConfigDto.getPassArea()));
            }
            cargoShipConfigDto.setYardBayObj(JSONObject.parseObject(cargoShipConfigDto.getYardBay()));
        });
        return dtos;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class )
    public void enableConfig(List<CargoShipConfigDto> cargoShipConfigDtos) {
        clearShipInfo(cargoShipConfigDtos);
        for (CargoShipConfigDto cargoShipConfigDto:cargoShipConfigDtos) {
            if(cargoShipConfigDto.getIsActive().compareTo((byte)1)==0){
                //激活或更新配置需先取消其它配置
//                this.baseMapper.updateByIsActive();
                //桥吊配置确认
                if(checkBridgeStatus(cargoShipConfigDto)){
                    continue;
                }
                //调用路径规划端接口，设置桥吊缓冲区、桥吊扭锁站对应
                if(StringUtils.isNotBlank(cargoShipConfigDto.getPassArea())){
                    JSONObject passAreaObj = JSONObject.parseObject(cargoShipConfigDto.getPassArea());
                    String upArea = passAreaObj.getString("upArea");
                    String downArea = passAreaObj.getString("downArea");
                    String upAreaId = passAreaObj.getString("upAreaId");
                    String downAreaId = passAreaObj.getString("downAreaId");
                    //激活当前配置删除其它配置
                    vmsAreaBusinessInfoService.remove(new QueryWrapper<VmsAreaBusinessInfo>().eq("area_type",5));
                    updatePassArea(cargoShipConfigDto.getBridgeNo(),upArea,upAreaId,"1");
                    updatePassArea(cargoShipConfigDto.getBridgeNo(),downArea,downAreaId,"2");

                }
                if(StringUtils.isNotBlank(cargoShipConfigDto.getLockArea())){
                    log.info("激活扭锁信息");
                    String[] lockAreas = cargoShipConfigDto.getLockArea().split(",");
                    Arrays.stream(lockAreas).forEach(lockArea->{
                        vmsCranePadBindService.updateCraneBind(null,null,lockArea,cargoShipConfigDto.getBridgeNo());
                    });
                }
            }else{//配置移除

            }
            //保存当前配置
            this.save(cargoShipConfigDto);
        }
    }

    @Override
    public JSONObject listByCondition(CargoShipConfigDto cargoShipConfigDto) {

        JSONObject result = new JSONObject();
        List<CargoShipConfigDto> dtos = this.baseMapper.queryOnWorkingList(cargoShipConfigDto);
        List<String> workBridge = new ArrayList<>();
        dtos.parallelStream().forEach(dto -> {
            if(StringUtils.isNotBlank(dto.getPassArea())){
                dto.setPassAreaObj(JSONObject.parseObject(dto.getPassArea()));
            }
            dto.setYardBayObj(JSONObject.parseObject(dto.getYardBay()));
            workBridge.add(dto.getBridgeNo());
        });
        result.put("workBridge",workBridge);
        result.put("result",dtos);
        return result;
    }

    private void clearShipInfo(List<CargoShipConfigDto> cargoShipConfigDtos) {
       /* CargoShipConfigInfo delEntity = new CargoShipConfigInfo();
        //delEntity.setCode(cargoShipConfigDtos.get(0).getCode());
        delEntity.setName(cargoShipConfigDtos.get(0).getName());*/
        this.baseMapper.delete(new LambdaQueryWrapper<CargoShipConfigInfo>().eq(CargoShipConfigInfo::getName,cargoShipConfigDtos.get(0).getName()));
    }

    private Boolean checkBridgeStatus(CargoShipConfigDto cargoShipConfigDto) {
        assert  cargoShipConfigDto.getBridgeNo() != null:"bridgeNo不能为null";

//        SSIPage ssiPage = vmsShipsDrawingService.queryPage(new HashMap<String, Object>() {{
//            put("pageSize", 999);
//            put("pageIndex", 1);
//        }});
//        List<VmsShipInfo> shipCodes = (List<VmsShipInfo>)ssiPage.getList();
//        List<String> voyageNo = shipCodes.stream().map(VmsShipInfo::getVoyageNo).collect(Collectors.toList());
        List<CargoShipConfigInfo> list = this.lambdaQuery().eq(CargoShipConfigInfo::getBridgeNo, cargoShipConfigDto.getBridgeNo())
                .ne(CargoShipConfigInfo::getCode,cargoShipConfigDto.getCode())
                .list();
        for (CargoShipConfigInfo oldCargo: list) {
//            if(oldCargo.getIsActive().compareTo((byte)1) == 0&&voyageNo.contains(oldCargo.getCode())){
//                return true;
//            }
            this.baseMapper.deleteById(oldCargo.getId());
        }
        return false;
    }

    @Override
    public List<JSONObject> qryEnableCacheArea(String beginLocation, String endLocation) {


        //计算坐标点
        List<LocationVo> orderLocations = getOrderLocation(beginLocation,endLocation);
        //绘制区域
        Path2D.Double ruleArea = drawRuleArea(orderLocations);
        //获取所有缓冲区
        List<JSONObject> bufferArray = qryCacheArea();
        Iterator<JSONObject> iterator = bufferArray.iterator();
        while (iterator.hasNext()){
            JSONObject buffeArea =iterator.next();
            boolean contains = ruleArea.contains(buffeArea.getDouble("centerlng"), buffeArea.getDouble("centerlat"));
            if(contains){
                iterator.remove();
                continue;
            }
          boolean locateCheck1 = ruleArea.contains(buffeArea.getDouble("lng1"), buffeArea.getDouble("lat1"));
          boolean locateCheck2 = ruleArea.contains(buffeArea.getDouble("lng2"), buffeArea.getDouble("lat2"));
          boolean locateCheck3 = ruleArea.contains(buffeArea.getDouble("lng3"), buffeArea.getDouble("lat3"));
          boolean locateCheck4 = ruleArea.contains(buffeArea.getDouble("lng4"), buffeArea.getDouble("lat4"));
          if(locateCheck1||locateCheck2||locateCheck3||locateCheck4){
                iterator.remove();
          }
        }
        return bufferArray;
    }

    private  List<JSONObject> qryCacheArea(){
        JSONObject param = new JSONObject();
        param.put("state", "0");// 可正常使用
        log.debug(String.format("绘制地图区域请求发送:----%s", param.toJSONString()));
        String result = RestTemplateUtil.post(String.format("%s%s", mapUrl, URL.MAP_QUERY_URL),  param.toJSONString(), null);
        log.debug(String.format("绘制地图区域请求返回结果:----%s", result));
        JSONObject jsonObject = JSONObject.parseObject(result);
        if(Objects.nonNull(jsonObject)&&jsonObject.getInteger("code")==1){
            JSONArray data = jsonObject.getJSONArray("data");
            List<JSONObject> results = data.toJavaList(JSONObject.class).stream().filter((obj) -> {
                if (!"1,5".contains(obj.getString("type"))) {
                    return false;
                }
                return true;
            }).sorted((a, b) -> {
                String oa = a.getString("name").replaceAll("^\\w","");
                String ob = b.getString("name").replaceAll("^\\w","");
                return Integer.valueOf(oa).compareTo(Integer.valueOf(ob));
            }).collect(Collectors.toList());
            return results;
        }
        return Collections.emptyList();
    }


    public  Path2D.Double drawRuleArea( List<LocationVo> orderLocations) {
        Path2D.Double generalPath = new Path2D.Double();
        Double startLng = null;
        Double startLat = null;
        for(LocationVo entity:orderLocations){
            if(startLng == null||startLat == null){
                startLng = entity.getLon();
                startLat = entity.getLat();
                generalPath.moveTo(startLng,startLat);
            }else {
                generalPath.lineTo(entity.getLon(),entity.getLat());
            }
        }
        generalPath.lineTo(startLng,startLat);
        generalPath.closePath();
        return generalPath;
    }

    private  List<LocationVo> getOrderLocation(String beginLocation, String endLocation) {
        
        String[] startLocation = beginLocation.replaceAll(";", "").split(",");
        String[] lastLocation = endLocation.replaceAll(";", "").split(",");
        if(startLocation.length!=2||lastLocation.length!=2){
            return null;
        }
        List<LocationVo> locationVos = new ArrayList<>();
        Double startLongitude = Double.valueOf(startLocation[0]);
        Double startLatitude = Double.valueOf(startLocation[1]);
        Double lastLongitude = Double.valueOf(lastLocation[0]);
        Double lastLatitude = Double.valueOf(lastLocation[1]);
        if(startLongitude > lastLongitude){
            startLongitude = startLongitude - 0.000175;
            lastLongitude = lastLongitude + 0.000175;
        }else {
            startLongitude = startLongitude + 0.000175;
            lastLongitude = lastLongitude - 0.000175;
        }
        locationVos.add(new LocationVo(startLongitude,startLatitude,null));
        locationVos.add(new LocationVo(startLongitude+0.000189,startLatitude+0.000720,null));
        locationVos.add(new LocationVo(lastLongitude+0.000189,lastLatitude+0.000720,null));
        locationVos.add(new LocationVo(lastLongitude,lastLatitude,null));
        return locationVos;
    }



    private String updatePassArea(String bridgeNos,String passArea,String passAreaId,String placeType){
        if(StringUtils.isBlank(passArea)){
            return null;
        }
        MapDataDto mapdata = new MapDataDto();
        mapdata.setId(passAreaId);
        mapdata.setNumber(passArea);
        mapdata.setTaskType(Integer.valueOf(placeType));
        mapdata.setBridge(bridgeNos);
        mapdata.setBridgeNo(bridgeNos);
        mapdata.setPlacetype(placeType);
        mapdata.setType("5");
        mapdata.setState("0");
        mapdata.setField("F");
        mapdata.setName(passArea);
        String paramData = JSONObject.toJSONString(mapdata);
		log.info(String.format("绘制地图区域update请求发送:----%s", paramData));
        String result = RestTemplateUtil.post(String.format("%s%s", mapUrl, URL.MAP_EDIT_URL), paramData, null);
		log.info(String.format("绘制地图区域update返回结果:----%s", result));
        JSONObject jsonObject = JSONObject.parseObject(result);
		if(Objects.nonNull(jsonObject)&&jsonObject.getInteger("code")==1){//成功则更新
            vmsAreaBusinessInfoService.updatePassArea(mapdata);
        }
		return result;
    }
}
