package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ssi.constant.RedisKey;
import com.ssi.entity.CraneInfo;
import com.ssi.entity.VmsIndividuationConfig;
import com.ssi.model.CraneInfoRedis;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIResponse;
import com.ssi.service.CraneInfoService;
import java.util.HashMap;

import com.ssi.service.VmsAreaBusinessInfoService;
import com.ssi.service.VmsCranePadBindService;
import com.ssi.service.VmsIndividuationConfigService;
import lombok.val;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 吊具信息服务层实现类
 */
@Service
public class CraneInfoServiceImpl implements CraneInfoService {

  @Autowired
  private RedisDataModel redisDataModel;

  @Autowired
  private VmsIndividuationConfigService individuationConfigService;

  @Value("${harbor.crane.realTimeInfoKeyPrefix}")
  private String craneRealTimeInfoKeyPrefix;

  @Value("${harbor.crane.confirm}")
  private String craneConfirmKeyPrefix;

  @Autowired
  private VmsAreaBusinessInfoService vmsAreaBusinessInfoService;

  /**
   * 获取桥吊、场吊信息列表
   */
  @Override
  public SSIResponse queryCraneList(CraneInfo craneInfo) {
    List<Object> resList = queryCraneListFromRedis(craneInfo);
    if (resList != null) {
      for (Object object: resList){
        JSONObject object1 = (JSONObject) object;
        int portType = object1.getIntValue("portType");
        if(1==portType){
          List<String> portCode = vmsAreaBusinessInfoService.getByCrane(object1.getString("portCode"),craneInfo.getTaskType(),craneInfo.getTaskTypes());
          object1.put("passArea",portCode);
          object1.put("taskType",craneInfo.getTaskType());
        }
      }
      return SSIResponse.ok(resList);
    }
    return SSIResponse.no("未查询到数据");
  }

  @Override
  public Map<String, CraneInfoRedis> queryCraneListFromRedis() {
    if (redisDataModel.keys(RedisKey.CRANE_INFO.getKeyPrefix() + "*").size() == 0){
      return null;
    }
    val d = redisDataModel.mgetForMap(
        redisDataModel.keys(RedisKey.CRANE_INFO.getKeyPrefix() + "*"));
    Map<String, CraneInfoRedis> map = new HashMap<>();
    d.forEach((key, value) -> {
      if (value != null) {
        map.put(StringUtils.removeStart(key, RedisKey.CRANE_INFO.getKeyPrefix()),
            JSON.parseObject((String) value, CraneInfoRedis.class));
      }
    });
    return map;
  }

  @Override
  public List<Object> queryCraneListFromRedis(CraneInfo craneInfo) {
    Set<String> keys = redisDataModel.keys(craneRealTimeInfoKeyPrefix + "*");
    if (CollectionUtils.isNotEmpty(keys)) {
      List<String> list = redisDataModel.mget(keys.toArray(new String[keys.size()]));
      Integer craneType = craneInfo != null ? craneInfo.getCraneType() : null;
      String craneNo = craneInfo != null ? craneInfo.getCraneNo() : null;
      if (CollectionUtils.isNotEmpty(list)) {
        List<Object> res = list.stream().map(s -> JSON.parseObject(s)).filter(obj -> {
          if (craneType != null) {
            Integer type = (Integer) obj.get("portType");
            if (!craneType.equals(type)) {
              return false;
            }
          }
          if (craneNo != null) {
            String code = String.valueOf(obj.get("portCode"));
            if (!code.startsWith(craneNo)) {
              return false;
            }
          }
          return true;
        }).collect(Collectors.toList());
        return res;
      }
    }
    return null;
  }

  /**
   * 获取桥吊、场吊、龙锁装卸区信息列表
   */
  @Override
  public SSIResponse queryCraneLockAreaList(CraneInfo craneInfo) {
    List<Object> resList = queryCraneListFromRedis(craneInfo);
    //暂时不用先注释掉
        /*String craneNo = craneInfo != null ? craneInfo.getCraneNo() : null;
        if(craneNo == null || "扭锁装卸区".startsWith(craneNo)){
            JSONObject object = new JSONObject();
            object.put("portCode", "扭锁装卸区");
            object.put("portType",3);
            resList.add(object);
        }*/
    if (resList != null) {
      return SSIResponse.ok(resList);
    }
    return SSIResponse.no("未查询到数据");
  }

  @Override
  public Map queryCraneConfirmInfo(String craneNo) {
    return redisDataModel.getJson2Map(craneConfirmKeyPrefix + ":" + craneNo);
  }

  @Override
  public SSIResponse queryCraneColor(String craneNo) {

    VmsIndividuationConfig vmsIndividuationConfig = individuationConfigService.lambdaQuery().eq(VmsIndividuationConfig::getConfigKey, "crane_color_config").one();
    String configValue = vmsIndividuationConfig.getConfigValue();
    JSONObject jsonObject = JSON.parseObject(configValue);
    if (jsonObject != null) {
      return SSIResponse.ok(StringUtils.isNotBlank(craneNo)? jsonObject.getString(craneNo):jsonObject);
    }
    return SSIResponse.no("未查询到数据");
  }
}
