package com.ssi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.controller.OperationAnalysisController;
import com.ssi.entity.VmsChargingTotal;
import com.ssi.entity.VmsKpiAnalysis;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.dto.AnalyseTaskOrderResp;
import com.ssi.entity.dto.VehicleTaskOrderReq;
import com.ssi.entity.vo.*;
import com.ssi.mapper.VmsCarryCapacityAnalysisMapper;
import com.ssi.mapper.VmsChargingTotalMapper;
import com.ssi.mapper.VmsTosOrdersAnalysisMapper;
import com.ssi.mapper.VmsTosOrdersMapper;
import com.ssi.response.SSIResponse;
import com.ssi.service.*;
import com.ssi.utils.ChargeDailyTimeUtil;
import com.ssi.utils.DateUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * 运营分析服务实现类
 */
@Service
public class OperationAnalysisServiceImpl implements OperationAnalysisService {

  @Autowired
  private VmsVehicleService vmsVehicleService;
  @Autowired
  private VmsChargingPileService vmsChargingPileService;
  @Autowired
  private VmsTosOrdersMapper vmsTosOrdersMapper;
  @Autowired
  private BigScreenService bigScreenService;
  @Autowired
  private VmsCarryCapacityAnalysisMapper vmsCarryCapacityAnalysisMapper;
  @Autowired
  private VmsChargingTotalMapper vmsChargingTotalMapper;
  @Autowired
  private VmsTosOrdersAnalysisMapper vmsTosOrdersAnalysisMapper;
  @Autowired
  private VmsKpiAnalysisService vmsKpiAnalysisService;

  /**
   * 各项总体数据
   * @param vins
   * @param interval
   */
  @Override
  public SSIResponse overallData(List<String> vins, List<Date> interval) {
    Map<String, Object> res = new HashMap();
    //车辆总数
    //int totalVehicle = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getStatus, 0).count();
    int totalVehicle = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getStatus, 0)
            .in(VmsVehicle::getVehicleType,2).count();
    res.put("totalVehicle", totalVehicle);
    Date startDate = new Date();
    Date endDate = new Date();
    if(CollectionUtils.isNotEmpty(interval)){
      startDate = interval.get(0);
      startDate.setHours(0);
      endDate = org.apache.commons.lang3.time.DateUtils.addDays(interval.get(1), 1);
      endDate.setHours(0);
    }else {
      startDate = org.apache.commons.lang3.time.DateUtils.addDays(startDate, -1);
      startDate.setHours(0);
      endDate.setHours(0);
    }
    //车辆有效使用率
    double vehicleEffectiveUseRate = vmsCarryCapacityAnalysisMapper.getVehicleEffectiveUseRate(vins,startDate,endDate);
    res.put("vehicleEffectiveUseRate", vehicleEffectiveUseRate);
    //充电桩总数
    int totalChargingPile = vmsChargingPileService.count();
    res.put("totalChargingPile", totalChargingPile);
    //充电桩利用率
    double chargingPileUseRate = vmsChargingTotalMapper.getChargingPileUseRate(vins,startDate,endDate);
    res.put("chargingPileUseRate", chargingPileUseRate);
    //充电次数
    res.put("chargingNum",vmsChargingTotalMapper.queryChargingTimes(vins,startDate,endDate));
/*    //行驶时长（小时）
    SSIResponse operateData = bigScreenService.getOperateData(vins,startDate,endDate);
    OperateDataVo operateDataVo = (OperateDataVo) operateData.getData();
    BigDecimal travelTime = operateDataVo.getTravelTime();
    res.put("travelTime", travelTime);
    //行驶里程（公里）
    BigDecimal travelMile = operateDataVo.getTravelmile();
    res.put("travelMile", travelMile);
    //装船箱数
    TaskAnalysisVo numVo = vmsTosOrdersMapper.queryContainerNum(vins,startDate,endDate);
    res.put("loadContainerNum", numVo.getLoadContainerNum());
    //卸船箱数
    res.put("unloadContainerNum", numVo.getUnloadContainerNum());
    //理货箱数
    res.put("handleContainerNum", numVo.getHandleContainerNum());
    //平均每箱装船时间
    TaskAnalysisVo timeVo = vmsTosOrdersMapper.queryContainerAvgTime(vins,startDate,endDate);
    res.put("loadContainerAvgTime", timeVo.getLoadContainerAvgTime());
    //平均每箱卸船时间
    res.put("unloadContainerAvgTime", timeVo.getUnloadContainerAvgTime());
    //平均每箱理货时间
    res.put("handleContainerAvgTime", timeVo.getHandleContainerAvgTime());
    TaskAnalysisVo orderNum = vmsTosOrdersMapper.queryOrderNum(vins,startDate,endDate);
    //装船任务数
    res.put("loadOrderNum", orderNum.getLoadContainerNum());
    //卸船任务数
    res.put("unloadOrderNum", orderNum.getUnloadContainerNum());
    //理货任务数
    res.put("handleOrderNum", orderNum.getHandleContainerNum());
    //总任务数
    res.put("totalOrderNum", orderNum.getHandleContainerNum()+ orderNum.getUnloadContainerNum()+orderNum.getLoadContainerNum());
    */
    KpiCostTimeVo qryKpi = new KpiCostTimeVo();
    qryKpi.setStartTime(startDate);
    qryKpi.setEndTime(endDate);
    qryKpi.setVins(vins);
    VmsKpiAnalysis vmsKpiAnalysis = vmsKpiAnalysisService.staticsSpeciParam(qryKpi);
    //总耗电量
    res.put("consume_total",Objects.nonNull(vmsKpiAnalysis)?vmsKpiAnalysis.getEnergyConsumption():0);
    //行驶时长（小时）
    res.put("travelTime",Objects.nonNull(vmsKpiAnalysis)?vmsKpiAnalysis.getWorkTime():0);
    //行驶里程（公里）
    res.put("travelMile",Objects.nonNull(vmsKpiAnalysis)?vmsKpiAnalysis.getMileage():0);
    JSONObject jsonObject = vmsKpiAnalysisService.taskIndexAnalysis(qryKpi);
    //总任务数
    //装船任务数
    //装船箱数（标准）
    //装船箱数（自然）
    //平均装箱时间
    //卸箱任务数
    //卸箱箱数（标准）
    //卸箱箱数（自然）
    //平均卸船时间
    //理货任务数
    //理货箱数（标准）
    //理货箱数（自然）
    //平均理货时间
    res.putAll(jsonObject);
    return SSIResponse.ok(res);
  }

  /**
   * 运输效率趋势图
   */
  @Override
  public SSIResponse getTaskTrend(Integer timeType) throws Exception {
    switch (timeType) {
      //日
      case 1:
        LocalDateTime startTime = LocalDateTime.of(
            LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
        LocalDateTime endTime = LocalDateTime.of(
            LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
        Long startTimeStamp = startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        List<TaskAnalysisVo> taskTrendByHour = vmsTosOrdersMapper.getTaskTrendByHour(startTime,
            endTime, startTimeStamp);
        return SSIResponse.ok(TaskAnalysisVo.fillBlankHour(taskTrendByHour, TaskAnalysisVo.class));
      //周
      case 2:
        startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MIN);
        endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MAX);
        List<TaskAnalysisVo> taskTrendByDay = vmsTosOrdersAnalysisMapper.getTaskTrendByDay(
            startTime, endTime);
        return SSIResponse.ok(
            TaskAnalysisVo.fillBlankDay(taskTrendByDay, TaskAnalysisVo.class, startTime, endTime));
      //月
      case 3:
        startTime = LocalDateTime.of(LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MIN);
        endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MAX);
        taskTrendByDay = vmsTosOrdersAnalysisMapper.getTaskTrendByDay(startTime, endTime);
        return SSIResponse.ok(
            TaskAnalysisVo.fillBlankDay(taskTrendByDay, TaskAnalysisVo.class, startTime, endTime));
    }
    return null;
  }

  /**
   * 停车等待时长分析
   */
  @Override
  public SSIResponse getStopWaitTimeRate(Long startTime) {
    Map stopWaitTimeRate = vmsCarryCapacityAnalysisMapper.getStopWaitTimeRate(startTime);
    return SSIResponse.ok(stopWaitTimeRate);
  }

  /**
   * 运载力趋势图（空载里程、满载里程）
   */
  @Override
  public SSIResponse getTaskMileTrend(Integer timeType) throws Exception {
    LocalDateTime startTime = null;
    LocalDateTime endTime = null;
    switch (timeType) {
      case 2:
        startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MIN);
        endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MAX);
        break;
      case 3:
        startTime = LocalDateTime.of(LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MIN);
        endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MAX);
        break;
    }
    List<TaskMileTrendByDayVo> taskMileTrendByDay = vmsCarryCapacityAnalysisMapper.getTaskMileTrendByDay(
        startTime, endTime);
    return SSIResponse.ok(
        TaskMileTrendByDayVo.fillBlankDay(taskMileTrendByDay, TaskMileTrendByDayVo.class, startTime,
            endTime));
  }

  /**
   * 充电桩利用率、充电时长趋势图
   */
  @Override
  public SSIResponse getChargingPileUseTrend(Integer timeType) throws Exception {
    switch (timeType) {
      case 1:
        LocalDateTime startTime = LocalDateTime.of(
            LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
        LocalDateTime endTime = LocalDateTime.of(
            LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
        List<VmsChargingTotal> startList = vmsChargingTotalMapper.selectList(
            new LambdaQueryWrapper<VmsChargingTotal>()
                .or(vmsChargingTotalLambdaQueryWrapper -> {
                  vmsChargingTotalLambdaQueryWrapper.gt(VmsChargingTotal::getStartTime,
                          startTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli())
                      .lt(VmsChargingTotal::getStartTime,
                          endTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli());
                })
                .or(vmsChargingTotalLambdaQueryWrapper -> {
                  vmsChargingTotalLambdaQueryWrapper.gt(VmsChargingTotal::getEndTime,
                          startTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli())
                      .lt(VmsChargingTotal::getEndTime,
                          endTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli());
                })
        );
        ChargeDailyTimeUtil chargeDailyTimeUtil = new ChargeDailyTimeUtil(
            Date.from(startTime.atZone(ZoneId.systemDefault()).toInstant()));
        startList.stream().forEach(vmsChargingTotal -> {
          Long startTime1 = vmsChargingTotal.getStartTime();
          Long endTime1 = vmsChargingTotal.getEndTime();
          chargeDailyTimeUtil.add(startTime1, endTime1,
              vmsChargingTotal.getChargingEnergy().doubleValue());
        });
        chargeDailyTimeUtil.finish(2);
        return SSIResponse.ok(chargeDailyTimeUtil.getDailyTimeBeanList());
      case 2:
        startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MIN);
        endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MAX);
        List<ChargingPileUseRateTrendByDayVo> chargingPileUseRateTrendByDay = vmsChargingTotalMapper.getChargingPileUseRateTrendByDay(
            startTime, endTime);
        return SSIResponse.ok(
            ChargingPileUseRateTrendByDayVo.fillBlankDay(chargingPileUseRateTrendByDay,
                ChargingPileUseRateTrendByDayVo.class, startTime, endTime));
      case 3:
        startTime = LocalDateTime.of(LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MIN);
        endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(),
            LocalTime.MAX);
        chargingPileUseRateTrendByDay = vmsChargingTotalMapper.getChargingPileUseRateTrendByDay(
            startTime, endTime);
        return SSIResponse.ok(
            ChargingPileUseRateTrendByDayVo.fillBlankDay(chargingPileUseRateTrendByDay,
                ChargingPileUseRateTrendByDayVo.class, startTime, endTime));
    }
    return SSIResponse.ok();
  }

  /**
   * 平均一周充电频次分析
   */
  @Override
  public SSIResponse getChargingFrequencyByVehicle() {
    List<Map> chargingFrequencyByVehicle = vmsChargingTotalMapper.getChargingFrequencyByVehicle();
    return SSIResponse.ok(chargingFrequencyByVehicle);
  }

  /**
   * 平均一周充电频次平均值
   */
  @Override
  public SSIResponse getOverallChargingFrequency() {
    double overallChargingFrequency = vmsChargingTotalMapper.getOverallChargingFrequency();
    return SSIResponse.ok(overallChargingFrequency);
  }

  /**
   * 车辆使用率/里程/时长/能耗分析
   */
  @Override
  public SSIResponse getUseInfoByVehicle() {
    List<UseInfoByVehicleVo> useInfoByVehicle = vmsCarryCapacityAnalysisMapper.getUseInfoByVehicle();
    double useRateTotal = 0;
    for (UseInfoByVehicleVo vehicleVo : useInfoByVehicle) {
      useRateTotal += vehicleVo.getUseRate();
    }
    double useRateLeft = 0;
    for (UseInfoByVehicleVo vehicleVo : useInfoByVehicle) {
      vehicleVo.setUseRate(new BigDecimal(vehicleVo.getUseRate() / useRateTotal)
          .setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue());
      useRateLeft += vehicleVo.getUseRate();
    }
    if (useInfoByVehicle.size()>0){//判断对象不为空
      useInfoByVehicle.get(0).setUseRate(useInfoByVehicle.get(0).getUseRate() - (useRateLeft - 1));
    }
    return SSIResponse.ok(useInfoByVehicle);
  }
  @Override

  public SSIResponse getVehicleTaskOrder(VehicleTaskOrderReq req) {

    Date startDate;
    Date endDate;
    if(Objects.nonNull(req.getStartDate())&&Objects.nonNull(req.getEndDate())){
      startDate = req.getStartDate();
      endDate = req.getEndDate();
      if(startDate.compareTo(endDate)==0){
        startDate.setHours(0);
        endDate.setHours(23);
      }
    }else {
      startDate = new Date();
      startDate.setHours(1);
      endDate = new Date();
    }
    Date indexDate = startDate;
    List<AnalyseTaskOrderResp> result = new ArrayList<>();
    do{
      AnalyseTaskOrderResp analyseTaskOrderResp = new AnalyseTaskOrderResp();
      String dateStr = "";
      Integer flag = 0;
      if(startDate.getDay()!=endDate.getDay()){
        dateStr = DateUtils.format(indexDate, DateUtils.DATE_PATTERN);
        analyseTaskOrderResp.setDate(DateUtils.format(indexDate, "MM.dd"));
      }else {
        flag = 1;
        dateStr = DateUtils.format(indexDate, "yyyyMMddHH");
        analyseTaskOrderResp.setHour(indexDate.getHours());
      }
      analyseTaskOrderResp.setDateStr(dateStr);
      result.add(analyseTaskOrderResp);
      //获取任务总数
      analyseTaskOrderResp.setTotalTask(calculateTotalList(req.getTotalType(),dateStr,req.getVins(),flag));
      //装船任务数据
      analyseTaskOrderResp.setLoadTask(calculateLoadList(req.getLoadType(),dateStr,req.getVins(),flag));
      //卸船任务数据
      analyseTaskOrderResp.setUnloadTask(calculateUnloadList(req.getUnloadType(),dateStr,req.getVins(),flag));
      //搬移任务数据
      analyseTaskOrderResp.setMoveTask(calculateMoveList(req.getMoveType(),dateStr,req.getVins(),flag));
      //指令数据
      analyseTaskOrderResp.setTotalOrder(calculateOrderList(req.getOrderType(),dateStr,req.getVins(),flag));
      //修改时间戳
      if(startDate.getDay()==endDate.getDay()){
        indexDate = org.apache.commons.lang3.time.DateUtils.addHours(indexDate, 1);
      }else {
        indexDate = org.apache.commons.lang3.time.DateUtils.addDays(indexDate, 1);
      }

    }while (indexDate.compareTo(endDate)<=0);
    return SSIResponse.ok(result);
  }

  private Integer calculateOrderList(Integer orderType, String indexDate, List<String> vin,Integer flag) {
    Integer result = 0;
    orderType = Objects.isNull(orderType) ? 0 : orderType;
    switch (orderType){
      case 1:
        result = vmsTosOrdersMapper.queryOrdersByDate(vin,indexDate,1,flag);
        break;
      case 2:
        result = vmsTosOrdersMapper.queryOrdersByDate(vin,indexDate,2,flag);
        break;
      case 3:
        result = vmsTosOrdersMapper.queryOrdersByDate(vin,indexDate,3,flag);
        break;
      case 4:
        result = vmsTosOrdersMapper.queryOrdersByDate(vin,indexDate,4,flag);
        break;
      default:
        result = vmsTosOrdersMapper.queryOrdersByDate(vin,indexDate,1,flag);
        break;
    }
    return result;
  }

  private Integer calculateMoveList(Integer moveType, String indexDate, List<String> vin, Integer flag) {
    Integer result = 0;
    moveType = Objects.isNull(moveType) ? 0 : moveType;
    switch (moveType){
      case 1:
        result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,4,flag);
        break;
      case 2:
        result = vmsTosOrdersMapper.queryTosOrderByDate(vin,indexDate,4,flag);
        break;
      case 3:
        result = vmsTosOrdersMapper.staticsAppCaveOrdersByDate(vin, indexDate, 4,flag);
        break;
      case 4:
        result = vmsTosOrdersMapper.staticsContainerNumsByDate(vin, indexDate,4,flag);
        break;
      default:
        result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,4,flag);
        break;
    }
    return result;
  }

  private Integer calculateUnloadList(Integer unloadType, String indexDate, List<String> vin, Integer flag) {
    Integer result = 0;
    unloadType = Objects.isNull(unloadType) ? 0 : unloadType;
    switch (unloadType){
      case 1:
        result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,3,flag);
        break;
      case 2:
        result = vmsTosOrdersMapper.queryTosOrderByDate(vin,indexDate,3,flag);
        break;
      case 3:
        result = vmsTosOrdersMapper.staticsAppCaveOrdersByDate(vin, indexDate, 3,flag);
        break;
      case 4:
        result = vmsTosOrdersMapper.staticsContainerNumsByDate(vin, indexDate,3,flag);
        break;
      default:
        result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,3,flag);
        break;
    }
    return result;
  }

  private Integer calculateLoadList(Integer loadType,String indexDate, List<String> vin, Integer flag) {
    Integer result = 0;
    loadType = Objects.isNull(loadType) ? 0 : loadType;
    switch (loadType){
      case 1:
        result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,2,flag);
        break;
      case 2:
        result = vmsTosOrdersMapper.queryTosOrderByDate(vin,indexDate,2,flag);
        break;
      case 3:
        result = vmsTosOrdersMapper.staticsAppCaveOrdersByDate(vin, indexDate, 2,flag);
        break;
      case 4:
        result = vmsTosOrdersMapper.staticsContainerNumsByDate(vin, indexDate,2,flag);
        break;
      default:
        result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,2,flag);
        break;
    }
    return result;
  }

  private Integer calculateTotalList(Integer totalType, String indexDate, List<String> vin, Integer flag) {
    Integer result = 0;
    totalType = Objects.isNull(totalType) ? 0 : totalType;
    switch (totalType){
        case 1:
          result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,1,flag);
          break;
        case 2:
          result = vmsTosOrdersMapper.queryTosOrderByDate(vin,indexDate,1,flag);
          break;
        case 3:
          result = vmsTosOrdersMapper.staticsAppCaveOrdersByDate(vin, indexDate, 1,flag);
          break;
        case 4:
          result = vmsTosOrdersMapper.staticsContainerNumsByDate(vin, indexDate,1,flag);
          break;
        default:
          result = vmsTosOrdersMapper.queryOrderNumByDate(vin,indexDate,1,flag);
          break;
      }
    return result;
  }

}
