package com.ssi.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.entity.VmsVehicleEmergencyStop;
import com.ssi.entity.vo.VmsVehicleEmergencyVo;
import com.ssi.mapper.VehicleEmergencyMapper;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.AccessResponse;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VehicleEmergencyService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 故障历史服务实现类
 */
@Service("vehicleEmergencyService")
@Slf4j
public class VehicleEmergencyServiceImpl extends ServiceImpl<VehicleEmergencyMapper, VmsVehicleEmergencyStop> implements VehicleEmergencyService {

    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private VmsVehicleMapper vmsVehicleMapper;
    @Autowired
    private RedisDataModel redisDataModel;
    @Value("${harbor.remote.emergencyParking}")
    private String emergencyParkingKeyPrefix;
    @Value("${command-url}")
    private String commandUrl;
    /**
     * 故障列表分页
     */
    @Override
    public SSIPage queryPage(Map<String, Object> params) {

        Integer pageSize = params.get("pageSize") == null ? 0 : (Integer) params.get("pageSize");
        Integer pageIndex = params.get("pageIndex") == null ? 10 : (Integer) params.get("pageIndex");
        Long startTime = (Long) params.get("startTime");
        Long stopTime = (Long) params.get("stopTime");
        Date startDate = startTime == null ? null : new Date(startTime);
        Date stopDate = stopTime == null ? null : new Date(stopTime);
        params.put("startTime",startDate);
        params.put("stopTime",stopDate);

        IPage<VmsVehicleEmergencyVo> page = new Page<>(pageIndex, pageSize);
        page = this.getBaseMapper().getEmergencyRecordList(page,params);
        return new SSIPage(page);
    }

    /**
     * 紧急停车
     * @param vin 车辆vin
     * @param emergencyType 紧停类型
     */
    @Override
    public SSIResponse emergency(String vin, Integer emergencyType) {
        // preCheck
        log.info(String.format("调用紧停接口，vin：%s，emergencyType：%s", vin, emergencyType));
        Map<String, Object> emergencyMap = redisDataModel.getJson2Map(emergencyParkingKeyPrefix + ":" + vin);
        if (emergencyMap != null) {
            Integer emergencyStatus = (Integer) emergencyMap.get("emergencyStatus");
            if (emergencyStatus == 1 && emergencyType == 1) {
                return SSIResponse.no("车辆已处于非紧停状态中");
            }
            if (emergencyStatus == 2 && emergencyType == 2) {
                return SSIResponse.no("车辆已处于紧停状态中");
            }
        }
        // execute
        String url = commandUrl + "/remote/app/emergency";
        Map param = new HashMap();
        param.put("vin", vin);
        //遥控类型 1-紧急停车;2-取消紧停
        param.put("emergencyType", emergencyType);
        AccessResponse accessResponse = restTemplate.postForObject(url, param, AccessResponse.class);
        if (accessResponse.isSuccess()) {
            log.info(String.format("调用紧停接口成功，vin：%s，emergencyType：%s", vin, emergencyType));
            return SSIResponse.ok();
        } else {
            String msg = accessResponse.getStatus().getDetails();
            log.info(String.format("调用紧停接口失败，vin：%s，emergencyType：%s，msg：%s", vin, emergencyType, msg));
            return SSIResponse.no(msg);
        }
    }
}
