package com.ssi.service.impl;

import cn.afterturn.easypoi.handler.impl.ExcelDataHandlerDefaultImpl;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.VmsVehicleAlertHistory;
import com.ssi.entity.vo.TaskAnalysisVo;
import com.ssi.entity.vo.VmsVehicleAlertHistoryVo;
import com.ssi.entity.vo.VmsVehicleFaultAnalysisVo;
import com.ssi.entity.vo.VmsVehicleFaultAnalysisVo.AnalysisInfoVo;
import com.ssi.entity.vo.VmsVehicleFaultVo;
import com.ssi.mapper.VehicleTroubleHistoryMapper;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VehicleTroubleService;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.DateUtils;
import com.ssi.utils.PoiUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 故障历史服务实现类
 */
@Service("vehicleTroubleService")
public class VehicleTroubleServiceImpl extends ServiceImpl<VehicleTroubleHistoryMapper, VmsVehicleAlertHistory> implements VehicleTroubleService {

    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private VehicleTroubleHistoryMapper vehicleTroubleHistoryMapper;
    private static String[] faultTypes = {"V","I","P","OBU"};
    private static String[] faultLevels = {"1","2","3","4"};

    /**
     * 故障列表分页
     */
    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        List<VmsVehicle> list = vmsVehicleService.list();
        Map<String, VmsVehicle> vinMap = list.stream().collect(Collectors.toMap(VmsVehicle::getVin, Function.identity()));
        String vehicleNum = (String) params.get("vehicleNum");
        Integer pageSize = params.get("pageSize") == null ? 0 : (Integer) params.get("pageSize");
        Integer pageIndex = params.get("pageIndex") == null ? 10 : (Integer) params.get("pageIndex");
        Long startTime = (Long) params.get("startTime");
        Long stopTime = (Long) params.get("stopTime");
        Date startDate = startTime == null ? null : new Date(startTime);
        Date stopDate = stopTime == null ? null : new Date(stopTime);
        params.put("startTime",startDate);
        params.put("stopTime",stopDate);
        if (StringUtils.isNotBlank(vehicleNum)) {
            List<String> vins = list.parallelStream().filter(vehicle -> vehicle.getVehicleNum().contains(vehicleNum)).map(VmsVehicle::getVin).collect(Collectors.toList());
            if (vins.isEmpty()) {
                return new SSIPage(Lists.newArrayList(), 0, pageSize, pageIndex);
            }
            params.put("vins",vins);
        }
        IPage<VmsVehicleAlertHistoryVo> page = new Page<>(pageIndex, pageSize);
        page = this.getBaseMapper().getAlertHistoryList(page,params);
//        page.getRecords().parallelStream().forEach(vmsVehicleAlertHistory -> {
//            VmsVehicle vehicle = vinMap.get(vmsVehicleAlertHistory.getVin());
//            vmsVehicleAlertHistory.setVehicleNum(vehicle == null ? null : vehicle.getVehicleNum());
//        });
        return new SSIPage(page);
    }
	@Override
	public void export(Map<String, Object> params, HttpServletResponse response) {
		// TODO Auto-generated method stub
	    Integer exportType = (Integer) params.get("exportType");
        List<VmsVehicleAlertHistoryVo> list;
        ExcelDataHandlerDefaultImpl<Object> excelDataHandler = new ExcelDataHandlerDefaultImpl<Object>() {
            @Override
			public String[] getNeedHandlerFields() {
                return new String[]{"报警时间","解除时间"};
            }

//            @Override
            public Object exportHandler(Object obj, String name, Object value) {
            	if("报警时间".equals(name)) {
            		VmsVehicleAlertHistoryVo vo =(VmsVehicleAlertHistoryVo) obj;
            		 return DateUtils.format(vo.getAlarmTime(), 2);
            	}
            	if("解除时间".equals(name)) {
            		VmsVehicleAlertHistoryVo vo =(VmsVehicleAlertHistoryVo) obj;
            		return DateUtils.format(vo.getRelieveAlarmTime(), 2);
            	}
                return super.exportHandler(obj, name, value);
            }
        };

        if (exportType == 0) {
            SSIPage ssiPage = queryPage(params);
            list = (List<VmsVehicleAlertHistoryVo>) ssiPage.getList();
            PoiUtils.exportExcel(excelDataHandler, list, "车辆异常报警", "VmsVehicle", VmsVehicleAlertHistoryVo.class, "车辆异常报警" + ".xls", true, response);
        } else {
        	   params.put("pageIndex", 1);
        	   params.put("pageSize", 1000000);
        	   SSIPage ssiPage = queryPage(params);
               list = (List<VmsVehicleAlertHistoryVo>) ssiPage.getList();
               PoiUtils.exportExcel(excelDataHandler, list, "车辆异常报警", "VmsVehicle", VmsVehicleAlertHistoryVo.class, "车辆异常报警" + ".xls", true, response);
        }
	}
    @Override
    public SSIResponse getFaultTrend(Map<String, Object> params) throws Exception {
        Integer timeType = (Integer) params.get("timeType");
        switch (timeType) {
            //日
            case 1:
                LocalDateTime startTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                LocalDateTime endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                Long startTimeStamp = startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli();
                List<VmsVehicleFaultVo> taskTrendByHour = vehicleTroubleHistoryMapper.getFaultTrendByHour(startTime, endTime,startTimeStamp,params);
                return SSIResponse.ok(TaskAnalysisVo.fillBlankHour(taskTrendByHour,VmsVehicleFaultVo.class));
            //周
            case 2:
                startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                List<VmsVehicleFaultVo> taskTrendByDay = vehicleTroubleHistoryMapper.getFaultTrendByDay(startTime, endTime,params);
                return SSIResponse.ok(TaskAnalysisVo.fillBlankDay(taskTrendByDay, VmsVehicleFaultVo.class, startTime, endTime));
            //月
            case 3:
                startTime = LocalDateTime.of(LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                taskTrendByDay = vehicleTroubleHistoryMapper.getFaultTrendByDay(startTime, endTime,params);
                return SSIResponse.ok(TaskAnalysisVo.fillBlankDay(taskTrendByDay, VmsVehicleFaultVo.class, startTime, endTime));
        }
        return null;
    }

    @Override
    public SSIResponse getFaultDistribution(Map<String, Object> params) throws Exception {
        Integer timeType = (Integer) params.get("timeType");
        LocalDate startTime = null;
        LocalDateTime endTime = null;
        switch (timeType) {
            //日
            case 1:
                startTime = LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate();
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                break;
            //周
            case 2:
                startTime = LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate();
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                break;
            //月
            case 3:
                startTime = LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate();
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                break;
            default:
                return null;
        }
        List<VmsVehicleFaultVo> distribution = vehicleTroubleHistoryMapper.getFaultDistribution(startTime, endTime,params);
        return SSIResponse.ok(distribution);
    }

    @Override
    public SSIResponse getFaultTypeTrend(Map<String, Object> params) throws Exception {
        Integer timeType = (Integer) params.get("timeType");
        switch (timeType) {
            //日
            case 1:
                LocalDateTime startTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                LocalDateTime endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                Long startTimeStamp = startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli();
                List<VmsVehicleFaultVo> faultTrendByHour = vehicleTroubleHistoryMapper.getFaultTypeTrendByHour(startTime, endTime,startTimeStamp,params);
                return SSIResponse.ok(this.fillFaultTypeList(faultTrendByHour));
            //周
            case 2:
                startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                List<VmsVehicleFaultVo> faultTrendByDay = vehicleTroubleHistoryMapper.getFaultTypeTrendByDay(startTime, endTime,params);
                return SSIResponse.ok(this.fillFaultTypeList(faultTrendByDay, startTime, endTime));
            //月
            case 3:
                startTime = LocalDateTime.of(LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                faultTrendByDay = vehicleTroubleHistoryMapper.getFaultTypeTrendByDay(startTime, endTime,params);
                return SSIResponse.ok(this.fillFaultTypeList(faultTrendByDay, startTime, endTime));
        }
        return null;
    }

    public static List<VmsVehicleFaultAnalysisVo> fillFaultTypeList(List<VmsVehicleFaultVo> list) throws Exception{
        List<VmsVehicleFaultAnalysisVo> resList = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            VmsVehicleFaultAnalysisVo faultAnalysisVo = new VmsVehicleFaultAnalysisVo();
            faultAnalysisVo.setHour(i);
            Map<String,Integer> hourMap = new HashMap<>();
            for (int j = 0; j < list.size(); j++) {
                VmsVehicleFaultVo vo1 = list.get(j);
                if (vo1.getHour() == i) {
                    hourMap.put(vo1.getFaultType(),vo1.getFaultNum());
                }
                if (vo1.getHour() > i){
                    break;
                }
            }
            List<AnalysisInfoVo> faultList = new ArrayList<>();
            for(String faultType : faultTypes){
                AnalysisInfoVo analysisInfoVo = new AnalysisInfoVo();
                analysisInfoVo.setFaultType(faultType);
                if (hourMap.get(faultType) != null) {
                    analysisInfoVo.setFaultNum(hourMap.get(faultType));
                }
                faultList.add(analysisInfoVo);
            }
            faultAnalysisVo.setList(faultList);
            resList.add(faultAnalysisVo);
        }
        return resList;
    }

    public static List<VmsVehicleFaultAnalysisVo> fillFaultTypeList(List<VmsVehicleFaultVo> list, LocalDateTime startTime, LocalDateTime endTime) throws Exception{
        List<VmsVehicleFaultAnalysisVo> resList = new ArrayList<>();
        LocalDate date = startTime.toLocalDate();
        do {
            VmsVehicleFaultAnalysisVo faultAnalysisVo = new VmsVehicleFaultAnalysisVo();
            faultAnalysisVo.setDate(date);
            Map<String,Integer> dayMap = new HashMap<>();
            for (int i = 0; i < list.size(); i++) {
                VmsVehicleFaultVo vo1 = list.get(i);
                if (date.compareTo(vo1.getDate()) == 0) {
                    dayMap.put(vo1.getFaultType(),vo1.getFaultNum());
                }
                if (date.compareTo(vo1.getDate()) < 0) {
                    break;
                }
            }
            List<AnalysisInfoVo> faultList = new ArrayList<>();
            for(String faultType : faultTypes) {
                AnalysisInfoVo analysisInfoVo = new AnalysisInfoVo();
                analysisInfoVo.setFaultType(faultType);
                if (dayMap.get(faultType) != null) {
                    analysisInfoVo.setFaultNum(dayMap.get(faultType));
                }
                faultList.add(analysisInfoVo);
            }
            faultAnalysisVo.setList(faultList);
            resList.add(faultAnalysisVo);
            date = date.plusDays(1);
        } while (date.compareTo(endTime.toLocalDate()) <= 0);
        return resList;
    }

    @Override
    public SSIResponse getFaultLevelTrend(Map<String, Object> params) throws Exception {
        Integer timeType = (Integer) params.get("timeType");
        switch (timeType) {
            //日
            case 1:
                LocalDateTime startTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                LocalDateTime endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                Long startTimeStamp = startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli();
                List<VmsVehicleFaultVo> faultTrendByHour = vehicleTroubleHistoryMapper.getFaultLevelTrendByHour(startTime, endTime,startTimeStamp,params);
                return SSIResponse.ok(this.fillFaultLevelList(faultTrendByHour));
            //周
            case 2:
                startTime = LocalDateTime.of(LocalDateTime.now().minus(7, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                List<VmsVehicleFaultVo> faultTrendByDay = vehicleTroubleHistoryMapper.getFaultLevelTrendByDay(startTime, endTime,params);
                return SSIResponse.ok(this.fillFaultLevelList(faultTrendByDay, startTime, endTime));
            //月
            case 3:
                startTime = LocalDateTime.of(LocalDateTime.now().minus(30, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
                endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
                faultTrendByDay = vehicleTroubleHistoryMapper.getFaultLevelTrendByDay(startTime, endTime,params);
                return SSIResponse.ok(this.fillFaultLevelList(faultTrendByDay, startTime, endTime));
        }
        return null;
    }

    public static List<VmsVehicleFaultAnalysisVo> fillFaultLevelList(List<VmsVehicleFaultVo> list) throws Exception{
        List<VmsVehicleFaultAnalysisVo> resList = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            VmsVehicleFaultAnalysisVo faultAnalysisVo = new VmsVehicleFaultAnalysisVo();
            faultAnalysisVo.setHour(i);
            Map<String,Integer> hourMap = new HashMap<>();
            for (int j = 0; j < list.size(); j++) {
                VmsVehicleFaultVo vo1 = list.get(j);
                if (vo1.getHour() == i) {
                    hourMap.put(vo1.getFaultGrade(),vo1.getFaultNum());
                }
                if (vo1.getHour() > i){
                    break;
                }
            }
            List<AnalysisInfoVo> faultList = new ArrayList<>();
            for(String faultLevel : faultLevels){
                AnalysisInfoVo analysisInfoVo = new AnalysisInfoVo();
                analysisInfoVo.setFaultGrade(faultLevel);
                if (hourMap.get(faultLevel) != null) {
                    analysisInfoVo.setFaultNum(hourMap.get(faultLevel));
                }
                faultList.add(analysisInfoVo);
            }
            faultAnalysisVo.setList(faultList);
            resList.add(faultAnalysisVo);
        }
        return resList;
    }

    public static List<VmsVehicleFaultAnalysisVo> fillFaultLevelList(List<VmsVehicleFaultVo> list, LocalDateTime startTime, LocalDateTime endTime) throws Exception{
        List<VmsVehicleFaultAnalysisVo> resList = new ArrayList<>();
        LocalDate date = startTime.toLocalDate();
        do {
            VmsVehicleFaultAnalysisVo faultAnalysisVo = new VmsVehicleFaultAnalysisVo();
            faultAnalysisVo.setDate(date);
            Map<String,Integer> dayMap = new HashMap<>();
            for (int i = 0; i < list.size(); i++) {
                VmsVehicleFaultVo vo1 = list.get(i);
                if (date.compareTo(vo1.getDate()) == 0) {
                    dayMap.put(vo1.getFaultGrade(),vo1.getFaultNum());
                }
                if (date.compareTo(vo1.getDate()) < 0) {
                    break;
                }
            }
            List<AnalysisInfoVo> faultList = new ArrayList<>();
            for(String faultLevel : faultLevels) {
                AnalysisInfoVo analysisInfoVo = new AnalysisInfoVo();
                analysisInfoVo.setFaultGrade(faultLevel);
                if (dayMap.get(faultLevel) != null) {
                    analysisInfoVo.setFaultNum(dayMap.get(faultLevel));
                }
                faultList.add(analysisInfoVo);
            }
            faultAnalysisVo.setList(faultList);
            resList.add(faultAnalysisVo);
            date = date.plusDays(1);
        } while (date.compareTo(endTime.toLocalDate()) <= 0);
        return resList;
    }


}
