package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.constant.URL;
import com.ssi.entity.VmsAreaBarrier;
import com.ssi.entity.map.MapDataDto;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsAreaBarrierService;
import com.ssi.mapper.VmsAreaBarrierMapper;
import com.ssi.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
*
*/
@Slf4j
@Service
public class VmsAreaBarrierServiceImpl extends ServiceImpl<VmsAreaBarrierMapper, VmsAreaBarrier>
implements VmsAreaBarrierService{

    @Value("${map-edit-url}")
    private String mapEditUrl;

    @Override
    public SSIResponse saveMapData(MapDataDto mapdata) {
//        List<VmsAreaBarrier> list = this.lambdaQuery().eq(VmsAreaBarrier::getName, mapdata.getNumber()).list();
//        if(CollectionUtils.isNotEmpty(list)){
//            return SSIResponse.no("已存在同名数据");
//        }
//        VmsAreaBarrier saveEntity = getVmsAreaBarrier(mapdata);
        log.info(String.format("绘制电子围栏区域保存请求发送:----%s", mapdata));
        String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.FENCE_ADD_URL),JSON.toJSONString(mapdata) , null);
        log.info(String.format("绘制电子围栏区域保存返回结果:----%s", result));
        JSONObject jsonObject = JSONObject.parseObject(result);
//        this.saveOrUpdate(saveEntity);
        return SSIResponse.ok(jsonObject);
    }

    private VmsAreaBarrier getVmsAreaBarrier(MapDataDto mapdata) {
        VmsAreaBarrier saveEntity = new VmsAreaBarrier();
        if(StringUtils.isNotBlank(mapdata.getId())){
            saveEntity.setId(Integer.valueOf(mapdata.getId()));
        }
        saveEntity.setEndPoint(mapdata.getField());
        saveEntity.setName(mapdata.getNumber());
        saveEntity.setAreaType(Integer.valueOf(mapdata.getPlacetype()));
        saveEntity.setType(Integer.valueOf(mapdata.getType()));
        saveEntity.setDr(Integer.valueOf(mapdata.getState()));
        String[] points = mapdata.getPoints().split(";");
        List<Map> maps = new ArrayList<>();
        for (String point : points) {
            Map<String, String> map = new HashMap<>();
            String[] loglat = point.split(",");
            map.put("lng",loglat[0]);
            map.put("lat", loglat[1]);
            maps.add(map);
        }
        saveEntity.setLocations(JSON.toJSONString(maps));
        return saveEntity;
    }

    @Override
    public SSIResponse deleteMapData(MapDataDto mapdata) {
//        this.removeById(mapdata.getId());
        log.info(String.format("删除电子围栏区域保存请求发送:----%s", mapdata));
        String result = RestTemplateUtil.get(String.format("%s%s?id=%s", mapEditUrl, URL.FENCE_DELETE_URL,mapdata.getId()), null);
        log.info(String.format("删除电子围栏区域保存返回结果:----%s", result));
        JSONObject jsonObject = JSONObject.parseObject(result);
        return SSIResponse.ok(jsonObject);
    }

    @Override
    public SSIResponse addBarrierMap(MapDataDto mapdata, JSONObject jsonObject) {
        if(StringUtils.isNotBlank(mapdata.getType())&&!"11".equalsIgnoreCase(mapdata.getType())){
            return SSIResponse.ok();
        }
        try {
            VmsAreaBarrier qryEntity = new VmsAreaBarrier();
            if (StringUtils.isNotBlank(mapdata.getNumber())) {
                qryEntity.setName(mapdata.getNumber());// 名称
            }
            if (StringUtils.isNotBlank((mapdata.getPlacetype()))) {
                qryEntity.setAreaType(Integer.valueOf(mapdata.getPlacetype()));// 状态。0可用；1不可用（初始状态）;2已计划;3有车
            }
            log.info(String.format("电子围栏区域查询请求发送:----%s", mapdata));
            String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.FENCE_QUERY_URL), JSON.toJSONString(mapdata), null);
//            log.info(String.format("子围栏区域查询返回结果:----%s", result));
//        List<VmsAreaBarrier> vmsAreaBarriers = this.baseMapper.selectList(new QueryWrapper<>(qryEntity));
            JSONArray dataList = jsonObject.getJSONArray("data");
            JSONObject jsonResult = JSONArray.parseObject(result, JSONObject.class);
            JSONArray data = jsonResult.getJSONArray("data");
            if(Objects.isNull(data)){
                return SSIResponse.ok();
            }
            for (int i = 0; i < data.size();i++){
                JSONObject tmpJsonObj = data.getJSONObject(i);
                JSONObject tmpObj = new JSONObject();
                dataList.add(tmpObj);
                tmpObj.put("id", tmpJsonObj.getIntValue("id"));
                tmpObj.put("name", tmpJsonObj.getString("name"));
                tmpObj.put("state", 1);
                tmpObj.put("type", 11);
                tmpObj.put("placeType", tmpJsonObj.getString("fenceType"));
                String points = tmpJsonObj.getString("points");
                if (StringUtils.isNotBlank(points)) {
                    String[] locations = points.split(";");
                    tmpObj.put("locateNum", locations.length);
                    for (int j = 0; j < locations.length; j++) {
                        String[] tmpLocation = locations[j].split(",");
                        tmpObj.put("lng" + (j + 1), tmpLocation[0]);
                        tmpObj.put("lat" + (j + 1), tmpLocation[1]);
                    }
                }
            }
            return SSIResponse.ok();
        }catch (Exception e){
            log.error("addBarrierMap occur error :",e);
            return SSIResponse.ok();
        }
    }

    @Override
    public SSIResponse updateMapData(MapDataDto mapdata) {
        log.info(String.format("更新电子围栏区域保存请求发送:----%s", mapdata));
        String result = RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.FENCE_UPDATE_URL),JSON.toJSONString(mapdata) , null);
        log.info(String.format("更新电子围栏区域保存返回结果:----%s", result));
        JSONObject jsonObject = JSONObject.parseObject(result);
//        this.saveOrUpdate(saveEntity);
        return SSIResponse.ok(jsonObject);
    }
}
