package com.ssi.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ssi.entity.VmsAreaPosition;
import com.ssi.entity.VmsChargingConnectorInfo;
import com.ssi.entity.VmsChargingEquipmentInfo;
import com.ssi.entity.VmsChargingPile;
import com.ssi.response.SSIPage;
import com.ssi.service.VmsChargingConnectorInfoService;
import com.ssi.mapper.VmsChargingConnectorInfoMapper;
import com.ssi.utils.QueryUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.Asserts;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
*
*/
@Service
public class VmsChargingConnectorInfoServiceImpl extends ServiceImpl<VmsChargingConnectorInfoMapper, VmsChargingConnectorInfo>
implements VmsChargingConnectorInfoService{

    @Override
    public Runnable saveConnectorInfo(JSONObject data) {
        return ()->{
            try {
                JSONArray stationInfos = data.getJSONArray("StationInfos");
                for (int i = 0; i < stationInfos.size(); i++) {
                    JSONObject stationInfo = stationInfos.getJSONObject(i);
                    JSONArray equipmentInfos = stationInfo.getJSONArray("EquipmentInfos");
                    for (int j = 0; j < equipmentInfos.size(); j++) {
                        JSONObject equipmentInfo = equipmentInfos.getJSONObject(j);
                        JSONArray connectorInfos = equipmentInfo.getJSONArray("ConnectorInfos");
                        for (int k = 0; k < connectorInfos.size(); k++) {
                            VmsChargingConnectorInfo chargingConnectorInfo = connectorInfos.getObject(i, VmsChargingConnectorInfo.class);
                            chargingConnectorInfo.setEquipmentId(equipmentInfo.getString("EquipmentID"));
                            chargingConnectorInfo.setStationId(stationInfo.getString("StationID"));
                            chargingConnectorInfo.setUpdateTime(new Date());
                            this.saveOrUpdate(chargingConnectorInfo);
                        }
                    }
                }
            }catch (Exception e){
                log.error("saveConnectorInfo occur error:",e);
            }
        };
    }

    @Override
    public void updateStatus(JSONObject jsonObject) {
        Asserts.notNull(jsonObject,"充电站");
        String stationID = jsonObject.getString("StationID");
        JSONArray connectorStatusInfos = jsonObject.getJSONArray("ConnectorStatusInfos");
        for (int i = 0; i < connectorStatusInfos.size(); i++) {
            VmsChargingConnectorInfo updateEntity = connectorStatusInfos.getObject(i, VmsChargingConnectorInfo.class);
            updateEntity.setStationId(stationID);
            this.updateById(updateEntity);
        }
    }

    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        Integer connectorType = (Integer) params.get("connectorType");
        Integer workStatus = (Integer) params.get("status");
        String pileNum = (String) params.get("equipmentId");
        String gunNum = (String) params.get("connectorId");
        IPage<VmsChargingConnectorInfo> page = this.page(
                new QueryUtils<VmsChargingConnectorInfo>().getPage(params),
                new LambdaQueryWrapper<VmsChargingConnectorInfo>()
                        .eq(workStatus != null, VmsChargingConnectorInfo::getStatus, workStatus)
                        .eq(connectorType != null, VmsChargingConnectorInfo::getConnectorType, connectorType)
                        .like(StringUtils.isNotBlank(gunNum), VmsChargingConnectorInfo::getConnectorId, gunNum)
                        .like(StringUtils.isNotBlank(pileNum), VmsChargingConnectorInfo::getEquipmentId, pileNum)
        );
        return new SSIPage(page);
    }
}
