package com.ssi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ssi.entity.VmsAreaPosition;
import com.ssi.mapper.VmsChargingPileMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.service.VmsAreaPositionService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ssi.utils.QueryUtils;

import com.ssi.entity.VmsChargingPile;
import com.ssi.service.VmsChargingPileService;

/**
 * 充电桩服务实现类
 */
@Service("chargingPileService")
public class VmsChargingPileServiceImpl extends ServiceImpl<VmsChargingPileMapper, VmsChargingPile> implements VmsChargingPileService {

    @Value("${charging-pile.latest_key_prefix:harbor:charging:pile:info}")
    private String latestKeyPrefix;

    @Value("${charging-pile.charging_history_prefix:harbor:charging:history}")
    private String chargingHistoryPrefix;

    @Value("${vehicle.latestData.redis.prefix:ivccs:vms:vehicle:latest}")
    private String latestRedisKeyPrefix;

    @Value("${vehicle.latestData.redis.postfix:harbor_D00A}")
    private String latestRedisKeyPostfix;

    @Value("${vehicle.cache.key:ivccs:vms:vehicle:cache}")
    private String vehicleCacheKey;

    @Autowired
    private RedisDataModel redisDataModel;
    @Autowired
    private VmsAreaPositionService vmsAreaPositionService;

    /**
     * 充电桩分页数据
     * @param params
     * @return
     */
    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        Integer onlineStatus = (Integer) params.get("onlineStatus");
        Integer workStatus = (Integer) params.get("workStatus");
        Integer faultStatus = (Integer) params.get("faultStatus");
        String pileNum = (String) params.get("pileNum");
        String gunNum = (String) params.get("gunNum");
        IPage<VmsChargingPile> page = this.page(
                new QueryUtils<VmsChargingPile>().getPage(params),
                new LambdaQueryWrapper<VmsChargingPile>().eq(onlineStatus != null, VmsChargingPile::getOnlineStatus, onlineStatus)
                        .eq(workStatus != null, VmsChargingPile::getWorkStatus, workStatus)
                        .eq(faultStatus != null, VmsChargingPile::getFaultStatus, faultStatus)
                        .like(StringUtils.isNotBlank(gunNum), VmsChargingPile::getGunNum, gunNum)
                        .like(StringUtils.isNotBlank(pileNum), VmsChargingPile::getPileNum, pileNum)
        );

        //获取充电桩实时充电指令
        Map<String, String> chargingHistoryInfo = getChargingHistoryInfo();

        //获取工作状态
        page.getRecords().stream().forEach(chargingPile -> {
            if(StringUtils.isNotBlank(chargingPile.getGunNum())){
                List<Integer> allStatus = Lists.newArrayList();
                String gunNumStr = chargingPile.getGunNum();
                String[] split = gunNumStr.split(",");
                List<String> gunNums = Arrays.asList(split);
                gunNums.stream().forEach(num -> {
                    String result = redisDataModel.get(String.format("%s:%s", latestKeyPrefix, num));
                    JSONObject jsonObject = JSONObject.parseObject(result);
                    if(jsonObject == null){
                        return;
                    }
                    Integer status = jsonObject.getInteger("status");
                    allStatus.add(status);
                });

                if(allStatus.contains(255) || allStatus.contains(0)){
                    //异常状态
                    chargingPile.setWorkStatus(2);
                }else if(allStatus.contains(2) || allStatus.contains(3) || allStatus.contains(4)){
                    //充电中
                    chargingPile.setWorkStatus(1);
                }else{
                    //空闲
                    chargingPile.setWorkStatus(0);
                }
                //查询充电车辆
                if(chargingPile.getWorkStatus() == 1){
                    //根据充电桩和停车区充电枪是否有对应查询对应关系
                    List<VmsAreaPosition> areaPositionList = vmsAreaPositionService.list();
                    String areaNum = null;
                    for(VmsAreaPosition areaPosition : areaPositionList){
                        boolean isCorrespondArea = false;
                        String chargingInterfaceIds = areaPosition.getChargingInterfaceIds();
                        for(String gunNum1 : gunNums){
                            if(StringUtils.isNotBlank(chargingInterfaceIds)
                                    && chargingInterfaceIds.indexOf(gunNum1) > -1){
                                isCorrespondArea = true;
                                break;
                            }
                        }
                        if(isCorrespondArea){
                            areaNum = areaPosition.getAreaNum();
                            break;
                        }
                    }
                    if(StringUtils.isNotBlank(areaNum)){
                        String vin = chargingHistoryInfo.get(areaNum);
                        if(StringUtils.isNotBlank(vin)){
                            Map<String, Object> vehicleInfoRecord = redisDataModel.hgetJson2Map(vehicleCacheKey, vin);
                            Map<String, Object> record = redisDataModel.getJson2Map(String.format("%s:%s-%s", latestRedisKeyPrefix, vin, latestRedisKeyPostfix));
                            chargingPile.setChargingVehicleNum((String) vehicleInfoRecord.get("vehicleNum"));
                            chargingPile.setChargingPer((Integer) record.get("soc"));
                        }
                    }
                }
            }else{
                chargingPile.setWorkStatus(2);
            }
        });

        return new SSIPage(page);
    }

    private Map<String, String> getChargingHistoryInfo() {
        Map<String, String> res = new HashMap<>();

        String prefix = String.format("%s:*", chargingHistoryPrefix);
        Set<String> keysUnResolveRepeat = redisDataModel.keys(prefix);
        keysUnResolveRepeat.stream().forEach(key -> {
            Map<String, Object> record = redisDataModel.getJson2Map(key);
            res.put((String)record.get("areaNum"),(String)record.get("vin"));
        });

        return res;
    }
}
