package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ssi.constant.URL;
import com.ssi.entity.*;
import com.ssi.entity.vo.CargoShipConfigDto;
import com.ssi.entity.vo.CranePadBindVo;
import com.ssi.mapper.VmsCranePadBindMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.CargoShipConfigInfoService;
import com.ssi.service.VmsCranePadBindService;
import com.ssi.service.VmsOrdersVehicleAlterService;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.RestTemplateUtil;
import com.ssi.utils.VehicleUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 港机pad绑定服务实现类
 */
@Service("cranePadBindService")
public class VmsCranePadBindServiceImpl extends ServiceImpl<VmsCranePadBindMapper, VmsCranePadBind> implements VmsCranePadBindService {

    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private VmsOrdersVehicleAlterService vmsOrdersVehicleAlterService;
    @Autowired
    private CargoShipConfigInfoService cargoShipConfigInfoService;
    @Autowired
    private RedisDataModel redisDataModel;
    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;
    @Value("${map-edit-url}")
    private String mapEditUrl;
    /**
     * 吊具pad绑定列表
     */
    @Override
    public SSIPage queryPage(CranePadBindVo cranePadBindVo) {
        IPage<VmsCranePadBind> page = this.lambdaQuery()
                .eq(Objects.nonNull(cranePadBindVo.getCraneType()),VmsCranePadBind::getCraneType,cranePadBindVo.getCraneType())
                .like(cranePadBindVo.getCraneNo() != null, VmsCranePadBind::getCraneNo, cranePadBindVo.getCraneNo())
                .orderByDesc(VmsCranePadBind::getCreateTime)
                .page(new Page<>(cranePadBindVo.getPageIndex(), cranePadBindVo.getPageSize()));
        CargoShipConfigDto queryDto = new CargoShipConfigDto();
        queryDto.setIsActive((byte)1);
        JSONObject resultMap = cargoShipConfigInfoService.listByCondition(queryDto);
        JSONArray jsonArray = resultMap.getJSONArray("result");
        if(jsonArray.size()>0){
            List<CargoShipConfigDto> cargoShipConfigDtos = jsonArray.toJavaList(CargoShipConfigDto.class);
            List<String> collect = cargoShipConfigDtos.stream().map(CargoShipConfigDto::getBridgeNo).collect(Collectors.toList());
            page.getRecords().stream().forEach(vmsCranePadBind -> {
                if(collect.contains(vmsCranePadBind.getCraneNo())){
                    vmsCranePadBind.setInUsing(1);
                }
            });
        }
        return new SSIPage(page);
    }

    /**
     * 根据mac获取桥吊信息
     * @param padMac pad的MAC地址
     */
    @Override
    public SSIResponse getCraneByMac(String padMac) {
        List<VmsCranePadBind> list = this.list(new LambdaQueryWrapper<VmsCranePadBind>().eq(VmsCranePadBind::getPadMac, padMac));

        if (list != null && !list.isEmpty()) {

            List<VmsVehicle> vehicleList = vmsVehicleService.list();
            Map<String, VmsVehicle> vinMap = vehicleList.stream().collect(Collectors.toMap(VmsVehicle::getVin, Function.identity()));

            List<Map<String, Object>> ordersList = getCraneOrderList(vinMap, list.get(0));

            list.get(0).setOrdersList(ordersList);
            return SSIResponse.ok(list.get(0));
        }
        return SSIResponse.ok();
    }

    /**
     * 获取港机对应的任务
     * @param vinMap
     * @param vmsCranePadBind
     * @return
     */
    @Override
    public List<Map<String, Object>> getCraneOrderList(Map<String, VmsVehicle> vinMap, VmsCranePadBind vmsCranePadBind) {
        List<Map<String, Object>> ordersList = Lists.newArrayList();

        List<VmsTosOrders> orders = getUnconfOrdersByPortCode(vmsCranePadBind);

        orders.stream().forEach(vmsTosOrders -> {
            Map<String, Object> map = new HashMap<>();
            map.put("taskNo", vmsTosOrders.getTaskNo());
            map.put("taskType", vmsTosOrders.getTaskType());
            map.put("portCode", vmsTosOrders.getPortCode());
            map.put("containerId", vmsTosOrders.getContainerId());
            map.put("vin", vmsTosOrders.getVin());
            map.put("status", vmsTosOrders.getStatus());
            VmsVehicle vmsVehicle = vinMap.get(vmsTosOrders.getVin());
            if(vmsVehicle != null){
                map.put("vehicleNum", vmsVehicle.getVehicleNum());
                map.put("vehicleType",vmsVehicle.getVehicleType() == null ?0:vmsVehicle.getVehicleType());
            }else{
                map.put("vehicleNum", "未知");
                map.put("vehicleType",0);
            }

            ordersList.add(map);
        });
        return ordersList;
    }

    @Override
    @Transactional
    public void updateCraneBind(List<Integer> craneIds, String id, String number,String bridgeNo) {
        try {
            //更新原有为null;
            this.baseMapper.clearByAreaNo(number);
            //更新
            String[] bridgeNos = StringUtils.split(bridgeNo, ",");
            this.baseMapper.updateByIds(id,number,Arrays.asList(bridgeNos));

            Map param = new HashMap();
            param.put("bridgeNo", bridgeNo);
            param.put("name", number);
            String paramData = JSONObject.toJSONString(param);
            RestTemplateUtil.post(String.format("%s%s", mapEditUrl, URL.MAP_EDIT_LOCK_URL), paramData, null);
        }catch (Exception e){
            log.error("更新桥吊扭锁区绑定关系错误",e);
        }
    }

    @Override
    public void addCarineInfo(JSONObject jsonObject) {
        JSONArray dataList = jsonObject.getJSONArray("data");
        for (int i =0;i<dataList.size();i++){
            JSONObject tmpObject = dataList.getJSONObject(i);
            tmpObject.put("bindCarine",Collections.EMPTY_LIST);
            if(Objects.nonNull(tmpObject)&&tmpObject.getInteger("type")==6){
                List<VmsCranePadBind> list = this.lambdaQuery().eq(VmsCranePadBind::getLockAreaId, tmpObject.getString("id")).list();
                tmpObject.put("bindCarine",list);
            }
        }
    }

    private List<VmsTosOrders> getUnconfOrdersByPortCode(VmsCranePadBind vmsCranePadBind) {

        if(vmsCranePadBind == null || vmsCranePadBind.getCraneType() == null
                || StringUtils.isBlank(vmsCranePadBind.getCraneNo())){
            //港机信息不存在
            return new ArrayList<VmsTosOrders>();
        }

        //获取当前所有任务
        Set<String> keys = redisDataModel.keys(latestOrderKeyPrefix + "*");
        List<String> list = redisDataModel.mget(keys.toArray(new String[keys.size()]));
        //获取已确认任务
        List<String> confOrderNos = vmsOrdersVehicleAlterService.list().stream().map(VmsOrdersVehicleAlter::getTaskNo).collect(Collectors.toList());
        //判断港机类型
        Integer taskLocationType = vmsCranePadBind.getCraneType() == 1 ? 5 : 1;
        //过滤任务
        if (CollectionUtils.isNotEmpty(keys)) {
            if (CollectionUtils.isNotEmpty(list)) {
                List<VmsTosOrders> res = list.stream().map(s -> JSON.parseObject(s, VmsTosOrders.class))
                        .filter(vmsTosOrders -> {
                            if (vmsCranePadBind.getCraneNo().equals(vmsTosOrders.getPortCode()) && !VehicleUtil.isTaskOver(vmsTosOrders.getStatus())
                                    && vmsTosOrders.getTaskLocationType() == taskLocationType
                                    && (confOrderNos.isEmpty() || !confOrderNos.contains(vmsTosOrders.getTaskNo()))
                                    //暂时注释
//                                    && (vmsTosOrders.getStatus() == VmsTosOrderStatusEnum.ARRIVED_DESTINATION.getCode()
//                                        || vmsTosOrders.getStatus() == VmsTosOrderStatusEnum.VEHICLE_LOCKUP.getCode())
                            ) {
                                return true;
                            }
                            return false;
                        }).collect(Collectors.toList());
                if (!res.isEmpty()) {
                    res = res.stream().sorted(Comparator.comparing(VmsTosOrders::getCollectTime)).collect(Collectors.toList());
                }
                return res;
            }
        }
        return new ArrayList<VmsTosOrders>();
    }

}
