package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.constant.VehicleConstant;
import com.ssi.entity.VmsVehicle;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.QueryUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.ssi.mapper.VmsDebugAppMapper;
import com.ssi.entity.VmsDebugApp;
import com.ssi.service.VmsDebugAppService;

/**
 * 调试APP服务实现类
 */
@Service("debugAppService")
@Slf4j
public class VmsDebugAppServiceImpl extends ServiceImpl<VmsDebugAppMapper, VmsDebugApp> implements VmsDebugAppService {

    @Value("${debug-app.check-num.prefix}")
    private String checkNumPrefix;
    @Value("${harbor.remote.vehicle_status}")
    private String remoteControlStatusKeyPrefix;

    @Autowired
    private RedisDataModel redisDataModel;
    @Autowired
    private VmsVehicleService vmsVehicleService;

    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        IPage<VmsDebugApp> page = this.page(
                new QueryUtils<VmsDebugApp>().getPage(params),
                new QueryWrapper<VmsDebugApp>()
        );
        Set<String> keys = redisDataModel.keys(remoteControlStatusKeyPrefix + "*");
        if(CollectionUtils.isNotEmpty(keys)){
            List<String> controlStatusList = redisDataModel.mget(keys.toArray(new String[keys.size()]));
            if(CollectionUtils.isNotEmpty(controlStatusList)){
                List<VmsDebugApp> records = page.getRecords();
                if (CollectionUtils.isNotEmpty(records)){
                    records.forEach(vmsDebugApp -> {
                        String appMac = vmsDebugApp.getAppMac();
                        for (int i = 0; i < controlStatusList.size(); i++) {
                            String res = controlStatusList.get(i);
                            JSONObject obj = JSON.parseObject(res);
                            String mac = obj.getString("appMac");
                            Integer appControl = obj.getInteger("appControl");
                            String vin = obj.getString("vin");
                            if(StringUtils.isNotBlank(mac) && appMac.equals(mac) && appControl == 1){
                                VmsVehicle one = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getVin, vin).one();
                                if(one != null){
                                    vmsDebugApp.setVehicleNum(one.getVehicleNum());
                                }
                                vmsDebugApp.setVin(vin);
                                vmsDebugApp.setAppControl(appControl);
                            }
                        }
                    });
                }
            }
        }
        return new SSIPage(page);
    }

    @Override
    public String getKey() {
        String key = baseMapper.getKey(VehicleConstant.DEBUG_APP_KEY, VehicleConstant.ADMIN_DATABASE_NAME);
        return key;
    }

    @Override
    public void updateKey(String key) {
        String curKey = this.getKey();
        boolean flag = StringUtils.isNotBlank(curKey);
        baseMapper.updateKey(key, flag, VehicleConstant.DEBUG_APP_KEY, VehicleConstant.ADMIN_DATABASE_NAME);
    }

    /**
     * 密钥校验
     */
    @Override
    public SSIResponse checkApp(String key, String appMac) {
        log.info(String.format("遥控APP验证请求发送->appMac:%s,密钥:%s",appMac,key));
        int count = this.count(new LambdaQueryWrapper<VmsDebugApp>().eq(VmsDebugApp::getAppMac, appMac));

        if (count <= 0) {
            log.info(String.format("遥控APP验证结果->设备未授权,appMac:%s",appMac));
            return SSIResponse.no("设备未授权！");
        }

        String checkNumKey = String.format("%s:%S", checkNumPrefix, appMac);

        Integer num = StringUtils.isNotBlank(redisDataModel.get(checkNumKey)) ? Integer.parseInt(redisDataModel.get(checkNumKey)) : 0;

        if (num >= 5) {
            log.info(String.format("遥控APP验证结果->当前设备密钥已输错5次,appMac:%s,key:%s",appMac,key));
            return SSIResponse.no("密钥已输错5次，请稍后再试！");
        }

        String curKey = this.getKey();
        if (!curKey.equalsIgnoreCase(key)) {
            //添加错误次数
            num++;
            redisDataModel.set(checkNumKey, String.valueOf(num), 5 * 60L);
            log.info(String.format("遥控APP验证结果->密钥不正确,appMac:%s,key:%s",appMac,key));
            return SSIResponse.no("密钥不正确！");
        }

        redisDataModel.del(checkNumKey);
        log.info(String.format("遥控APP验证成功->appMac:%s,密钥:%s",appMac,key));
        return SSIResponse.ok();

    }

    /**
     * 修改遥控接管最大距离
     */
    @Override
    public void updateDiStance(Integer distance) {
        Integer curDistance = this.getDistance();
        boolean flag = curDistance != null;
        baseMapper.updateDiStance(distance, flag, VehicleConstant.DEBUG_APP_DISTANCE, VehicleConstant.ADMIN_DATABASE_NAME);

    }

    /**
     * 获取遥控接管最大距离
     */
    @Override
    public Integer getDistance() {
        String distance = baseMapper.getDistance(VehicleConstant.DEBUG_APP_DISTANCE, VehicleConstant.ADMIN_DATABASE_NAME);
        return StringUtils.isNotBlank(distance) ? Integer.parseInt(distance) : null;
    }

}
