package com.ssi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.response.SSIPage;
import com.ssi.utils.QueryUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.ssi.mapper.VmsEventMapper;
import com.ssi.entity.VmsEvent;
import com.ssi.service.VmsEventService;

/**
 * 事件服务实现类
 */
@Service("eventService")
public class VmsEventServiceImpl extends ServiceImpl<VmsEventMapper, VmsEvent> implements VmsEventService {

    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        String eventType = (String) params.get("eventType");
        Integer eventSource = (Integer) params.get("eventSource");
        Integer eventStatus = (Integer) params.get("eventStatus");
        Long startTime = (Long) params.get("startTime");
        Long stopTime = (Long) params.get("stopTime");
        Date startDate = startTime == null ? null : new Date(startTime);
        Date stopDate = stopTime == null ? null : new Date(stopTime);
        IPage<VmsEvent> page = this.page(
                new QueryUtils<VmsEvent>().getPage(params),
                new LambdaQueryWrapper<VmsEvent>()
                        .eq(StringUtils.isNotBlank(eventType), VmsEvent::getEventType, eventType)
                        .eq(eventSource != null, VmsEvent::getEventSource, eventSource)
                        //未来事件
                        .gt(eventStatus != null && eventStatus == 0, VmsEvent::getEventStartTime, new Date())
                        //活动事件
                        .le(eventStatus != null && eventStatus == 1, VmsEvent::getEventStartTime, new Date())
                        .and(eventStatus != null && eventStatus == 1, wrapper -> wrapper.ge(VmsEvent::getEventStopTime, new Date()))
                        //d过期事件
                        .lt(eventStatus != null && eventStatus == 2, VmsEvent::getEventStopTime, new Date())

                        .between(startTime != null && stopTime != null, VmsEvent::getEventStartTime, startDate, stopDate)
        );
        page.getRecords().parallelStream().forEach(event -> {
            if (event.getEventStatus() == null) {
                if (event.getEventStartTime().getTime() < new Date().getTime()) {
                    if (event.getEventStopTime().getTime() > new Date().getTime()) {
                        event.setEventStatus(1);
                    } else {
                        event.setEventStatus(2);
                    }
                } else {
                    event.setEventStatus(0);
                }
            }
        });
        return new SSIPage(page);
    }

}
