package com.ssi.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.constant.enums.VmsTosOrderStatusEnum;
import com.ssi.entity.VmsHangerStatusInfo;
import com.ssi.mapper.VmsHangerStatusInfoMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsHangerStatusInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import java.util.Map;

/**
 * 吊具状态服务实现类
 */
@Service("vmsHangerStatusInfoService")
public class VmsHangerStatusInfoServiceImpl extends ServiceImpl<VmsHangerStatusInfoMapper, VmsHangerStatusInfo> implements VmsHangerStatusInfoService {

    @Autowired
    private RedisDataModel redisDataModel;

    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;

    @Value("${harbor.vehicle.confirm_status_prefix}")
    private String harborVehicleConfirmStatusPrefix;

    @Value("${harbor.crane.realTimeInfoKeyPrefix}")
    private String craneRealTimeInfoKeyPrefix;

    @Override
    public IPage<VmsHangerStatusInfo> queryPage(int current, int size, VmsHangerStatusInfo vmsHangerStatusInfo) {
        IPage<VmsHangerStatusInfo> page = new Page<>(current, size);
        QueryWrapper<VmsHangerStatusInfo> wrapper = new QueryWrapper<>();
        return this.page(page,wrapper);
    }

    /**
     * 查询车辆装卸箱时吊具状态信息
     */
    @Override
    public SSIResponse getHangerStatusByVin(String vin) {
        String orderKey = String.format("%s:%s", latestOrderKeyPrefix, vin);
        Map<String, Object> map = redisDataModel.getJson2Map(orderKey);
        if(map != null){
            int status = (int) map.get("status");
            if(VmsTosOrderStatusEnum.isLoadUnloadStatus(status)){
                String portCode = (String) map.get("portCode");
                if(portCode == null){
                    return SSIResponse.no("未查询到吊具状态信息");
                }
                Map<String, Object> hangerStatusMap = redisDataModel.getJson2Map(String.format("%s:%s", craneRealTimeInfoKeyPrefix, portCode));
                hangerStatusMap.put("orderStatus",status);
                String confirmStatusKey = String.format("%s:%s", harborVehicleConfirmStatusPrefix, vin);
                Map<String, Object> confirmStatusMap = redisDataModel.getJson2Map(confirmStatusKey);
                hangerStatusMap.putAll(confirmStatusMap);
                return SSIResponse.ok(hangerStatusMap);
            }else{
                return SSIResponse.no("等待车辆停稳锁死");
            }
        }
        return SSIResponse.no("未查询到车辆任务");
    }
}
