package com.ssi.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.entity.VmsKpiAnalysis;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.vo.KpiCostTimeVo;
import com.ssi.mapper.VmsKpiAnalysisMapper;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsKpiAnalysisService;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.DateUtils;
import com.ssi.utils.PoiUtils;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
* @author Admin
* @description 针对表【vms_kpi_analysis】的数据库操作Service实现
* @createDate 2023-02-28 14:17:30
*/
@Slf4j
@Service
public class VmsKpiAnalysisServiceImpl extends ServiceImpl<VmsKpiAnalysisMapper, VmsKpiAnalysis>
    implements VmsKpiAnalysisService{


    @Autowired
    private VmsVehicleService vmsVehicleService;

    @Override
    public List<JSONObject> staticsWorkTime(KpiCostTimeVo kpiCostTimeVo) {
       return this.baseMapper.staticsWorkTime(kpiCostTimeVo);
    }

    @Override
    public List<JSONObject> taskAnalysis(KpiCostTimeVo kpiCostTimeVo) {
        kpiCostTimeVo.setVehicleTaskLabels(Arrays.asList(TaskType.LOAD_TASK.code,TaskType.UNLOAD_TASK.code));
        List<VmsKpiAnalysis> vmsKpiAnalyses = this.baseMapper.taskAnalysis(kpiCostTimeVo);
        if(CollectionUtils.isEmpty(vmsKpiAnalyses)){
            return Collections.emptyList();
        }
//        List<VmsKpiAnalysis> mergedList = componentWholeOrder(vmsKpiAnalyses);
        //计算统计数据
        return kaiCalculate(vmsKpiAnalyses);
    }

    @Override
    public VmsKpiAnalysis staticsSpeciParam(KpiCostTimeVo qryKpi) {
        return this.baseMapper.staticsSpeciParam(qryKpi);
    }

    @Override
    public JSONObject taskIndexAnalysis(KpiCostTimeVo qryKpi) {
        qryKpi.setVehicleTaskLabels(Arrays.asList(TaskType.LOAD_TASK.code,TaskType.UNLOAD_TASK.code,TaskType.COMPETE_TASK.code));
        List<VmsKpiAnalysis> vmsKpiAnalyses = this.baseMapper.taskAnalysis(qryKpi);
        JSONObject result = new JSONObject();

        if(CollectionUtils.isEmpty(vmsKpiAnalyses)){
            return result;
        }
        List<VmsKpiAnalysis> mergedList = componentWholeOrder(vmsKpiAnalyses);
        result.put("totalTask",mergedList.size());
        Map<Integer, List<VmsKpiAnalysis>> lableCollect = mergedList.stream().collect(Collectors.groupingBy(VmsKpiAnalysis::getVehicleTaskLabel, Collectors.toList()));
        result.put("loadTask",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?lableCollect.get(TaskType.LOAD_TASK.code).size():0);
        result.put("loadNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?lableCollect.get(TaskType.LOAD_TASK.code).size():0);
        result.put("loadStandNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?lableCollect.get(TaskType.LOAD_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getStandardBoxNum).sum():0);
        result.put("loadAvaTime",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?new BigDecimal(lableCollect.get(TaskType.LOAD_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getWorkTime).sum()/lableCollect.get(TaskType.LOAD_TASK.code).size()).setScale(2,RoundingMode.HALF_UP):0);

        result.put("unloadTask",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?lableCollect.get(TaskType.UNLOAD_TASK.code).size():0);
        result.put("unloadNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?lableCollect.get(TaskType.UNLOAD_TASK.code).size():0);
        result.put("unloadStandNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?lableCollect.get(TaskType.UNLOAD_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getStandardBoxNum).sum():0);
        result.put("unloadAvaTime",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?new BigDecimal(lableCollect.get(TaskType.UNLOAD_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getWorkTime).sum()/lableCollect.get(TaskType.UNLOAD_TASK.code).size()).setScale(2,RoundingMode.HALF_UP):0);

        result.put("competeTask",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?lableCollect.get(TaskType.COMPETE_TASK.code).size():0);
        result.put("competeNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?lableCollect.get(TaskType.COMPETE_TASK.code).size():0);
        result.put("competeStandNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?lableCollect.get(TaskType.COMPETE_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getStandardBoxNum).sum():0);
        result.put("competeAvaTime",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?new BigDecimal(lableCollect.get(TaskType.COMPETE_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getWorkTime).sum()/lableCollect.get(TaskType.COMPETE_TASK.code).size()).setScale(2,RoundingMode.HALF_UP):0);
        return result;
    }

    @Override
    public SSIResponse getTaskTrend(Integer timeType, Integer groupType) {
        assert timeType!=null:"时间不能为空";
        assert groupType!=null:"分组类型不能为空";

        int deltDate = timeType==2?7:timeType==3?30:1;
        LocalDateTime startTime = LocalDateTime.of(
                LocalDateTime.now().minus(deltDate, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
        LocalDateTime endTime = LocalDateTime.of(
                LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
        List<VmsKpiAnalysis> taskTrends = this.baseMapper.getTaskTrend(startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli(), endTime.toInstant(ZoneOffset.of("+8")).toEpochMilli());
        List<VmsKpiAnalysis> vmsKpiAnalyses = componentWholeOrder(taskTrends);
        //分组
        Map<String, List<VmsKpiAnalysis>> groupMap = vmsKpiAnalyses.stream().collect(Collectors.groupingBy(vmsKpiAnalysis -> {
            if (groupType == 1) {
                String portCode = vmsKpiAnalysis.getPortCode();
                return StringUtils.isNotBlank(portCode)?portCode:"other";
            } else if (groupType == 2) {
                String voyageNo = vmsKpiAnalysis.getVoyageNo();
                return StringUtils.isNotBlank(voyageNo)?voyageNo:"other";
            } else if (groupType == 3) {
                Long collectTime = vmsKpiAnalysis.getCollectTime();
                Date date = new Date(collectTime);
                if (timeType == 1) {
                    return DateUtils.format(date, "HH");
                }
                return DateUtils.format(date, DateUtils.DATE_PATTERN);
            }
            VmsVehicle one = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getVin, vmsKpiAnalysis.getVin()).one();
            return Objects.isNull(one)?vmsKpiAnalysis.getVin():one.getVehicleNum();
        }, Collectors.toList()));

        List<JSONObject> jsonObjects = calTaskTrend(groupMap);
        if(groupType==3){//时间填充坐标轴
            fillTimeLine(startTime,endTime,jsonObjects);
        }
        jsonObjects.sort(Comparator.comparing(a -> a.getString("xAis")));
        return SSIResponse.ok(jsonObjects);
    }

    @Override
    public void exportExcel(KpiCostTimeVo kpiCostTimeVo, HttpServletResponse response) {

        List<VmsKpiAnalysis> vmsKpiAnalyses = this.baseMapper.taskAnalysis(kpiCostTimeVo);

        String fileName = "KPI数据详情" + DateUtils.format(new Date(), 2) + ".xls";
        PoiUtils.exportExcel(vmsKpiAnalyses, null,
                "orderList", VmsKpiAnalysis.class,
                fileName, true, response);

    }

    @Override
    public List<String> getVoyageNo() {

        return this.baseMapper.getVoyageNo();
    }

    private void fillTimeLine(LocalDateTime startTime, LocalDateTime endTime, List<JSONObject> jsonObjects) {

        String formatter = "yyyy-MM-dd";
        if(startTime.toLocalDate().compareTo(endTime.toLocalDate())==0){
            formatter = "HH";
        }
        do{
            String format = startTime.format(DateTimeFormatter.ofPattern(formatter));

            Boolean xAis = jsonObjects.stream().filter(jsonObject -> jsonObject.getString("xAis").equalsIgnoreCase(format)).findAny().isPresent();
            log.info("~~{} 存在吗 {}",format,xAis);
            if(!xAis){
                JSONObject tmpObj = new JSONObject();
                tmpObj.put("xAis",format);
                tmpObj.put("loadNum",0);
                tmpObj.put("unloadNum",0);
                tmpObj.put("competeNum",0);
                tmpObj.put("loadAvaTime",0);
                tmpObj.put("unloadAvaTime",0);
                tmpObj.put("competeAvaTime",0);
                jsonObjects.add(tmpObj);
            }
            if(startTime.toLocalDate().compareTo(endTime.toLocalDate())==0){
                startTime = startTime.plusHours(1l);
            }else {
                startTime = startTime.plusDays(1l);
            }
        }while (startTime.compareTo(endTime)<0);

    }

    private List<JSONObject> calTaskTrend(Map<String, List<VmsKpiAnalysis>> groupMap) {
        List<JSONObject> jsonObjects = new ArrayList<>();
        for (Map.Entry<String, List<VmsKpiAnalysis>> entry:groupMap.entrySet()){
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("xAis",entry.getKey());
            List<VmsKpiAnalysis> mergedList = entry.getValue();
            Map<Integer, List<VmsKpiAnalysis>> lableCollect = mergedList.stream().collect(Collectors.groupingBy(VmsKpiAnalysis::getVehicleTaskLabel, Collectors.toList()));
            jsonObject.put("loadNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?lableCollect.get(TaskType.LOAD_TASK.code).size():0);
            jsonObject.put("unloadNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?lableCollect.get(TaskType.UNLOAD_TASK.code).size():0);
            jsonObject.put("competeNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?lableCollect.get(TaskType.COMPETE_TASK.code).size():0);
            jsonObject.put("loadAvaTime",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?new BigDecimal(lableCollect.get(TaskType.LOAD_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getWorkTime).sum()/lableCollect.get(TaskType.LOAD_TASK.code).size()).setScale(2,RoundingMode.HALF_UP):0);
            jsonObject.put("unloadAvaTime",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?new BigDecimal(lableCollect.get(TaskType.UNLOAD_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getWorkTime).sum()/lableCollect.get(TaskType.UNLOAD_TASK.code).size()).setScale(2,RoundingMode.HALF_UP):0);
            jsonObject.put("competeAvaTime",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?new BigDecimal(lableCollect.get(TaskType.COMPETE_TASK.code).stream().mapToDouble(VmsKpiAnalysis::getWorkTime).sum()/lableCollect.get(TaskType.COMPETE_TASK.code).size()).setScale(2,RoundingMode.HALF_UP):0);

            jsonObject.put("loadStandNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.LOAD_TASK.code))?lableCollect.get(TaskType.LOAD_TASK.code).stream().filter(e->Objects.nonNull(e.getStandardBoxNum())).mapToDouble(VmsKpiAnalysis::getStandardBoxNum).sum():0);
            jsonObject.put("unloadStandNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.UNLOAD_TASK.code))?lableCollect.get(TaskType.UNLOAD_TASK.code).stream().filter(e->Objects.nonNull(e.getStandardBoxNum())).mapToDouble(VmsKpiAnalysis::getStandardBoxNum).sum():0);
            jsonObject.put("competeStandNum",CollectionUtils.isNotEmpty(lableCollect.get(TaskType.COMPETE_TASK.code))?lableCollect.get(TaskType.COMPETE_TASK.code).stream().filter(e->Objects.nonNull(e.getStandardBoxNum())).mapToDouble(VmsKpiAnalysis::getStandardBoxNum).sum():0);
            jsonObjects.add(jsonObject);
        }
        return jsonObjects;
    }


    private List<JSONObject> kaiCalculate(List<VmsKpiAnalysis> mergedList) {

        List<JSONObject> resultList = new ArrayList<>();
        if(CollectionUtils.isEmpty(mergedList)){
            return resultList;
        }
        Map<String, List<VmsKpiAnalysis>> vinListMap = mergedList.stream().collect(Collectors.groupingBy(VmsKpiAnalysis::getVin, Collectors.toList()));
        for (Map.Entry<String, List<VmsKpiAnalysis>> entry : vinListMap.entrySet()){
            //平均收箱里程， 平均送箱里程， 单圈里程  重载率（%）空载率（%）
            List<VmsKpiAnalysis> tmpList =  entry.getValue();
            double receiveMile = tmpList.stream().filter(vmsKpiAnalysis -> vmsKpiAnalysis.getTaskType() == TaskType.LOAD_TASK.code).mapToDouble(VmsKpiAnalysis::getMileage).sum();
            double senMile = tmpList.stream().filter(vmsKpiAnalysis -> vmsKpiAnalysis.getTaskType() == TaskType.UNLOAD_TASK.code).mapToDouble(VmsKpiAnalysis::getMileage).sum();
            double receiveTime = tmpList.stream().filter(vmsKpiAnalysis -> vmsKpiAnalysis.getTaskType() == TaskType.LOAD_TASK.code).mapToDouble(VmsKpiAnalysis::getWorkTime).sum();
            double senTime = tmpList.stream().filter(vmsKpiAnalysis -> vmsKpiAnalysis.getTaskType() == TaskType.UNLOAD_TASK.code).mapToDouble(VmsKpiAnalysis::getWorkTime).sum();
            double total = receiveMile+senMile;
            JSONObject jsonObject = new JSONObject();
            VmsVehicle one = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getVin, entry.getKey()).one();
            jsonObject.put("vin",Objects.nonNull(one)?one.getVehicleNum():entry.getKey());
            jsonObject.put("avgReciveMile", new BigDecimal(receiveMile/tmpList.size()).setScale(2, RoundingMode.HALF_UP));
            jsonObject.put("avgSendMile", new BigDecimal(senMile/tmpList.size()).setScale(2, RoundingMode.HALF_UP));
            jsonObject.put("singleMile", new BigDecimal(total/tmpList.size()).setScale(2, RoundingMode.HALF_UP));
            jsonObject.put("loadRatio", new BigDecimal(receiveTime/(receiveTime+senTime)).setScale(2, RoundingMode.HALF_UP));
            jsonObject.put("unloadRatio", new BigDecimal(senTime/(receiveTime+senTime)).setScale(2, RoundingMode.HALF_UP));
            resultList.add(jsonObject);
        }
        return resultList;
    }

    /**
     *
     * @param vmsKpiAnalyses 未组装任务列表
     * @return 组装好的任务列表
     */
    private List<VmsKpiAnalysis> componentWholeOrder(List<VmsKpiAnalysis> vmsKpiAnalyses) {
        List<VmsKpiAnalysis> mergedList = new ArrayList<>();
        Map<String,VmsKpiAnalysis> tmpMap = new HashMap<>();
        for (VmsKpiAnalysis vmsKpiAnalysis: vmsKpiAnalyses) {
            VmsKpiAnalysis oldKpi = tmpMap.get(vmsKpiAnalysis.getVin());
            if(Objects.isNull(oldKpi)){
                tmpMap.put(vmsKpiAnalysis.getVin(),vmsKpiAnalysis);
                continue;
            }
            //任务合并逻辑
            int mergeFlag = mergeTaskInfo(mergedList, oldKpi, vmsKpiAnalysis);
            if(mergeFlag==0){
                tmpMap.remove(vmsKpiAnalysis.getVin());
            }
            //处理任务异常逻辑
            if(mergeFlag==1){
                tmpMap.put(vmsKpiAnalysis.getVin(),vmsKpiAnalysis);
            }
        }
        //处理剩余数据
        tmpMap.forEach((k,v)->{
            if((v.getVehicleTaskLabel()==TaskType.LOAD_TASK.code&&v.getPortType()==1)
                    ||(v.getVehicleTaskLabel()==TaskType.UNLOAD_TASK.code&&v.getPortType()==2)
                    ||(v.getVehicleTaskLabel()==TaskType.COMPETE_TASK.code&&v.getPortType()==2)
            ){
                mergedList.add(v);
            }
        });
        return mergedList;
    }
    /**
     *
     * @param mergedList
     * @param oldKpi
     * @param vmsKpiAnalysis
     * @return 0: 完成任务链，List新增任务，移除tmpMap; 1: 任务变更,List新增任务，覆盖旧任务;2：任务周期中，无需操作
     */
    private int mergeTaskInfo(List<VmsKpiAnalysis> mergedList, VmsKpiAnalysis oldKpi, VmsKpiAnalysis vmsKpiAnalysis) {
        //非当前任务||旧任务为结束任务
        if(oldKpi.getVehicleTaskLabel()!=vmsKpiAnalysis.getVehicleTaskLabel()
                ||(oldKpi.getVehicleTaskLabel()==TaskType.LOAD_TASK.code&&oldKpi.getPortType()==1)
                ||(oldKpi.getVehicleTaskLabel()==TaskType.UNLOAD_TASK.code&&oldKpi.getPortType()==2)
                ||(oldKpi.getVehicleTaskLabel()==TaskType.COMPETE_TASK.code&&oldKpi.getPortType()==2)
        ){
            mergedList.add(oldKpi);
            return  1;
        }
        //当前任务||新任务为起始任务, 多个起始任务统计为一个任务，
        oldKpi.setMileage(oldKpi.getMileage()+vmsKpiAnalysis.getMileage());
        oldKpi.setEnergyConsumption(oldKpi.getEnergyConsumption()+vmsKpiAnalysis.getEnergyConsumption());
        oldKpi.setWorkTime(oldKpi.getWorkTime()+ vmsKpiAnalysis.getWorkTime());
        oldKpi.setVehicleTaskLabel(vmsKpiAnalysis.getVehicleTaskLabel());
        oldKpi.setPortType(vmsKpiAnalysis.getPortType());
        if(StringUtils.isNotBlank(vmsKpiAnalysis.getPortCode())&&vmsKpiAnalysis.getPortType()==1) oldKpi.setPortCode(vmsKpiAnalysis.getPortCode());
        if(StringUtils.isNotBlank(vmsKpiAnalysis.getVoyageNo())) oldKpi.setVoyageNo(vmsKpiAnalysis.getVoyageNo());

        //相同任务类型， 判断是否为完整任务链
        if((oldKpi.getVehicleTaskLabel()==TaskType.LOAD_TASK.code&&vmsKpiAnalysis.getPortType()==1)
                ||(oldKpi.getVehicleTaskLabel()==TaskType.UNLOAD_TASK.code&&vmsKpiAnalysis.getPortType()==2)
                ||(oldKpi.getVehicleTaskLabel()==TaskType.COMPETE_TASK.code&&vmsKpiAnalysis.getPortType()==2)
        ){
            mergedList.add(oldKpi);
            return 0;
        }else {
            return 2;
        }

    }

     enum TaskType{
        LOAD_TASK(1),
         UNLOAD_TASK(2),
         COMPETE_TASK(3);

         private Integer code;
         TaskType(Integer code) {
             this.code = code;
         }

         public Integer getCode() {
             return code;
         }
     }
}




