package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.entity.VmsTosOrders;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIResponse;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.ssi.mapper.VmsOrdersVehicleAlterMapper;
import com.ssi.entity.VmsOrdersVehicleAlter;
import com.ssi.service.VmsOrdersVehicleAlterService;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


@Service("ordersVehicleAlterService")
public class VmsOrdersVehicleAlterServiceImpl extends ServiceImpl<VmsOrdersVehicleAlterMapper, VmsOrdersVehicleAlter> implements VmsOrdersVehicleAlterService {

    @Autowired
    private RedisDataModel redisDataModel;
    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;

    /**
     * 确认任务车辆
     */
    @Override
    public SSIResponse changeOrdersVehicle(String taskNo, String vin) {
        int count = this.count(new LambdaQueryWrapper<VmsOrdersVehicleAlter>().eq(VmsOrdersVehicleAlter::getTaskNo, taskNo));
        if(count > 0){
            return SSIResponse.no("该任务已确认，请勿重复确认");
        }

        //获取任务原vin
        String originalVin = null;
        Set<String> keys = redisDataModel.keys(latestOrderKeyPrefix + "*");
        List<String> list = redisDataModel.mget(keys.toArray(new String[keys.size()]));
        if (CollectionUtils.isNotEmpty(keys)) {
            if (CollectionUtils.isNotEmpty(list)) {
                List<VmsTosOrders> res = list.stream().map(s -> JSON.parseObject(s,VmsTosOrders.class)).collect(Collectors.toList());
                for(VmsTosOrders vmsTosOrders : res){
                    if(vmsTosOrders.getTaskNo().equals(taskNo)){
                        originalVin = vmsTosOrders.getVin();
                    }
                }
            }
        }

        VmsOrdersVehicleAlter vmsOrdersVehicleAlter = new VmsOrdersVehicleAlter();
        vmsOrdersVehicleAlter.setTaskNo(taskNo);
        vmsOrdersVehicleAlter.setVin(vin);
        vmsOrdersVehicleAlter.setOriginalVin(originalVin);
        this.save(vmsOrdersVehicleAlter);
        return SSIResponse.ok();
    }
}
