package com.ssi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.ssi.constant.URL;
import com.ssi.entity.vo.VmsRoadDeviceVo;
import com.ssi.mapper.VmsRoadDeviceMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.utils.RestTemplateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.utils.QueryUtils;

import com.ssi.entity.VmsRoadDevice;
import com.ssi.service.VmsRoadDeviceService;

/**
 * 路侧设备服务实现类
 */
@Service("roadDeviceService")
public class VmsRoadDeviceServiceImpl extends ServiceImpl<VmsRoadDeviceMapper, VmsRoadDevice> implements VmsRoadDeviceService {
    @Autowired
    private RedisDataModel redisDataModel;

    @Value("${road-device.video-platform-url}")
    private String videoPlatformUrl;
    @Value("${road-device.need-get-video}")
    private boolean needGetVideo;
    @Value("${harbor.v2x.rsm}")
    private String rsmPrefix;
    @Value("${BridgeCrane-url}")
    private String bcUrl;// 桥吊 url
    /**
     * 路侧设备分页数据
     * @param params
     * @return
     */
    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        String deviceNum = (String) params.get("deviceNum");
        String deviceType = (String) params.get("deviceType");
        Integer onlineStatus = (Integer) params.get("onlineStatus");
        Integer workStatus = (Integer) params.get("workStatus");
        IPage<VmsRoadDevice> page = this.page(
                new QueryUtils<VmsRoadDevice>().getPage(params),
                new LambdaQueryWrapper<VmsRoadDevice>().like(StringUtils.isNotBlank(deviceNum), VmsRoadDevice::getDeviceNum, deviceNum)
                        .eq(StringUtils.isNotBlank(deviceType), VmsRoadDevice::getDeviceType, deviceType)
                        .eq(onlineStatus != null, VmsRoadDevice::getOnlineStatus, onlineStatus)
                        .eq(workStatus != null, VmsRoadDevice::getWorkStatus, workStatus)
        );

        if(needGetVideo){
            page.getRecords().parallelStream().forEach(vmsRoadDevice -> {
                getVideoPlayUrl(vmsRoadDevice);
            });
        }
        page.getRecords().parallelStream().forEach(vmsRoadDevice -> {
            if(StringUtils.startsWith(vmsRoadDevice.getType(),"C")){
              Map m =   getBridgeCrane(StringUtils.substring(vmsRoadDevice.getType(),1,4));
              vmsRoadDevice.setLat(Double.valueOf(m.get("latitude")+""));
              vmsRoadDevice.setLon(Double.valueOf(m.get("longitude")+""));
            }
        });
        return new SSIPage(page);
    }

    private  Map  getBridgeCrane(String  number){
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("craneNum", number);
        jsonObject.put("laneNum", 4);
        jsonObject.put("containerSize", 1);
        jsonObject.put("containerPosition", 2);
        String result = "";
        try {
            result = RestTemplateUtil.post(bcUrl, jsonObject.toString(), null);
        } catch (Exception e) {
            result =  null;
        }
        if(result!=null){
            Map m =   JSONObject.parseObject(result,Map.class);
            return m;
        }
        return new HashMap<>();
    }


    private void getVideoPlayUrl(VmsRoadDevice vmsRoadDevice) {
        vmsRoadDevice.setVideoOnline(false);
        List<String> playVideoAddressList = Lists.newArrayList();
        try {
            String videoAddress = vmsRoadDevice.getVideoAddress();
            if(StringUtils.isNotBlank(videoAddress)){
                List<String> videoAddressList = Arrays.asList(videoAddress.split(","));
                videoAddressList.stream().forEach(curAddr -> {
                    String playVideoAddress = getPlayVideoAddress(curAddr);
                    if(StringUtils.isNotBlank(playVideoAddress)){
                        playVideoAddressList.add(playVideoAddress);
                    }
                });
            }
            if(!playVideoAddressList.isEmpty()){
                vmsRoadDevice.setVideoOnline(true);
                vmsRoadDevice.setPlayVideoAddress(playVideoAddressList);
            }
        } catch (Exception e) {
            log.error("路测设备请求视频地址接口失败：vmsRoadDevice："+JSONObject.toJSONString(vmsRoadDevice));
        }

    }

    private String getPlayVideoAddress(String videoAddress) {
        boolean rtspAddr = videoAddress.startsWith("rtsp:");
        if(rtspAddr){
            String checkResult = RestTemplateUtil.get(String.format("%s/%s%s", videoPlatformUrl, URL.ROAD_DEVICE_CHECKONLINE, videoAddress),null);
            JSONObject checkResultJson = JSONObject.parseObject(checkResult);
            if(0 == checkResultJson.getInteger("code")){
                Boolean online = checkResultJson.getJSONObject("data").getBoolean("online");
                if(online){
                    String urlResult = RestTemplateUtil.get(String.format("%s/%s%s", videoPlatformUrl, URL.ROAD_DEVICE_RTSPSTART, videoAddress),null);
                    JSONObject urlResultJson = JSONObject.parseObject(urlResult);

                    if(0 == urlResultJson.getInteger("code")
                            || 201 == urlResultJson.getInteger("code")){
                        return urlResultJson.getJSONObject("data").toJSONString();
                    }
                }
            }
        }
        boolean rtmpAddr = videoAddress.startsWith("rtmp:");
        if(rtmpAddr){
            String checkResult = RestTemplateUtil.get(String.format("%s/%s%s", videoPlatformUrl, URL.RTMP_CHECKONLINE, videoAddress),null);
            JSONObject checkResultJson = JSONObject.parseObject(checkResult);
            if(0 == checkResultJson.getInteger("code")){
                Boolean online = checkResultJson.getJSONObject("data").getBoolean("online");
                if(online){
                    String urlResult = RestTemplateUtil.get(String.format("%s/%s%s", videoPlatformUrl, URL.RTMP_PLAY, videoAddress),null);
                    JSONObject urlResultJson = JSONObject.parseObject(urlResult);
                    if(0 == urlResultJson.getInteger("code")
                            || 201 == urlResultJson.getInteger("code")){
                        return urlResultJson.getJSONObject("data").toJSONString();
                    }
                }
            }
        }
        return null;
    }

    /**
     * 获取视频播放地址
     * @return
     */
    @Override
    public List<VmsRoadDeviceVo> videoPlayUrlList(){
        List<VmsRoadDevice> list = this.list();
        List<VmsRoadDeviceVo> collect = list.stream().flatMap(vmsRoadDevice -> {
            getVideoPlayUrl(vmsRoadDevice);
            List<String> playVideoAddress = vmsRoadDevice.getPlayVideoAddress();
            if(playVideoAddress != null){
                return playVideoAddress.stream().map(url -> {
                    VmsRoadDeviceVo vmsRoadDeviceVo = new VmsRoadDeviceVo();
                    vmsRoadDeviceVo.setDeviceName(vmsRoadDevice.getDeviceName());
                    vmsRoadDeviceVo.setUrl(url);
                    return vmsRoadDeviceVo;
                });
            }
            return Stream.empty();
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * 路测安全消息信息
     */
    @Override
    public Map getRsmInfo(Long id) {
        Map<String, Object> map = redisDataModel.getJson2Map(rsmPrefix + ":" + id);
        return map;
    }
}
