package com.ssi.service.impl;



import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.entity.*;
import com.ssi.entity.vo.CargoShipConfigDto;
import com.ssi.mapper.VmsShipsDrawingMapper;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.CargoShipConfigInfoService;
import com.ssi.service.VmsShipsDrawingService;
import com.ssi.utils.AuthorizationUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;


/**
 * 船模图
 */
@Service("shipsDrawingService")
@DS("oracle")
public class VmsShipsDrawingServiceImpl extends ServiceImpl<VmsShipsDrawingMapper, VmsShipsDrawing> implements VmsShipsDrawingService {

	@Autowired
	private CargoShipConfigInfoService cargoShipConfigInfoService;

	@Override
	public VmsShipsDrawing getVmsShipsDrawing(Map<String, Object> params) {
		//参数解析
		String shipCod = String.valueOf(params.get("shipCod"));
		String voyageNo = String.valueOf(params.get("voyageNo"));
		String workStatus = String.valueOf(params.get("workStatus"));
		String[] bayNo = parseBayId(String.valueOf(params.get("bayNo")));

		Map<String, VmsShipNcl> shipBayNcl = new HashMap<>(0);
		if("1".equals(workStatus)){
			shipBayNcl = getShipBayNcl(voyageNo, bayNo[0]);
			if(!bayNo[0].equals(bayNo[1])){
				shipBayNcl.putAll(getShipBayNcl(voyageNo, bayNo[1]));
			}
		}

		VmsShipsDrawing shipDraw = new VmsShipsDrawing();
		searchShipUDrawByBayNo(shipDraw, shipCod, bayNo, shipBayNcl);
		searchShipDDrawByBayNo(shipDraw, shipCod, bayNo, shipBayNcl);

		return shipDraw;
	}

	private Map<String, VmsShipNcl> getShipBayNcl(String voyageNo,  String shortBayNo){
		//贝位处于工作状态
		List<VmsShipNcl> vmsShipNcls =  this.baseMapper.queryShipNclByShortBayNo(voyageNo, shortBayNo);
		Map<String, VmsShipNcl> bayNclMap;
		if(vmsShipNcls != null){
			int size = vmsShipNcls.size();
			bayNclMap = new HashMap<>(size);
			VmsShipNcl vmsShipNcl;
			for (int i = 0; i < size; i++) {
				vmsShipNcl = vmsShipNcls.get(i);
				//强制上船状态解析
				if(vmsShipNcl.getAllowFlag() != null){
					if("Y".equalsIgnoreCase(vmsShipNcl.getAllowFlag())){
						//强制上船
						vmsShipNcl.setCommStatus("7");
					}else if("N".equalsIgnoreCase(vmsShipNcl.getAllowFlag())){
						//禁止上船
						vmsShipNcl.setCommStatus("8");
					}
				}
				bayNclMap.put(vmsShipNcl.getBayNo(), vmsShipNcl);
			}
		}else{
			bayNclMap = new HashMap<>(0);
		}

		return bayNclMap;
	}


	public void searchShipUDrawByBayNo(VmsShipsDrawing shipDraw, String shipCod, String[] bayNo, Map<String, VmsShipNcl> shipBayNcl){

		List<VmsShipsDrawing> layNoU = this.baseMapper.queryShipDrawBylayNo(shipCod,"U", bayNo[0]);
		if(layNoU == null){return;}

		shipDraw.setU(layNoU);
		List<String> layNolist = new ArrayList<>();
		int maxRow = 0;

		for(VmsShipsDrawing layNo : layNoU) {
			// 添加列编号
			layNolist.add(layNo.getLayNo());
			List<VmsShipsDrawing> rowNoList = this.baseMapper.queryShipDrawByRowNo(shipCod, "U", bayNo[0], layNo.getLayNo());
			if(rowNoList == null){
				continue;
			}

			int size = rowNoList.size();
			if(size > maxRow){
				maxRow = size;
			}

			layNo.setU(rowNoList);

			// 根据船号 U舱体，贝号，层号 获取 箱体的信息
			for(VmsShipsDrawing row : rowNoList) {
				VmsShipNcl vmsShipNcl = shipBayNcl.get(String.format("%s%s%s", bayNo[0], row.getRowNo(), layNo.getLayNo()));
				if(vmsShipNcl == null){
					vmsShipNcl = shipBayNcl.get(String.format("%s%s%s", bayNo[1], row.getRowNo(), layNo.getLayNo()));
				}
				if(vmsShipNcl != null){
					List<VmsShipNcl> tempnclList1 = row.getShipNcl();
					if(tempnclList1 == null){
						tempnclList1 = new ArrayList<>();
						row.setShipNcl(tempnclList1);
					}
					tempnclList1.add(vmsShipNcl);
					row.setWorkStatus(1);
				}
			}
		}
		shipDraw.setUlayNoList(layNolist);
		shipDraw.setUmaxRow(maxRow);
	}

	public void searchShipDDrawByBayNo(VmsShipsDrawing shipDraw, String shipCod, String[] bayNo, Map<String, VmsShipNcl> shipBayNcl){

		List<VmsShipsDrawing> layNoD = this.baseMapper.queryShipDrawBylayNo(shipCod,"D", bayNo[0]);
		if(layNoD == null){return;}

		shipDraw.setD(layNoD);
		List<String> layNolist = new ArrayList<>();
		int maxRow = 0;

		for(VmsShipsDrawing layNo : layNoD) {
			// 添加列编号
			layNolist.add(layNo.getLayNo());

			List<VmsShipsDrawing> rowNoList = this.baseMapper.queryShipDrawByRowNo(shipCod, "D", bayNo[0], layNo.getLayNo());
			if(rowNoList == null){
				continue;
			}

			int size = rowNoList.size();
			if(size > maxRow){
				maxRow = size;
			}

			layNo.setD(rowNoList);

			// 根据船号 U舱体，贝号，层号 获取 箱体的信息
			for(VmsShipsDrawing row : rowNoList) {
				VmsShipNcl vmsShipNcl = shipBayNcl.get(String.format("%s%s%s", bayNo[0], row.getRowNo(), layNo.getLayNo()));
				if(vmsShipNcl == null){
					vmsShipNcl = shipBayNcl.get(String.format("%s%s%s", bayNo[1], row.getRowNo(), layNo.getLayNo()));
				}
				if(vmsShipNcl != null){
					List<VmsShipNcl> tempnclList1 = row.getShipNcl();
					if(tempnclList1 == null){
						tempnclList1 = new ArrayList<>();
						row.setShipNcl(tempnclList1);
					}
					tempnclList1.add(vmsShipNcl);
					row.setWorkStatus(1);
				}
			}
		}
		shipDraw.setDlayNoList(layNolist);
		shipDraw.setDmaxRow(maxRow);
	}

	public VmsShipsDrawing test(String shipCod,String bayNo,String voyageNo) {
		VmsShipsDrawing shipDraw=new VmsShipsDrawing();
	    	 List<VmsShipsDrawing> shipMaplist=this.baseMapper.findShipMap(shipCod,bayNo); // 根据船号和贝号 获取所有的箱体信息
	    	 List<VmsShipNcl> shipNclList=this.baseMapper.findShipNcl(voyageNo,bayNo); // 根据船号和贝号 获取所有的箱体信息
	    	 List<VmsShipsDrawing> layNoU=new ArrayList();
	    	 List<VmsShipsDrawing> layNoD=new ArrayList();
	    	 VmsShipsDrawing layNo;
	    	 VmsShipsDrawing rowNo;
	    	 Map<String ,VmsShipsDrawing> keyMap=new HashMap<String ,VmsShipsDrawing >();
	    	 for(VmsShipsDrawing smp:shipMaplist) {
	    		 if(smp.getDeckId().equals("D")) {
	    			  rowNo=new VmsShipsDrawing();
	    			  rowNo.setBayNo(smp.getBayNo());
	    			  rowNo.setLayNo(smp.getLayNo());
	    			  rowNo.setRowNo(smp.getRowNo());
//	    			  rowNo.setShipsmp(shipsmp);
	    			  String tempBayNo=bayNo+smp.getLayNo()+smp.getRowNo();
//	    			  String temp=bayNo+ncl.getLayNo()+
	    			  List<VmsShipNcl> teml=new ArrayList<VmsShipNcl>();
	    			  for(VmsShipNcl ncl: shipNclList) {
	    				  if(tempBayNo.equals(ncl.getBayNo())) {
	    					  teml.add(ncl);
	    					  rowNo.setShipNcl(teml);
	    				  }
	    			  }
	    			  if (!keyMap.containsKey(smp.getLayNo())) {
	    				  layNo= new VmsShipsDrawing();
	    				   List<VmsShipsDrawing> childrenList = new ArrayList<VmsShipsDrawing>();
	    				  layNo.setLayNo(smp.getLayNo());
	    				  layNo.setD(childrenList);
	    				  keyMap.put(smp.getLayNo(), layNo);
//	    				  if() {
//	    					  
//	    				  }
	    			  }
	    			  keyMap.get(smp.getLayNo()).getD().add(rowNo);
	    			 
//	    			 layNoU.add(sd);
//	    			 shipDraw.setBayNo(bayNo);
	    		 }
	    	 }
//	    	 VmsShipsDrawing tt=	 (VmsShipsDrawing) keyMap.values();
	    	 List<VmsShipsDrawing> cc=new ArrayList<VmsShipsDrawing>();
	    	 for(VmsShipsDrawing tt :keyMap.values()) {
	    		 cc.add(tt);
	    	 }
	    	 shipDraw.setD(cc);
//	    	 shipDraw.setD(keyMap.values());
	    	 
	    	 
	    	 
	        return shipDraw;
		 
	}

	@Override
	public SSIPage getVmsShipsDrawingBayNo(Map<String, Object> params) {
		// TODO Auto-generated method stub
		Integer pageSize = params.get("pageSize") == null ? 10 : (Integer) params.get("pageSize");
		Integer pageIndex = params.get("pageIndex") == null ? 1 : (Integer) params.get("pageIndex");
		params.put("shipCod", (String) params.get("shipCod"));
		params.put("bayNo", (String) params.get("bayNo"));
		IPage<VmsShipsDrawing> page = new Page<>(pageIndex, pageSize);
		page=  this.baseMapper.queryShipDrawByCode(page,params);

		List<VmsShipsDrawing> records = page.getRecords();
		if(records != null && !records.isEmpty()){
			//获取需作业的贝位列表, 将偶数贝 减一 调整为 奇数贝
			Map<String, String> workBayIdMap = getWorkBayIdMap(String.valueOf(params.get("voyageNo")));
			//更新船图贝位状态信息
			records.forEach(record ->{	
				String bayNo = workBayIdMap.get(record.getBayNo());
				if(bayNo != null){
					record.setWorkStatus(1);
					record.setBayNo(bayNo);
				}else{
					record.setWorkStatus(0);
				}
			});
		}
		return new SSIPage(page);
	}

	private Map<String, String> getWorkBayIdMap(String voyageNo){
		List<String> workBays = this.baseMapper.queryShipWorkBay(voyageNo);
		Map<String, String> workBayIdMap = new HashMap<>();
		String bayId;
		//获取需作业的贝位列表, 将偶数贝 减一 调整为 奇数贝
		for (String workBay : workBays) {
			try {
				int bayNum = Integer.parseInt(workBay);
				if(bayNum % 2 == 0){
					//偶数贝，大箱,
					bayNum = bayNum - 1;
					bayId = String.format("%03d", bayNum);
					workBayIdMap.put(bayId, String.format("%s:%s", bayId, workBay));
				}else if(!workBayIdMap.containsKey(workBay)){
					//避免顺序问题，导致奇数贝覆盖了偶数贝信息
					workBayIdMap.put(workBay, workBay);
				}
			} catch (NumberFormatException e) {
				log.error(String.format("贝位编号转整数失败:%s", workBay), e);
			}
		}
		return workBayIdMap;
	}

	private String[] parseBayId(String bayIdStr){
		String[] bayId = new String[2];
		if(StringUtils.isNotBlank(bayIdStr)){
			String[] bayIds = bayIdStr.split(":");
			if(bayIds.length > 1){
				bayId = bayIds;
			}else{
				bayId[0] = bayIds[0];
				bayId[1] = bayIds[0];
			}
		}
		return bayId;
	}
	
 
	@Override
	public List<VmsCable> getCableList() {
		// TODO Auto-generated method stub
		 List<VmsCable> cablist=this.baseMapper.cableList(); // 根据船号和贝号 获取所有的箱体信息
		return cablist;
	}
	@Override
	public List<VmsShipInfo> getShipInfoCoor() {
		// TODO Auto-generated method stub
		 List<VmsShipInfo> cablist=this.baseMapper.queryshipInfoCoor(); // 根据船号和贝号 获取所有的箱体信息
		List<CargoShipConfigDto> cargoShipConfigDtos = cargoShipConfigInfoService.queryOnWorkingList();
		List<String> defaultConfigs = cargoShipConfigDtos.stream().map(CargoShipConfigDto::getName).collect(Collectors.toList());
		cablist.parallelStream().forEach(shipInfoCoor -> {
			shipInfoCoor.setHasConfig(0);
			if(defaultConfigs.contains(shipInfoCoor.getShipNam())){
				shipInfoCoor.setHasConfig(1);
			}
		});
		return cablist;
	}
    @Override
    public SSIPage queryPage(Map<String, Object> params) {

        Integer pageSize = params.get("pageSize") == null ? 10 : (Integer) params.get("pageSize");
        Integer pageIndex = params.get("pageIndex") == null ? 1 : (Integer) params.get("pageIndex");
        params.put("voyageNo", (String) params.get("voyageNo"));
        params.put("shipCod", (String) params.get("shipCod"));
        IPage<VmsShipInfo> page = new Page<>(pageIndex, pageSize);
        page = this.getBaseMapper().queryshipInfoList(page, params);
        return new SSIPage(page);
    }

	@Override
	public SSIResponse<?> saveShipTx(DcvAllowGoShipLaneTxVo vo) {
		// TODO Auto-generated method stub
		List<DcvAllowGoShipLaneTx> tempList=vo.getShipList();
		   JSONObject curUser = AuthorizationUtils.getCurUser();
//        Integer userId = (Integer) curUser.get("userId");
		   String username ="";
		   if(null!=curUser) {
			     username = (String) curUser.get("username");
		   }
      
        List<String> t=new ArrayList<String>();
        for(DcvAllowGoShipLaneTx temp : tempList) {
        	temp.setRecUser(username);
        	temp.getCntr();
        	t.add(temp.getCntr());
        	VmsShipNcl tempncl=	this.getBaseMapper().queryShipNclByCntr(temp.getCntr());
        	temp.setBayNo(tempncl.getBayNo());
        }
//      List<VmsShipNcl> tempNcl= this.getBaseMapper().queryShipNclByCntr(t);
//      tempList=new ArrayList<DcvAllowGoShipLaneTx>();
//      for(VmsShipNcl c : tempNcl ) {
//    	  DcvAllowGoShipLaneTx tx=new DcvAllowGoShipLaneTx();
//    	  tx.setAllowFlag(list.get(0).getAllowFlag());
//    	  tx.setRecUser(username);
//    	  tx.setCntr(c.getCntr());
//    	  tx.setBayNo(c.getBayNo());
//    	  tempList.add(tx);
//      }
		int result = 0;
		if(Objects.nonNull(vo.getDeleteItem())&&1 == vo.getDeleteItem()){//删除对应船贝位设置
			this.baseMapper.deleteBatch(tempList);
		}else{
			result=  this.getBaseMapper().saveBatch(tempList);
		}
//		int result=this.getBaseMapper().updateShipTx(list);
//		if(result!=1) {
//			result=this.getBaseMapper().saveShipTx(list);
//		}
		 
		
        return SSIResponse.ok(result);
	}



	@Override
	public Map getPileInfo(String portCode) {
		// TODO Auto-generated method stub
		Map map=  this.baseMapper.getPileInfo(portCode);
		return map;
	}

	@Override
	public String getShipWorkingBay(String voyageNo, String portCode) {
		// TODO Auto-generated method stub
		String bayNo =  this.baseMapper.getShipWorkingBay(voyageNo, portCode);
		return bayNo;
	}
}
