package com.ssi.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ssi.constant.VehicleConstant;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.dto.VmsVehicleDto;
import com.ssi.entity.vo.VideoMonitorParams;
import com.ssi.mapper.VmsTerminalMapper;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.mqtt.MQTTPublishClient;
import com.ssi.response.AccessResponse;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import org.apache.commons.lang3.StringUtils;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ssi.utils.QueryUtils;
import com.ssi.entity.VmsTerminal;
import com.ssi.service.VmsTerminalService;
import org.springframework.web.client.RestTemplate;

/**
 * 终端服务实现类
 */
@Service("terminalService")
public class VmsTerminalServiceImpl extends ServiceImpl<VmsTerminalMapper, VmsTerminal> implements VmsTerminalService {

    @Value("${command-url}")
    private String commandUrl;
    @Value("${mqtt.address}")
    private String mqttAddress;
    @Value("${mqtt.serverId}")
    private String mqttServerId;
    @Value("${mqtt.name}")
    private String mqttName;
    @Value("${mqtt.password}")
    private String mqttPassword;
    @Value("${mqtt.url}")
    private String mqttUrl;
    @Value("${mqtt.CAPath}")
    private String mqttCAPath;

    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private VmsVehicleMapper vmsVehicleMapper;

    /**
     * 终端分页数据
     * @param params
     * @return
     */
    @Override
    public SSIPage queryPage(Map<String, Object> params) {
        String terminalNum = (String) params.get("terminalNum");

        IPage<VmsTerminal> page = this.page(
                new QueryUtils<VmsTerminal>().getPage(params),
                new LambdaQueryWrapper<VmsTerminal>().like(StringUtils.isNotBlank(terminalNum), VmsTerminal::getTerminalNum, terminalNum)
                        .orderByDesc(VmsTerminal::getCreateTime)

        );

        return new SSIPage(page);
    }

    @Override
    public SSIResponse saveTerminal(VmsTerminal vmsTerminal) {
        Map<String, Object> repeatMap = checkRepeat(vmsTerminal);
        if ((Boolean) repeatMap.get("repeat")) {
            return SSIResponse.no((String) repeatMap.get("repeatInfo"));
        }
        this.saveOrUpdate(vmsTerminal);
        return SSIResponse.ok();
    }

    private Map<String, Object> checkRepeat(VmsTerminal vmsTerminal) {
        Map<String, Object> repeatMap = Maps.newHashMap();
        repeatMap.put("repeat", true);

        List<VmsTerminal> list = this.list();
        Map<String, VmsTerminal> simMap = Maps.newHashMap();
        Map<String, VmsTerminal> tNumMap = Maps.newHashMap();
        Map<String, VmsTerminal> tTypeMap = Maps.newHashMap();
        Map<String, VmsTerminal> iccidMap = Maps.newHashMap();
        list.parallelStream().forEach(vmsTerminal1 -> {
            simMap.put(vmsTerminal1.getSim(), vmsTerminal1);
            tNumMap.put(vmsTerminal1.getTerminalNum(), vmsTerminal1);
            tTypeMap.put(vmsTerminal1.getTerminalType(), vmsTerminal1);
            iccidMap.put(vmsTerminal1.getIccid(), vmsTerminal1);
        });
        VmsTerminal repeatSim = simMap.get(vmsTerminal.getSim());
        VmsTerminal repeatTnum = tNumMap.get(vmsTerminal.getTerminalNum());
        VmsTerminal repeatTtype = tTypeMap.get(vmsTerminal.getTerminalType());
        VmsTerminal repeatIccid = iccidMap.get(vmsTerminal.getIccid());
        if (repeatSim != null) {
            if (vmsTerminal.getId() == null || (vmsTerminal.getId() != null && vmsTerminal.getId() != repeatSim.getId())) {
                repeatMap.put("repeatInfo", "sim卡号不能重复");
                return repeatMap;
            }
        }
        if (repeatTnum != null) {
            if (vmsTerminal.getId() == null || (vmsTerminal.getId() != null && vmsTerminal.getId() != repeatTnum.getId())) {
                repeatMap.put("repeatInfo", "终端编号不能重复");
                return repeatMap;
            }
        }
        if (repeatTtype != null) {
            if (vmsTerminal.getId() == null || (vmsTerminal.getId() != null && vmsTerminal.getId() != repeatTtype.getId())) {
                repeatMap.put("repeatInfo", "终端型号不能重复");
                return repeatMap;
            }
        }
        if (repeatIccid != null) {
            if (vmsTerminal.getId() == null || (vmsTerminal.getId() != null && vmsTerminal.getId() != repeatIccid.getId())) {
                repeatMap.put("repeatInfo", "iccid不能重复");
                return repeatMap;
            }
        }
        repeatMap.put("repeat", false);
        return repeatMap;
    }

    @Override
    public SSIResponse videoMonitor(String vin) {

        VmsVehicleDto vehicle = vmsVehicleMapper.queryVehicleByVin(vin);
        if(vehicle == null){
            return SSIResponse.no("车辆不存在");
        }
        //发起mqtt请求，请求后视摄像头
        MQTTPublishClient mqttClientSend = new MQTTPublishClient(mqttAddress, mqttServerId,mqttCAPath,mqttName,mqttPassword,mqttUrl);
        mqttClientSend.mqttRequest(vehicle, VehicleConstant.START_PUSHER);

        String videoTerminalNum = vehicle.getVideoTerminalNum();
        VmsTerminal vmsTerminal = this.lambdaQuery().eq(VmsTerminal::getTerminalNum, videoTerminalNum).one();
        if(vmsTerminal != null){
            List<Object> result = Lists.newArrayList();
            String sim = vmsTerminal.getSim();
            VideoMonitorParams.VideoMonitorParamsBuilder builder = VideoMonitorParams.builder().sim(sim).dataType(1).controlType(0).hdType(0);
            for (int i = 0; i < 3; i++) {
                VideoMonitorParams params = builder.channel(i + 1).build();
                AccessResponse accessResponse = restTemplate.postForObject(commandUrl + "/media/9101", params, AccessResponse.class);
                if(accessResponse.isSuccess()){
                    Map data = (Map) accessResponse.getData();
                    data.put("channel",i + 1);
                    result.add(data);
                }else{
                    Map data = new HashMap();
                    data.put("result",0);
                    result.add(data);
                }
            }
            return SSIResponse.ok(result);
        }else{
            return SSIResponse.no("车辆视频卡终端不在线");
        }
    }

}
