package com.ssi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.entity.VmsTosOrdersAnalysis;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.vo.TaskAnalysisVo;
import com.ssi.mapper.VmsTosOrdersAnalysisMapper;
import com.ssi.mapper.VmsTosOrdersMapper;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.service.VmsTosOrdersAnalysisService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.util.List;

/**
 * 任务分析实现类
 */
@Service("vmsTosOrdersAnalysisService")
public class VmsTosOrdersAnalysisServiceImpl extends ServiceImpl<VmsTosOrdersAnalysisMapper, VmsTosOrdersAnalysis> implements VmsTosOrdersAnalysisService {

    @Autowired
    private VmsTosOrdersMapper vmsTosOrdersMapper;
    @Autowired
    private VmsVehicleMapper vmsVehicleMapper;

    @Override
    public IPage<VmsTosOrdersAnalysis> queryPage(int current, int size, VmsTosOrdersAnalysis vmsTosOrdersAnalysis) {
        IPage<VmsTosOrdersAnalysis> page = new Page<>(current, size);
        QueryWrapper<VmsTosOrdersAnalysis> wrapper = new QueryWrapper<>();
        return this.page(page, wrapper);
    }

    /**
     * 根据时间段执行任务分析
     * @param startTime
     * @param endTime
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void runTosAnalysisTask(LocalDateTime startTime, LocalDateTime endTime) {
        Long startTimeStamp = startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        List<TaskAnalysisVo> taskNumTimeByDay = vmsTosOrdersMapper.getTaskNumTimeByDay(startTime, endTime,startTimeStamp);
        LocalDate startDate = startTime.toLocalDate();
        LocalDate endDate = endTime.toLocalDate();
        List<VmsVehicle> vehicleList = vmsVehicleMapper.selectList(new LambdaQueryWrapper<VmsVehicle>()
                .eq(VmsVehicle::getStatus, 0));
        for (LocalDate ld = startDate; ld.compareTo(endDate) <= 0; ld = ld.plusDays(1)) {
            LocalDate ld1 = ld;
            lambdaUpdate().set(VmsTosOrdersAnalysis::getDeleted, 1)
                    .eq(VmsTosOrdersAnalysis::getAnalysisDate, ld1)
                    .update();
            for (int i = 0; i < vehicleList.size(); i++) {
                VmsVehicle v = vehicleList.get(i);
                String vin = v.getVin();
                boolean present = taskNumTimeByDay.stream().anyMatch(vo -> vo.getVin().equals(vin) && ld1.isEqual(vo.getDate()));
                if (!present) {
                    VmsTosOrdersAnalysis analysis = new VmsTosOrdersAnalysis();
                    analysis.setAnalysisDate(ld);
                    analysis.setVin(vin);
                    getBaseMapper().insert(analysis);
                }
            }
        }
        if (CollectionUtils.isNotEmpty(taskNumTimeByDay)) {
            taskNumTimeByDay.forEach(this::accept);
        }
    }

    /**
     * 重新计算任务数据
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void reAnalysis(LocalDate date) {
        LocalDateTime startTime = LocalDateTime.of(date, LocalTime.MIN);
        Long startTimeStamp = startTime.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        LocalDateTime endTime = LocalDateTime.of(date, LocalTime.MAX);
        List<TaskAnalysisVo> taskNumTimeByDay = vmsTosOrdersMapper.getTaskNumTimeByDay(startTime, endTime,startTimeStamp);
        if (CollectionUtils.isNotEmpty(taskNumTimeByDay)) {
            //原始数据未删除的情况下，可以进行重新分析
            lambdaUpdate().set(VmsTosOrdersAnalysis::getDeleted, 1)
                    .eq(VmsTosOrdersAnalysis::getAnalysisDate, date)
                    .update();
            List<VmsVehicle> vehicleList = vmsVehicleMapper.selectList(new LambdaQueryWrapper<VmsVehicle>()
                    .eq(VmsVehicle::getStatus, 0));
            for (int i = 0; i < vehicleList.size(); i++) {
                String vin = vehicleList.get(i).getVin();
                boolean present = taskNumTimeByDay.stream().anyMatch(vo -> vo.getVin().equals(vin));
                if (!present) {
                    VmsTosOrdersAnalysis analysis = new VmsTosOrdersAnalysis();
                    analysis.setAnalysisDate(date);
                    analysis.setVin(vin);
                    getBaseMapper().insert(analysis);
                }
            }
            taskNumTimeByDay.forEach(this::accept);
        }
    }

    private void accept(TaskAnalysisVo dayHourTrend) {
        VmsTosOrdersAnalysis analysis = new VmsTosOrdersAnalysis();
        analysis.setAnalysisDate(dayHourTrend.getDate());
        analysis.setVin(dayHourTrend.getVin());
        analysis.setLoadContainerNum(dayHourTrend.getLoadContainerNum());
        analysis.setLoadContainerTime(dayHourTrend.getLoadContainerTime());
        analysis.setUnloadContainerNum(dayHourTrend.getUnloadContainerNum());
        analysis.setUnloadContainerTime(dayHourTrend.getUnloadContainerTime());
        analysis.setHandleContainerNum(dayHourTrend.getHandleContainerNum());
        analysis.setHandleContainerTime(dayHourTrend.getHandleContainerTime());
        getBaseMapper().insert(analysis);
    }
}
