package com.ssi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.constant.enums.VmsTosOrderStatusEnum;
import com.ssi.entity.VmsTosOrders;
import com.ssi.entity.VmsTosOrdersOee;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.vo.TaskKPIVo;
import com.ssi.entity.vo.TaskStatusHistory;
import com.ssi.entity.vo.VmsTosOrdersKPIVo;
import com.ssi.mapper.VmsTosOrdersOeeMapper;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.model.VehicleElasticSearchModel;
import com.ssi.service.VmsTosOrdersOeeService;
import com.ssi.service.VmsTosOrdersService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.*;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;

/**
 * OEE分析任务实现类
 */
@Service("vmsTosOrdersOeeService")
@Slf4j
public class VmsTosOrdersOeeServiceImpl extends ServiceImpl<VmsTosOrdersOeeMapper, VmsTosOrdersOee> implements VmsTosOrdersOeeService {

    @Autowired
    private VmsTosOrdersService vmsTosOrdersService;
    @Autowired
    private VehicleElasticSearchModel vehicleElasticSearchModel;
    @Autowired
    private VmsVehicleMapper vmsVehicleMapper;

    /**
     * 根据时间段执行OEE分析
     * @param startTime
     * @param endTime
     */
    @Override
    public void runTosOeeTask(LocalDateTime startTime, LocalDateTime endTime) {
        List<VmsTosOrders> list = vmsTosOrdersService.lambdaQuery()
                .eq(VmsTosOrders::getDeleted, 0)
                .in(VmsTosOrders::getStatus, VmsTosOrderStatusEnum.FINISH_CONFIRM.getCode(),VmsTosOrderStatusEnum.ARRIVED_DESTINATION.getCode(),
                        VmsTosOrderStatusEnum.VEHICLE_LOCKUP.getCode(),VmsTosOrderStatusEnum.FINISH_CRANE_CONFIRM.getCode(),
                        VmsTosOrderStatusEnum.FINISH_DRIVER_CONFIRM.getCode())
                .in(VmsTosOrders::getVehicleTaskLabel, 1, 2, 3)
                .ge(VmsTosOrders::getCollectTime, Date.from(startTime.atZone(ZoneId.systemDefault()).toInstant()).getTime())
                .lt(VmsTosOrders::getCollectTime, Date.from(endTime.atZone(ZoneId.systemDefault()).toInstant()).getTime())
                .list();
        if (CollectionUtils.isNotEmpty(list)) {
            list.stream().forEach(vmsTosOrders -> {
                try {
                    String taskNo = vmsTosOrders.getTaskNo();
                    Integer count = this.lambdaQuery()
                            .eq(VmsTosOrdersOee::getTaskNo, taskNo)
                            .eq(VmsTosOrdersOee::getDeleted, 0)
                            .ge(VmsTosOrdersOee::getCollectTime, Date.from(startTime.atZone(ZoneId.systemDefault()).toInstant()).getTime())
                            .lt(VmsTosOrdersOee::getCollectTime, Date.from(endTime.atZone(ZoneId.systemDefault()).toInstant()).getTime())
                            .count();
                    if (count == 0) {
                        VmsTosOrdersOee vmsTosOrdersOee = new VmsTosOrdersOee();
                        vmsTosOrders.setId(null);
                        BeanUtils.copyProperties(vmsTosOrdersOee, vmsTosOrders);
                        String vin = vmsTosOrders.getVin();
                        VmsVehicle vmsVehicle = vmsVehicleMapper.selectOne(new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getVin, vin));
                        vmsTosOrdersOee.setVehicleNum(vmsVehicle.getVehicleNum());
                        int year = startTime.getYear();
                        List<TaskStatusHistory> taskStatusHistoryList = vehicleElasticSearchModel.searchTaskHistory(taskNo, String.valueOf(year));
                        Long collectTime = vmsTosOrders.getCollectTime();
                        Long lastCollectTime = collectTime;
                        for (int j = 0; j < taskStatusHistoryList.size(); j++) {
                            Integer status = taskStatusHistoryList.get(j).getStatus();
                            Long currentCollectTime = taskStatusHistoryList.get(j).getCollectTime();
                            if (VmsTosOrderStatusEnum.OBU_RECEIVED.getCode() == status) {//车辆接受 43;
                                //指令等待时间 =  车辆接收时间 - collect_time
                                vmsTosOrdersOee.setInstructionWaitingTime(calTime(lastCollectTime, currentCollectTime));
                                lastCollectTime = currentCollectTime;
                            }
                            if (VmsTosOrderStatusEnum.START_TASK.getCode() == status) {//车辆开始行驶 65;
                                //作业准备时间 = 任务开始(开始行驶)-车辆接受
                                vmsTosOrdersOee.setHomeworkPreparationTime(calTime(lastCollectTime, currentCollectTime));
                                lastCollectTime = currentCollectTime;
                            }
                            if (VmsTosOrderStatusEnum.ARRIVED_DESTINATION.getCode() == status) {//到达目的地 3;
                                //移动到取箱点时间 = 到达目的地或者车辆停稳锁死-任务开始或车辆接受时间（任务开始时间为空）
                                vmsTosOrdersOee.setMoveTheBoxTime(calTime(lastCollectTime, currentCollectTime));
                                lastCollectTime = currentCollectTime;
                            }
                            if (VmsTosOrderStatusEnum.VEHICLE_LOCKUP.getCode() == status) {//车辆停稳锁死 49;
                                if (vmsTosOrdersOee.getMoveTheBoxTime() == null) {
                                    vmsTosOrdersOee.setMoveTheBoxTime(calTime(lastCollectTime, currentCollectTime));
                                }
                                lastCollectTime = currentCollectTime;
                            }
                            if (VmsTosOrderStatusEnum.FINISH_CONFIRM.getCode() == status) {//三重确认完成 37;
                                //取箱点等待时间 = 三重确认完成-到达目的地或者车辆停稳锁死
                                vmsTosOrdersOee.setPickingTime(calTime(lastCollectTime, currentCollectTime));
                            }
                        }
                        this.save(vmsTosOrdersOee);
                    }
                } catch (Exception e) {
                    log.error("runTosOeeTask出错,vmsTosOrders.TaskNo:" + vmsTosOrders.getTaskNo(), e);
                }
            });
        }
    }

    /**
     * 任务KPI-效率OEE分析
     */
    @Override
    public List taskOee(VmsTosOrdersKPIVo vmsTosOrders) {
        if (vmsTosOrders.getStartTime() == null && vmsTosOrders.getEndTime() == null) {
            LocalDateTime startTime = LocalDateTime.of(LocalDateTime.now().minus(3, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
            vmsTosOrders.setStartTime(startTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli());
            vmsTosOrders.setEndTime(endTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli());
        }
        List<TaskKPIVo> taskKPIVos = this.getBaseMapper().taskOee(vmsTosOrders);
        return taskKPIVos;
    }

    private long calTime(Long startDate, Long endDate) {
        if (startDate == null || endDate == null) {
            return 0;
        }
        long diff = endDate - startDate;
        return diff > 0 ? diff : 0;
    }
}
