package com.ssi.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.export.ExcelBatchExportService;
import cn.afterturn.easypoi.handler.impl.ExcelDataHandlerDefaultImpl;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alicp.jetcache.anno.CacheRefresh;
import com.alicp.jetcache.anno.Cached;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.ssi.constant.VehicleConstant;
import com.ssi.constant.enums.VmsTosOrderStatusEnum;
import com.ssi.entity.*;
import com.ssi.entity.dto.VehicleTerminalData;
import com.ssi.entity.dto.VmsTosOrdersDto;
import com.ssi.entity.dto.kpi.VehicleDateStageDto;
import com.ssi.entity.dto.kpi.VmsTosOrdersKpiDto;
import com.ssi.entity.vo.*;
import com.ssi.mapper.*;
import com.ssi.model.RedisDataModel;
import com.ssi.model.VehicleElasticSearchModel;
import com.ssi.response.AccessResponse;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsCranePadBindService;
import com.ssi.service.VehicleTroubleService;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.elasticsearch.action.search.*;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.Scroll;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.service.VmsTosOrdersService;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 任务服务实现类
 */
@Service("vmsTosOrdersService")
public class VmsTosOrdersServiceImpl extends ServiceImpl<VmsTosOrdersMapper, VmsTosOrders> implements VmsTosOrdersService {
    private static Logger log = LoggerFactory.getLogger(VmsTosOrdersServiceImpl.class);

    @Autowired
    VmsTaskJourneyInfoMapper taskJourneyInfoMapper;
    @Autowired
    private VehicleTroubleService vehicleTroubleService;
    @Autowired
    private RestHighLevelClient restHighLevelClient;
    @Autowired
    private VmsCranePadBindService vmsCranePadBindService;
    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private RedisDataModel redisDataModel;
    @Autowired
    private VehicleTroubleHistoryMapper vehicleTroubleHistoryMapper;
    @Autowired
    private VehicleElasticSearchModel vehicleElasticSearchModel;
    @Autowired
    private VehicleDataUtil vehicleDataUtil;
    @Autowired
    private VmsTosOrdersAnalysisMapper vmsTosOrdersAnalysisMapper;
    @Autowired
    private VmsTosOrdersMapper vmsTosOrdersMapper;
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private VehicleRemoteInstructionMapper vehicleRemoteInstructionMapper;

    @Value("${analysis.vehicle-data.es-index-name}")
    private String esVehicleDataIndexName;
    @Value("${analysis.vehicle-data.index-rule:year}")
    private String indexRule;
    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;
    @Value("${vehicle.cache.key:ivccs:vms:vehicle:cache}")
    private String vehicleCacheKey;
    @Value("${command-url}")
    private String commandUrl;
    private ThreadLocal<VehicleDateStageDto> threadStage = new ThreadLocal<VehicleDateStageDto>();


    @Override
    public Map<String, VmsTosOrders> getAllVehicleLatestWork() {
        Map<String, VmsTosOrders> resultMap = Maps.newHashMap();
        List<VmsTosOrders> list = this.list();
        list.parallelStream().forEach(vmsTosOrders -> {
            VmsTosOrders tempOrders = resultMap.get(vmsTosOrders.getVin());
            if (tempOrders == null || tempOrders.getReceiveTime().getTime() < vmsTosOrders.getReceiveTime().getTime()) {
                resultMap.put(vmsTosOrders.getVin(), vmsTosOrders);
            }
        });
        return resultMap;
    }

    /**
     * 获取车辆当前任务
     * @param vin 车辆vin
     */
    @Override
    public Map<String, Object> getOrdersByVin(String vin) {
        Map<String, Object> resultMap = Maps.newHashMap();

        Map<String, Object> json2Map = redisDataModel.getJson2Map(String.format("%s:%s", latestOrderKeyPrefix, vin));
        JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(json2Map));
        VmsTosOrders vmsTosOrders = JSONObject.toJavaObject(jsonObject, VmsTosOrders.class);

        if (vmsTosOrders != null) {
            resultMap.put("vmsTosOrders", vmsTosOrders);
            resultMap.put("workStatus", !VehicleUtil.isTaskOver(vmsTosOrders.getStatus()) ? 0 : 1);
//            resultMap.put("taskAssociation", vmsTosOrders.getTaskAssociation());
            resultMap.put("taskType", vmsTosOrders.getTaskType());
            if (!VehicleUtil.isTaskOver(vmsTosOrders.getStatus())) {
                if (!(vmsTosOrders.getStartTime() == null)) {
                    resultMap.put("workTime", System.currentTimeMillis() - vmsTosOrders.getStartTime().getTime());
                }
            }
        } else {
            resultMap.put("workStatus", 1);
        }
        return resultMap;
    }

    /**
     * 车辆行程查询
     * @param vin 车辆vin
     * @param startTime 查询起始时间
     * @param stopTime 查询截止时间
     * @param faultGrade 故障等级
     */
    @Override
    public SSIResponse getVehicleTrip(String vin, Long startTime, Long stopTime, String faultGrade) {
        Date startDate = startTime == null ? null : new Date(startTime);
        Date stopDate = stopTime == null ? null : new Date(stopTime);

        List<VmsTosOrders> list = baseMapper.getVehicleTrip(vin, startDate, stopDate);
        Map<String, Object> params = new HashMap<>();
        params.put("startTime", startDate);
        params.put("stopTime", stopDate);
        params.put("faultGrade", faultGrade);
        List<String> vins = new ArrayList<>();
        vins.add(vin);
        params.put("vins", vins);
        List<VmsVehicleAlertHistoryVo> vehicleAlertHistoryList = vehicleTroubleHistoryMapper.getAlertHistoryList(params);

        //异常过滤任务
        List<VmsTosOrders> resultList = filterOrders(list, faultGrade, vehicleAlertHistoryList);

        //查询里程能耗信息
        resultList.parallelStream().forEach(vmsTosOrders -> {
            VmsTaskJourneyInfo vmsTaskJourneyInfo = taskJourneyInfoMapper.selectOne(new LambdaQueryWrapper<VmsTaskJourneyInfo>()
                    .eq(VmsTaskJourneyInfo::getTaskId, vmsTosOrders.getTaskNo()));
            changeUnit(vmsTaskJourneyInfo);
            //计算百公里能耗
            calcHundredEnergy
                    (vmsTaskJourneyInfo);
            vmsTosOrders.setVmsTaskJourneyInfo(vmsTaskJourneyInfo);
        });
        return SSIResponse.ok(resultList);
    }

    private void changeUnit(VmsTaskJourneyInfo vmsTaskJourneyInfo) {
        if (vmsTaskJourneyInfo == null) {
            return;
        }
        //mile单位已经是千米了，不用再除以1000
        /*if(vmsTaskJourneyInfo.getMile() != null){
            vmsTaskJourneyInfo.setMile(vmsTaskJourneyInfo.getMile().divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP));
        }*/
        if (vmsTaskJourneyInfo.getTimeConsuming() != null) {
            BigDecimal timeSeconds = new BigDecimal(vmsTaskJourneyInfo.getTimeConsuming()).divide(new BigDecimal(1000), 0, BigDecimal.ROUND_CEILING);
            vmsTaskJourneyInfo.setTime(formatTime(timeSeconds));
        }
    }

    private List<VmsTosOrders> filterOrders(List<VmsTosOrders> list, String faultGrade, List<VmsVehicleAlertHistoryVo> alertList) {
        //时长一分钟以内的任务过滤掉
        list = list.stream().filter(vmsTosOrders -> {
            Date startTime = vmsTosOrders.getStartTime();
            Date endTime = vmsTosOrders.getEndTime();
            if (startTime != null && endTime != null) {
                return vmsTosOrders.getEndTime().getTime() - vmsTosOrders.getStartTime().getTime() > 60000;
            }
            return true;
        }).collect(Collectors.toList());
        if (StringUtils.isBlank(faultGrade)) {
            return list;
        } else {
            if (alertList == null || alertList.isEmpty()) {
                return new ArrayList<VmsTosOrders>();
            }
        }
        List<VmsTosOrders> resultList = Lists.newArrayList();
        list.stream().forEach(vmsTosOrders -> {
            if (vmsTosOrders.getStartTime() == null || vmsTosOrders.getEndTime() == null) {
                return;
            }
            if (vmsTosOrders.getStartTime().getTime() > alertList.get(0).getAlarmTime().getTime()) {
                //任务结束时间大于最大异常开始时间
            } else if (vmsTosOrders.getEndTime().getTime() < alertList.get(alertList.size() - 1).getAlarmTime().getTime()) {
                //任务结束时间小于最小异常开始时间
            } else {
                resultList.add(vmsTosOrders);
            }
        });
        return resultList;
    }

    /**
     * 根据时间查询车辆能耗里程
     */
    @Override
    public SSIResponse getMileTimeByTime(String vin, Long startTime, Long stopTime) {
        VmsTaskJourneyInfo vmsTaskJourneyInfo = execAnalysisForVmsTosOrders(vin, startTime, stopTime);
        return SSIResponse.ok(vmsTaskJourneyInfo);
    }

    private VmsTaskJourneyInfo execAnalysisForVmsTosOrders(String vin, Long startTime, Long stopTime) {
        VmsTaskJourneyInfo vmsTaskJourneyInfo = new VmsTaskJourneyInfo();
        try {
            vmsTaskJourneyInfo.setMileVal(0D);
            vmsTaskJourneyInfo.setEnergyConsumingVal(0D);
            vmsTaskJourneyInfo.setTimeConsuming(0L);
            calc(vmsTaskJourneyInfo, vin, startTime, stopTime);
            //单位转换
            changeUnit(vmsTaskJourneyInfo);

            vmsTaskJourneyInfo.setMile(new BigDecimal(vmsTaskJourneyInfo.getMileVal()).setScale(2, BigDecimal.ROUND_UP));
            vmsTaskJourneyInfo.setEnergyConsuming(new BigDecimal(vmsTaskJourneyInfo.getEnergyConsumingVal()).setScale(2,
                    BigDecimal.ROUND_HALF_UP));

            calcHundredEnergy(vmsTaskJourneyInfo);

        } catch (Exception e) {
            log.error("Fail to analysis vmstoorders.", e);
        }
        return vmsTaskJourneyInfo;
    }

    private void calc(VmsTaskJourneyInfo vmsTaskJourneyInfo, String vin, Long startTime, Long stopTime) throws IOException {
        Gson gson = new GsonBuilder().create();
        int size = 10000;
        final Scroll scroll = new Scroll(TimeValue.timeValueMinutes(10L));
        SearchResponse response = getSearchResponse(vin, startTime, stopTime, size, scroll);
        String scrollId = response.getScrollId();
        SearchHit[] searchHits = response.getHits().getHits();
        VehicleTerminalData lastVehicleTerminalData = null;
        while (searchHits != null && searchHits.length > 0) {
            for (SearchHit hit : searchHits) {
                VehicleTerminalData currentVehicleTerminalData = null;
                String sourceAsString = hit.getSourceAsString();
                if (sourceAsString != null) {
                    currentVehicleTerminalData = parseVehicleTerminalDate(gson, sourceAsString);
                }

                if (lastVehicleTerminalData != null && currentVehicleTerminalData != null) {
                    long time = currentVehicleTerminalData.getCollectTime() - lastVehicleTerminalData.getCollectTime();
                    if (time <= 60000) {
                        calcMile(vmsTaskJourneyInfo, lastVehicleTerminalData, currentVehicleTerminalData);
                        caclTime(vmsTaskJourneyInfo, lastVehicleTerminalData, currentVehicleTerminalData);
                        calcEnergy(vmsTaskJourneyInfo, lastVehicleTerminalData, currentVehicleTerminalData);
                    }

                }

                lastVehicleTerminalData = currentVehicleTerminalData;
            }

            /* Next batch data. */
            if (searchHits.length == size) {
                response = getNextScrollResponse(scrollId, scroll);
                searchHits = response.getHits().getHits();
            } else {
                break;
            }
        }
        clearScroll(scrollId);
    }

    private void calcHundredEnergy(VmsTaskJourneyInfo vmsTaskJourneyInfo) {
        if (vmsTaskJourneyInfo != null && vmsTaskJourneyInfo.getEnergyConsuming() != null &&
                vmsTaskJourneyInfo.getMile() != null && vmsTaskJourneyInfo.getMile().doubleValue() > 0) {
            vmsTaskJourneyInfo.setHundredEnergyConsuming(vmsTaskJourneyInfo.getEnergyConsuming()
                    .divide(vmsTaskJourneyInfo.getMile(), 2, RoundingMode.HALF_UP).multiply(new BigDecimal(100)));
        }
    }

    private void calcMile(VmsTaskJourneyInfo vmsTaskJourneyInfo, VehicleTerminalData lastVehicleTerminalData,
                          VehicleTerminalData currentVehicleTerminalData) {
//        Double dist = GpsUtil.getDistance(lastVehicleTerminalData.getLatitude(),
//                lastVehicleTerminalData.getLongitude(), currentVehicleTerminalData.getLatitude(),
//                currentVehicleTerminalData.getLongitude());
//        if (dist > 0 && dist < 1 && currentVehicleTerminalData.getSpeed().compareTo(BigDecimal.ZERO)!=0) {
//            vmsTaskJourneyInfo.setMileVal(vmsTaskJourneyInfo.getMileVal() + dist);
//        }
        if (lastVehicleTerminalData.getSpeed().doubleValue() >= 0.0
                && currentVehicleTerminalData.getSpeed().doubleValue() >= 0.0) {
            double curSpeed = currentVehicleTerminalData.getSpeed().doubleValue();
            double lastSpeed = lastVehicleTerminalData.getSpeed().doubleValue();
            double avgSpeed = (lastSpeed + curSpeed) / 2;
            long time = currentVehicleTerminalData.getCollectTime() - lastVehicleTerminalData.getCollectTime();
            double mile = avgSpeed / 60 / 60 * (time / 1000);
            vmsTaskJourneyInfo.setMileVal(vmsTaskJourneyInfo.getMileVal() + mile);

        }
    }

    private void caclTime(VmsTaskJourneyInfo vmsTaskJourneyInfo, VehicleTerminalData lastVehicleTerminalData, VehicleTerminalData currentVehicleTerminalData) {
        long time = currentVehicleTerminalData.getCollectTime() - lastVehicleTerminalData.getCollectTime();
        vmsTaskJourneyInfo.setTimeConsuming(vmsTaskJourneyInfo.getTimeConsuming() + time);
    }

    private void calcEnergy(VmsTaskJourneyInfo vmsTaskJourneyInfo, VehicleTerminalData lastVehicleTerminalData,
                            VehicleTerminalData currentVehicleTerminalData) {
        double energyConsuming = calcEnergy(lastVehicleTerminalData, currentVehicleTerminalData);
        vmsTaskJourneyInfo.setEnergyConsumingVal(energyConsuming + vmsTaskJourneyInfo.getEnergyConsumingVal());
    }

    public static double calcEnergy(VehicleTerminalData lastVehicleTerminalData,
                                    VehicleTerminalData currentVehicleTerminalData) {
        double totalElectricity = lastVehicleTerminalData.getTotalElectricity();
        double totalVoltage = lastVehicleTerminalData.getTotalVoltage();
        if (totalElectricity == -1 || totalVoltage == -1) {
            return 0;
        }
        return Math.abs((totalElectricity * totalVoltage) / 1000 *
                (Double.parseDouble(String.valueOf(currentVehicleTerminalData.getCollectTime()
                        - lastVehicleTerminalData.getCollectTime())) / (1000 * 60 * 60)));
    }

    private VehicleTerminalData parseVehicleTerminalDate(Gson gson, String jsonStr) {
        try {
            return gson.fromJson(jsonStr, VehicleTerminalData.class);
        } catch (Throwable t) {
            log.error("[Analysis Executor]Fail to convert VehicleTerminalDate vmstoorders.\n" +
                    jsonStr, t);
        }

        return null;
    }

    private SearchResponse getSearchResponse(String vin, Long startTime, Long stopTime, int size, Scroll scroll) throws IOException {
        BoolQueryBuilder boolBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        sourceBuilder.size(size);

        MatchQueryBuilder matchQueryBuilder = QueryBuilders.matchQuery("vin", vin);

        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("collectTime");
        rangeQueryBuilder.gte(startTime);
        rangeQueryBuilder.lte(stopTime);

        boolBuilder.must(rangeQueryBuilder);
        boolBuilder.must(matchQueryBuilder);

        FieldSortBuilder fieldSortBuilder = new FieldSortBuilder("collectTime");

        sourceBuilder.query(boolBuilder);
        sourceBuilder.sort(fieldSortBuilder);
        sourceBuilder.timeout(new TimeValue(600, TimeUnit.SECONDS));

        String[] indexs = ElasticSearchUtil.getIndexs(startTime, stopTime, esVehicleDataIndexName, 1, true, indexRule);
        SearchRequest searchRequest = new SearchRequest(indexs);
        searchRequest.scroll(scroll);
        searchRequest.source(sourceBuilder);

        return restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
    }

    private SearchResponse getNextScrollResponse(String scrollId, Scroll scroll) throws IOException {
        SearchScrollRequest scrollRequest = new SearchScrollRequest(scrollId);
        scrollRequest.scroll(scroll);

        return restHighLevelClient.scroll(scrollRequest, RequestOptions.DEFAULT);
    }

    private void clearScroll(String scrollId) throws IOException {
        ClearScrollRequest clearScrollRequest = new ClearScrollRequest();
        clearScrollRequest.addScrollId(scrollId);

        restHighLevelClient.clearScroll(clearScrollRequest, RequestOptions.DEFAULT);

    }

    @Override
    public SSIPage queryPage(VmsTosOrdersVo ordersVo) {
        String vehicleNum = ordersVo.getVehicleNum();
        if (StringUtils.isNotBlank(vehicleNum)) {
            VmsVehicle vehicle = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getVehicleNum, vehicleNum).one();
            if (vehicle == null) {
                return new SSIPage(new Page<>());
            }
            String vin = vehicle.getVin();
            ordersVo.setVin(vin);
        }
        IPage<VmsTosOrders> iPage = new Page<VmsTosOrders>(ordersVo.getPageIndex(), ordersVo.getPageSize());
        IPage<VmsTosOrders> page = baseMapper.pageQuery(iPage,ordersVo);
        List<VmsTosOrders> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return new SSIPage(page);
        }
        records.forEach(vmsTosOrders -> {
            try {
                vmsTosOrders.setStartTimeStamp(vmsTosOrders.getStartTime().getTime()/1000);
                String costTime = getCostTime(vmsTosOrders);
                vmsTosOrders.setCostTime(costTime);
                Map<String, Object> map = redisDataModel.hgetJson2Map(vehicleCacheKey, vmsTosOrders.getVin());
                vmsTosOrders.setVehicleNum((String) map.get("vehicleNum"));
                String description = vehicleDataUtil.generateTaskLocationDescription(vmsTosOrders.getTaskLocationType(), vmsTosOrders.getVehicleLocation());
                vmsTosOrders.setLocationDescription(description);
                //添加lockLabel
                Map<String, Object> orderMap = redisDataModel.getJson2Map(String.format("%s:%s", latestOrderKeyPrefix,  vmsTosOrders.getVin()));
                vmsTosOrders.setLockLabel(Objects.isNull(orderMap.get("lockLabel"))?null:(Integer)orderMap.get("lockLabel"));
            } catch (Exception e) {
                log.error("获取时间、车辆编号、地点描述异常:", e);
            }
        });
        return new SSIPage(page);
    }

    /**
     * 搬运集装箱的列表
     */
    @Override
    public SSIPage queryPageByAllStatus(VmsTosOrdersVo ordersVo) {
        //查询车辆任务数据
        List<VmsTosOrdersDto> ordersDtosList = vmsTosOrdersMapper.queryPageByAllStatus(ordersVo);

        IPage<VmsTosOrdersKpiDto> vmsPage = new Page<>(ordersVo.getPageIndex(), ordersVo.getPageSize());
        if (CollectionUtils.isEmpty(ordersDtosList)) {
            return new SSIPage(vmsPage);
        }
        //查询任务的所有状态信息
        List<String> taskNoList = ordersDtosList.stream().map(vmsTosOrders->vmsTosOrders.getTaskNo()).collect(Collectors.toList());
        String year = DateUtils.format(ordersVo.getQueryStartTime(), "yyyy");
        Map<String,List<TaskStatusHistory>> statusMap = vehicleElasticSearchModel.searchTaskHistory(taskNoList, year);
        //封装大任务方法
        List<VmsTosOrdersKpiDto> kpiOrderList = getVmsTosOrdersKpiDtos(ordersDtosList, statusMap);
        //将队列逆序
        Collections.reverse(kpiOrderList);
        vmsPage.setRecords(kpiOrderList);
//        threadStage.remove();
        return new SSIPage(vmsPage);
    }

    /**
     * 任务封装逻辑
     * @param ordersDtosList
     * @param statusMap
     * @return
     */
    public List<VmsTosOrdersKpiDto> getVmsTosOrdersKpiDtos(List<VmsTosOrdersDto> ordersDtosList, Map<String, List<TaskStatusHistory>> statusMap) {
        /**
         * 封装KPI数据
         */
        List<VmsTosOrdersKpiDto> kpiOrderList = new ArrayList<>();   //结果集
        VmsTosOrdersKpiDto kpiParentDto = new VmsTosOrdersKpiDto();  //大任务信息
        kpiParentDto.setCollectTime(ordersDtosList.get(0).getCollectTime());
        List<VmsTosOrdersDto> orderList = new ArrayList<>();    //大任务下子任务集合
        List<VehicleDateStageDto> dateList = new ArrayList<>();    //大任务下步骤时间
        VehicleDateStageDto dateStageDto = null;
        int startPortType = 0;
        int lastPortType = 0;//记录前一条记录的portType
        int lastVehicleTaskLabel = 0;//记录前一条记录的任务标签 装船or卸船
        int appControl = 0;
        for(int i =0 ;i < ordersDtosList.size();i++){
            VmsTosOrdersDto vmsTosOrdersDto = ordersDtosList.get(i);
            //装船任务，任务从堆场作业开始;卸船任务，任务从桥吊开始;搬移任务，从第一个PortCode开始
            if (vmsTosOrdersDto.getVehicleTaskLabel() == 1){//装船
                startPortType = 2;//起点为场吊
            }else if (vmsTosOrdersDto.getVehicleTaskLabel() == 2){//卸船
                startPortType = 1;//起点轮胎吊
            }
            //APP控制
            if(vmsTosOrdersDto.getAppControl()==1){
                appControl = 1;
            }
            //封装当前任务的状态链
            List<TaskStatusHistory> list = statusMap.get(vmsTosOrdersDto.getTaskNo());
            if (CollectionUtils.isEmpty(list)) {
                continue;
            }
            //转换状态链，并判断是否有车辆启动状态，  这个方法的代码似乎有些问题
            TaskStatusHistory startTask = this.transferTaskStatus(list);
            vmsTosOrdersDto.setStatusHistoryList(list);

            //封装当前子任务的描述、耗时
            long endTime = list.get(list.size()-1).getCollectTime();//最后状态的时间
            vmsTosOrdersDto.setCostTime(getCostTime(vmsTosOrdersDto.getCollectTime(),endTime));//最后状态时间-任务指令时间
            String description = vehicleDataUtil.generateTaskLocationDescription(vmsTosOrdersDto.getTaskLocationType(), vmsTosOrdersDto.getVehicleLocation());
            vmsTosOrdersDto.setLocationDescription(description);

            //大任务处理  非第一次进入， 结束上一个大任务，并开始新的大任务，根据任务的起点portType和任务类型标签VehicleTaskLabel来判断
            /** 例如
             * portType  VehicleTaskLabel
             * 1			2          |
             * 1			2          | 大任务一
             * 2			2          |————————
             * 1			2           |
             * 1			2           |大任务二
             * 2			2           |——————
             * 1			2           |
             * 1			2           |大任务三
             * 1			2           |
             * 2			2           |______
             */
            if ((vmsTosOrdersDto.getPortType() == startPortType
                    && lastPortType != vmsTosOrdersDto.getPortType() && lastPortType != 0)
                    || (lastVehicleTaskLabel != vmsTosOrdersDto.getVehicleTaskLabel() && lastVehicleTaskLabel != 0)){
                //保存上一个大任务
                kpiParentDto.setCostTime(getCostTime(kpiParentDto.getCollectTime(),kpiParentDto.getEndTime()));
                dateList.add(dateStageDto);
                kpiParentDto.setStatusDateList(dateList);
                kpiParentDto.setVmsOrderList(orderList);
                kpiOrderList.add(kpiParentDto);

                //新建一个大任务
                kpiParentDto = new VmsTosOrdersKpiDto();
                //新建任务列表
                orderList= new ArrayList<>();
                //新建时间列表
                dateList = new ArrayList<>();
                //大任务开始时间
                kpiParentDto.setCollectTime(vmsTosOrdersDto.getCollectTime());
                kpiParentDto.setTaskNo(vmsTosOrdersDto.getTaskNo());
                kpiParentDto.setContainerId(vmsTosOrdersDto.getContainerId());
                kpiParentDto.setContainerSize(vmsTosOrdersDto.getContainerSize());
                kpiParentDto.setContainerWeight(vmsTosOrdersDto.getContainerWeight());
                kpiParentDto.setVehicleNum(vmsTosOrdersDto.getVehicleNum());
                kpiParentDto.setVehicleTaskLabel(vmsTosOrdersDto.getVehicleTaskLabel());
                kpiParentDto.setTaskSource(vmsTosOrdersDto.getTaskSource());
                kpiParentDto.setAppControl(appControl);
            }

            //新建一个子任务时间对象，并赋值子任务开始时间
            if (vmsTosOrdersDto.getPortType() != lastPortType){
                if (dateStageDto != null && vmsTosOrdersDto.getPortType() != startPortType){
                    dateList.add(dateStageDto);
                }
                dateStageDto = new VehicleDateStageDto();
                dateStageDto.setReceiveTosDate(vmsTosOrdersDto.getCollectTime());
            }
            //赋值子任务车辆启动时间
            if (startTask != null){
                dateStageDto.setVehicleStartDate(startTask.getCollectTime());
            }
            //赋值子任务结束时间
            if ((i < ordersDtosList.size() -1
                    && ordersDtosList.get(i+1).getPortType() != vmsTosOrdersDto.getPortType()
                    && vmsTosOrdersDto.getEndTime() != null) || i == ordersDtosList.size() -1){
                dateStageDto.setFinishDate(vmsTosOrdersDto.getEndTime() != null?vmsTosOrdersDto.getEndTime().getTime():null);
            }

            //设置大任务信息(存在第二段任务，将第一段任务覆盖)
            if (vmsTosOrdersDto.getPortType() != startPortType){
                kpiParentDto.setContainerId(vmsTosOrdersDto.getContainerId());
                kpiParentDto.setContainerSize(vmsTosOrdersDto.getContainerSize());
                kpiParentDto.setContainerWeight(vmsTosOrdersDto.getContainerWeight());
                kpiParentDto.setVehicleNum(vmsTosOrdersDto.getVehicleNum());
                kpiParentDto.setVehicleTaskLabel(vmsTosOrdersDto.getVehicleTaskLabel());
                kpiParentDto.setAppControl(appControl);
            }else{
                kpiParentDto.setTaskNo(vmsTosOrdersDto.getTaskNo());
                kpiParentDto.setTaskSource(vmsTosOrdersDto.getTaskSource());
            }
            kpiParentDto.setEndTime(endTime);
            orderList.add(vmsTosOrdersDto);
            lastPortType = vmsTosOrdersDto.getPortType();
            lastVehicleTaskLabel = vmsTosOrdersDto.getVehicleTaskLabel();
        }

        kpiParentDto.setCostTime("0");
        //保存最后一个大任务
        if(Objects.nonNull(kpiParentDto.getCollectTime())&&Objects.nonNull(kpiParentDto.getEndTime())){
            kpiParentDto.setCostTime(getCostTime(kpiParentDto.getCollectTime(),kpiParentDto.getEndTime()));
        }
        if(Objects.nonNull(dateStageDto)){
            dateList.add(dateStageDto);
        }
        kpiParentDto.setStatusDateList(dateList);
        kpiParentDto.setVmsOrderList(orderList);
        kpiOrderList.add(kpiParentDto);
        return kpiOrderList;
    }

    //根据状态模板转化
    private TaskStatusHistory transferTaskStatus(List<TaskStatusHistory> list) {
        int transferNum = 0;
        TaskStatusHistory startTask = null;
        for(Integer status : VehicleConstant.statusTemp){
            TaskStatusHistory historyStatus = null;
            if (transferNum < list.size()){
                historyStatus = list.get(transferNum);
                if (historyStatus.getStatus() == VehicleConstant.START_TASK){
                    startTask = historyStatus;
                }
                if (historyStatus.getStatus() == VehicleConstant.TASK_FAIL && transferNum < list.size()-1){
                    list.remove(0);
                }

                //任务终止、取消成功状态，直接返回
                if (historyStatus.getStatus() == VehicleConstant.TASK_FAIL || historyStatus.getStatus() == VehicleConstant.TASK_CANCEL_SUCCESS
                        || historyStatus.getStatus() == VehicleConstant.FORCE_CANCEL_SUCCESS){
                    break;
                }
                //状态不存在于模板中,跳过该状态
                if (!VehicleConstant.statusTemp.contains(historyStatus.getStatus())){
                    transferNum ++;
                    if (transferNum >= list.size()){
                        continue;
                    }
                    historyStatus = list.get(transferNum);//???跳过当前状态，进入下一状态
                }
                //当前状态与模板一致
                if (historyStatus.getStatus() == status){
                    transferNum ++;
                    continue;
                }
            }

            //构建缺失的状态数据
            TaskStatusHistory transferStatus = new TaskStatusHistory();
            VmsTosOrderStatusEnum anEnum = VmsTosOrderStatusEnum.find(status);//获取当前状态的状态描述
            if (anEnum != null) {
                transferStatus.setStatusDescription(anEnum.getDescription());
            }
            transferStatus.setStatus(status);
            if (transferNum -1 >= 0){
                historyStatus = list.get(transferNum-1);
            }
            transferStatus.setVehicleNum(historyStatus.getVehicleNum());
            transferStatus.setCollectTime(historyStatus.getCollectTime());
            transferStatus.setSn(historyStatus.getSn());
            transferStatus.setVin(historyStatus.getVin());
            transferStatus.setExisted(false);
            list.add(transferNum,transferStatus);
            transferNum ++;
        }
        return startTask;
    }



    @Override
    public void export(VmsTosOrdersVo vmsTosOrdersVo, HttpServletResponse response) {
        int exportType = vmsTosOrdersVo.getExportType();
        List<VmsTosOrders> list;
        ExcelDataHandlerDefaultImpl<Object> excelDataHandler = new ExcelDataHandlerDefaultImpl<Object>() {
            @Override
			public String[] getNeedHandlerFields() {
                return new String[]{"作业时间","指令来源","APP介入"};
            }

            @Override
            public Object exportHandler(Object obj, String name, Object value) {
                if ("作业时间".equals(name)) {
                    VmsTosOrders order = (VmsTosOrders) obj;
                    return getCostTime(order);
                }
                if("指令来源".equals(name)){
                    if(1 == (Integer) value){
                        return "TOS下发";}
                    else if (2 == (Integer) value){
                        return "手动下发";}
                    else if (3 == (Integer) value){
                        return "VMS下发";}
                }
                if("APP介入".equals(name)){
                   return 1 == (Integer) value?"是":"否";
                }
                return super.exportHandler(obj, name, value);
            }
        };
        if (vmsTosOrdersVo.getStartTime() == null){
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.MONTH,-3);
            vmsTosOrdersVo.setStartTime(calendar.getTime());
        }
        if (exportType == 0) {
            SSIPage ssiPage = queryPage(vmsTosOrdersVo);
            list = (List<VmsTosOrders>) ssiPage.getList();
            PoiUtils.exportExcel(excelDataHandler, list, "任务清单", "ChargeOrder", VmsTosOrders.class, "任务清单" + ".xls", true, response);
        } else {
        	  vmsTosOrdersVo.setPageIndex(1);
        	  vmsTosOrdersVo.setPageSize(1000000);
        	  SSIPage ssiPage = queryPage(vmsTosOrdersVo);
              list = (List<VmsTosOrders>) ssiPage.getList();
              PoiUtils.exportExcel(excelDataHandler, list, "任务清单", "ChargeOrder", VmsTosOrders.class, "任务清单" + ".xls", true, response);
//            ExcelBatchExportService batchExportService = new ExcelBatchExportService();
//            ExportParams exportParams = new ExportParams("任务清单", "任务清单");
//            exportParams.setDataHandler(excelDataHandler);
//            exportParams.setMaxNum(1000000);
//            batchExportService.init(exportParams, VmsTosOrdersVo.class);
//            this.getBaseMapper().getListAsStream(vmsTosOrdersVo, resultContext -> {
//                VmsTosOrders orders = resultContext.getResultObject();
//                batchExportService.appendData(Arrays.asList(orders));
//            });
//            Workbook workbook = batchExportService.closeExportBigExcel();
//            PoiUtils.downLoadExcel("任务清单.xlsx", response, workbook);
        }
    }

    private String formatTime(BigDecimal seconds) {
        int sec = seconds.intValue();
        if (sec < 0) {
            return "0秒";
        }
        StringBuilder sb = new StringBuilder();
        if (sec < 60) {
            sb.append(seconds + "秒");
        } else if (sec >= 60 && sec < 3600) {
            int m = sec / 60;
            int s = sec % 60;
            sb.append(m + "分");
            if (s > 0) {
                sb.append(s + "秒");
            }
        } else {
            int h = sec / 3600;
            int m = (sec % 3600) / 60;
            int s = (sec % 3600) % 60;
            sb.append(h + "小时");
            if (m > 0) {
                sb.append(m + "分");
            }
            if (s > 0) {
                sb.append(s + "秒");
            }
        }
        return sb.toString();
    }

    /**
     * 根据异常报警查询行程
     */
    @Override
    public SSIResponse getTripByAlert(String vin, String alertId) {
        VmsVehicleAlertHistory alertHistory = vehicleTroubleService.getById(alertId);
        if (alertHistory == null) {
            return SSIResponse.ok();
        }
        List<VmsTosOrders> list = this.list(new LambdaQueryWrapper<VmsTosOrders>()
                .eq(StringUtils.isNotBlank(vin), VmsTosOrders::getVin, vin)
                .le(alertHistory.getAlarmTime() != null, VmsTosOrders::getStartTime, alertHistory.getAlarmTime())
                .ge(alertHistory.getAlarmTime() != null, VmsTosOrders::getEndTime, alertHistory.getAlarmTime())
                .orderByDesc(VmsTosOrders::getStartTime));
        if (list != null && !list.isEmpty()) {
            return SSIResponse.ok(list.get(0));
        }
        return SSIResponse.ok();
    }

    /**
     * 获得各业务箱数
     */
    @Override
    @Cached(name = "vmsTosOrdersServiceImpl:queryContainerNumOneDay", expire = 60, timeUnit = TimeUnit.SECONDS)
    @CacheRefresh(refresh = 30, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.SECONDS)
    public SSIResponse queryContainerNumOneDay(LocalDateTime startTime, LocalDateTime endTime) {
        List<TaskAnalysisVo> list = this.getBaseMapper().queryContainerNumByDay(startTime, endTime);
        if (CollectionUtils.isEmpty(list)) {
            TaskAnalysisVo vo = new TaskAnalysisVo();
            vo.setDate(startTime.toLocalDate());
            vo.setHandleContainerNum(0);
            vo.setLoadContainerNum(0);
            vo.setUnloadContainerNum(0);
            return SSIResponse.ok(vo);
        }
        return SSIResponse.ok(list.get(0));
    }

    @Override
    @Cached(name = "vmsTosOrdersServiceImpl:queryContainerNum", expire = 60, timeUnit = TimeUnit.SECONDS)
    @CacheRefresh(refresh = 30, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.SECONDS)
    public SSIResponse queryContainerNum() {
        TaskAnalysisVo vo = this.getBaseMapper().queryContainerNum(null,null,null);
        return SSIResponse.ok(vo);
    }

    /**
     * 获得最近n天各业务箱数
     */
    @Override
    @Cached(name = "vmsTosOrdersServiceImpl:queryContainerNumByDay", expire = 60, timeUnit = TimeUnit.SECONDS)
    @CacheRefresh(refresh = 30, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.SECONDS)
    public SSIResponse queryContainerNumByDay(LocalDateTime startTime, LocalDateTime endTime) throws Exception {
        if (startTime.isAfter(endTime)) {
            return SSIResponse.no("查询起始时间需在结束时间之前");
        }
        List<TaskAnalysisVo> list = vmsTosOrdersAnalysisMapper.getTaskTrendByDay(startTime, endTime);
        List<DayHourTrend> resList = TaskAnalysisVo.fillBlankDay(list, TaskAnalysisVo.class, startTime, endTime);
        return SSIResponse.ok(resList);
    }

    /**
     * 获得各业务运输效率
     */
    @Override
    @Cached(name = "vmsTosOrdersServiceImpl:queryContainerAvgTimeOneDay", expire = 60, timeUnit = TimeUnit.SECONDS)
    @CacheRefresh(refresh = 30, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.SECONDS)
    public SSIResponse queryContainerAvgTimeOneDay(LocalDateTime startTime, LocalDateTime endTime) {
        List<TaskAnalysisVo> list = this.getBaseMapper().queryContainerAvgTimeByDay(startTime, endTime);
        if (CollectionUtils.isEmpty(list)) {
            TaskAnalysisVo vo = new TaskAnalysisVo();
            vo.setDate(startTime.toLocalDate());
            vo.setHandleContainerAvgTime(0d);
            vo.setLoadContainerAvgTime(0d);
            vo.setUnloadContainerAvgTime(0d);
            return SSIResponse.ok(vo);
        }
        return SSIResponse.ok(list.get(0));
    }

    @Override
    @Cached(name = "vmsTosOrdersServiceImpl:queryContainerAvgTime", expire = 60, timeUnit = TimeUnit.SECONDS)
    @CacheRefresh(refresh = 30, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.SECONDS)
    public SSIResponse queryContainerAvgTime() {
        TaskAnalysisVo vo = this.getBaseMapper().queryContainerAvgTime(null,null,null);
        return SSIResponse.ok(vo);
    }

    @Override
    public SSIResponse getNumByVin(String vin) {
        Map<String, Integer> resultMap = new HashMap<>();

        AtomicInteger loadingGoodsNum = new AtomicInteger(0);
        AtomicInteger unloadingGoodsNum = new AtomicInteger(0);
        AtomicInteger arrangeGoodsNum = new AtomicInteger(0);

        List<VmsTosOrders> list = this.list(new LambdaQueryWrapper<VmsTosOrders>().eq(VmsTosOrders::getVin, vin));

        list.parallelStream().forEach(vmsTosOrders -> {
            if (vmsTosOrders.getTaskType() != null && vmsTosOrders.getVehicleTaskLabel() != null) {
                Integer containerNum = vmsTosOrders.getContainerNum() != null ? vmsTosOrders.getContainerNum() : 0;
                Integer vehicleTaskLabel = vmsTosOrders.getVehicleTaskLabel();
                Integer taskType = vmsTosOrders.getTaskType();
                if (vehicleTaskLabel == 2 && taskType == 1) {
                    unloadingGoodsNum.getAndAdd(containerNum);
                } else if (vehicleTaskLabel == 1 && taskType == 2) {
                    loadingGoodsNum.getAndAdd(containerNum);
                } else if (vehicleTaskLabel == 3 && taskType == 1) {
                    arrangeGoodsNum.getAndAdd(containerNum);
                }
            }
        });

        resultMap.put("loadingGoodsNum", loadingGoodsNum.intValue());
        resultMap.put("unloadingGoodsNum", unloadingGoodsNum.intValue());
        resultMap.put("arrangeGoodsNum", arrangeGoodsNum.intValue());
        return SSIResponse.ok(resultMap);
    }

    /**
     * 获取当前pad对应桥吊可选车辆
     */
    @Override
    public SSIResponse getVehicleByPadMac(String padMac, String taskNo) {
//        List<VehicleOrderVo> list = baseMapper.getVehicleByPadMac(padMac);
//        VehicleOrderVo cur = null;
//        for (int i = 0; i < list.size(); i++) {
//            if (taskNo.equals(list.get(i).getTaskNo())) {
//                cur = list.get(i);
//                list.remove(i);
//            }
//        }
//        list.add(0, cur);
//        return SSIResponse.ok(list);

        List<VmsVehicle> normalVehicle = vmsVehicleService.getNormalVehicle();
        Map<String, VmsVehicle> vinMap = normalVehicle.stream().collect(Collectors.toMap(VmsVehicle::getVin, Function.identity()));
        List<VmsCranePadBind> padBindList = vmsCranePadBindService.list(new LambdaQueryWrapper<VmsCranePadBind>().eq(VmsCranePadBind::getPadMac, padMac));
        String craneNo = padBindList.get(0).getCraneNo();
        //判断港机类型
        Integer taskLocationType = padBindList.get(0).getCraneType() == 1 ? 5 : 1;
        //获取当前所有任务
        Set<String> keys = redisDataModel.keys(latestOrderKeyPrefix + "*");
        List<String> orderList = redisDataModel.mget(keys.toArray(new String[keys.size()]));
        List<VehicleOrderVo> vehicleOrderVos = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(keys)) {
            if (CollectionUtils.isNotEmpty(orderList)) {
                List<VmsTosOrders> res = orderList.stream().map(s -> JSON.parseObject(s, VmsTosOrders.class))
                        .filter(vmsTosOrders -> {
                            if (craneNo.equals(vmsTosOrders.getPortCode()) && !VehicleUtil.isTaskOver(vmsTosOrders.getStatus())
                                    && vmsTosOrders.getTaskLocationType() == taskLocationType) {
                                return true;
                            }
                            return false;
                        }).collect(Collectors.toList());
                if (!res.isEmpty()) {
                    res = res.stream().sorted(Comparator.comparing(VmsTosOrders::getCollectTime)).collect(Collectors.toList());
                    res.stream().forEach(vmsTosOrders -> {
                        VmsVehicle vmsVehicle = vinMap.get(vmsTosOrders.getVin());
                        if (vmsVehicle == null) {
                            return;
                        }
                        VehicleOrderVo vehicleOrderVo = new VehicleOrderVo();
                        BeanUtils.copyProperties(vmsVehicle, vehicleOrderVo);
                        vehicleOrderVo.setStartTime(vmsTosOrders.getStartTime());
                        vehicleOrderVo.setTaskNo(vmsTosOrders.getTaskNo());
                        vehicleOrderVos.add(vehicleOrderVo);
                    });
                    VehicleOrderVo cur = null;
                    for (int i = 0; i < vehicleOrderVos.size(); i++) {
                        if (taskNo.equals(vehicleOrderVos.get(i).getTaskNo())) {
                            cur = vehicleOrderVos.get(i);
                            vehicleOrderVos.remove(i);
                        }
                    }
                    if (cur != null) {
                        vehicleOrderVos.add(0, cur);
                    }
                }
            }
        }
        return SSIResponse.ok(vehicleOrderVos);
    }

    /**
     * 查询任务状态历史
     */
    @Override
    public SSIResponse getTaskHistory(String taskNo) {
        List<VmsTosOrders> orders = this.lambdaQuery().eq(VmsTosOrders::getTaskNo, taskNo)
//                .eq(VmsTosOrders::getDeleted, 0)
            .list();
        if (CollectionUtils.isEmpty(orders)) {
            return SSIResponse.no("任务不存在");
        }
        Date collectTime = new Date(orders.get(0).getCollectTime());
        String year = DateUtils.format(collectTime, "yyyy");
        List<TaskStatusHistory> list = vehicleElasticSearchModel.searchTaskHistory(taskNo, year);
        if (CollectionUtils.isNotEmpty(list)) {
            list.forEach(taskStatusHistory -> {
                Map<String, Object> map = redisDataModel.hgetJson2Map(vehicleCacheKey, taskStatusHistory.getVin());
                String num = (String) map.get("vehicleNum");
                taskStatusHistory.setVehicleNum(num);
            });
        }
        return SSIResponse.ok(list);
    }

    /**
     * 进行中的任务列表
     */
    @Override
    public SSIPage listInProcess(VmsTosOrdersVo vmsTosOrdersVo) {
        if (vmsTosOrdersVo == null) {
            vmsTosOrdersVo = new VmsTosOrdersVo();
        }
        String vehicleNum = vmsTosOrdersVo.getVehicleNum();
        if (StringUtils.isNotBlank(vehicleNum)) {
            VmsVehicle vehicle = vmsVehicleService.lambdaQuery().eq(VmsVehicle::getVehicleNum, vehicleNum).one();
            if (vehicle == null) {
                return new SSIPage(new Page<>());
            }
            vmsTosOrdersVo.setVin(vehicle.getVin());
        }
        LocalDateTime dateTime = LocalDateTime.now().minus(24, ChronoUnit.HOURS);
        Date startTime = Date.from(dateTime.atZone(ZoneId.systemDefault()).toInstant());
        IPage<VmsTosOrders> page = this.lambdaQuery()
                .eq(vmsTosOrdersVo.getTaskType() != null, VmsTosOrders::getTaskType, vmsTosOrdersVo.getTaskType())
                .eq(vmsTosOrdersVo.getStatus() != null, VmsTosOrders::getStatus, vmsTosOrdersVo.getStatus())
                .eq(VmsTosOrders::getDeleted, 0)
                .like(StringUtils.isNotBlank(vmsTosOrdersVo.getTaskNo()), VmsTosOrders::getTaskNo, vmsTosOrdersVo.getTaskNo())
                .like(StringUtils.isNotBlank(vmsTosOrdersVo.getVin()), VmsTosOrders::getVin, vmsTosOrdersVo.getVin())
                .ge(VmsTosOrders::getCollectTime, startTime)
                .notIn(VmsTosOrders::getStatus, 35, 36, 37)
                .orderByDesc(VmsTosOrders::getCollectTime)
                .page(new Page<>(vmsTosOrdersVo.getPageIndex(), vmsTosOrdersVo.getPageSize()));
        List<VmsTosOrders> records = page.getRecords();
        if (CollectionUtils.isNotEmpty(records)) {
            records.forEach(vmsTosOrders -> {
                try {
                    String costTime = getCostTime(vmsTosOrders);
                    vmsTosOrders.setCostTime(costTime);
                    Map<String, Object> map = redisDataModel.hgetJson2Map(vehicleCacheKey, vmsTosOrders.getVin());
                    String num = (String) map.get("vehicleNum");
                    vmsTosOrders.setVehicleNum(num);
                } catch (Exception e) {
                    log.error("获取时间、车辆编号异常:", e);
                }
            });
        }
        return new SSIPage(page);
    }

    private String getCostTime(long startTime, long endTime) {
        Date startDate = new Date(startTime);
        Date endDate = new Date(endTime);
        return getCostTime(startDate, endDate);
    }

    private String getCostTime(VmsTosOrders vmsTosOrders) {
        Date startTime = new Date(vmsTosOrders.getCollectTime());
        Date endTime = vmsTosOrders.getEndTime();
        if (endTime == null) {
            endTime = vmsTosOrders.getUpdateTime();
        }
        if (startTime == null) {
            return "";
        }
        long diff = 0;
        if (endTime != null) {
            diff = endTime.getTime() - startTime.getTime();
        }
        BigDecimal timeSeconds = new BigDecimal(diff).divide(new BigDecimal(1000), 0, BigDecimal.ROUND_HALF_UP);
        vmsTosOrders.setCostTimeSeconds(timeSeconds.intValue());
        return formatTime(timeSeconds);
    }

    private String getCostTime(Date startTime, Date endTime) {
        if (startTime == null) {
            return "";
        }
        long diff = 0;
        if (endTime != null) {
            diff = endTime.getTime() - startTime.getTime();
        }
        BigDecimal timeSeconds = new BigDecimal(diff).divide(new BigDecimal(1000), 0, BigDecimal.ROUND_HALF_UP);
        String time = formatTime(timeSeconds);
        return time;
    }

    /**
     * 实时任务列表
     */
    @Override
    public SSIPage realTimeTasks(VmsTosOrdersVo vmsTosOrdersVo) {
        Page<VmsVehicle> page = vmsVehicleService.page(new Page<>(vmsTosOrdersVo.getPageIndex(), vmsTosOrdersVo.getPageSize()),
                new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getStatus, 0));
        List<VmsVehicle> vehicles = page.getRecords();
        if (CollectionUtils.isNotEmpty(vehicles)) {
            List<Map<String, Object>> list = vehicles.stream().map(vmsVehicle -> {
                Map<String, Object> map = redisDataModel.getJson2Map(String.format("%s:%s", latestOrderKeyPrefix, vmsVehicle.getVin()));
                if (map != null) {
                    String description = vehicleDataUtil.generateTaskLocationDescription(map);
                    map.put("locationDescription", description);
                    Date collectTime = null;
                    if (map.get("collectTime") != null) {
                        collectTime = new Date((long) map.get("collectTime"));
                    }
                    Date endTime = null;
                    if (map.get("endTime") != null) {
                        endTime = new Date((long) map.get("endTime"));
                    }
                    map.put("costTime", getCostTime(collectTime, endTime));
                    map.put("vehicleNum", vmsVehicle.getVehicleNum());
                    VmsVehicleRemoteInstruction queryEntity = new VmsVehicleRemoteInstruction();
                    queryEntity.setInstructionType("APP");
                    queryEntity.setResult((byte)1);
                    queryEntity.setTaskId(map.get("taskNo")+","+map.get("taskNoMapping"));
                    Integer count = vehicleRemoteInstructionMapper.selectCount(new QueryWrapper<>(queryEntity));
                    map.put("appControl",0);
                    if(count>0){
                        map.put("appControl",1);
                    }
                    if(Objects.isNull(map.get("taskSource"))){
                        map.put("taskSource",1);
                    }
                } else {
                    map = new HashMap<>();
                    map.put("vehicleNum", vmsVehicle.getVehicleNum());
                    map.put("collectTime", 0L);
                }
                return map;
            }).sorted((o1, o2) -> (long) o1.get("collectTime") > (long) o2.get("collectTime") ? -1 : 1)
                    .collect(Collectors.toList());
            return new SSIPage(list, (int) page.getTotal(), (int) page.getSize(), (int) page.getCurrent());
        }
        return new SSIPage(null, (int) page.getTotal(), (int) page.getSize(), (int) page.getCurrent());
    }

    /**
     * 获取某一天的任务KPI
     */
    @Override
    public List getDailyKpi(LocalDateTime startTime, LocalDateTime endTime, String vehicleNum) {
        List<VmsVehicle> vehicles = vmsVehicleService.list(new LambdaQueryWrapper<VmsVehicle>()
                .eq(VmsVehicle::getStatus, 0)
                .like(StringUtils.isNotBlank(vehicleNum), VmsVehicle::getVehicleNum, vehicleNum)
                .orderByAsc(VmsVehicle::getVehicleNum));
        List<VehicleKpiVo.HourKpi> list = getBaseMapper().getDailyKpi(startTime, endTime);
        List<Map<String, Object>> containerList = getBaseMapper().getContainNum(startTime,endTime);
        Map<String, String> containerMap = new HashedMap();
        for (Map<String, Object> map : containerList) {
            containerMap.put(String.valueOf(map.get("vin")),String.valueOf(map.get("count")));
        }
        if (CollectionUtils.isNotEmpty(vehicles)) {
            List<Map> resList = vehicles.stream().map(vmsVehicle -> {
                try {
                    String vNum = vmsVehicle.getVehicleNum();
                    String vin = vmsVehicle.getVin();
                    VehicleKpiVo vehicleKpiVo = new VehicleKpiVo(vin, vNum);
                    vehicleKpiVo.setDate(startTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                    vehicleKpiVo.fillData(list);
                    Map<String, String> map = vehicleKpiVo.toMap();
                    map.put("containerNum",containerMap.get(vin) != null ? containerMap.get(vin) : "0");
                    return map;
                } catch (Exception e) {
                    log.error("获取daily kpi异常", e);
                }
                return null;
            }).filter(map -> map != null).collect(Collectors.toList());
            return resList;
        }
        return null;
    }

    @Override
    public List taskKPI(VmsTosOrdersKPIVo vmsTosOrders) throws Exception {
        //任务KPI-效率OEE分析
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<TaskKPIVo> resLists = new ArrayList<>();
        if (vmsTosOrders.getStartTime() == null || vmsTosOrders.getEndTime() == null) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH) - 1, 23, 59, 59);//昨天的时间
            long nowStartTime = calendar.getTime().getTime();//毫秒级时间戳
            calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH) - 3, 00, 00, 00);//三天前的时间
            long nowEndTime = calendar.getTime().getTime();//毫秒级时间戳

            resLists = vmsTosOrdersMapper.taskKPI(vmsTosOrders, String.valueOf(nowStartTime), String.valueOf(nowEndTime));//查询 任务KPI-效率OEE分析
        } else {
            resLists = vmsTosOrdersMapper.taskKPI(vmsTosOrders, null, null);//查询 任务KPI-效率OEE分析
        }
        List<TaskKPIVo> rest = new ArrayList<>();
        for (int i = 0; i < resLists.size(); i++) {

            TaskKPIVo taskKPIVo = new TaskKPIVo();
            taskKPIVo.setVin(resLists.get(i).getVin());//VIN
            taskKPIVo.setVehicleNum(resLists.get(i).getVehicleNum());//车辆编号
            taskKPIVo.setTaskNo(resLists.get(i).getTaskNo());//任务编号

            Date collectTimeYear = format.parse(resLists.get(i).getInstructionWaitingTime());
            String year = DateUtils.format(collectTimeYear, "yyyy");
            List<TaskStatusHistory> list = vehicleElasticSearchModel.searchTaskHistory(resLists.get(i).getTaskNo(), year);

            Date collectTime = format.parse(resLists.get(i).getInstructionWaitingTime());
            Long vehicleReceivingTime = null;//车辆接收时间
            Long taskBegins = null;//任务开始
            Long destination = null;//到达目的地
            Long vehicleStoppingLocking = null;//车辆停稳锁死
            Long threeConfirmedCompletion = null;//三重确认完成
            for (int j = 0; j < list.size(); j++) {
                if (list.get(j).getStatus() == 43) {//车辆接受 43;
                    vehicleReceivingTime = list.get(j).getCollectTime();
                }
                if (list.get(j).getStatus() == 15) {//任务开始 15;
                    taskBegins = list.get(j).getCollectTime();
                }
                if (list.get(j).getStatus() == 3) {//到达目的地 3;
                    destination = list.get(j).getCollectTime();
                }
                if (list.get(j).getStatus() == 49) {//车辆停稳锁死 49;
                    vehicleStoppingLocking = list.get(j).getCollectTime();
                }
                if (list.get(j).getStatus() == 37) {//三重确认完成 37;
                    threeConfirmedCompletion = list.get(j).getCollectTime();
                }
            }

            //指令等待时间 =  车辆接收时间 - collect_time
            if (vehicleReceivingTime == null) {
                taskKPIVo.setInstructionWaitingTime("0");
            } else {
                long diff1 = vehicleReceivingTime - collectTime.getTime();
                String instruc = String.valueOf(diff1 / (60 * 1000) % 60);
                taskKPIVo.setInstructionWaitingTime(isNumeric(instruc) ? instruc : "0");
            }


            //作业准备时间 = 任务开始-车辆接受
            if (vehicleReceivingTime == null || taskBegins == null) {
                taskKPIVo.setHomeworkPreparationTime("0");
            } else {
                long diff2 = taskBegins - vehicleReceivingTime;
                String homework = String.valueOf(diff2 / (60 * 1000) % 60);
                taskKPIVo.setHomeworkPreparationTime(isNumeric(homework) ? homework : "0");
            }


            //移动到去箱点时间 = 到达目的地或者车辆停稳锁死-任务开始
            if (taskBegins == null) {
                taskKPIVo.setMoveTheBoxTime("0");
            } else {
                if (vehicleStoppingLocking != null) {//车辆停稳锁死
                    long diff3 = vehicleStoppingLocking - taskBegins;
                    String moveThe = String.valueOf(diff3 / (60 * 1000) % 60);
                    taskKPIVo.setMoveTheBoxTime(isNumeric(moveThe) ? moveThe : "0");
                } else if (destination != null) {//到达目的地
                    long diff3 = destination - taskBegins;
                    String moveThe = String.valueOf(diff3 / (60 * 1000) % 60);
                    taskKPIVo.setMoveTheBoxTime(isNumeric(moveThe) ? moveThe : "0");
                } else {
                    //            	long diff3 = taskBegins.getTime();
                    //            	String moveThe = String.valueOf(diff3 / (60 * 1000) % 60);
                    //            	taskKPIVo.setMoveTheBoxTime(isNumeric(moveThe)? moveThe : "0");
                    taskKPIVo.setMoveTheBoxTime("0");
                }
            }

            //取箱点等待时间 = 三重确认完成-到达目的地或者车辆停稳锁死
            if (threeConfirmedCompletion == null) {
                taskKPIVo.setPickingTime("0");
            } else {
                if (vehicleStoppingLocking != null) {//车辆停稳锁死
                    long diff4 = threeConfirmedCompletion - vehicleStoppingLocking;
                    String pickingTime = String.valueOf(diff4 / (60 * 1000) % 60);
                    taskKPIVo.setPickingTime(isNumeric(pickingTime) ? pickingTime : "0");
                } else if (destination != null) {//到达目的地
                    long diff4 = threeConfirmedCompletion - destination;
                    String pickingTime = String.valueOf(diff4 / (60 * 1000) % 60);
                    taskKPIVo.setPickingTime(isNumeric(pickingTime) ? pickingTime : "0");
                } else {
                    //            	long diff4 = threeConfirmedCompletion.getTime();
                    //            	String pickingTime = String.valueOf(diff4 / (60 * 1000) % 60);
                    //            	taskKPIVo.setPickingTime(isNumeric(pickingTime)? pickingTime : "0");
                    taskKPIVo.setPickingTime("0");
                }
            }

            taskKPIVo.setBoxQuantity(resLists.get(i).getBoxQuantity());
            rest.add(taskKPIVo);

        }
        return rest;
    }

    /**
     * 判断是否为数字，包含负数情况
     *
     * @param str
     * @return
     */
    private boolean isNumeric(String str) {
        Boolean flag = false;
        String tmp;
        if (StringUtils.isNotBlank(str)) {
            if (str.startsWith("-")) {
                tmp = str.substring(1);
                flag = false;
                return flag;
            } else {
                tmp = str;
            }
            flag = tmp.matches("^[0.0-9.0]+$");
        }
        return flag;
    }

    /**
     * 车管平台强制取消
     */
    @Override
    public SSIResponse forceCancel(String taskNo) {
        log.info("强制取消任务，任务编号:{}", taskNo);
        VmsTosOrders order = lambdaQuery().eq(VmsTosOrders::getTaskNo, taskNo).one();
        if (order == null) {
            return SSIResponse.no("任务不存在");
        }
        String vin = order.getVin();
        String url = commandUrl + "/command/force/cancel";
        HashMap<String, Object> param = new HashMap<>();
        param.put("vin", vin);
        param.put("taskNo", taskNo);
        param.put("taskOperation", 2);
        AccessResponse accessResponse = restTemplate.postForObject(url, param, AccessResponse.class);
        log.info("强制取消任务，任务编号：" + taskNo + ",返回结果：" + JSON.toJSONString(accessResponse));
        if (accessResponse.isSuccess()) {
            Map data = (Map) accessResponse.getData();
            Integer res = (Integer) data.get("result");
            if (res == 1) {
                return SSIResponse.ok("取消成功");
            } else if (res == 2) {
                return SSIResponse.no("取消失败");
            }
        } else {
            String msg = accessResponse.getStatus().getDetails();
            return SSIResponse.no(msg);
        }
        return SSIResponse.no("取消成功");
    }
}
