package com.ssi.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.entity.VmsTosOrders;
import com.ssi.entity.VmsTosOriginalOrders;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.vo.VmsTosOrdersVo;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.service.VmsTosOriginalOrdersService;
import com.ssi.mapper.VmsTosOriginalOrdersMapper;
import com.ssi.service.VmsVehicleService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
*
*/
@Service
public class VmsTosOriginalOrdersServiceImpl extends ServiceImpl<VmsTosOriginalOrdersMapper, VmsTosOriginalOrders>
implements VmsTosOriginalOrdersService{

    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private RedisDataModel redisDataModel;
    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;
    @Value("${vehicle.cache.key:ivccs:vms:vehicle:cache}")
    private String vehicleCacheKey;

    @Override
    public SSIPage queryPageByCondition(VmsTosOrdersVo ordersVo) {

        List<VmsVehicle> vehicles = vmsVehicleService.list();
        Map<String, String> vin2Num = vehicles.stream().collect(Collectors.toMap(VmsVehicle::getVin, VmsVehicle::getVehicleNum));
        String vehicleNum = ordersVo.getVehicleNum();
        if (StringUtils.isNotBlank(vehicleNum)) {
            Map.Entry<String, String> currentEntry = vin2Num.entrySet().stream().filter(e -> e.getValue().equalsIgnoreCase("vehicleNum")).findAny().orElse(null);
            if (currentEntry == null) {
                return new SSIPage(new Page<>());
            }
            String vin = currentEntry.getKey();
            ordersVo.setVin(vin);
        }
        IPage<VmsTosOriginalOrders> iPage = new Page<>(ordersVo.getPageIndex(), ordersVo.getPageSize());
        IPage<VmsTosOriginalOrders> page = baseMapper.pageQuery(iPage,ordersVo);
        List<VmsTosOriginalOrders> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return new SSIPage(page);
        }
        records.forEach(vmsTosOrders -> {
            try {
                vmsTosOrders.setStartTimeStamp(vmsTosOrders.getStartTime().getTime()/1000);
                String costTime = getCostTime(vmsTosOrders);
                vmsTosOrders.setCostTime(costTime);
                vmsTosOrders.setVehicleNum(vin2Num.get(vmsTosOrders.getVin()));
                //添加lockLabel
                Map<String, Object> orderMap = redisDataModel.getJson2Map(String.format("%s:%s", latestOrderKeyPrefix,  vmsTosOrders.getVin()));
                vmsTosOrders.setLockLabel(Objects.isNull(orderMap.get("lockLabel"))?null:(Integer)orderMap.get("lockLabel"));
            } catch (Exception e) {
                log.error("获取时间、车辆编号、地点描述异常:", e);
            }
        });
        return new SSIPage(page);
    }

    private String getCostTime(VmsTosOriginalOrders vmsTosOrders) {
        Date startTime = new Date(vmsTosOrders.getCollectTime());
        Date endTime = vmsTosOrders.getEndTime();
        if (endTime == null) {
            endTime = vmsTosOrders.getUpdateTime();
        }
        if (startTime == null) {
            return "";
        }
        long diff = 0;
        if (endTime != null) {
            diff = endTime.getTime() - startTime.getTime();
        }
        BigDecimal timeSeconds = new BigDecimal(diff).divide(new BigDecimal(1000), 0, BigDecimal.ROUND_HALF_UP);
        vmsTosOrders.setCostTimeSeconds(timeSeconds.intValue());
        return formatTime(timeSeconds);
    }

    private String formatTime(BigDecimal seconds) {
        int sec = seconds.intValue();
        if (sec < 0) {
            return "0秒";
        }
        StringBuilder sb = new StringBuilder();
        if (sec < 60) {
            sb.append(seconds + "秒");
        } else if (sec >= 60 && sec < 3600) {
            int m = sec / 60;
            int s = sec % 60;
            sb.append(m + "分");
            if (s > 0) {
                sb.append(s + "秒");
            }
        } else {
            int h = sec / 3600;
            int m = (sec % 3600) / 60;
            int s = (sec % 3600) % 60;
            sb.append(h + "小时");
            if (m > 0) {
                sb.append(m + "分");
            }
            if (s > 0) {
                sb.append(s + "秒");
            }
        }
        return sb.toString();
    }

}
