package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ssi.constant.URL;
import com.ssi.constant.enums.TrafficLightEnum;
import com.ssi.entity.VmsTrafficLightInfo;
import com.ssi.mapper.VmsTrafficLightInfoMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.response.SSIPage;
import com.ssi.service.VmsTrafficLightInfoService;
import com.ssi.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

/**
*
*/
@Service
@Slf4j
public class VmsTrafficLightInfoServiceImpl extends ServiceImpl<VmsTrafficLightInfoMapper, VmsTrafficLightInfo>
implements VmsTrafficLightInfoService{

    @Autowired
    private RedisDataModel redisDataModel;

    private ReentrantLock lock = new ReentrantLock();

    @Value("${map-edit-url}")
    private String mapEditUrl;

    @Override
    public SSIPage queryPage(VmsTrafficLightInfo req) {

        IPage<VmsTrafficLightInfo> page = new Page(req.getPageIndex(), req.getPageSize());
        IPage<VmsTrafficLightInfo> quryResult = this.baseMapper.selectPage(page, new QueryWrapper<VmsTrafficLightInfo>()
                .eq(Objects.nonNull(req.getStatus()), "status", req.getStatus())
                .eq(StringUtils.isNotBlank(req.getCrossId()), "cross_id", req.getCrossId()));
        //查询接入接口
        List<VmsTrafficLightInfo> records = quryResult.getRecords();
        for (VmsTrafficLightInfo info: records) {
           /* Map<String, Object> map = redisDataModel.hgetJson2Map("ivccs:vms:traffic:light", info.getCrossId());
            if(MapUtils.isEmpty(map)){
                continue;
            }
            List<JSONObject> lists = (List<JSONObject>)map.get("list");*/
            List<JSONObject> lists = new ArrayList<>();
            fillLights("北向",lists);
            fillLights("南向",lists);
            fillLights("东向",lists);
            fillLights("西向",lists);
            info.setLights(lists);
        }
        return new SSIPage(quryResult);
    }

    private void fillLights(String faceDirect, List<JSONObject> lists) {
        JSONObject jsonObject = new JSONObject();
        List<TrafficLightEnum> enums = TrafficLightEnum.getEnumsByPhaseNoFlagName(faceDirect);
        jsonObject.put("direction",faceDirect);
        for (int i = 0; i < enums.size(); i++) {
            jsonObject.put(enums.get(i).getDirectName(),enums.get(i).getPhaseNo());
        }
        lists.add(jsonObject);
    }

    @Override
    public String manual(VmsTrafficLightInfo req) {
        if(!lock.tryLock()){
            return "操作频繁，请稍后";
        }
        try {
            Asserts.notNull(req.getCrossId(), "CrossId");
            String lamps = req.getLampNos();
            lamps = getLamps(req, lamps);
            VmsTrafficLightInfo one = this.lambdaQuery().eq(VmsTrafficLightInfo::getCrossId, req.getCrossId()).eq(VmsTrafficLightInfo::getIsLock, 1).one();
            if (Objects.nonNull(one)) {
                //解锁操作
                boolean b = sendRequest(req.getCrossId(), 0, null);
                if (!b) {
                    return "解锁失败";
                }
                try {
                   Thread.sleep(3000);//
                } catch (Exception e) {
                    log.error("解锁等待出现异常：", e);
                }
            }
            //人工控制
            boolean b = sendRequest(req.getCrossId(), 1, lamps);
            if (!b) {
                return "控制失败";
            }
            Map<String, List> saveObj = new HashMap<>();
            saveObj.put("list",StringUtils.isNotBlank(lamps)?Arrays.asList(lamps.split(",")):Collections.EMPTY_LIST);
            redisDataModel.hset("ivccs:vms:traffic:light", req.getCrossId(),JSON.toJSONString(saveObj));
            //锁 定
            VmsTrafficLightInfo updateEntity = new VmsTrafficLightInfo();
            updateEntity.setCrossId(req.getCrossId());
            updateEntity.setIsLock((byte) 1);
            this.saveOrUpdate(updateEntity);
            return null;
        }catch (Exception e){
            return "控制失败";
        }finally {
            lock.unlock();
        }
    }

    private String getLamps(VmsTrafficLightInfo req, final String lamps) {
        String newLamps = lamps;
        Map<String, Object> map = redisDataModel.hgetJson2Map("ivccs:vms:traffic:light", req.getCrossId());
        if(!MapUtils.isEmpty(map)&&CollectionUtils.isNotEmpty((List<String>)map.get("list"))){
            List<String> olamps = (List<String>)map.get("list");
             if(req.getFlag()==2){
                 olamps.add(req.getLampNos());
             }else {
                 olamps.removeIf(e->e.equalsIgnoreCase(lamps));
             }
            newLamps = StringUtils.join(olamps,",") ;
        }else if(req.getFlag()==0){
            newLamps = "";
        }
        return newLamps;
    }

    private boolean sendRequest(String crossId,Integer enable,String objData){

        JSONObject object = new JSONObject();
        object.put("controlType",4);
        object.put("custom",1);
        object.put("planId",1);
        object.put("delay",0);
        object.put("sys","UICP");
        object.put("crossId",crossId);
        object.put("enable",enable);
        object.put("objData",objData);
        log.info(String.format("控制交通灯保存请求发送:----%s", object.toJSONString()));
        String result = RestTemplateUtil.post(String.format("%s%s","http://10.11.10.32:8090/", URL.TRAFFIC_LIGHT), object.toJSONString() , null);
        log.info(String.format("控制交通灯保存请求结果:----%s", result));
        JSONObject qryResult = JSON.parseObject(result);
        if(qryResult.getString("code").equalsIgnoreCase("1")){
            return false;
        }
        return true;
    }
}
