package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alicp.jetcache.anno.CacheRefresh;
import com.alicp.jetcache.anno.Cached;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ssi.constant.VehicleConstant;
import com.ssi.entity.VmsAlertThreshold;
import com.ssi.entity.VmsAlertThresholdNew;
import com.ssi.entity.VmsTosOrders;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.vo.*;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.model.TelecontrolModel;
import com.ssi.model.VehicleElasticSearchModel;
import com.ssi.model.VehicleLatestDataModel;
import com.ssi.response.SSIPage;
import com.ssi.response.SSIResponse;
import com.ssi.service.VmsAlertThresholdNewService;
import com.ssi.service.VmsAlertThresholdService;
import com.ssi.service.VmsTosOrdersService;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.tools.Tool;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 车辆服务实现类
 */
@Service("vehicleService")
@Slf4j
public class VmsVehicleServiceImpl extends ServiceImpl<VmsVehicleMapper, VmsVehicle> implements VmsVehicleService {

	@Value("${ivccs.vehicle.latestData.redis.prefix:ivccs:vms:vehicle:latest}")
	private String latestRedisKeyPrefix;
	@Value("${vehicle.status.redis.keyPrefixOn}")
	private String statusRedisKeyPrefix;
	@Value("${vehicle.list.redis.keyPrefix:harbor:vehicle:list}")
	private String vehicleListKeyPrefix;
	@Value("${vehicle.latestData.redis.postfix:harbor_D00A}")
	private String realPostfix;
	@Value("${order.latestOrderKeyPrefix:harbor:command:status}")
	private String latestOrderKeyPrefix;
	@Value("${harbor.remote.emergencyParking}")
	private String emergencyParkingKeyPrefix;
	@Value("${harbor.vehicle.confirm_status_prefix}")
	private String harborVehicleConfirmStatusPrefix;
	@Value("${harbor.vehicle.onstatusKey:harbor:vehicle:onstatus}")
	private String onstatusKey;
	@Autowired
	private RedisDataModel redisDataModel;
	@Autowired
	private VmsTosOrdersService vmsTosOrdersService;
	@Autowired
	private VmsAlertThresholdService vmsAlertThresholdService;
	@Autowired
	private VmsAlertThresholdNewService vmsAlertThresholdNewService;
	@Autowired
	private VehicleLatestDataModel vehicleLatestDataModel;
	@Autowired
	private VehicleDataUtil vehicleDataUtil;
	@Autowired
	private TelecontrolModel telecontrolModel;
	@Autowired
	private VehicleElasticSearchModel vehicleElasticSearchModel;

	private final String CACHE_NAME = "vehicle:vehicleInfo";

	@Override
	public SSIPage queryPage(Map<String, Object> params) {
		String vehicleNum = (String) params.get("vehicleNum");
		Integer vehicleStatus = (Integer) params.get("vehicleStatus");
		Integer vehicleType = (Integer) params.get("vehicleType");
		IPage<VmsVehicle> page = this.page(new QueryUtils<VmsVehicle>().getPage(params),
				new LambdaQueryWrapper<VmsVehicle>()
						.like(StringUtils.isNotBlank(vehicleNum), VmsVehicle::getVehicleNum, vehicleNum)
						.eq(vehicleStatus != null, VmsVehicle::getStatus, vehicleStatus)
						.eq(vehicleType != null, VmsVehicle::getVehicleType, vehicleType)
						.orderByAsc(VmsVehicle::getVehicleNum));

		List<VmsVehicle> records = page.getRecords();
		List<VehicleOrderVo> vehicleOrderVos = new ArrayList<>();
		if(CollectionUtils.isNotEmpty(records)){
			records.stream().forEach(vehicle ->{
				VehicleOrderVo vehicleOrderVo = new VehicleOrderVo();
				BeanUtils.copyProperties(vehicle,vehicleOrderVo);
				Map<String, Object> map = handleVehicleData(vehicle.getVin());
				vehicleOrderVo.setSoc(Objects.isNull(map.get("soc"))?Integer.valueOf(-1):(Integer) map.get("soc"));
				vehicleOrderVo.setOnlineStatus((Integer)map.get("state"));
				vehicleOrderVos.add(vehicleOrderVo);
			});
		}
		return new SSIPage(vehicleOrderVos,(int)page.getTotal(),(int)page.getSize(),(int)page.getCurrent());
	}

	@Override
	public List<VmsVehicle> getNormalVehicle() {
		// 先从缓存中取
		String vehicleString = redisDataModel.get(String.format("%s", vehicleListKeyPrefix));
		if (StringUtils.isNotBlank(vehicleString)) {
			return this.toListOfObject(vehicleString, VmsVehicle.class);
		}

		LambdaQueryWrapper<VmsVehicle> wrapper = new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getStatus, 0)
				.in(VmsVehicle::getVehicleType, 1, 2);
		List<VmsVehicle> vehicleList = this.list(wrapper);
		if (CollectionUtils.isNotEmpty(vehicleList)) {
			redisDataModel.set(String.format("%s", vehicleListKeyPrefix), JSON.toJSONString(vehicleList),
					(long) VehicleConstant.DEFAULT_EXPIRATION_TIME);
		}
		return vehicleList;
	}

	public static <T> List<T> toListOfObject(String json, Class<T> clazz) {
		ObjectMapper om = new ObjectMapper();
		// 反序列化时，忽略Javabean中不存在的属性，而不是抛出异常
		om.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
		// 忽略入参没有任何属性导致的序列化报错
		om.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
		try {
			Class<T[]> arrayClass = (Class<T[]>) Class.forName("[L" + clazz.getName() + ";");
			return Lists.newArrayList(om.readValue(json, arrayClass));
		} catch (IOException | ClassNotFoundException e) {
			log.error("json={}, clazz={}", json, clazz, e);
		}
		return null;
	}

	@Override
	public List<VmsVehicle> getNormalVehicleByVehicleNum(String vehicleNum, Integer vehicleType) {
		LambdaQueryWrapper<VmsVehicle> wrapper = new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getStatus, 0)
				.in(VmsVehicle::getVehicleType, 1, 2);
		if (StringUtils.isNotBlank(vehicleNum)) {
			wrapper.like(VmsVehicle::getVehicleNum, vehicleNum);
		}
		List<VmsVehicle> vehicleList = this.list(wrapper);
		return vehicleList;
	}

	@Cached(name = CACHE_NAME, expire = 120, timeUnit = TimeUnit.MINUTES)
	@CacheRefresh(refresh = 1, stopRefreshAfterLastAccess = 1440, timeUnit = TimeUnit.MINUTES)
	@Override
	public List<VmsVehicle> listAndCache() {
		return this.list(new LambdaQueryWrapper<VmsVehicle>().eq(VmsVehicle::getStatus, 0));
	}

	/**
	 * 车辆监控列表接口
	 */
	@Override
	public SSIResponse listForMonitor(List<VmsVehicle> vehicleList) {
		VmsAlertThreshold info = new VmsAlertThreshold();
		info.setParamKey("wait_overtime_energy_key");
		VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
//        VehicleVo vehicleVo = new VehicleVo();
		List<VehicleVo.vehicleTemp> vehicleTempList = Lists.newArrayList();
		vehicleList.stream().forEach(vehicle -> {
			Map<String, Object> realMap = handleVehicleData(vehicle, current);
			VehicleVo.vehicleTemp vehicleTemp = vehicleDataUtil.toVehicleTemp(realMap);
			vehicleTempList.add(vehicleTemp);
		});

		Collections.sort(vehicleTempList, new Comparator<VehicleVo.vehicleTemp>() {
			@Override
			public int compare(VehicleVo.vehicleTemp x, VehicleVo.vehicleTemp y) {
				if (x.getOnlineStatus() < y.getOnlineStatus()) {
					return 1;
				} else if (x.getOnlineStatus() == y.getOnlineStatus()) {
					if (x.getWorkStatus() < y.getWorkStatus()) {
						return 1;
					} else if (x.getWorkStatus() == y.getWorkStatus()) {
						return x.getVehicleNum().compareTo(y.getVehicleNum());
					} else {
						return -1;
					}
				} else {
					return -1;
				}
			}
		});
//        vehicleVo.setVehicleList(vehicleTempList);
		return SSIResponse.ok(vehicleTempList);
	}

	@Override
	public SSIResponse vehicleStatusForMonitor(List<VmsVehicle> vehicleList) {
		VehicleStatusVo vehicleStatusVo = vehicleStatusDist(vehicleList);
		return SSIResponse.ok(vehicleStatusVo);
	}

	@Override
	public VehicleStatusVo vehicleStatusDist(List<VmsVehicle> vehicleList) {
		VehicleStatusVo vehicleStatusVo = new VehicleStatusVo();
		AtomicInteger parkNum = new AtomicInteger(0);
		AtomicInteger loadingGoodsNum = new AtomicInteger(0);
		AtomicInteger unloadingGoodsNum = new AtomicInteger(0);
		AtomicInteger arrangeGoodsNum = new AtomicInteger(0);
		AtomicInteger chargeNum = new AtomicInteger(0);
		AtomicInteger temporaryParkNum = new AtomicInteger(0);
		AtomicInteger offlineNum = new AtomicInteger(0);
		vehicleList.parallelStream().forEach(vehicle -> {
			Map<String, Object> realMap = redisDataModel
					.getJson2Map(String.format("%s:%s-%s", latestRedisKeyPrefix, vehicle.getVin(), realPostfix));
			// 不在线
			if (realMap == null) {
				offlineNum.incrementAndGet();
				return;
			}
			//String onlineStatus = redisDataModel.get(String.format("%s:%s", statusRedisKeyPrefix, vehicle.getVin()));
			//离线状态
			String onlineStatus = redisDataModel.get(String.format("%s:%s", onstatusKey, vehicle.getVin()));
			if (onlineStatus == null && StringUtils.isBlank(onlineStatus)|| !onlineStatus.equals("1")) {
				realMap.put("state", 0);
				offlineNum.incrementAndGet();
				return;
			} else {
				realMap.put("state", Integer.parseInt(onlineStatus));
			}
//			int online = vehicleDataUtil.checkOnline(realMap.get("state") == null ? 0 : (Integer) realMap.get("state"),
//					(Long) realMap.get("collectTime"));
//			if (online == 0) {
//				offlineNum.incrementAndGet();
//				return;
//			}
			// 在线
			VmsTosOrders vmsTosOrders = getLatestOrderByVin(vehicle.getVin());
			if (vmsTosOrders != null && !VehicleUtil.isTaskOver(vmsTosOrders.getStatus())) {
				if (vmsTosOrders.getVehicleTaskLabel() == null) {
					log.error(String.format("车辆:%s:%s最新任务 车辆任务标签 为null, 任务信息如下：\n %s", vehicle.getVehicleNum(),
							vehicle.getVin(), vmsTosOrders));
					return;
				}
				switch (vmsTosOrders.getVehicleTaskLabel()) {
				case 1:
					loadingGoodsNum.incrementAndGet();
					break;
				case 2:
					unloadingGoodsNum.incrementAndGet();
					break;
				case 3:
					arrangeGoodsNum.incrementAndGet();
					break;
				case 4:
					parkNum.incrementAndGet();
					break;
				case 5:
					chargeNum.incrementAndGet();
					break;
				case 6:
					temporaryParkNum.incrementAndGet();
					break;
				}
			} else {
				parkNum.incrementAndGet();
			}
		});
		vehicleStatusVo.setVehicleNum(vehicleList.size());
		vehicleStatusVo.setLoadingGoodsNum(loadingGoodsNum.get());
		vehicleStatusVo.setUnloadingGoodsNum(unloadingGoodsNum.get());
		vehicleStatusVo.setArrangeGoodsNum(arrangeGoodsNum.get());
		vehicleStatusVo.setOfflineNum(offlineNum.get());
		vehicleStatusVo.setParkNum(parkNum.get());
		vehicleStatusVo.setChargeNum(chargeNum.get());
		vehicleStatusVo.setTemporaryParkNum(temporaryParkNum.get());
		vehicleStatusVo.setOtherOrdersNum(vehicleStatusVo.getOfflineNum() + vehicleStatusVo.getParkNum()
				+ vehicleStatusVo.getChargeNum() + vehicleStatusVo.getTemporaryParkNum());

		DecimalFormat df = new DecimalFormat("######0.00");
		vehicleStatusVo.setLoadingGoodsPer(
				df.format(vehicleStatusVo.getLoadingGoodsNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setUnloadingGoodsPer(
				df.format(vehicleStatusVo.getUnloadingGoodsNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setArrangeGoodsPer(
				df.format(vehicleStatusVo.getArrangeGoodsNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setOfflinePer(
				df.format(vehicleStatusVo.getOfflineNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setParkPer(
				df.format(vehicleStatusVo.getParkNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setChargePer(
				df.format(vehicleStatusVo.getChargeNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setTemporaryParkPer(
				df.format(vehicleStatusVo.getTemporaryParkNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		vehicleStatusVo.setOtherOrdersPer(
				df.format(vehicleStatusVo.getOtherOrdersNum() * 1.0 / vehicleStatusVo.getVehicleNum() * 100) + "%");
		return vehicleStatusVo;
	}

	@Override
	public SSIResponse saveVehicle(VmsVehicle vehicle) {

		Map<String, Object> repeatMap = checkRepeat(vehicle);
		if ((Boolean) repeatMap.get("repeat")) {
			return SSIResponse.no((String) repeatMap.get("repeatInfo"));
		}
		this.saveOrUpdate(vehicle);
		return SSIResponse.ok();
	}

	private Map<String, Object> checkRepeat(VmsVehicle vehicle) {
		Map<String, Object> repeatMap = Maps.newHashMap();
		repeatMap.put("repeat", true);
		List<VmsVehicle> list = new ArrayList<VmsVehicle>();
		if (null != vehicle.getId()) {
			LambdaQueryWrapper<VmsVehicle> wrapper = new LambdaQueryWrapper<VmsVehicle>().ne(VmsVehicle::getId,vehicle.getId());
			list = this.list(wrapper);
		} else {
			list = this.list();
		}

		Map<String, VmsVehicle> vNumMap = Maps.newHashMap();
		Map<String, VmsVehicle> vTnumMap = Maps.newHashMap();
		Map<String, VmsVehicle> chassisMap = Maps.newHashMap();
		Map<String, VmsVehicle> platNumMap = Maps.newHashMap();
		Map<String, VmsVehicle> vinMap = Maps.newHashMap();
		list.parallelStream().forEach(vehicle1 -> {
			vNumMap.put(vehicle1.getVehicleNum(), vehicle1);
			vTnumMap.put(vehicle1.getTerminalNum(), vehicle1);
			chassisMap.put(vehicle1.getChassisNum(), vehicle1);
			platNumMap.put(vehicle1.getPlateNo(), vehicle1);
			vinMap.put(vehicle1.getVin(), vehicle1);
		});
		VmsVehicle repeatVnum = vNumMap.get(vehicle.getVehicleNum());
		VmsVehicle repeatVtnum = vTnumMap.get(vehicle.getTerminalNum());
		VmsVehicle repeatChassis = chassisMap.get(vehicle.getChassisNum());
		VmsVehicle repeatPlatNum = platNumMap.get(vehicle.getPlateNo());
		VmsVehicle repeatVinNum = platNumMap.get(vehicle.getVin());
		if (repeatVnum != null) {
			if (vehicle.getVehicleNum() == null
					|| (vehicle.getVehicleNum() != null && vehicle.getVehicleNum() != repeatVnum.getVehicleNum())) {
				repeatMap.put("repeatInfo", "车辆编号不能重复");
				return repeatMap;
			}
		}
		if (repeatVtnum != null) {
			if (vehicle.getTerminalNum() == null
					|| (vehicle.getTerminalNum() != null && vehicle.getTerminalNum() != repeatVtnum.getTerminalNum())) {
				repeatMap.put("repeatInfo", "一个终端只能绑定一个车辆");
				return repeatMap;
			}
		}
		if (repeatChassis != null) {
			if (vehicle.getChassisNum() == null
					|| (vehicle.getChassisNum() != null && vehicle.getChassisNum() != repeatChassis.getChassisNum())) {
				repeatMap.put("repeatInfo", "底盘号不能重复");
				return repeatMap;
			}
		}
		if (repeatPlatNum != null) {
			if (vehicle.getPlateNo() == null
					|| (vehicle.getPlateNo() != null && vehicle.getPlateNo() != repeatPlatNum.getPlateNo())) {
				repeatMap.put("repeatInfo", "车牌号不能重复");
				return repeatMap;
			}
		}
		if (repeatVinNum != null) {
			if (vehicle.getVin() == null || (vehicle.getVin() != null && vehicle.getVin() != repeatVinNum.getVin())) {
				repeatMap.put("repeatInfo", "VIN不能重复");
				return repeatMap;
			}
		}
		repeatMap.put("repeat", false);
		return repeatMap;
	}

	private Map<String, Object> handleVehicleData(String vin){

		if(StringUtils.isBlank(vin)){
			return Collections.emptyMap();
		}
		Map<String, Object> realMap = redisDataModel
				.getJson2Map(String.format("%s:%s-%s", latestRedisKeyPrefix, vin, realPostfix));
		if (realMap == null) {
			realMap = vehicleElasticSearchModel.searchLatestVehicleData(vin);
		}
		if (realMap == null) {
			realMap = Maps.newHashMap();
		}

		String onlineStatus = redisDataModel.get(String.format("%s:%s", "harbor:vehicle:online",vin));
		if (onlineStatus==null && StringUtils.isBlank(onlineStatus) || !onlineStatus.equals("1")) {
			realMap.put("state", 0);
		} else {
			realMap.put("state", Integer.parseInt(onlineStatus));
		}
		return realMap;
	}

	private Map<String, Object> handleVehicleData(VmsVehicle vehicle, VmsAlertThreshold current) {
		Map<String, Object> realMap = redisDataModel
				.getJson2Map(String.format("%s:%s-%s", latestRedisKeyPrefix, vehicle.getVin(), realPostfix));
		if (realMap == null) {
			realMap = vehicleElasticSearchModel.searchLatestVehicleData(vehicle.getVin());
		}
		if (realMap == null) {
			realMap = Maps.newHashMap();
		}
		VmsTosOrdersVo latestOrders = getLatestOrderByVin(vehicle.getVin());

		realMap.put("vehicleNum", vehicle.getVehicleNum());
		realMap.put("vehicleType",vehicle.getVehicleType());
		realMap.put("vin", vehicle.getVin());
		// 查询在线状态
		//String onlineStatus = redisDataModel.get(String.format("%s:%s", statusRedisKeyPrefix, vehicle.getVin()));
		String onlineStatus = redisDataModel.get(String.format("%s:%s", "harbor:vehicle:onstatus", vehicle.getVin()));
		if (onlineStatus==null && StringUtils.isBlank(onlineStatus) || !onlineStatus.equals("1")) {
			realMap.put("state", 0);
		} else {
			realMap.put("state", Integer.parseInt(onlineStatus));
		}
		if (latestOrders == null || latestOrders.getStatus() == 35 || latestOrders.getStatus() == 36
				|| latestOrders.getStatus() == 37 || latestOrders.getStatus() == 41) {
			realMap.put("workStatus", 0);
		} else {
			realMap.put("orderData", latestOrders);
			realMap.put("workStatus", 1);
			// 是否超时
			if (latestOrders.getStartTime() == null) {
				realMap.put("waitOvertimeStatus", 0);
			} else {
				Long startTime = latestOrders.getStartTime().getTime();
				long time;
				time = startTime + Integer.parseInt(current.getParamValue()) * 60 * 1000;
				if (time < System.currentTimeMillis()) {
					realMap.put("waitOvertimeStatus", 1);
				} else {
					realMap.put("waitOvertimeStatus", 0);
				}
			}

		}
		VmsAlertThreshold info = new VmsAlertThreshold();
		info.setParamKey("low_energy_key");
		VmsAlertThreshold vmsAlertThreshold = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
		realMap.put("lowEnergy", Integer.parseInt(vmsAlertThreshold.getParamValue()));
		// 添加紧急停车状态
		Map<String, Object> emergencyStatusMap = redisDataModel
				.getJson2Map(String.format("%s:%s", emergencyParkingKeyPrefix, vehicle.getVin()));
		if (emergencyStatusMap != null) {
			Object emergencyStatus = emergencyStatusMap.get("emergencyStatus");
			realMap.put("emergencyType", emergencyStatus);
		} else {
			realMap.put("emergencyType", 2);
		}
		// 三重确认状态添加
		Map<String, Object> threeConfirmStatus = redisDataModel
				.getJson2Map(String.format("%s:%s", harborVehicleConfirmStatusPrefix, vehicle.getVin()));
		realMap.put("threeConfirmStatus", threeConfirmStatus);
		return realMap;
	}

	private VmsTosOrdersVo getLatestOrderByVin(String vin) {
//        List<VmsTosOrders> list = vmsTosOrdersService.list(new LambdaQueryWrapper<VmsTosOrders>()
//                .eq(VmsTosOrders::getVin, vin)
//                .orderByDesc(VmsTosOrders::getReceiveTime));
//        VmsTosOrders latestOrders = list == null || list.isEmpty() ? null:list.get(0);
//        return latestOrders;
		Map<String, Object> json2Map = redisDataModel.getJson2Map(String.format("%s:%s", latestOrderKeyPrefix, vin));
		JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(json2Map));
		VmsTosOrdersVo vmsTosOrders = JSONObject.toJavaObject(jsonObject, VmsTosOrdersVo.class);
		return vmsTosOrders;
	}

	/**
	 * 运力分析
	 */
	@Override
	public SSIResponse transportCapacity() {
		List<VmsVehicle> vehicleList = this.getNormalVehicle();
		VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>());
		Double socSum = 0.0;
		Integer vehicleSum = 0;
		TransportCapacityAnalysisVo capacityAnalysisVo = new TransportCapacityAnalysisVo();
		for (VmsVehicle vehicle : vehicleList) {
			Map<String, Object> realMap = handleVehicleData(vehicle, current);
			int online = vehicleDataUtil.checkOnline(realMap.get("state") == null ? 0 : (Integer) realMap.get("state"),
					(Long) realMap.get("collectTime"));
			if (online == 1) {
				vehicleSum++;
				Integer workStatus = (Integer) realMap.get("workStatus");
				Integer faultStatus = realMap.get("equipStatus") == null ? 1
						: (Integer) realMap.get("equipStatus") == 1 ? 1 : 0;
				Integer chargingStatus = (Integer) realMap.get("chargingStatus");
				chargingStatus = chargingStatus == null ? 0 : chargingStatus;
				Integer soc = (Integer) realMap.get("soc");
				if (soc == null) {
					soc = 0;
				}
				// 车辆任务超时也算入运力里
				Object waitOvertimeStatus = realMap.get("waitOvertimeStatus");
				if (waitOvertimeStatus == null) {
					waitOvertimeStatus = 0;
				}
				// chargingStatus 3:未充电，4：充电完成
//                if ((workStatus != 1 || (int)waitOvertimeStatus == 1) && faultStatus != 1 && (chargingStatus == 3 || chargingStatus == 4)) {
				capacityAnalysisVo.rangeMatch(soc);
				socSum = socSum + soc;
//                }
			}
		}
		Double transportCapacity = vehicleSum != 0 ? socSum / vehicleSum : 0;
		BigDecimal bg = new BigDecimal(transportCapacity).setScale(1, BigDecimal.ROUND_HALF_UP);
		capacityAnalysisVo.setTransportCapacity(bg);
		return SSIResponse.ok(capacityAnalysisVo);
	}

	/**
	 * 获取当前定位周边车辆
	 */
	@Override
	public List<VehicleControlStatusVo> getVehicleByLocation(String location, final Integer distance) {
		log.info(String.format("%s:%s,%s:%s", "根据经纬度获取车辆接口调用:经纬度", location, "遥控距离", distance));
		List<VehicleControlStatusVo> resultList = Lists.newArrayList();
		String[] split = location.split(",");
		double[] curLoc = new double[] { Double.parseDouble(split[0]), Double.parseDouble(split[1]) };
		List<VmsVehicle> list = this.list(new LambdaQueryWrapper<VmsVehicle>().orderByAsc(VmsVehicle::getVehicleNum));
		list.stream().forEach(vehicle -> {
			Map<String, Object> realMap = redisDataModel
					.getJson2Map(String.format("%s:%s-%s", latestRedisKeyPrefix, vehicle.getVin(), realPostfix));
			// 查询在线状态
			if (realMap != null && !realMap.isEmpty() && realMap.get("longitude") != null
					&& realMap.get("latitude") != null) {
				String onlineStatus = redisDataModel
						.get(String.format("%s:%s", statusRedisKeyPrefix, vehicle.getVin()));
				if (StringUtils.isBlank(onlineStatus)) {
					realMap.put("state", 0);
				} else {
					realMap.put("state", Integer.parseInt(onlineStatus));
				}
				int online = vehicleDataUtil.checkOnline(
						realMap.get("state") == null ? 0 : (Integer) realMap.get("state"),
						(Long) realMap.get("collectTime"));
				if (online == 1) {
					Double longitude = ConfigUtils.getAsDoubleWithDefault(realMap, "longitude", 0.0);
					Double latitude = ConfigUtils.getAsDoubleWithDefault(realMap, "latitude", 0.0);
					double vehicleDistance = GpsUtil.getDistance(curLoc[0], curLoc[1], latitude, longitude);
					if (vehicleDistance * 1000 < distance) {
						// 接管状态
						Map controlStatus = telecontrolModel.getControlStatus(vehicle.getVin());
						VehicleControlStatusVo vehicleControlStatusVo = new VehicleControlStatusVo();
						BeanUtils.copyProperties(vehicle, vehicleControlStatusVo);
						if (controlStatus != null) {
							vehicleControlStatusVo.setAppControl((Integer) controlStatus.get("appControl"));
							vehicleControlStatusVo.setAppActive((Integer) controlStatus.get("appActive"));
							vehicleControlStatusVo.setAppMac((String) controlStatus.get("appMac"));
						}
						// 紧停状态
						Map emergencyStatusMap = telecontrolModel.getEmergencyStatus(vehicle.getVin());
						Integer emergencyStatus = 2;
						if (emergencyStatusMap != null) {
							emergencyStatus = (Integer) emergencyStatusMap.get("emergencyStatus");
						}
						vehicleControlStatusVo.setEmergencyStatus(emergencyStatus);
						// 切换自动驾驶状态
						Map autoPilotMap = telecontrolModel.getAutoPilotStatus(vehicle.getVin());
						Integer autoPilotStatus = 4;
						if (autoPilotMap != null) {
							autoPilotStatus = (Integer) autoPilotMap.get("autopilot");
						}
						vehicleControlStatusVo.setAutopilot(autoPilotStatus);
						int soc = ConfigUtils.getAsIntegerWithDefault(realMap, "soc", 0);
						vehicleControlStatusVo.setSoc(soc);
						resultList.add(vehicleControlStatusVo);
					}
				}
			}
		});
		return resultList;
	}

	@Override
	public Map getVmsCranePosMapInfo(String name) {
		Map map = this.baseMapper.getVmsCranePosMapInfo(name);
		return map;
	}

	@Override
	public List<Map> getVmsVehicle() {
		// TODO Auto-generated method stub
		List<Map>result=this.baseMapper.getVmsVehicle();
		return result;
	}

}
