package com.ssi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.ssi.utils.ElasticSearchUtil;
import com.ssi.websocket.WebBackSocketService;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.admin.indices.exists.indices.IndicesExistsRequest;
import org.elasticsearch.action.search.SearchRequestBuilder;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.sort.SortOrder;
import org.joda.time.DateTime;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * Description:
 *    用于读取kafka数据回放
 * @author LiXiaoCong
 * @version 2019/10/12 12:47
 */
@Slf4j
public class WebSocketDataEsBackor {

    Set<String> sessions;
    String mainSession;

    private String backorId;
    private  String scenesId;
    private WebSocketDataBackServiceImpl webSocketDataBackService;
    private Set<String> vins;

    private TransportClient transportClient;
    private String indexBase;
    private Map<String, Map<String, Object>> latestMap;

    private Map<String, String> latestJson;

    private AtomicBoolean closed = new AtomicBoolean(false);

    private boolean randomDelay;
    private double randomDelayScale;
    private String randomDelayScenesId;
    private String playBackFolder;

    public WebSocketDataEsBackor(String backorId,
                                 String scenesId,
                                 WebSocketDataBackServiceImpl webSocketDataBackService,
                                 Set<String> vins,
                                 String mainSession,
                                 Set<String> sessions,
                                 TransportClient transportClient,
                                 String indexBase,
                                 String playBackFolder){
        this.backorId = backorId;
        this.scenesId = scenesId;
        this.webSocketDataBackService = webSocketDataBackService;
        this.vins = vins;
        this.mainSession = mainSession;
        this.sessions = sessions;
        this.transportClient = transportClient;
        this.indexBase = indexBase;
        this.latestMap = new HashMap<>();
        this.latestJson = new HashMap<>();
        this.playBackFolder = playBackFolder;
    }

    public WebSocketDataEsBackor() {
    }

    public void playBack(Long startTime, Long stopTime, long delay, boolean normal){

        try {
            //初始化结果容器
            Map<String, List<Map<String, Object>>> records = Maps.newHashMap();
            for(String vin : vins){
                records.put(vin, Lists.newLinkedList());
            }
            Map<String, List<Map<String, Object>>> res = getRecordsFromFile(startTime,stopTime);
            if(res == null){
                getRecords(startTime, stopTime, records);
                if(!records.isEmpty()){
                    //这个地方把数据保存到文件
                    String filePath = playBackFolder + "/" + startTime + "-" + stopTime + ".json";
                    JSON.writeJSONString(new FileOutputStream(filePath),records);
                }
            }else{
                records = res;
            }
            Iterator<Map.Entry<String, List<Map<String, Object>>>> iterator = records.entrySet().iterator();
            while (iterator.hasNext()){
                if(iterator.next().getValue().size() == 0){
                    iterator.remove();
                }
            }
            if(!records.isEmpty()) {
                if(normal) {
                   playBackByNormal(records, delay);
                }/*else if(randomDelay && randomDelayScenesId != null){
                    playBackInThread(records, delay);
                }*/ else {
                    playBack(records, delay);
                }
            }else{
                log.warn(String.format("车辆%s的数据查询成功，时间段：[%s, %s]， 无数据， 无需回放。", vins, startTime, stopTime));
            }
        } catch (Exception e) {
            log.warn(String.format("车辆%s的数据回放失败，时间段：[%s, %s]。", vins, startTime, stopTime), e);
            shutDown();
            WebBackSocketService.messageBroadcast(backorId, Sets.newHashSet(this.sessions), Lists.newArrayList(), true);

        }
    }

    Map<String, List<Map<String, Object>>> getRecordsFromFile(Long startTime,Long stopTime) throws Exception{
        String filePath = playBackFolder + "/" + startTime + "-" + stopTime + ".json";
        boolean exists = new File(filePath).exists();
        if(!exists){
            return null;
        }
        Map<String, List<Map<String, Object>>> records = JSON.parseObject(new FileInputStream(filePath), Map.class);
        return records;
    }

    private void playBack(Map<String, List<Map<String, Object>>> data, long delay){

        log.info(String.format("开始回放车辆%s的数据.", vins));

        String recordJson;
        long sleepTime;

        Map<String, Object> map;

        //获取最多数据集合个数
        Set<Map.Entry<String, List<Map<String, Object>>>> entries = data.entrySet();
        Map<String, Integer> sizeMap = Maps.newHashMap();
        Integer max = entries.stream().map(entry ->{
            int size = entry.getValue().size();
            sizeMap.put(entry.getKey(), size);
            return size;
        }).max(Integer::compareTo).get();

        Integer vinSize;
        for (int i = 0; i < max; i++) {

            long t = System.currentTimeMillis();
            log.info(String.format("开始%s的第%s次推送数据：%s", backorId, i + 1, t));
            List<String> recordList = Lists.newArrayList();
            for (Map.Entry<String, List<Map<String, Object>>> entry : entries){

                vinSize = sizeMap.get(entry.getKey());
                vinSize = (i >= vinSize) ? (vinSize - 1) : i;
                map = entry.getValue().get(vinSize);

                if(!closed.get()) {
                    int i1 = checkRandomDelay(map);
                    recordJson = JSON.toJSONString(map);
                    this.latestJson.put(entry.getKey(), recordJson);
                    recordList.add(recordJson);
                }else {
                    return;
                }
            }

            WebBackSocketService.messageBroadcast(backorId, Sets.newHashSet(sessions), recordList);


            //计算休眠时间
            sleepTime = delay - (System.currentTimeMillis() - t);
            sleep(sleepTime > 0 ? sleepTime : 1000L);
        }

        pushLast();
        log.info(String.format("回放车辆数据完成：%s, 回放数据记录情况：%s。", vins, sizeMap));
    }

    /**
     *   仅适合车辆少的情况， 用于测试
     * @param data
     * @param delay
     */
    private void playBackInThread(Map<String, List<Map<String, Object>>> data, long delay){

        log.info(String.format("开始使用多线程回放车辆%s的数据.", vins));

        CountDownLatch countDownLatch = new CountDownLatch(data.size());
        data.forEach((vin, records) ->{
            new Thread(() -> {
                try {
                    String recordJson;
                    long sleepTime;
                    long t = System.currentTimeMillis();
                    log.info(String.format("开始%s的车辆%s推送数据： %s", backorId, vin, t));
                    for (Map<String, Object> map : records){
                        if(!closed.get()) {
                            int i1 = checkRandomDelay(map);
                            recordJson = JSON.toJSONString(map);
                            this.latestJson.put(vin, recordJson);
                            WebBackSocketService.messageBroadcast(backorId, Sets.newHashSet(sessions), recordJson);

                            if(i1 > 100){
                                sleep(i1);
                            }
                        }else {
                            return;
                        }
                        //计算休眠时间
                        sleepTime = delay - (System.currentTimeMillis() - t);
                        sleep(sleepTime > 0 ? sleepTime : 1000L);
                    }
                } finally {
                    countDownLatch.countDown();
                }
            }).start();
        });

        try {
            countDownLatch.await();
        } catch (InterruptedException e) {
        }

        pushLast();
        List<String> collect = data.entrySet()
                .stream()
                .map(entry -> String.format("%s:%s", entry.getKey(), entry.getValue().size()))
                .collect(Collectors.toList());
        log.info(String.format("回放车辆数据完成：%s, 回放数据记录情况：%s。", vins, collect));
    }

    private int checkRandomDelay(Map<String, Object> map){
        int delay = 0;
        if(randomDelay && randomDelayScenesId.equals(scenesId)){
            double random = Math.random();
            if(random < randomDelayScale){
                map.put("netError", true);
                delay = (int) (random * 100000);
                log.warn(String.format("触发随机延迟：%s, 延迟时长：%s。", map, delay));
            }
        }
        return delay;
    }

    private void playBackByNormal(Map<String, List<Map<String, Object>>> data, long delay){

        log.info(String.format("开始使用队列回放车辆%s的数据.", vins));

        String json;
        long sleepTime;

        LinkedList<Map<String, Object>> res;
        Map<String, Object> map;
        boolean empty;

        Iterator<Map.Entry<String, List<Map<String, Object>>>> iterator;
        Map.Entry<String, List<Map<String, Object>>> entry;
        int i = 0;
        while (!data.isEmpty()){
            long t = System.currentTimeMillis();
            log.info(String.format("开始%s的第%s次推送数据：%s", backorId, i++, t));

            iterator = data.entrySet().iterator();
            while (iterator.hasNext()){
                entry = iterator.next();
                res = (LinkedList<Map<String, Object>>) entry.getValue();
                map = res.poll();
                empty = res.isEmpty();
                if(map != null){
                    if(!closed.get()) {
                        checkRandomDelay(map);
                        json = JSON.toJSONString(map);
                        if(empty){
                            json = String.format("#%s", json);
                        }
                        WebBackSocketService.messageBroadcast(backorId, Sets.newHashSet(sessions), json);
                    }else {
                        return;
                    }
                }

                if(empty){
                    log.info(String.format("车辆%s的数据回放完成。", entry.getKey()));
                    this.latestJson.put(entry.getKey(), null);
                    iterator.remove();
                }
            }

            //计算休眠时间
            sleepTime = delay - (System.currentTimeMillis() - t);
            sleep(sleepTime > 0 ? sleepTime : 1000L);
        }

        pushLast();
        log.info(String.format("回放车辆数据完成：%s。", vins));
    }

    void getRecords(Long startTime,
                    Long stopTime,
                    Map<String, List<Map<String, Object>>> resultContain){

        SearchRequestBuilder searchRequestBuilder = getSearchRequestBuilder(startTime, stopTime);
        if(searchRequestBuilder == null){
            resultContain.clear();
            return;
        }
        log.info(String.format("开始查询车辆%s的数据， 查询索引为：%s.",
                vins, Arrays.asList(searchRequestBuilder.request().indices())));

        searchRequestBuilder.setFetchSource(new String[]{"timestamp", "value"}, null);

        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        queryBuilder.must(QueryBuilders.termQuery("key", "all"));

        RangeQueryBuilder rangeQueryBuilder = ElasticSearchUtil.createRangeQueryBuilder("timestamp", startTime, stopTime);
        queryBuilder.must(rangeQueryBuilder);

        SearchResponse searchResponse = searchRequestBuilder.setQuery(queryBuilder)
                .setScroll(TimeValue.timeValueMinutes(5))
                .addSort("timestamp", SortOrder.ASC)
                .setSize(1000)
                .get();

        SearchHits hits = searchResponse.getHits();
        SearchHit[] hitsHits = hits.getHits();
        Object line;
        Map<String, Object> record;
        Map<String, Object> sourceAsMap;
        while (hitsHits.length > 0) {

            for (int i = 0; i < hitsHits.length; i++) {
                sourceAsMap = hitsHits[i].getSourceAsMap();
                line = sourceAsMap.get("value");
                if(line != null){
                    record = jsonToMap(line.toString());
                    if(record != null){
                        checkAndPushResultContain(record, sourceAsMap.get("timestamp"), resultContain);
                    }
                }
            }
            searchResponse = transportClient.prepareSearchScroll(searchResponse.getScrollId())
                    .setScroll(TimeValue.timeValueMinutes(5)).get();
            hitsHits = searchResponse.getHits().getHits();
        }

        List<String> collect = resultContain.entrySet()
                .stream()
                .map(entry -> String.format("%s:%s", entry.getKey(), entry.getValue().size()))
                .collect(Collectors.toList());
        log.info(String.format("车辆%s的数据查询成功，时间段：[%s, %s]， 数据量：%s", vins, startTime, stopTime, collect));
    }


    private void checkAndPushResultContain(Map<String, Object> record,
                                           Object time,
                                           Map<String, List<Map<String, Object>>> resultContains){

        resultContains.forEach((vin, resultContain) ->{
            Object map = record.get(vin);
            if(map != null && isValid(vin, map)){
                Map<String, Object> res = (Map<String, Object>) map;
                res.put("timestamp", time);
                resultContain.add(res);
            }
        });
    }

    private SearchRequestBuilder getSearchRequestBuilder(Long startTime, Long stopTime) {
        SearchRequestBuilder searchRequestBuilder;
        if(stopTime >= 1570982400000L) {
            String startDay = new DateTime(startTime).toString("yyyyMMdd");
            String stopDay = new DateTime(stopTime).toString("yyyyMMdd");
            boolean exists = transportClient.admin().indices()
                    .exists(new IndicesExistsRequest()
                            .indices(new String[]{String.format("%s_%s", indexBase, startDay), String.format("%s_%s", indexBase, stopDay)}))
                    .actionGet()
                    .isExists();
            if(!exists){
                return null;
            }
            if(startDay.equals(stopDay)){
                searchRequestBuilder = transportClient.prepareSearch(String.format("%s_%s", indexBase, startDay));
            }else{
                searchRequestBuilder = transportClient.prepareSearch(String.format("%s_%s", indexBase, startDay),
                        String.format("%s_%s", indexBase, stopDay));
            }
        }else{
            searchRequestBuilder = transportClient.prepareSearch(String.format("%s_exception_2019", indexBase));
        }
        return searchRequestBuilder;
    }

    private Map<String, Object> jsonToMap(String json){
        Map<String, Object> map = null;
        try {
            map = JSON.parseObject(json, new TypeReference<Map<String, Object>>() {
            });
        } catch (Exception e) {
            log.error(String.format("json数据转换Map失败：%s", json), e);
        }
        return map;
    }

    public void shutDown() {
        this.closed.set(true);
    }


    private boolean isValid(String vin, Object record){
        boolean valid;
        try {
            Map<String, Object> map = (Map<String, Object>) record;
            valid = ("在线".equals(map.get("clientStatus")));

            if(valid){
                Map<String, Object> preMap = this.latestMap.get(vin);
                if(preMap != null){
                    String prelocation = String.format("%s,%s", preMap.get("latitude"), preMap.get("longitude"));
                    String location = String.format("%s,%s", map.get("latitude"), map.get("longitude"));
                    double speed = Double.parseDouble(map.getOrDefault("speed", "0").toString());
                    valid = !(prelocation.equals(location) && speed != 0);
                }

                if(valid) {
                    this.latestMap.put(vin, map);
                }
            }
        } catch (Exception e) {
            log.error("数据检查异常。", e);
            valid = false;
        }
        return valid;
    }

    private void sleep(long time){
        try {
            Thread.sleep(time);
        } catch (InterruptedException e) { }
    }

    private void pushLast(){
        this.latestJson.forEach((key, json) ->{
            if(!closed.get()) {
                HashSet<String> sessions = Sets.newHashSet(this.sessions);
                WebBackSocketService.messageBroadcast(backorId, sessions, json, true);
                log.info(String.format("推送%s的车辆%s的最后一条数据。", sessions, key));
            }else{
                return;
            }
        });
        shutDown();
    }

    public void setRandomDelay(boolean randomDelay) {
        this.randomDelay = randomDelay;
    }

    public void setRandomDelayScale(double randomDelayScale) {
        this.randomDelayScale = randomDelayScale;
    }

    public void setRandomDelayScenesId(String randomDelayScenesId) {
        this.randomDelayScenesId = randomDelayScenesId;
    }
}
