package com.ssi.service.platform.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ssi.kafka.model.KafkaRecordModel;
import com.ssi.mapper.VehicleVideoMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.model.VehicleBaseInfoModel;
import com.ssi.model.VehicleElasticSearchModel;
import com.ssi.service.platform.VehicleMonitorService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class VehicleMonitorServiceImpl implements VehicleMonitorService {

    @Value("${v2x.rsu.transform.video.request:http://172.16.15.26:10080/rtspstart}")
    private String videoTransformRquestUrl;
    @Value("${v2x.history.video.request:http://172.16.15.26:10080/historyvideo}")
    private String videoListformRquestUrl;

    @Autowired
    private VehicleVideoMapper vehicleVideoMapper;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisDataModel redisDataModel;

    @Autowired
    private VehicleBaseInfoModel vehicleBvaseInfoModel;

    @Value("${command.url:http://172.16.15.13:8040/}")
    private String commandUrl;

    @Autowired
    private VehicleElasticSearchModel vehicleElasticSearchModel;

    @Value("${meilan.data.esIndex:v2x_meilan_vehicle}")
    private String meilanDataIndex;
    @Value("${meilan.data.esIndex.timeField:collectTime}")
    private String timeField;

    @Value("${rtmp.video.lan.url:rtmp://192.168.6.25:1935/}")
    private String rtmpBaseLanUrl;

    @Value("${flv.video.lan.url:http://192.168.6.25:8080/}")
    private String flvBaseLanUrl;

    @Value("${video.online.keyPrefix:v2x:meilan:video:}")
    private String videoPrefixKey;
    @Autowired
    private KafkaRecordModel kafkaRecordModel;

    @Override
    public Object getCarHistoryVideoList(String vin) {
        return getHistoryVideoList(vin);
    }

    @Override
    public Object getCarVideoList(String vin) {
        return getCarVideoList(vin, rtmpBaseLanUrl, flvBaseLanUrl);
    }

    @Override
    public Map<String, Object> getOfflineVehicleList(String vehicleCompany, String vehicleType, Integer offlineDays, int page, int size) {
        Map<String, Object> searchResult = vehicleElasticSearchModel.getOfflineVehicleList(meilanDataIndex, timeField, vehicleCompany, vehicleType, offlineDays, page, size);
        List<Map<String, Object>> records = (List<Map<String, Object>>) searchResult.get("records");
        List<Map<String, Object>> result = records.stream().map(r -> {
            Long collectTime = (Long) r.get(timeField);
            Integer days = Double.valueOf((System.currentTimeMillis() - collectTime) / (24 * 60 * 60 * 1000)).intValue();
            r.put("offlineDays", days);
            return r;
        }).collect(Collectors.toList());
        searchResult.put("records", result);
        return searchResult;
    }

    @Override
    public Object getCarVideoPlayingList(String vin) {
        List<Object> result = Lists.newArrayList();
        for (int i = 1; i <= 4; i++) {
            try {
                Object videoUrl = playingCarVideo(vin, i);
                result.add(videoUrl);
            } catch (Exception e) {
                log.error("获取视频失败!", e);
            }
        }
        return result;
    }

    public Object getHistoryVideoList(String vin) {
        List results = Lists.newArrayList();
        try {
            String url = String.format("%s?vin=%s", videoListformRquestUrl, vin);
            //激活回放流
            String responseString = restTemplate.getForObject(url, String.class);
            log.info(responseString);
            results = getVideoHistoryList(vin);
        } catch (RestClientException e) {
            log.error(String.format("调用接口失败：\n %s", vin), e);
        }
        return results;
    }

    public Object playingCarVideo(String vin, Integer channel) {
        Map<String, Object> result = Maps.newHashMap();
        try {
            Map<String, Object> vehicleInfo = vehicleBvaseInfoModel.getVehicleBaseInfo(vin);
            if (vehicleInfo != null) {
                String url = String.format("%s/media/9101", commandUrl);
                JSONObject jo = new JSONObject();
                jo.put("sim", vehicleInfo.get("sim"));
                jo.put("channel", channel);
                jo.put("dataType", 1);
                jo.put("controlType", 0);
                jo.put("hdType", 0);
                String responseString = restTemplate.postForObject(url, jo, String.class);
                JSONObject resJo = JSON.parseObject(responseString);
                String code = resJo.getString("code");
                if ("0".equals(code)) {
                    JSONObject dataJo = resJo.getJSONObject("data");
                    result.put("vin", vin);
                    result.put("sim", vehicleInfo.get("sim"));
                    result.put("channel", channel);
                    result.put("rtmpUrl", dataJo.getString("playUrl"));
                    result.put("sn", dataJo.getString("sn"));
                    result.put("result", dataJo.getString("result"));
                } else {
                    //JSONObject dataJo = resJo.getJSONObject("data");
                    result.put("vin", vin);
                    result.put("channel", channel);
                    result.put("sim", vehicleInfo.get("sim"));
                }
            }
        } catch (Exception e) {
            log.error(String.format("调用接口失败：\n %s", vin), e);
        }
        return result;
    }


    private List getVideoHistoryList(String vin) {
        List<Map<String, Object>> videolist = vehicleVideoMapper.getHistoryVideoByVin(vin);
        return videolist;

    }

    public List<Map<String, Object>> getCarVideoList(String vin, String rtmpBase, String flvBase) {

        List<Map<String, Object>> result = Lists.newArrayList();
        if (StringUtils.isNotBlank(vin)) {
            result = vehicleVideoMapper.getVideoListByVins(Arrays.asList(vin.split(",")));
        }

        if (!result.isEmpty()) {
            result = result.parallelStream().map(video -> {

                String url = String.valueOf(video.get("url"));
                if (url != null && url.indexOf("sdp") == -1) {
                    Map<String, Object> videoMsg = videoMsgByRedis(url);
                    if (videoMsg != null) {
                        String rtmpUrl = (String) videoMsg.remove("rtmpUrl");
                        String flvUrl = (String) videoMsg.remove("flvUrl");
                        if (rtmpUrl.indexOf("dfcv") > -1) {
                            rtmpUrl = String.format("%s%s", rtmpBase, rtmpUrl.substring(rtmpUrl.indexOf("dfcv")));
                        }

                        if (flvUrl.indexOf("dfcv") > -1) {
                            flvUrl = String.format("%s%s", flvBase, flvUrl.substring(flvUrl.indexOf("dfcv")));
                        }
                        video.put("url", rtmpUrl);
                        video.put("flvUrl", flvUrl);
                        video.putAll(videoMsg);
                    } else {
                        video.put("url", String.format("%s%s", rtmpBase, url));
                        video.put("flvUrl", String.format("%s%s.flv", flvBase, url));
                        video.put("online", "0");
                    }
                } else {
                    Map<String, Object> videoMap = transfromVideoUrl(url);
                    if (videoMap != null && !videoMap.isEmpty()) {
                        video.put("url", videoMap.get("rtmp"));
                        video.put("flvUrl", videoMap.get("httpflv"));
                        video.put("online", "1");
                    }
                }

                return video;
            }).collect(Collectors.toList());
        }
        return result;
    }

    public Map<String, Object> videoMsgByRedis(String videoUrl) {
        String key = String.format("%s%s", videoPrefixKey, videoUrl);
        return redisDataModel.getJson2Map(key);
    }

    public Map<String, Object> transfromVideoUrl(String rtsp) {
        try {

            String url = String.format("%s?rtsp=%s", videoTransformRquestUrl, rtsp);
            String responseString = restTemplate.getForObject(url, String.class);
            if (StringUtils.isNotBlank(responseString)) {
                log.info(JSON.toJSONString(responseString));
                Map<String, Object> res = JSON.parseObject(responseString, Map.class);
                if (res != null) {
                    Object data = res.get("data");
                    if (data != null && data instanceof Map) {
                        Map map = (Map<String, Object>) data;
                        return map;
                    }
                }
            }
        } catch (RestClientException e) {
            log.error(String.format("调用接口失败：\n %s", rtsp), e);
        }
        return Collections.emptyMap();
    }

}