package com.ssi.task;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

import com.ssi.entity.dto.TelecontrolParamDto;
import com.ssi.model.RedisDataModel;
import com.ssi.model.TelecontrolModel;
import com.ssi.websocket.WebSocketServiceForDebugApp;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.listener.KeyExpirationEventMessageListener;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.stereotype.Component;

/**
 * 会话超时执行车辆刹车，确保安全性
 */
@Component
@Slf4j
public class KeyExpiredListener extends KeyExpirationEventMessageListener {

  private static final Logger LOGGER = LoggerFactory.getLogger(KeyExpiredListener.class);

  @Value("${harbor.remote.overtime}")
  private String debugAppOvertimePrefix;
  @Value("${harbor.remote.socketOvertime}")
  private String socketOvertimePrefix;
  @Autowired
  private TelecontrolModel telecontrolModel;
  @Autowired
  private RedisDataModel redisDataModel;

  public KeyExpiredListener(RedisMessageListenerContainer listenerContainer) {
    super(listenerContainer);
  }

  @Override
  public void onMessage(Message message, byte[] pattern) {
    String channel = new String(message.getChannel(), StandardCharsets.UTF_8);
    String key = new String(message.getBody(), StandardCharsets.UTF_8);
//        if (key.startsWith(debugAppOvertimePrefix)) {
//            LOGGER.info("redis key 过期：pattern={},channel={},key={}", new String(pattern), channel, key);
//            String vinMac = key.replace(debugAppOvertimePrefix + ":", "");
//            String[] strings = vinMac.split("&");
//            String vin = strings[0];
//            String appMac = strings[1];
//            TelecontrolParamDto paramDto = new TelecontrolParamDto();
//            paramDto.setVin(vin);
//            paramDto.setAppMac(appMac);
//            paramDto.setControlType(1);
//            paramDto.setOperationValue(2);
//            telecontrolModel.vehicleTelecontrol(paramDto);
//            Map msg = new HashMap<>();
//            msg.put("msgType", 1);
//            msg.put("vin", vin);
//            msg.put("appMac", appMac);
//            msg.put("message", "超时退出接管");
//            WebSocketServiceForDebugApp.messageBroadcast(vin, msg);
//        }
    if (key.startsWith(socketOvertimePrefix)) {
      String[] arr = key.split(":");
      String vin = arr[arr.length - 1];
      String sessionId = arr[arr.length - 2];
      log.info("判断sessionId：{},vin:{}是否超时", sessionId, vin);
      String socketOvertime = redisDataModel.get(
          socketOvertimePrefix + ":" + sessionId + ":" + vin);
      if (StringUtils.isNotBlank(socketOvertime)) {
        return;
      }
      LOGGER.error(String.format("会话：%s 超时.刹车：%s", sessionId, vin));
      telecontrolModel.breakVehicle(vin);
    }
  }
}