package com.ssi.task;

import com.ssi.constant.VehicleConstant;
import com.ssi.entity.dto.VmsVehicleDto;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.mqtt.MQTTPublishClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.util.Set;

/**
 * 车辆下电后关闭后视视频流
 *
 * @author ZhangLiYao
 * @version 1.0
 * @date 2020/8/6 10:00
 */
@Component
@EnableScheduling
public class StopVideoTask {

  @Autowired
  private RedisDataModel redisDataModel;
  @Value("${vehicle.video.keyPrefix:harbor:vehicle:video:status}")
  private String videoStatusKey;
  @Value("${vehicle.status:redis:keyPrefix:harbor:harbor:vehicle:online}")
  private String vehicleStatusKey;
  @Value("${mqtt.address}")
  private String mqttAddress;
  @Value("${mqtt.serverId}")
  private String mqttServerId;
  @Value("${mqtt.name}")
  private String mqttName;
  @Value("${mqtt.password}")
  private String mqttPassword;
  @Value("${mqtt.url}")
  private String mqttUrl;
  @Value("${mqtt.CAPath}")
  private String mqttCAPath;
  @Autowired
  private VmsVehicleMapper vmsVehicleMapper;

  @Scheduled(cron = "${task.stopVideoTask.cron}")
  private void getOrdersOfTask() {

    Set<String> keySet = redisDataModel.keys(videoStatusKey + "*");
    if (CollectionUtils.isEmpty(keySet)) {
      return;
    }
    for (String cacheKey : keySet) {
      if (StringUtils.isBlank(cacheKey)) {
        continue;
      }
      String videoStatus = redisDataModel.get(cacheKey);
      if (VehicleConstant.VIDEO_STATUS_OFF == Integer.valueOf(videoStatus)) {
        continue;
      }
      String vin = cacheKey.substring(22);
      String online = redisDataModel.get(String.format("%s:%s", vehicleStatusKey, vin));
      if (Integer.valueOf(online) == VehicleConstant.VEHICLE_STATUS_ONLINE) {
        continue;
      }

      //车辆不在线，关闭后视推流
      VmsVehicleDto vehicle = vmsVehicleMapper.queryVehicleByVin(vin);
      MQTTPublishClient mqttClientSend = new MQTTPublishClient(mqttAddress, mqttServerId,
          mqttCAPath, mqttName, mqttPassword, mqttUrl);
      mqttClientSend.mqttRequest(vehicle, VehicleConstant.START_PUSHER);
    }
  }
}
