package com.ssi.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Sets;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.ssi.entity.VmsVehicle;
import com.ssi.entity.VmsVehicleOperateTask;
import com.ssi.entity.dto.VehicleTerminalData;
import com.ssi.mapper.VmsVehicleOperateTaskMapper;
import com.ssi.service.VmsVehicleService;
import com.ssi.utils.DateUtils;
import com.ssi.utils.ElasticSearchUtil;
import com.ssi.utils.GpsUtil;
import org.elasticsearch.action.search.ClearScrollRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.Scroll;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author ZhangLiYao
 * @version 1.0
 * @date 2020/8/6 10:03
 */
@Component
public class TaskOperateExecutor {

  private static Logger log = LoggerFactory.getLogger(TaskOperateExecutor.class);

  @Autowired
  private VmsVehicleService vmsVehicleService;
  @Autowired
  private VmsVehicleOperateTaskMapper vmsVehicleOperateTaskMapper;

  @Autowired
  private RestHighLevelClient restHighLevelClient;

  @Value("${analysis.vehicle-data.es-index-name}")
  private String esVehicleDataIndexName;

  @Value("${analysis.vehicle-data.index-rule:year}")
  private String indexRule;

  public void submitOpreateAnalysis() {
    log.info("车辆运营数据定时任务开始执行-------");
    List<VmsVehicle> list = vmsVehicleService.list();
    list.stream().forEach(vehicle -> {

      VmsVehicleOperateTask vmsVehicleOperateTask = getVehicleOpreate(vehicle.getVin());
      int count = vmsVehicleOperateTaskMapper.selectCount(
          new LambdaQueryWrapper<VmsVehicleOperateTask>()
              .eq(VmsVehicleOperateTask::getVin, vehicle.getVin())
              .eq(VmsVehicleOperateTask::getTaskDate, vmsVehicleOperateTask.getTaskDate()));
      if (count <= 0) {
        vmsVehicleOperateTaskMapper.insert(vmsVehicleOperateTask);
      }
    });
    log.info("车辆运营数据定时任务执行完毕-------");

  }

  private VmsVehicleOperateTask getVehicleOpreate(String vin) {
    long[] yesterday = getYesterday();
    VmsVehicleOperateTask vmsVehicleOperateTask = new VmsVehicleOperateTask();
    vmsVehicleOperateTask = getVehicleOpreateByTime(vin, yesterday[0], yesterday[1]);
    return vmsVehicleOperateTask;
  }

  public VmsVehicleOperateTask getVehicleOpreateByTime(String vin, Long startTime, Long stopTime) {
    VmsVehicleOperateTask vmsVehicleOperateTask = new VmsVehicleOperateTask();
    vmsVehicleOperateTask.setVin(vin);
    vmsVehicleOperateTask.setOperateMile(new BigDecimal(0));
    vmsVehicleOperateTask.setOperateTime(new BigDecimal(0));
    vmsVehicleOperateTask.setMileVal(0D);
    vmsVehicleOperateTask.setTimeVal(0L);
    vmsVehicleOperateTask.setTaskDate(
        DateUtils.format(new Date(startTime), DateUtils.DATE_PATTERN));

    try {
      searchByTime(vmsVehicleOperateTask, startTime, stopTime);
    } catch (IOException e) {
      e.printStackTrace();
      log.error("查询数据失败，vin:" + vmsVehicleOperateTask.getVin());
    }

    vmsVehicleOperateTask.setOperateMile(
        BigDecimal.valueOf(vmsVehicleOperateTask.getMileVal()).setScale(2, BigDecimal.ROUND_UP));
    vmsVehicleOperateTask.setOperateTime(BigDecimal.valueOf(vmsVehicleOperateTask.getTimeVal())
        .divide(new BigDecimal(60 * 60 * 1000), 2, BigDecimal.ROUND_HALF_UP));
    return vmsVehicleOperateTask;
  }

  private void searchByTime(VmsVehicleOperateTask vmsVehicleOperateTask, Long startTime,
      Long stopTime) throws IOException {
    Set<String> columns = Sets.newHashSet("vin", "location", "longitude", "latitude", "collectTime"
        , "receiveTime", "speed");
    Gson gson = new GsonBuilder().create();
    int size = 10000;
    final Scroll scroll = new Scroll(TimeValue.timeValueMinutes(10L));
    SearchResponse response = getSearchResponse(vmsVehicleOperateTask.getVin(), startTime, stopTime,
        size, scroll, columns);
    String scrollId = response.getScrollId();
    SearchHit[] searchHits = response.getHits().getHits();
    VehicleTerminalData lastVehicleTerminalData = null;
    while (searchHits != null && searchHits.length > 0) {
      for (SearchHit hit : searchHits) {
        VehicleTerminalData currentVehicleTerminalData = null;
        String sourceAsString = hit.getSourceAsString();
        if (sourceAsString != null) {
          currentVehicleTerminalData = parseVehicleTerminalDate(gson, sourceAsString);
        }
        if (lastVehicleTerminalData != null && currentVehicleTerminalData != null) {
          long time = currentVehicleTerminalData.getCollectTime()
              - lastVehicleTerminalData.getCollectTime();
          if (time <= 60000) {
            caclMile(vmsVehicleOperateTask, lastVehicleTerminalData, currentVehicleTerminalData);

            caclTime(vmsVehicleOperateTask, lastVehicleTerminalData, currentVehicleTerminalData);
          }
        }
        lastVehicleTerminalData = currentVehicleTerminalData;
      }

      /* Next batch data. */
      if (searchHits.length == size) {
        response = getNextScrollResponse(scrollId, scroll);
        searchHits = response.getHits().getHits();
      } else {
        break;
      }
    }
    clearScroll(scrollId);
  }

  private void caclTime(VmsVehicleOperateTask vmsVehicleOperateTask,
      VehicleTerminalData lastVehicleTerminalData, VehicleTerminalData currentVehicleTerminalData) {
    long time =
        currentVehicleTerminalData.getCollectTime() - lastVehicleTerminalData.getCollectTime();
    vmsVehicleOperateTask.setTimeVal(vmsVehicleOperateTask.getTimeVal() + time);
  }

  private void caclMile(VmsVehicleOperateTask vmsVehicleOperateTask,
      VehicleTerminalData lastVehicleTerminalData, VehicleTerminalData currentVehicleTerminalData) {
//        Double dist = GpsUtil.getDistance(lastVehicleTerminalData.getLatitude(),
//                lastVehicleTerminalData.getLongitude(), currentVehicleTerminalData.getLatitude(),
//                currentVehicleTerminalData.getLongitude());
//        if (dist > 0 && dist < 1 && currentVehicleTerminalData.getSpeed().compareTo(BigDecimal.ZERO)!=0) {
//            vmsVehicleOperateTask.setMileVal(vmsVehicleOperateTask.getMileVal() + dist);
//        }
    if (lastVehicleTerminalData.getSpeed().doubleValue() >= 0.0
        && currentVehicleTerminalData.getSpeed().doubleValue() >= 0.0) {
      double curSpeed = currentVehicleTerminalData.getSpeed().doubleValue();
      double lastSpeed = lastVehicleTerminalData.getSpeed().doubleValue();
      double avgSpeed = (lastSpeed + curSpeed) / 2;
      long time =
          currentVehicleTerminalData.getCollectTime() - lastVehicleTerminalData.getCollectTime();
      double mile = avgSpeed / 60 / 60 * (time / 1000);
      vmsVehicleOperateTask.setMileVal(vmsVehicleOperateTask.getMileVal() + mile);

    }
  }

  private long[] getYesterday() {
    Calendar calendar = Calendar.getInstance();
    calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
        calendar.get(Calendar.DAY_OF_MONTH) - 1, 0, 0, 0);
    long start = calendar.getTime().getTime();
    Calendar calendar2 = Calendar.getInstance();
    calendar2.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
        calendar.get(Calendar.DAY_OF_MONTH) - 1, 23, 59, 59);
    long end = calendar2.getTime().getTime();
    long[] result = new long[]{start, end};
    return result;
  }

  private VehicleTerminalData parseVehicleTerminalDate(Gson gson, String jsonStr) {
    try {
      return gson.fromJson(jsonStr, VehicleTerminalData.class);
    } catch (Throwable t) {
      log.error("[Analysis Executor]Fail to convert VehicleTerminalDate vmstoorders.\n" +
          jsonStr, t);
    }

    return null;
  }

  private SearchResponse getSearchResponse(String vin, Long startTime, Long stopTime, int size,
      Scroll scroll, Set<String> columns) throws IOException {
    BoolQueryBuilder boolBuilder = QueryBuilders.boolQuery();
    SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
    sourceBuilder.size(size);

    MatchQueryBuilder matchQueryBuilder = QueryBuilders.matchQuery("vin", vin);

    RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("collectTime");
    rangeQueryBuilder.gte(startTime);
    rangeQueryBuilder.lte(stopTime);

    boolBuilder.must(rangeQueryBuilder);
    boolBuilder.must(matchQueryBuilder);

    FieldSortBuilder fieldSortBuilder = new FieldSortBuilder("collectTime");

    sourceBuilder.query(boolBuilder);
    sourceBuilder.sort(fieldSortBuilder);
    sourceBuilder.timeout(new TimeValue(600, TimeUnit.SECONDS));
    sourceBuilder.fetchSource(columns.toArray(new String[columns.size()]), null);

    String[] indexs = ElasticSearchUtil.getIndexs(startTime, stopTime, esVehicleDataIndexName, 1,
        true, indexRule);
    SearchRequest searchRequest = new SearchRequest(indexs);
    searchRequest.scroll(scroll);
    searchRequest.source(sourceBuilder);

    return restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
  }

  private SearchResponse getNextScrollResponse(String scrollId, Scroll scroll) throws IOException {
    SearchScrollRequest scrollRequest = new SearchScrollRequest(scrollId);
    scrollRequest.scroll(scroll);

    return restHighLevelClient.scroll(scrollRequest, RequestOptions.DEFAULT);
  }

  private void clearScroll(String scrollId) throws IOException {
    ClearScrollRequest clearScrollRequest = new ClearScrollRequest();
    clearScrollRequest.addScrollId(scrollId);

    restHighLevelClient.clearScroll(clearScrollRequest, RequestOptions.DEFAULT);

  }


}
