package com.ssi.task;

import com.ssi.service.VmsTosOrdersAnalysisService;
import com.ssi.service.VmsTosOrdersOeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;

/**
 * 车辆任务分析任务
 */
@Component
@EnableScheduling
public class TosAnalysisTask {

  @Autowired
  private VmsTosOrdersAnalysisService vmsTosOrdersAnalysisService;
  @Autowired
  private VmsTosOrdersOeeService vmsTosOrdersOeeService;

  /**
   * 装卸箱、理货分析任务
   */
  @Scheduled(cron = "${task.tosAnalysisTask.cron}")
  public void runTosAnalysisTask() {
    LocalDateTime startTime = LocalDateTime.of(
        LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MIN);
    LocalDateTime endTime = LocalDateTime.of(
        LocalDateTime.now().minus(1, ChronoUnit.DAYS).toLocalDate(), LocalTime.MAX);
    vmsTosOrdersAnalysisService.runTosAnalysisTask(startTime, endTime);
  }

  /**
   * 任务执行OEE分析任务
   */
  @Scheduled(cron = "${task.tosOeeTask.cron}")
  public void runTosOeeTask() {
    LocalDateTime endTime = LocalDateTime.now();
    LocalDateTime startTime = endTime.minus(24, ChronoUnit.HOURS);
    vmsTosOrdersOeeService.runTosOeeTask(startTime, endTime);
  }
}
