package com.ssi.utils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
/**
 * 贝位方法相关工具类
 */
public class BayNumUtil {
    private static final double EARTH_RADIUS = 6378.137;

    /**
     * 根据经纬度，计算两点间的距离
     *
     * @param longitude1 第一个点的经度
     * @param latitude1  第一个点的纬度
     * @param longitude2 第二个点的经度
     * @param latitude2  第二个点的纬度
     * @return 返回距离 单位千米
     */
    public static double getDistance(double longitude1, double latitude1, double longitude2, double latitude2) {
        // 纬度
        double lat1 = Math.toRadians(latitude1);
        double lat2 = Math.toRadians(latitude2);
        // 经度
        double lng1 = Math.toRadians(longitude1);
        double lng2 = Math.toRadians(longitude2);
        // 纬度之差
        double a = lat1 - lat2;
        // 经度之差
        double b = lng1 - lng2;
        // 计算两点距离的公式
        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) +
                Math.cos(lat1) * Math.cos(lat2) * Math.pow(Math.sin(b / 2), 2)));
        // 弧长乘地球半径, 返回单位: 千米
        s =  s * EARTH_RADIUS;
        return s;
    }
    public static void main(String[] args) {
        //获取四个经纬点换算出的中心点
        bayNumUtil("C:\\Users\\hekai\\Desktop\\采集定位点\\BayNum.json");
        //locationTrans("D:\\locationTrans.geojson");
    }

    /**
     * 根据四个点换算出经纬度
     * @param path 本地文件
     */
    public static  void bayNumUtil(String path) {

        //[[[117.96810563174562,24.45700681481371],
        // [117.96810435443824,24.457001924664915],
        // [117.96811146404536,24.45700069948276],
        // [117.96811260436358,24.457005485843936],
        // [117.96810563174562,24.45700681481371]]]}
        //        double lng=117.96810563174562;
        //        double lat=24.45700681481371;
        //        double lng1=117.96810435443824;
        //        double lat1=24.457001924664915;
        //        double lng2=117.96811146404536;
        //        double lat2=24.45700069948276;
        //        double lng3=117.96811260436358;
        //        double lat3=24.457005485843936;
        //        double totleLng=(lng+lng1+lng2 +lng3)/4;
        //        double totleLat=(lat+lat1+lat2 +lat3)/4;
        //        System.out.println((totleLng+0.00000435));
        //        System.out.println((totleLat+0.0000185));
        String s = readJsonFile(path);
        JSONObject jobj = JSON.parseObject(s);
        JSONArray features = jobj.getJSONArray("features");
        for (int i = 0; i < features.size(); i++) {
            JSONObject key1 = (JSONObject) features.get(i);
            JSONObject geometry = key1.getJSONObject("geometry");
            JSONObject properties = key1.getJSONObject("properties");
            String Yard = (String) properties.get("Yard");
            String Num = (String) properties.get("Num");
            if (Yard.equals("617")) {
                JSONArray jsonArray = geometry.getJSONArray("coordinates");
                JSONArray jsonArray2 = jsonArray.getJSONArray(0);
                double lng = 0;
                double lat = 0;
                double lng1 = 0;
                double lat1 = 0;
                double lng2 = 0;
                double lat2 = 0;
                double lng3 = 0;
                double lat3 = 0;
                for (int j = 0; j < jsonArray2.size(); j++) {
                    if (j == 0) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(0);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng = Double.parseDouble(jsonArray1.get(0).toString());
                            lat = Double.parseDouble(jsonArray1.get(1).toString());
                        }

                    }
                    if (j == 1) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(1);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng1 = Double.parseDouble(jsonArray1.get(0).toString());
                            lat1 = Double.parseDouble(jsonArray1.get(1).toString());
                        }

                    }
                    if (j == 2) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(2);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng2 = Double.parseDouble(jsonArray1.get(0).toString());
                            lat2 = Double.parseDouble(jsonArray1.get(1).toString());
                        }

                    }
                    if (j == 3) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(3);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng3 = Double.parseDouble(jsonArray1.get(0).toString());
                            lat3 = Double.parseDouble(jsonArray1.get(1).toString());
                        }
                    }
                }
                double totleLng = (lng + lng1 + lng2 + lng3) / 4;
                double totleLat = (lat + lat1 + lat2 + lat3) / 4;
                //向上是用减去对应的距离
                System.out.println("经度 : " + new BigDecimal(totleLng - 0.00000435).setScale(8,BigDecimal.ROUND_HALF_UP)
                                +" ,纬度 : " + new BigDecimal(totleLat - 0.0000185).setScale(8,BigDecimal.ROUND_HALF_UP)
                                + " ,"+ "Yard : " + Yard + " ,Num : " + Num);
            }
            if (Yard.equals("604")) {
                JSONArray jsonArray = geometry.getJSONArray("coordinates");
                JSONArray jsonArray2 = jsonArray.getJSONArray(0);
                double lng = 0;
                double lat = 0;
                double lng1 = 0;
                double lat1 = 0;
                double lng2 = 0;
                double lat2 = 0;
                double lng3 = 0;
                double lat3 = 0;
                for (int j = 0; j < jsonArray2.size(); j++) {
                    if (j == 0) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(0);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng = Double.parseDouble(jsonArray1.get(0).toString());
                            lat = Double.parseDouble(jsonArray1.get(1).toString());
                        }

                    }
                    if (j == 1) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(1);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng1 = Double.parseDouble(jsonArray1.get(0).toString());
                            lat1 = Double.parseDouble(jsonArray1.get(1).toString());
                        }

                    }
                    if (j == 2) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(2);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng2 = Double.parseDouble(jsonArray1.get(0).toString());
                            lat2 = Double.parseDouble(jsonArray1.get(1).toString());
                        }

                    }
                    if (j == 3) {
                        JSONArray jsonArray1 = jsonArray2.getJSONArray(3);
                        for (int k = 0; k < jsonArray1.size(); k++) {
                            lng3 = Double.parseDouble(jsonArray1.get(0).toString());
                            lat3 = Double.parseDouble(jsonArray1.get(1).toString());
                        }
                    }
                }
                double totleLng =(lng + lng1 + lng2 + lng3) / 4;
                double totleLat = (lat + lat1 + lat2 + lat3) / 4;
                //向下是用加上对应的距离
                System.out.println("经度 : "+ new BigDecimal(totleLng + 0.00000435).setScale(8,BigDecimal.ROUND_HALF_UP)
                                 + " ,纬度 : "+ new BigDecimal(totleLat + 0.0000185).setScale(8,BigDecimal.ROUND_HALF_UP)
                                 + " ,Yard : " + Yard + " ,Num : " + Num );
            }
        }
    }

    /**
     * 根据文件换算出重复的贝位点重新排序
     * @param path
     */
    public static void locationTrans(String path){
        String s = readJsonFile(path);
        JSONObject jobj = JSON.parseObject(s);
        JSONArray features = jobj.getJSONArray("features");
        Integer num = 0;
        String lats="0.0000065";//纬度差距
        String lngs="0.000031";//经纬差距
        Integer type=0;
        Integer eType=0;
        Integer eType01=0;
        Integer eType03=0;
        Integer eType04=0;
        Integer eType05=0;
        Integer eType06=0;
        Integer eType07=0;
        Integer eType08=0;
        Integer eType09=0;
        Integer dType03=0;
        Integer dType05=0;
        Integer dType07=0;
        Integer dType09=0;
        Integer dType001=0;
        Integer dType003=0;
        for (int i = 0; i < features.size(); i++) {
            JSONObject key1 = (JSONObject)features.get(i);
            JSONObject properties=key1.getJSONObject("properties");
            String stockDumpNum  = (String)properties.get("堆场编号");
            String stockDumpRanking = (String)properties.get("堆场排位");
            String bayNum=(String)properties.get("贝位编号");
            //A区
//            if (
//                    (stockDumpNum.contains("A01") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A02") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A03") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A04") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A05") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A06") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A07") && stockDumpRanking.equals("01"))||
//                    (stockDumpNum.contains("A08") && stockDumpRanking.equals("01"))
//            ){
//                if (bayNum.contains("04")){
//                    if (type==0){
//                        properties.put("贝位编号","61");
//                        String longitudes=(String)properties.get("贝位中心点");
//                        String lons[]=longitudes.split(",");
//                        String lat=lons[0];//经度
//                        String lng=lons[1];//纬度
//                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
//                        type=1;
//                    }else{
//                        properties.put("贝位编号","63");
//                        String longitudes=(String)properties.get("贝位中心点");
//                        String lons[]=longitudes.split(",");
//                        Integer fid=(Integer)properties.get("FID");
//                        properties.put("FID",fid+12580);
//                        key1.put("id",fid+12580);
//                        String lat=lons[0];//经度
//                        String lng=lons[1];//纬度
//                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
//                        type=0;
//                    }
//                }else{
//                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
//                        num = Integer.parseInt(bayNum) + 4;
//                        if (num < 10) {
//                            properties.put("贝位编号", "0" + num.toString());
//                        } else {
//                            properties.put("贝位编号", num.toString());
//                        }
//                    }
//                }
//                if(bayNum.contains("001")){
//                    properties.put("贝位编号","01");
//                    String longitudes=(String)properties.get("贝位中心点");
//                    String lons[]=longitudes.split(",");
//                    String lat=lons[0];//经度
//                    String lng=lons[1];//纬度
//                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
//                }
//                if(bayNum.contains("003")){
//                    properties.put("贝位编号","03");
//                    Integer fid=(Integer)properties.get("FID");
//                    properties.put("FID",fid+12580);
//                    key1.put("id",fid+12580);
//                    String longitudes=(String)properties.get("贝位中心点");
//                    String lons[]=longitudes.split(",");
//                    String lat=lons[0];//经度
//                    String lng=lons[1];//纬度
//                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
//                }
//            }
            //B区
            if (
                    stockDumpNum.contains("B01") && stockDumpRanking.equals("01")
            ){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","65");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        num = Integer.parseInt(bayNum) + 4;
                        if (num < 10) {
                            properties.put("贝位编号", "0" + num.toString());
                        } else {
                            properties.put("贝位编号", num.toString());
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            //C区
            if (
                    (stockDumpNum.contains("C01") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C02") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C03") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C04") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C05") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C06") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C07") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C08") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("C09") && stockDumpRanking.equals("01"))
            ){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","65");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","67");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        num = Integer.parseInt(bayNum) + 4;
                        if (num < 10) {
                            properties.put("贝位编号", "0" + num.toString());
                        } else {
                            properties.put("贝位编号", num.toString());
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            //D区
            if (
                    (stockDumpNum.contains("D01") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D02") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D03") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D04") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D05") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D06") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D07") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D08") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D09") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D13") && stockDumpRanking.equals("01"))||
                            (stockDumpNum.contains("D14") && stockDumpRanking.equals("01"))

            ){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","65");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","67");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }  else {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("D12") && stockDumpRanking.equals("01")){
                if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                    if (bayNum.equals("03")){
                        if(dType03==0){
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                            dType03=1;
                        }else{
                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        }
                    }else if(bayNum.equals("05")){
                        if(dType05==0){
                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                            dType05=1;
                        }else{
                            num = Integer.parseInt(bayNum) + 8;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        }
                    }else if(bayNum.equals("07")){
                        if(dType07==0){
                            num = Integer.parseInt(bayNum) + 8;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                            dType07=1;
                        }else{
                            num = Integer.parseInt(bayNum) + 10;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        }
                    }else if(bayNum.equals("09")){
                        if(dType09==0){
                            num = Integer.parseInt(bayNum) + 10;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                            dType09=1;
                        }else{
                            num = Integer.parseInt(bayNum) + 12;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                            String longitudes=(String)properties.get("贝位中心点");
                            String lons[]=longitudes.split(",");
                            String lat=lons[0];//经度
                            String lng=lons[1];//纬度
                            properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        }
                    }else{
                        if (bayNum.equals("01")){
                            num = Integer.parseInt(bayNum) + 4;
                        }else{
                            num = Integer.parseInt(bayNum) + 12;
                        }

                        if (num < 10) {
                            properties.put("贝位编号", "0" + num.toString());
                        } else {
                            properties.put("贝位编号", num.toString());
                        }
                    }
                }

                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }

            }
            if (stockDumpNum.contains("D11") && stockDumpRanking.equals("01")){
                if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                    num = Integer.parseInt(bayNum) + 8;
                    if (num < 10) {
                        properties.put("贝位编号", "0" + num.toString());
                    } else {
                        properties.put("贝位编号", num.toString());
                    }
                }
                if(bayNum.contains("001")){
                    if (dType001==0){
                        properties.put("贝位编号","01");
                        dType001=1;
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                    }else{
                        properties.put("贝位编号","03");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                    }


                }
                if(bayNum.contains("003")){
                    if (dType003==0){
                        properties.put("贝位编号","05");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        dType003=1;
                    }else{
                        properties.put("贝位编号","07");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                    }


                }

            }
            ///////////////E区
            if (stockDumpNum.contains("E01") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType01 == 0) {
                                num = Integer.parseInt(bayNum) + 6;
                                eType01 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 4;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {
                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E02") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E03") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType03 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType03 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E04") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType04 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType04 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E05") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType05 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType05 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E06") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType06 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType06 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E07") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType07 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType07 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E08") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType08 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType08 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E09") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        if (Integer.parseInt(bayNum) < 17) {
                            num = Integer.parseInt(bayNum) + 4;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        } else if (Integer.parseInt(bayNum) == 17) {
                            if (eType09 == 0) {
                                num = Integer.parseInt(bayNum) + 4;
                                eType09 = 1;
                            } else {
                                num = Integer.parseInt(bayNum) + 6;
                            }
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }

                        } else {

                            num = Integer.parseInt(bayNum) + 6;
                            if (num < 10) {
                                properties.put("贝位编号", "0" + num.toString());
                            } else {
                                properties.put("贝位编号", num.toString());
                            }
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E12") && stockDumpRanking.equals("01")){
                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")) {
                        num = Integer.parseInt(bayNum) + 4;
                        if (num < 10) {
                            properties.put("贝位编号", "0" + num.toString());
                        } else {
                            properties.put("贝位编号", num.toString());
                        }
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E14") && stockDumpRanking.equals("01")){
                if(bayNum.contains("01") ){
                    num = Integer.parseInt(bayNum) + 4;
                    if (num < 10) {
                        properties.put("贝位编号", "29");
                    } else {
                        properties.put("贝位编号", num.toString());
                    }
                }else if (bayNum.contains("03")){

                    properties.put("贝位编号", "31");
                }else{
                    if (!bayNum.contains("001")||!bayNum.contains("003")){
                        num = Integer.parseInt(bayNum) + 1;
                        if (num < 10) {
                            properties.put("贝位编号", "0" + num.toString());
                        } else {
                            properties.put("贝位编号", num.toString());
                        }
                    }

                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            if (stockDumpNum.contains("E15") && stockDumpRanking.equals("01")){
                if(Integer.parseInt(bayNum)%2==1 && !bayNum.contains("001") && !bayNum.contains("003")){
//                    num = Integer.parseInt(bayNum) + 4;
//                    if (num < 10) {
//                        properties.put("贝位编号", "0" + num.toString());
//                    } else {
//                        properties.put("贝位编号", num.toString());
//                    }
                }else{
                    num = Integer.parseInt(bayNum) + 21;
                    if (num < 10) {
                        properties.put("贝位编号", "0" + num.toString());
                    } else {
                        properties.put("贝位编号", num.toString());
                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","21");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","23");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
            //F区
            if ((stockDumpNum.contains("F01") && stockDumpRanking.equals("01")) ||
                    (stockDumpNum.contains("F02") && stockDumpRanking.equals("01")) ||
                    (stockDumpNum.contains("F03") && stockDumpRanking.equals("01")) ||
                    (stockDumpNum.contains("F04") && stockDumpRanking.equals("01")) ||
                    (stockDumpNum.contains("F05") && stockDumpRanking.equals("01")) ||
                    (stockDumpNum.contains("F06") && stockDumpRanking.equals("01")) ||
                    (stockDumpNum.contains("F07") && stockDumpRanking.equals("01"))
            ){

                if (bayNum.contains("04")){
                    if (type==0){
                        properties.put("贝位编号","61");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));
                        type=1;
                    }else{
                        properties.put("贝位编号","63");
                        String longitudes=(String)properties.get("贝位中心点");
                        String lons[]=longitudes.split(",");
                        Integer fid=(Integer)properties.get("FID");
                        properties.put("FID",fid+12580);
                        key1.put("id",fid+12580);
                        String lat=lons[0];//经度
                        String lng=lons[1];//纬度
                        properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                        type=0;
                    }
                }else{
                    if(!bayNum.equals("001")&&!bayNum.equals("003")){
                        num=Integer.parseInt(bayNum)+4;
                        if (num<10){
                            properties.put("贝位编号","0"+num.toString());
                        }else {
                            properties.put("贝位编号",num.toString());
                        }

                    }
                }
                if(bayNum.contains("001")){
                    properties.put("贝位编号","01");
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).add(new BigDecimal(lats))+","+new BigDecimal(lng).subtract(new BigDecimal(lngs)));

                }
                if(bayNum.contains("003")){
                    properties.put("贝位编号","03");
                    Integer fid=(Integer)properties.get("FID");
                    properties.put("FID",fid+12580);
                    key1.put("id",fid+12580);
                    String longitudes=(String)properties.get("贝位中心点");
                    String lons[]=longitudes.split(",");
                    String lat=lons[0];//经度
                    String lng=lons[1];//纬度
                    properties.put("贝位中心点",new BigDecimal(lat).subtract(new BigDecimal(lats))+","+new BigDecimal(lng).add(new BigDecimal(lngs)));
                }
            }
        }
        num=0;
        for (int i = 0; i < features.size(); i++) {
            JSONObject key1 = (JSONObject)features.get(i);
            JSONObject properties=key1.getJSONObject("properties");
            String stockDumpNum  = (String)properties.get("堆场编号");
            String stockDumpRanking = (String)properties.get("堆场排位");
            String bayNum=(String)properties.get("贝位编号");
            String longitudes=(String)properties.get("贝位中心点");
            Integer fid=(Integer)properties.get("FID");
//E15,E14双数
            if (stockDumpNum.contains("E01") && stockDumpRanking.equals("01")){
                num=num+1;
                System.out.println(" 堆场编号:"+stockDumpNum+" 堆场排位:"+stockDumpRanking+" 贝位编号："+bayNum+" 贝位中心点:"+longitudes+" FID:"+fid+" num:"+num);
            }
        }


        BufferedWriter writer = null;
        File file = new File("d:\\test.geojson");
        //如果文件不存在，则新建一个
        if(!file.exists()){
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        //写入
        try {
            writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file,false), "UTF-8"));
            writer.write(JSONObject.toJSONString(jobj));
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            try {
                if(writer != null){
                    writer.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        System.out.println("文件写入成功！");


    }
    public static String readJsonFile(String fileName) {
        String jsonStr = "";
        try {
            File jsonFile = new File(fileName);
            FileReader fileReader = new FileReader(jsonFile);

            Reader reader = new InputStreamReader(new FileInputStream(jsonFile),"utf-8");
            int ch = 0;
            StringBuffer sb = new StringBuffer();
            while ((ch = reader.read()) != -1) {
                sb.append((char) ch);
            }
            fileReader.close();
            reader.close();
            jsonStr = sb.toString();
            return jsonStr;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
}
