package com.ssi.utils;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Getter;
import lombok.Setter;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Getter
@Setter
public class ChargeDailyTimeUtil {

  private Date day;

  private List<ChargeDailyTimeHourBean> dailyTimeBeanList = new ArrayList<>();

  public ChargeDailyTimeUtil(Date date) {
    this.day = date;
    this.init();
  }

  private void init() {
    for (int i = 0; i < 24; i++) {
      ChargeDailyTimeHourBean dailyTimeBean = new ChargeDailyTimeHourBean(day, i);
      dailyTimeBeanList.add(dailyTimeBean);
    }
  }

  public void add(long startTime, long endTime, double energy) {
    for (int i = 0; i < dailyTimeBeanList.size(); i++) {
      ChargeDailyTimeHourBean dailyTimeBean = dailyTimeBeanList.get(i);
      long rangeStartTime = dailyTimeBean.getRangeStartTime();
      long rangeEndTime = dailyTimeBean.getRangeEndTime();
      long s = startTime >= rangeStartTime ? startTime : rangeStartTime;
      long e = endTime <= rangeEndTime ? endTime : rangeEndTime;
      if (e > s) {
        dailyTimeBean.addDuration((double) (e - s) / 1000 / 3600);
        dailyTimeBean.addEnergy((double) (e - s) / (endTime - startTime) * energy);
      }
    }
  }

  public void finish(int pileNum) {
    for (int i = 0; i < dailyTimeBeanList.size(); i++) {
      ChargeDailyTimeHourBean dailyTimeHourBean = dailyTimeBeanList.get(i);
      dailyTimeHourBean.chargingPileUseRate = new BigDecimal(
          dailyTimeHourBean.getDuration() / pileNum)
          .setScale(4, RoundingMode.HALF_UP).doubleValue();
    }
  }

  @Getter
  @Setter
  public class ChargeDailyTimeHourBean {

    private Date date;

    private int hour;

    @JsonIgnore
    private long rangeStartTime;

    @JsonIgnore
    private long rangeEndTime;

    private double duration;

    private double chargingPileUseRate;

    private double chargingEnergy;

    public ChargeDailyTimeHourBean(Date date, int hour) {
      this.date = date;
      this.hour = hour;
      long dayStartTime = date.getTime();
      this.rangeStartTime = dayStartTime + hour * 3600 * 1000;
      this.rangeEndTime = dayStartTime + (hour + 1) * 3600 * 1000;
    }

    public void addDuration(double value) {
      this.duration = duration + value;
    }

    public void addEnergy(double value) {
      this.chargingEnergy = chargingEnergy + value;
    }
  }
}
