package com.ssi.utils;

import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;

/**
 * 日期处理
 */
public class DateUtils {

  /**
   * 时间格式(yyyy-MM-dd)
   */
  public final static String DATE_PATTERN = "yyyy-MM-dd";

  /**
   * 年份格式
   */
  public static final String DATE_PATTERN_yyyy = "yyyy";

  /**
   * 时间格式(yyyy-MM-dd HH:mm:ss)
   */
  public final static String DATE_TIME_PATTERN = "yyyy-MM-dd HH:mm:ss";

  private static final ZoneId zone = ZoneId.systemDefault();

  public static String format(Date date, int flag) {
    if (flag == 1) {
      return format(date, DATE_PATTERN);
    } else if (flag == 2) {
      return format(date, DATE_TIME_PATTERN);
    }
    return null;
  }

  public static String format(Date date, String pattern) {
    if (date != null) {
      SimpleDateFormat df = new SimpleDateFormat(pattern);
      return df.format(date);
    }
    return null;
  }

  /**
   * 将时间戳转换为 LocalDateTime
   * @param timestamp
   * @return
   */
  public static LocalDateTime parseLocalDateTime(Long timestamp) {
    Instant instant = Instant.ofEpochMilli(timestamp);
    return LocalDateTime.ofInstant(instant, zone);
  }

  /**
   * 获取年份数据
   * @param timestamp
   * @return
   */
  public static String getYear(Long timestamp) {
    return parseLocalDateTime(timestamp).format(DateTimeFormatter.ofPattern(DATE_PATTERN_yyyy));
  }

}
