package com.ssi.utils;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.*;

public class FTPUtil {

  private String ip;
  private int port;
  private String user;
  private String pwd;
  private FTPClient ftpClient;

  //省略get set

  private static final Logger logger = LoggerFactory.getLogger(FTPUtil.class);

  //获取配置文件中的配置参数
//    private static String ftpIp = PropertiesUtil.getProperty("ftp.ip");
//    private static String ftpUser = PropertiesUtil.getProperty("ftp.user");
//    private static String ftpPwd = PropertiesUtil.getProperty("ftp.password");
//    private static int ftpPort = Integer.valueOf(PropertiesUtil.getProperty("ftp.port")).intValue();
  //构造方法，赋值value
  public FTPUtil(String ip, int port, String user, String pwd) {
    this.ip = ip;
    this.port = port;
    this.user = user;
    this.pwd = pwd;
  }

  //外部调用方法
//    public static boolean uploadFile(List<File> files) throws IOException {
//        FTPUtil ftpUtil = new FTPUtil(ftpIp,ftpPort,ftpUser,ftpPwd);
//        logger.info("开始连接ftp服务器！");
//        //当天的年月日作为目录名称
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
//        String catalogue = sdf.format(new Date());
//        boolean result = ftpUtil.uploadFile(catalogue,files);
//        logger.info("开始连接ftp服务器,结束上传,上传结果:{}",result);
//        return result;
//    }
  //外部调用方法
  public static boolean uploadFile(List<MultipartFile> files, String ip, int port, String user,
      String password, String remotePath, List<String> fileNames) throws IOException {
    FTPUtil ftpUtil = new FTPUtil(ip, port, user, password);
    logger.info("开始连接ftp服务器！");
    //当天的年月日作为目录名称
        /*SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String catalogue = sdf.format(new Date());*/
    boolean result = ftpUtil.uploadFiles(remotePath, files, fileNames);
    logger.info("开始连接ftp服务器,结束上传,上传结果:{}", result);
    return result;
  }

  //外部调用方法
  public static boolean uploadSingleFile(MultipartFile file, String ip, int port, String user,
      String password, String remotePath, String fileName) throws IOException {
    FTPUtil ftpUtil = new FTPUtil(ip, port, user, password);
    logger.info("开始连接ftp服务器！");
    //当天的年月日作为目录名称
        /*SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String catalogue = sdf.format(new Date());*/
    boolean result = ftpUtil.uploadSingleFile(remotePath, file, fileName);
    logger.info("开始连接ftp服务器,结束上传,上传结果:{}", result);
    return result;
  }


  //外部调用方法
  public static Map uploadFileForAPK(List<MultipartFile> files, String ip, int port, String user,
      String password, String remotePath, List<String> fileNames) throws IOException {
    FTPUtil ftpUtil = new FTPUtil(ip, port, user, password);
    logger.info("开始连接ftp服务器！");
    //当天的年月日作为目录名称
        /*SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String catalogue = sdf.format(new Date());*/
    Map map = ftpUtil.uploadFilesForAPK(remotePath, files, fileNames);
    logger.info("开始连接ftp服务器,结束上传,上传结果:{}", map.get("uploaded"));
    return map;
  }

  //内部调用方法，连接ftp
  private boolean uploadFile(String remotePath, List<File> files) throws IOException {
    boolean uploaded = true;
    FileInputStream fis = null;
    if (connectServer(this.ip, this.port, this.user, this.pwd)) {
      try {
        //判断这个目录是否存在，不存在则进行创建
        ftpClient.makeDirectory(remotePath);
        //切换到remodePath目录
        ftpClient.changeWorkingDirectory(remotePath);
        //设置1m缓冲
        ftpClient.setBufferSize(1024);
        //设置编码
        ftpClient.setControlEncoding("UTF-8");
        //设置文件类型：二进制文件
        ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
        //被动模式
        ftpClient.enterLocalPassiveMode();
        for (File file : files) {
          fis = new FileInputStream(file);
          ftpClient.storeFile(file.getName(), fis);
        }
      } catch (IOException e) {
        logger.error("上传文件异常", e);
        uploaded = false;
      } finally {
        if (fis != null) {
          fis.close();
        }
        ftpClient.disconnect();
      }
    }
    return uploaded;
  }

  //内部调用方法，连接ftp
  public boolean uploadFile(String remotePath, String fileHttpPath) throws IOException {
    boolean uploaded = true;
    FileInputStream fis = null;
    if (connectServer(this.ip, this.port, this.user, this.pwd)) {
      try {
        //切换到remodePath目录
        boolean flag4 = ftpClient.changeWorkingDirectory(remotePath);
//                boolean flag5 = ftpClient.changeWorkingDirectory("/PicHouse/LFWADRJF011002346/samplePic/");
        //设置1m缓冲
        ftpClient.setBufferSize(1024);
        //设置编码
        ftpClient.setControlEncoding("UTF-8");
        //设置文件类型：二进制文件
        ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
        //被动模式
        ftpClient.enterLocalPassiveMode();
        String imgAddress = fileHttpPath;
        URL url = new URL(imgAddress);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        // 设置连接超时时间
        conn.setConnectTimeout(3000);
        InputStream inputStream = null;

        // 正常响应时获取输入流, 在这里也就是图片对应的字节流
        if (conn.getResponseCode() == 200) {
          inputStream = conn.getInputStream();
        }
        String[] path = fileHttpPath.split("/");
        uploaded = ftpClient.storeFile(path[path.length - 1], inputStream);
      } catch (IOException e) {
        logger.error("上传文件异常", e);
        uploaded = false;
      } finally {
        if (fis != null) {
          fis.close();
        }
        ftpClient.disconnect();
      }
    }
    return uploaded;
  }

  private boolean uploadFiles(String remotePath, List<MultipartFile> files, List<String> fileNames)
      throws IOException {
    boolean uploaded = true;
    if (connectServer(this.ip, this.port, this.user, this.pwd)) {
      try {
        //判断这个目录是否存在，不存在则进行创建
        ftpClient.makeDirectory(remotePath);
        //切换到remodePath目录
        ftpClient.changeWorkingDirectory(remotePath);
        //设置1m缓冲
        ftpClient.setBufferSize(10240);
        //设置编码
        ftpClient.setControlEncoding("UTF-8");
        //设置文件类型：二进制文件
        ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
        //被动模式
        ftpClient.enterLocalPassiveMode();

        for (int i = 0; i < files.size(); i++) {
          MultipartFile file = files.get(i);
          if (StringUtils.isBlank(file.getOriginalFilename())) {
            continue;
          }
          ftpClient.storeFile(fileNames.get(i), file.getInputStream());
        }

        return true;
      } catch (IOException e) {
        logger.error("上传文件异常", e);
        uploaded = false;
      } finally {
        ftpClient.disconnect();
      }
    }
    return uploaded;
  }

  private boolean uploadSingleFile(String remotePath, MultipartFile file, String fileName)
      throws IOException {
    if (connectServer(this.ip, this.port, this.user, this.pwd)) {
      try {
        //判断这个目录是否存在，不存在则进行创建
        ftpClient.makeDirectory(remotePath);
        //切换到remodePath目录
        ftpClient.changeWorkingDirectory(remotePath);
        //设置1m缓冲
        ftpClient.setBufferSize(10240);
        //设置编码
        ftpClient.setControlEncoding("UTF-8");
        //设置文件类型：二进制文件
        ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
        //被动模式
        ftpClient.enterLocalPassiveMode();
        ftpClient.storeFile(fileName, file.getInputStream());
        return true;
      } catch (IOException e) {
        logger.error("上传文件异常", e);
        return false;
      } finally {
        ftpClient.disconnect();
      }
    } else {
      return false;
    }
  }


  private Map uploadFilesForAPK(String remotePath, List<MultipartFile> files,
      List<String> fileNames) throws IOException {
    Map<String, Object> resultMap = new HashMap<>();
    boolean uploaded = true;
    String name = null;
    if (connectServer(this.ip, this.port, this.user, this.pwd)) {
      try {
        //判断这个目录是否存在，不存在则进行创建
        ftpClient.makeDirectory(remotePath);
        //切换到remodePath目录
        ftpClient.changeWorkingDirectory(remotePath);
        //设置1m缓冲
        ftpClient.setBufferSize(10240);
        //设置编码
        ftpClient.setControlEncoding("UTF-8");
        //设置文件类型：二进制文件
        ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
        //被动模式
        ftpClient.enterLocalPassiveMode();

        for (MultipartFile file : files) {
          if (StringUtils.isBlank(file.getOriginalFilename())) {
            continue;
          }
          name = UUID.randomUUID().toString().replace("-", "");
//                    name = name + "."  + file.getOriginalFilename().split("\\.")[1];
          name = name + file.getOriginalFilename()
              .substring(file.getOriginalFilename().lastIndexOf("."));
          fileNames.add(name);
          ftpClient.storeFile(name, file.getInputStream());
        }
        resultMap.put("uploaded", uploaded);
        resultMap.put("name", name);
        return resultMap;
      } catch (IOException e) {
        logger.error("上传文件异常", e);
        uploaded = false;
      } finally {
        ftpClient.disconnect();
      }
    }
    resultMap.put("uploaded", uploaded);
    resultMap.put("name", name);
    return resultMap;
  }

  //连接ftp服务器
  private boolean connectServer(String ip, int port, String user, String pwd) {
    boolean isSuccess = false;
    ftpClient = new FTPClient();
    ftpClient.setDataTimeout(5000);
    try {
      ftpClient.connect(ip, port);
      isSuccess = ftpClient.login(user, pwd);
    } catch (IOException e) {
      logger.error("连接FTP服务器异常", e);
    }
    return isSuccess;
  }

  public static String upload(MultipartFile file, String path) {
    logger.info("upload的绝对地址：{}", path);
    //获取文件名
    String fileName = file.getOriginalFilename();
    //获取文件名的扩展名
    String fileExtensionName = fileName.substring(fileName.lastIndexOf(".") + 1);
    //创建一个新名称，等下用于上传文件的名称
    String uploadFileName = UUID.randomUUID().toString() + "." + fileExtensionName;
    logger.info("开始上传文件,上传文件的文件名:{},上传的路径:{},新文件名:{}", fileName, path, uploadFileName);

    File fileDir = new File(path);
    //如果目录不存在，则创建该目录
    if (!fileDir.exists()) {
      //获取权限
      fileDir.setWritable(true);
      //创建目录
      fileDir.mkdirs();
    }
    //把上传路径和新文件名传入新file对象
    File targetFile = new File(fileDir.getAbsoluteFile(), uploadFileName);
    try {
      //文件写入传入的目录中
      file.transferTo(targetFile);
    } catch (IOException e) {
      logger.error("上传文件异常", e);
      return null;
    }
    return targetFile.getName();
  }

  /**
   * 方法描述：检验指定路径的文件是否存在ftp服务器中
   *
   * @param filePath 指定绝对路径的文件 127.0.0.1:21/TEST/20161010/test_20161010.zip
   * @return 存在返回true，不存在返回false
   * @author guoxk
   * @createTime 2017年5月11日 上午11:26:44
   */
  public boolean isFTPFileExist(String filePath) {
    try {
      if (connectServer(this.ip, this.port, this.user, this.pwd)) {
        // 检验登陆操作的返回码是否正确
        if (!FTPReply.isPositiveCompletion(ftpClient.getReplyCode())) {
          ftpClient.disconnect();
          return false;
        }

        ftpClient.enterLocalPassiveMode(); //开启本地被动模式--Linux，windowns开启主动enterLocalActiveMode

        // 设置文件类型为二进制，与ASCII有区别
        ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
        // 设置编码格式
        ftpClient.setControlEncoding("GBK");

        // 提取绝对地址的目录以及文件名
        filePath = filePath.replace(ip + ":" + port + "/", "");
        String dir = filePath.substring(0, filePath.lastIndexOf("/"));
        String file = filePath.substring(filePath.lastIndexOf("/") + 1);

        // 进入文件所在目录，注意编码格式，以能够正确识别中文目录
        ftpClient.changeWorkingDirectory(new String(dir.getBytes("GBK"),
            FTP.DEFAULT_CONTROL_ENCODING));

        // 检验文件是否存在
        InputStream is = ftpClient.retrieveFileStream(new String(file
            .getBytes("GBK"), FTP.DEFAULT_CONTROL_ENCODING));
        if (is == null || ftpClient.getReplyCode() == FTPReply.FILE_UNAVAILABLE) {
          return false;
        }

        if (is != null) {
          is.close();
          ftpClient.completePendingCommand();
        }
        return true;
      }
    } catch (Exception e) {
      //logger.error(e.getMessage());
    } finally {
      if (ftpClient != null) {
        try {
          ftpClient.disconnect();
        } catch (IOException e) {
          //logger.error(e.getMessage());
        }
      }
    }

    return false;
  }

  public static void main(String[] args) {
    SimpleDateFormat sd = new SimpleDateFormat("yyyyMMdd");
    System.out.println(sd.format(new Date()));
  }
}
