package com.ssi.utils;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.afterturn.easypoi.handler.inter.IExcelDataHandler;
import com.ssi.exception.CustomizeException;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

/**
 * Created by csq on 2018/8/18.
 */
public class PoiUtils {

  public static final String[] EXCEL_EXT = {"xls", "xlsx"};

  public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass,
      String fileName, boolean isCreateHeader, HttpServletResponse response) {
    ExportParams exportParams = new ExportParams(title, sheetName);
    exportParams.setCreateHeadRows(isCreateHeader);
    defaultExport(list, pojoClass, fileName, response, exportParams);

  }

  public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass,
      String fileName, HttpServletResponse response) {
    defaultExport(list, pojoClass, fileName, response, new ExportParams(title, sheetName));
  }

  /**
   * 一个excel 创建多个sheet
   *
   * @param list 多个Map key title 对应表格Title key entity 对应表格对应实体 key data Collection 数据
   */
  public static void exportExcel(List<Map<String, Object>> list, String fileName,
      HttpServletResponse response) {
    defaultExport(list, fileName, response);
  }

  /**
   * 用poi导出Excel文件的静态方法
   *
   * @param list 数据：只能是List<Map<String, Object>>类型
   * @param sheetname Excel的sheet名字
   */
  public static void exportExcel(List<Map<String, Object>> list, String sheetname, String fileName,
      HttpServletResponse response) throws IOException {
    //新建工作簿
    HSSFWorkbook workbook = new HSSFWorkbook();
    //创建Excel的sheet
    HSSFSheet sheet = workbook.createSheet(sheetname);

    //从list任意一个Map对象里获取标题（字段名或属性名）放到sheet的第一行上，若第一条记录某字段值没有，则会没有该字段
    Map<String, Object> map = list.get(0);
    int num = 0;
    HSSFRow first = sheet.createRow(0);//创建sheet的第一行
    for (String key : map.keySet()) {
      first.createCell(num).setCellValue(key);//创建num+1行并在第num+1列上赋值（字段名）
      num++;
    }

    //从list取第一行到最后一行的内容并放到对应的Excel里，若记录里某字段值没有会有问题
    int rownum = 1;//行数
    for (Map<String, Object> data : list) {
      HSSFRow row = sheet.createRow(rownum);//创建sheet的第rownum+1行
      int n = 0;//列数
      for (String key : data.keySet()) {
        row.createCell(n).setCellValue(data.get(key).toString());//创建n+1行并在第n+1列上赋值
        n++;
      }
      rownum++;
    }
    downLoadExcel(fileName, response, workbook);

  }

  private static void defaultExport(List<?> list, Class<?> pojoClass, String fileName,
      HttpServletResponse response, ExportParams exportParams) {
    Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
    if (workbook != null)
      ;
    downLoadExcel(fileName, response, workbook);
  }

  public static void downLoadExcel(String fileName, HttpServletResponse response,
      Workbook workbook) {
    try {
      response.setCharacterEncoding("UTF-8");
      response.setHeader("content-Type", "application/vnd.ms-excel");
      response.setHeader("Content-Disposition",
          "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
      workbook.write(response.getOutputStream());
    } catch (IOException e) {
      throw new CustomizeException(e.getMessage());
    }
  }

  /**
   * 一个excel 创建多个sheet
   *
   * @param list 多个Map key title 对应表格Title key entity 对应表格对应实体 key data Collection 数据
   */
  private static void defaultExport(List<Map<String, Object>> list, String fileName,
      HttpServletResponse response) {
    Workbook workbook = ExcelExportUtil.exportExcel(list, ExcelType.HSSF);
    if (workbook != null)
      ;
    downLoadExcel(fileName, response, workbook);
  }

  public static <T> List<T> importExcel(String filePath, Integer titleRows, Integer headerRows,
      Class<T> pojoClass) {
    if (StringUtils.isBlank(filePath)) {
      return null;
    }
    ImportParams params = new ImportParams();
    params.setTitleRows(titleRows);
    params.setHeadRows(headerRows);
    List<T> list = null;
    try {
      list = ExcelImportUtil.importExcel(new File(filePath), pojoClass, params);
    } catch (NoSuchElementException e) {
      throw new CustomizeException("模板不能为空");
    } catch (Exception e) {
      e.printStackTrace();
      throw new CustomizeException(e.getMessage());
    }
    return list;
  }

  public static <T> List<T> importExcel(MultipartFile file, Integer titleRows, Integer headerRows,
      Class<T> pojoClass) {
    if (file == null) {
      return null;
    }
    ImportParams params = new ImportParams();
    params.setTitleRows(titleRows);
    params.setHeadRows(headerRows);
    List<T> list = null;
    try {
      list = ExcelImportUtil.importExcel(file.getInputStream(), pojoClass, params);
    } catch (NoSuchElementException e) {
      throw new CustomizeException("excel文件不能为空");
    } catch (Exception e) {
      throw new CustomizeException(e.getMessage());
    }
    return list;
  }

  /**
   * 检查文件名是否符合要求
   *
   * @param fileName 待监测文件名
   * @param exts 后缀名集合
   */
  public static boolean checkExt(String fileName, String[] exts) {
    if (StringUtils.isBlank(fileName)) {
      throw new CustomizeException("待校验文件名不能为空");
    }
    if (exts == null || exts.length == 0) {
      throw new CustomizeException("文件后缀名数组不能为空");
    }
    String ext = fileName.substring(fileName.lastIndexOf(".") + 1);
    for (int i = 0; i < exts.length; i++) {
      if (ext.equals(exts[i])) {
        return true;
      }
    }
    return false;
  }

  public static void exportExcel(IExcelDataHandler<?> handler, List<?> list, String title,
      String sheetName, Class<?> pojoClass, String fileName, boolean isCreateHeader,
      HttpServletResponse response) {
    ExportParams exportParams = new ExportParams(title, sheetName);
    exportParams.setDataHandler(handler);
    exportParams.setCreateHeadRows(isCreateHeader);
    defaultExport(list, pojoClass, fileName, response, exportParams);
  }

}
