package com.ssi.utils;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

/**
 * @author ZhangLiYao
 * @version 1.0
 * @date 2020/3/4 10:54
 */

public class RestTemplateUtil {

  private static class SingletonRestTemplate {

    static final RestTemplate INSTANCE = new RestTemplate();
  }

  private RestTemplateUtil() {
  }


  public static RestTemplate getInstance() {
    return SingletonRestTemplate.INSTANCE;
  }

  /**
   * post 请求
   *
   * @param url 请求路径
   * @param data body数据
   * @param token JWT所需的Token，不需要的可去掉
   */
  public static String post(String url, String data, String token) {
    HttpHeaders headers = new HttpHeaders();
    headers.add("Accept", "application/json");
    headers.add("Content-Encoding", "UTF-8");
    headers.add("Content-Type", "application/json; charset=UTF-8");
    if (token != null) {
      headers.add("Authorization", token);
    }
    HttpEntity<String> requestEntity = new HttpEntity<>(data, headers);
    return RestTemplateUtil.getInstance().postForObject(url, requestEntity, String.class);
  }

  /**
   * get 请求
   *
   * @param url 请求路径
   * @param token JWT所需的Token，不需要的可去掉
   */
  public static String get(String url, String token) {
    HttpHeaders headers = new HttpHeaders();
    headers.add("Accept", "application/json");
    headers.add("Content-Encoding", "UTF-8");
    headers.add("Content-Type", "application/json; charset=UTF-8");
    if (token != null) {
      headers.add("Authorization", token);
    }
    HttpEntity<String> requestEntity = new HttpEntity<>(null, headers);
    ResponseEntity<String> response = RestTemplateUtil.getInstance()
        .exchange(url, HttpMethod.GET, requestEntity, String.class);
    String responseBody = response.getBody();
    return responseBody;
  }
}