package com.ssi.utils;

import com.ssi.constant.enums.TaskLocationTypeEnum;
import com.ssi.constant.enums.TaskTypeEnum;
import com.ssi.entity.vo.VehicleVo;
import com.ssi.entity.vo.VmsTosOrdersVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Map;

@Component
@Slf4j
public class VehicleDataUtil {

  @Value("${client.onlineTime}")
  private Long onlineTime;

  public Integer checkOnline(Integer state, Long collectime) {
    /** 检查终端状态*/
    boolean dataOnline = (collectime != null
        && System.currentTimeMillis() - collectime < onlineTime);

    boolean isOnline = dataOnline && state == 1;

    return isOnline ? 1 : 0;
  }

  public VehicleVo.vehicleTemp toVehicleTemp(Map<String, Object> realDataMap) {
    VehicleVo.vehicleTemp vehicleTemp = new VehicleVo().new vehicleTemp();
    vehicleTemp.setVehicleNum(
        realDataMap.get("vehicleNum") == null ? null : (String) realDataMap.get("vehicleNum"));
    vehicleTemp.setVehicleType(
            realDataMap.get("vehicleType") == null ? 0 : (Integer) realDataMap.get("vehicleType"));
    vehicleTemp.setVin(realDataMap.get("vin") == null ? null : (String) realDataMap.get("vin"));
    vehicleTemp.setSoc((Integer) realDataMap.get("soc"));
    vehicleTemp.setFaultStatus(realDataMap.get("equipStatus") == null
        ? 1 : (Integer) realDataMap.get("equipStatus") == 1 ? 1 : 0);
    if (vehicleTemp.getFaultStatus() == 1) {
      vehicleTemp.setFaultLevel("一级故障");
    }
    vehicleTemp.setWorkStatus(
        realDataMap.get("workStatus") == null ? 0 : (Integer) realDataMap.get("workStatus"));
    Integer onlineStatus = checkOnline(
        realDataMap.get("state") == null ? 0 : (Integer) realDataMap.get("state")
        , (Long) realDataMap.get("collectTime"));
    vehicleTemp.setOnlineStatus(onlineStatus);
    vehicleTemp.setWaitOvertimeStatus(realDataMap.get("waitOvertimeStatus") == null ? 0
        : (Integer) realDataMap.get("waitOvertimeStatus"));
    vehicleTemp.setLowEnergyStatus(
        realDataMap.get("lowEnergy") == null || realDataMap.get("soc") == null ? 0
            : (Integer) realDataMap.get("soc") < (Integer) realDataMap.get("lowEnergy") ? 1 : 0);
    if (onlineStatus == 1) {
      VmsTosOrdersVo vmsTosOrders = (VmsTosOrdersVo) realDataMap.get("orderData");
      if (vmsTosOrders != null) {
        //加个判断subTaskType 0：缓冲区，1 2：扭锁站  其他不改变
        String description="";
        if(vmsTosOrders.getSubTaskType()==null){
          description = generateOrderDescription(vmsTosOrders);
        }else{
          if (vmsTosOrders.getSubTaskType() == 0){
            description="去缓冲区";
          }else if(vmsTosOrders.getSubTaskType() == 1 || vmsTosOrders.getSubTaskType()==2){
            description="去扭锁站";
          }else{
            description = generateOrderDescription(vmsTosOrders);
          }
        }


        vmsTosOrders.setOrderDescription(description);
        vehicleTemp.setOrderData(vmsTosOrders);
      }
    }
    vehicleTemp.setEmergencyType(
        realDataMap.get("emergencyType") == null ? 2 : (Integer) realDataMap.get("emergencyType"));
    vehicleTemp.setThreeConfirmStatus((Map) realDataMap.get("threeConfirmStatus"));
    return vehicleTemp;
  }

  public String generateOrderDescription(VmsTosOrdersVo vmsTosOrders) {
    if (vmsTosOrders != null) {
      try {
        Integer taskLocationType = vmsTosOrders.getTaskLocationType();
        String containerId = vmsTosOrders.getContainerId();
        Integer taskType = vmsTosOrders.getTaskType();
        String vehicleLocation = vmsTosOrders.getVehicleLocation();
        return generateOrderDescription(taskLocationType, taskType, vehicleLocation, containerId);
      } catch (Exception e) {
        log.error("生成任务信息描述异常", e);
      }
    }
    return null;
  }

  public String generateOrderDescription(Map taskOrderInfo) {
    if (taskOrderInfo != null) {
      try {
        Integer taskLocationType = (Integer) taskOrderInfo.get("taskLocationType");
        Integer taskType = (Integer) taskOrderInfo.get("taskType");
        String vehicleLocation = (String) taskOrderInfo.get("vehicleLocation");
        String containerId = (String) taskOrderInfo.get("containerId");
        return generateOrderDescription(taskLocationType, taskType, vehicleLocation, containerId);
      } catch (Exception e) {
        log.error("生成任务信息描述异常", e);
      }
    }
    return null;
  }

  public String generateTaskLocationDescription(Map taskOrderInfo) {
    if (taskOrderInfo != null) {
      try {
        Integer taskLocationType = (Integer) taskOrderInfo.get("taskLocationType");
        String vehicleLocation = (String) taskOrderInfo.get("vehicleLocation");
        return generateTaskLocationDescription(taskLocationType, vehicleLocation);
      } catch (Exception e) {
        log.error("生成任务地点描述异常", e);
      }
    }
    return null;
  }

  /**
   * taskLocationType:任务类型, 1-去堆场（装箱或卸箱）;2-去停车点（上扭锁，解扭锁，停车）;3-去固定停车区; 4-去临时停车区;5-去桥吊（装箱或卸箱）；6-去充电
   * taskType:作业类型,1:装箱，2:卸箱，3:充电，4:上扭锁，5:解扭锁,6:停车，7掉头
   */
  private String generateOrderDescription(Integer taskLocationType, Integer taskType,
      String vehicleLocation, String containerId) {
    TaskLocationTypeEnum taskLocationTypeEnum = TaskLocationTypeEnum.find(taskLocationType);
    TaskTypeEnum taskTypeEnum = TaskTypeEnum.find(taskType);
    String str1 = null;
    String str2 = null;
    if (StringUtils.isNotBlank(vehicleLocation) && vehicleLocation.length() >= 6) {
      str1 = vehicleLocation.substring(0, 3);
      str2 = vehicleLocation.substring(3, 6);
    }
    StringBuilder sb = new StringBuilder();
    switch (taskLocationType) {
      case 1:
        sb.append("去" + str1 + "堆场" + str2 + "贝位");
//            .append("将集装箱").append(containerId)
//            .append(taskTypeEnum.getDescription());
        break;
      case 2:
//        sb.append(taskLocationTypeEnum.getDescription())
//            .append(taskTypeEnum.getDescription());
       // break;
      case 3:
      case 4:
      case 6:
        sb.append(taskLocationTypeEnum.getDescription());
        break;
      case 5:
        sb.append("去" + str1 + "桥吊" + str2 + "车道");
//            .append("将集装箱").append(containerId)
//            .append(taskTypeEnum.getDescription());
        break;
    }
    return sb.toString();
  }

  public String generateTaskLocationDescription(Integer taskLocationType, String vehicleLocation) {
    TaskLocationTypeEnum taskLocationTypeEnum = TaskLocationTypeEnum.find(taskLocationType);
    String str1 = null;
    String str2 = null;
    if (StringUtils.isNotBlank(vehicleLocation)) {
      str1 = vehicleLocation.substring(0, 3);
      str2 = vehicleLocation.substring(3, 6);
    }
    StringBuilder sb = new StringBuilder();
    switch (taskLocationType) {
      case 1:
        sb.append(str1 + "堆场" + str2 + "贝位");
        break;
      case 2:
      case 3:
      case 4:
      case 6:
        sb.append(taskLocationTypeEnum.getDescription());
        break;
      case 5:
        sb.append(str1 + "桥吊" + str2 + "车道");
        break;
    }
    return sb.toString();
  }
}
