package com.ssi.utils.grid;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.util.*;

/**
 * 车辆数据，即网格用户
 */
public class MercatorVehicle implements Serializable {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    private final String vid;
    private final EventBus eventBus;

    private Set<MercatorGrid> grids = new HashSet<>();

    //车辆实时数据
    private MercatorPoint point;
    private List<MercatorGrid> waitGrids = new ArrayList<>();

    private Integer versionNo;
    private long timeStamp;
    private double speed;
    private boolean isActive =false;

    public MercatorVehicle(EventBus eventBus, String vid, Collection<MercatorGrid> grids,
                           MercatorPoint point, double speed, long timeStamp){
        this.vid = vid;
        this.eventBus = eventBus;
        update(grids, point, speed, timeStamp);
    }

    public int waitGridSize(){
        return this.waitGrids.size();
    }

    public boolean update(Collection<MercatorGrid> grids,
                         MercatorPoint point, double speed, long timeStamp){

        int waitGridSize = waitGridSize();
        //无等待 则可执行更新
        if(waitGridSize > 0){
            freeAllWaitGrids();
        }
        Iterator<MercatorGrid> iterator = this.grids.iterator();
        MercatorGrid next;
        while (iterator.hasNext()){
            next = iterator.next();
            if(!grids.contains(next)){
                //注销占用,让位给下一约定车辆， 并移除
                if(next.freeToNext(this.vid)){
                    String currentUser = next.getCurrentUser();
                    if(currentUser != null) {
                        //通知对应车辆
                        this.eventBus.post(new GridFreeEvent(this.vid, next));
                    }
                }
                iterator.remove();
            }
        }
        StringBuilder sb = new StringBuilder();
        grids.forEach(grid -> {
            if(this.grids.add(grid)){
                //添加成功，则为新占用网格
                grid.reserve(vid);
                String currentUser = grid.getCurrentUser();
                //检查网格当前用户是否为本车, 不是本车则将网格记录到等待队列
                if(!this.vid.equals(currentUser)){
                    sb.append(currentUser).append(" ");
                    this.waitGrids.add(grid);
//                    this.eventBus.post(new GridFreeEvent(currentUser, grid));
                }
            }
        });

        this.point = point;
        this.speed = speed;
        this.timeStamp = timeStamp;
        logger.info(String.format("车辆%s更新网格集合成功，占用网格数：%s， 等待网格数：%s, speed：%s, timeStamp: %s,等待网格占有者：[%s].",
                this.vid, this.grids.size(), waitGridSize(), speed, timeStamp,sb));
        return true;
    }

    public void freeWaitGrid(String frontVid, MercatorGrid grid){
        this.waitGrids.remove(grid);
        if(!this.grids.contains(grid))
            grid.freeToNext(this.vid);
        if(waitGridSize()==0){
            this.isActive = true;
        }
        logger.info(String.format("车辆%s 释放网格%s，当前车辆%s占用，移出等待集合， 当前等待网格数：%s", frontVid, grid, this.vid, waitGridSize()));
    }

    public void freeOwnedGrids(){
        this.grids.forEach((grid)->{
            if(!this.waitGrids.contains(grid)){
                grid.freeToNext(vid);
            }
        });
        grids.clear();
    }

    public void freeAllWaitGrids(){
        this.waitGrids.forEach((grid)->{
            if(vid.equalsIgnoreCase(grid.getCurrentUser())){
                grid.freeToNext(vid);
            }else {
                grid.cancelReserve(vid);
            }
        });
        this.waitGrids.clear();
    }


    public void freeAll(){
        Iterator<MercatorGrid> iterator = this.grids.iterator();
        MercatorGrid next;
        while (iterator.hasNext()){
            next = iterator.next();
            if(this.waitGrids.contains(next)){
                next.cancelReserve(this.vid);
            }else{
                if(next.freeToNext(this.vid)){
                    String currentUser = next.getCurrentUser();
                    if(currentUser != null) {
                        //通知对应车辆
                        this.eventBus.post(new GridFreeEvent(this.vid, next));
                    }
                }
            }
            iterator.remove();
        }
        this.waitGrids.clear();
    }

    public String getVid() {
        return vid;
    }

    public int getGridsSize() {
        return grids.size();
    }

    public Integer getVersionNo() {
        return versionNo;
    }

    public void setVersionNo(Integer versionNo) {
        this.versionNo = versionNo;
    }

    public boolean isActive() {
        return isActive;
    }

    public void setActive(boolean active) {
        isActive = active;
    }

    @Override
    public String toString() {
        final StringBuffer sb = new StringBuffer("MercatorVehicle{");
        sb.append("vid='").append(vid).append('\'');
        sb.append('}');
        return sb.toString();
    }
}
