package com.ssi.utils.telaidian;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
public class ConnectorStatusInfo {

    /**
     * 0：离网
     * 1：空闲
     * 2：占用（未充电）
     * 3：占用（充电中）
     * 4：占用（预约锁定）
     * 255：故障
     */
    public final static int STATUS_0 = 0;
    public final static int STATUS_1 = 1;
    public final static int STATUS_2 = 2;
    public final static int STATUS_3 = 3;
    public final static int STATUS_4 = 4;
    public final static int STATUS_255 = 255;
    /**
     * 0：未知
     * 10：空闲
     * 50：占用
     */
    public final static int PARKSTATUS_0 = 0;
    public final static int PARKSTATUS_10 = 10;
    public final static int PARKSTATUS_50 = 50;
    /**
     * 0：未知
     * 10：已解锁
     * 50：已上锁
     */
    public final static int LOCKSTATUS_0 = 0;
    public final static int LOCKSTATUS_10 = 10;
    public final static int LOCKSTATUS_50 = 50;

    private String ConnectorID;
    private Integer Status;
    private Integer ParkStatus;
    private Integer LockStatus;

    @JsonIgnore
    public boolean isAway() {
        return Status == STATUS_0;
    }

    @JsonIgnore
    public boolean isFree() {
        return Status == STATUS_1;
    }

    @JsonIgnore
    public boolean isOccupation() {
        return Status == STATUS_2;
    }

    @JsonIgnore
    public boolean isCharging() {
        return Status == STATUS_3;
    }

    @JsonIgnore
    public boolean isReserveLocked() {
        return Status == STATUS_4;
    }

    @JsonIgnore
    public boolean isTrouble() {
        return Status == STATUS_255;
    }

    @JsonIgnore
    public boolean isUnconnectedStatus() {
        return isAway() || isFree();
    }

    @JsonIgnore
    public boolean isConnectedStatus() {
        return !isUnconnectedStatus();
    }

}
