package com.ssi.utils.telaidian;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 充电设备状态刷新调度任务
 */
@Component
@ConditionalOnProperty(prefix = "app.context.charging-schedule", value = "enable", matchIfMissing = true)
public class DeviceStatusSchedule {

    private Logger logger = LoggerFactory.getLogger(getClass());

    public static final String LOG_PREFIX_TELAIDIAN = "[Telaidian]";

    private static final String CHARGING_PILE_INFO_REDIS_PREFIX = "harbor:charging:pile:info:";

    private boolean lock = false;
    @Autowired
    private Gson gson;
    private JsonParser jsonParser = new JsonParser();
    @Autowired
    private TelaidianRequestTpl telaidianRequestTpl;
    @Autowired


    /**
     * 调度任务
     */
//    @Scheduled(cron = "${telaidian.status-refresh.cron}")
    private void refreshDeviceStatus() {
        if (lock) {
            return;
        }

        try {
            lock = true;
           JSONObject statusMap = queryDeviceStatus();
            if ("0".equalsIgnoreCase(statusMap.getString("Ret"))) {
                List<ConnectorStatusInfo> connectorStatusInfoList = new ArrayList<>();
                String data = statusMap.getString("Data");
                JsonObject jsonObject = jsonParser.parse(data).getAsJsonObject();
                JsonArray stationStatusArray = jsonObject.getAsJsonArray("StationStatusInfos");
                if (stationStatusArray != null) {
                    for (JsonElement stationStatusEl : stationStatusArray) {
                        JsonArray connectorStatusArray = stationStatusEl.getAsJsonObject().getAsJsonArray("ConnectorStatusInfos");
                        if (connectorStatusArray != null) {
                            for (JsonElement connectorStatusEl : connectorStatusArray) {
                                ConnectorStatusInfo connectorStatusInfo = gson.fromJson(connectorStatusEl.getAsJsonObject(),
                                        ConnectorStatusInfo.class);
                                connectorStatusInfoList.add(connectorStatusInfo);
                            }
                        }
                    }
                }

                updateChargingPileInfoInRedis(connectorStatusInfoList);

//                logger.debug(LOG_PREFIX_TELAIDIAN + appContext.objToJsonStr(connectorStatusInfoList));
            } else {
                logger.warn(LOG_PREFIX_TELAIDIAN + gson.toJson(statusMap));
            }
        } catch (Exception e) {
            logger.error(LOG_PREFIX_TELAIDIAN + "Fail to refresh device status from telaidian.", e);
        } finally {
            lock = false;
        }
    }

    /**
     * 更新状态
     *
     * @param connectorStatusInfoList
     * @throws Exception
     */
    private void updateChargingPileInfoInRedis(List<ConnectorStatusInfo> connectorStatusInfoList) throws Exception {
        for (ConnectorStatusInfo connectorStatusInfo : connectorStatusInfoList) {
            
        }
    }

    /**
     * 查询状态
     *
     * @return
     * @throws Exception
     */
    private JSONObject queryDeviceStatus() throws Exception {
        String methodName = "query_station_status";
        Map<String, Object> paramMap = new HashMap<>();
        List<String> stationIdList = new ArrayList<>();
        stationIdList.add("3502050037");
        paramMap.put("StationIDs", stationIdList);

        return telaidianRequestTpl.queryData(paramMap, methodName, 3);
    }


}
