package com.ssi.utils.telaidian;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 特来电请求工具模板
 */
@Component
public class TelaidianRequestTpl {
    protected Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private RestTemplate restTemplate;
    private int tokenCount = 0;
    @Autowired
    private Gson gson;
    private JsonParser parser = new JsonParser();

    @Value("${telaidian.operator-id-telaidian}")
    private String operatorIdOfTelaidian;
    @Value("${telaidian.operator-id-xmg}")
    private String operatorIdOfXmg;
    @Value("${telaidian.assign-key}")
    private String assignKey;
    @Value("${telaidian.operator-secret}")
    private String operatorSecret;
    @Value("${telaidian.AES-key}")
    private String AESSecretOfXmg;
    @Value("${telaidian.AES-vector}")
    private String AESSecretIVOfXmg;
    @Value("${telaidian.AES-key}")
    private String AESSecretOfTelaidian;
    @Value("${telaidian.AES-vector}")
    private String AESSecretIVOfTelaidian;
    @Value("${telaidian.base-url}")
    private String baseUrl;

    /**
     * 获取token
     *
     * @param url
     * @return
     * @throws Exception
     */
    public String getToken(String url) throws Exception {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("OperatorID", operatorIdOfXmg);
        paramMap.put("OperatorSecret", operatorSecret);
        Map<String, Object> returnBody = sendRest(gson.toJson(paramMap), null, url);
        if (!("0").equals(String.valueOf(returnBody.get("Ret")))) {
            logger.warn(DeviceStatusSchedule.LOG_PREFIX_TELAIDIAN + gson.toJson(returnBody));
            throw new Exception("Fail to get token from telaidian.");
        }
        String data = AESOperator.decrypt(returnBody.get("Data").toString(), AESSecretOfXmg, AESSecretIVOfXmg);
        JsonObject jsonObject = parser.parse(data).getAsJsonObject();
        if (jsonObject.get("SuccStat") != null && jsonObject.get("SuccStat").getAsDouble() != 0.0) {
            logger.warn(DeviceStatusSchedule.LOG_PREFIX_TELAIDIAN + jsonObject.toString());
            throw new Exception("Fail to get token from telaidian.");
        }

        return jsonObject.get("AccessToken").getAsString();
    }

    /**
     * 执行发送请求
     *
     * @param params
     * @param token
     * @param url
     * @return
     * @throws Exception
     */
    public Map<String, Object> sendRest(String params, String token, String url) throws Exception {
        String Data = AESOperator.encrypt(params, AESSecretOfXmg, AESSecretIVOfXmg);
        String TimeStamp = DateFormatUtils.format(new Date(), "yyyyMMddHHmmss");
        String Seq = "0001";

        HttpHeaders headers = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        headers.setContentType(type);
        headers.add("Accept", MediaType.APPLICATION_JSON.toString());
        headers.add("Authorization", token == null ? "Bearer Token" : "Bearer " + token);

        JsonObject jsonObj = new JsonObject();
        jsonObj.addProperty("OperatorID", operatorIdOfXmg);
        jsonObj.addProperty("Data", Data);
        jsonObj.addProperty("TimeStamp", TimeStamp);
        jsonObj.addProperty("Seq", Seq);
        jsonObj.addProperty("Sig", getSig(Data, TimeStamp, Seq));

        logger.debug(DeviceStatusSchedule.LOG_PREFIX_TELAIDIAN + "接口[" + url + "]下发报文：" + jsonObj.toString() +
                "，下发的token为：" + token);

        HttpEntity<String> requestEntity = new HttpEntity<>(jsonObj.toString(), headers);
        Object obj = null;
        try {
            obj = restTemplate.postForObject(url, requestEntity, Object.class);
        } catch (RestClientException e) {
            logger.error(DeviceStatusSchedule.LOG_PREFIX_TELAIDIAN + "访问错误：" + e.getMessage());
        }
        if (obj instanceof String) {
            Map<String, Object> map = new HashMap<>();
            map = gson.fromJson(obj.toString(), map.getClass());
            if (map.get("Ret").toString().equals("0.0")) {
                map.put("Ret", "0");
            }
            return map;
        } else {
            Map<String, Object> object = restTemplate.postForObject(url, requestEntity, LinkedHashMap.class);
            return object;
        }
    }

    /**
     * 获取请求签名
     *
     * @param Data
     * @param TimeStamp
     * @param Seq
     * @return
     */
    private String getSig(String Data, String TimeStamp, String Seq) {
        StringBuilder builder = new StringBuilder();
        builder.append(operatorIdOfXmg).append(Data).append(TimeStamp).append(Seq);

        return HMacMD5.getHmacMd5Str(assignKey, builder.toString());
    }

    /**
     * 查询数据
     *
     * @param params
     * @param methodName
     * @param retryNums
     * @return
     * @throws Exception
     */
    public JSONObject queryData(String params, String methodName, int retryNums) throws Exception {
        JSONObject result = new JSONObject();
        if (retryNums <= 0) {
            logger.error(DeviceStatusSchedule.LOG_PREFIX_TELAIDIAN + "token连续3次认证失败.");
            throw new Exception("token连续3次认证失败.");
        }

        String token = getToken(baseUrl + "query_token");
        Map<String, Object> qryResult = sendRest(params, token, baseUrl + methodName);
        /* retry */
        String retStr = qryResult.get("Ret").toString();
        if ("4002".equals(retStr)) {
            return queryData(params, methodName, retryNums - 1);
        }

        if (!"0".equals(retStr)) {
            result.put("Ret", retStr);
            result.put("Msg", qryResult.get("Msg").toString());
            return result;
        }
        String data = AESOperator.decrypt(qryResult.get("Data").toString(), AESSecretOfXmg, AESSecretIVOfXmg);
        result.put("Ret", "0");
        result.put("Data", data);

        return result;
    }

    public JSONObject queryData(Map<String, Object> paramMap, String methodName,
                                         int retryNums) throws Exception {
        return queryData(gson.toJson(paramMap), methodName, retryNums);
    }

}
