package com.ssi.websocket;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.entity.VmsCranePadBind;
import com.ssi.service.VmsCranePadBindService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * 吊具APP任务推送websocket
 * @author zhangly
 * @since 2020-02-10
 * 相当于ws协议的Controller
 */
@ServerEndpoint("/craneAppWebSocketServer/{padMac}")
@Component
public class CraneAppWebSocketServer {

    private final static Logger log = LoggerFactory.getLogger(CraneAppWebSocketServer.class);

    @Autowired
    private VmsCranePadBindService vmsCranePadBindService;


    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;
    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
    private static CopyOnWriteArraySet<CraneAppWebSocketServer> webSocketSet = new CopyOnWriteArraySet<CraneAppWebSocketServer>();

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    //接收sid
    private String sid = "";

    private String padMac = "";

    private String craneNo = "";

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("sid") String sid, @PathParam("padMac") String padMac) {
        this.session = session;
        webSocketSet.add(this);     //加入set中
        addOnlineCount();           //在线数加1
        log.info("有新窗口开始监听:" + sid + ",当前在线人数为" + getOnlineCount());
        this.sid = sid;
        this.padMac = padMac;
        try {
            sendMessage("连接成功");
        } catch (IOException e) {
            log.error("websocket IO异常");
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        webSocketSet.remove(this);  //从set中删除
        subOnlineCount();           //在线数减1
        log.info("有一连接关闭！当前在线人数为" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        JSONObject messageJson = JSONObject.parseObject(message);
        String padMac = messageJson.getString("padMac");
        //群发消息
        for (CraneAppWebSocketServer item : webSocketSet) {
            try {
                if(item.session.getId().equals(session.getId())){
                    VmsCranePadBind crane = vmsCranePadBindService.getOne(new LambdaQueryWrapper<VmsCranePadBind>()
                            .eq(VmsCranePadBind::getPadMac, padMac));
                    this.craneNo = crane.getCraneNo();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("发生错误");
        error.printStackTrace();
    }

    /**
     * 实现服务器主动推送
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static CopyOnWriteArraySet<CraneAppWebSocketServer> getWebSocketSet(){
        return webSocketSet;
    }

    public String getCraneNo(){
        return this.craneNo;
    }
    public String getPadMac(){
        return this.padMac;
    }

    public static synchronized void addOnlineCount() {
        CraneAppWebSocketServer.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        CraneAppWebSocketServer.onlineCount--;
    }

    public void sendInfo() {
        log.info("桥吊APP推送任务信息:");


    }
}
