package com.ssi.websocket;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * 吊具实时信息websocket
 */
@ServerEndpoint("/craneWebSocket")
@Component
public class CraneInfoWebSocketServer {

    private final static Logger log = LoggerFactory.getLogger(CraneInfoWebSocketServer.class);

    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;

    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
    private static CopyOnWriteArraySet<CraneInfoWebSocketServer> webSocketSet = new CopyOnWriteArraySet<CraneInfoWebSocketServer>();

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    private String craneNo;

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session) {
        this.session = session;
        webSocketSet.add(this);     //加入set中
        addOnlineCount();           //在线数加1
        log.info("CraneWebSocketServer有新窗口开始监听:" + session.getId() + ",当前在线人数为" + getOnlineCount());
        try {
            sendMessage("连接成功");
        } catch (IOException e) {
            log.error("CraneWebSocketServer websocket IO异常");
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        webSocketSet.remove(this);  //从set中删除
        subOnlineCount();           //在线数减1
        log.info("CraneWebSocketServer有一连接关闭！当前在线人数为" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        log.info("CraneWebSocketServer收到来自窗口" + session.getId() + "的信息:" + message);
        this.craneNo = message;
    }

    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("CraneInfoWebSocketServer发生错误", error);
    }

    /**
     * 实现服务器主动推送
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        CraneInfoWebSocketServer.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        CraneInfoWebSocketServer.onlineCount--;
    }

    public static CopyOnWriteArraySet<CraneInfoWebSocketServer> getWebSocketSet() {
        return webSocketSet;
    }

    public String getCraneNo() {
        return craneNo;
    }
}
