package com.ssi.websocket;

import com.alibaba.fastjson.JSONObject;
import com.ssi.controller.platform.VehicleTrackController;
import com.ssi.response.SSIResponse;
import com.ssi.service.platform.impl.VehicleTrackServiceImpl;
import com.ssi.utils.SpringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 推动交通灯信息到前端
 */
@ServerEndpoint(value = "/ReplaySocketServer")
@Component
public class ReplaySocketServer {

    private static Logger log = LoggerFactory.getLogger(ReplaySocketServer.class);

    private static AtomicInteger onlineCount = new AtomicInteger(0);
    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
    private static CopyOnWriteArraySet<ReplaySocketServer> webSocketSet = new CopyOnWriteArraySet<>();

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    private Queue<Map<String, Object>> replayCache = new LinkedBlockingQueue<>();

    public static void messageBroadcast(String messages) {
        webSocketSet.parallelStream().forEach(webSocketSession -> {
                try {
//                    webSocketSession.session.getBasicRemote().sendText(messages);
                    webSocketSession.sendMessage(null);
                }catch (Exception e){
                    log.error("ReplaySocketServer服务发送消息失败:",e);
                }
            });

    }

    public Queue<Map<String, Object>> getReplayCache() {
        return replayCache;
    }

    public void setReplayCache(List<Map<String, Object>> replayCaches) {
        replayCache.clear();
        replayCache.addAll(replayCaches);
    }

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session) throws Exception {
        log.info("ReplaySocketServer服务新增用户连接；{}",session.getId());
        this.session=session;
        webSocketSet.add(this);
        log.info("当前用户数为:{}",onlineCount.incrementAndGet());
//        session.getBasicRemote().sendText("连接成功");

    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        log.info("用户离线当前用户数为：{}", onlineCount.decrementAndGet());
        webSocketSet.remove(this);
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(message);
        VehicleTrackServiceImpl vehicleTrackService = SpringUtils.getObject(VehicleTrackServiceImpl.class);
        SSIResponse response = vehicleTrackService.smallTrack(jsonObject.getString("vin"), jsonObject.getLong("startTime"), jsonObject.getLong("stopTime"), null);
        List<Map<String, Object>> listData = (List<Map<String, Object>>) response.getData();
        if(CollectionUtils.isNotEmpty(listData)){
            setReplayCache(listData);
        }
    }

    public void sendMessage(String message){
        Queue<Map<String, Object>> replayCache = getReplayCache();
        if(replayCache.size()<1){
            return;
        }
        Map<String, Object> poll = replayCache.poll();
        try {
            this.session.getBasicRemote().sendText(JSONObject.toJSONString(poll));
        }catch (Exception e){
            log.error("ReplaySocketServer服务发送消息失败:",e);
        }
    }

    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error(String.format("会话：%s 发生错误.", session.getId()), error);
    }

}

