package com.ssi.websocket;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ssi.model.RedisDataModel;
import com.ssi.utils.ConfigUtils;
import com.ssi.utils.SpringUtils;
import com.ssi.websocket.vo.CraneLocation;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.websocket.WsSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 推动交通灯信息到前端
 */
@ServerEndpoint(value = "/Websocket/TrafficLampInfo")
@Component
public class TrafficLampInfoWebSocketServer {

    private static Logger log = LoggerFactory.getLogger(TrafficLampInfoWebSocketServer.class);

    private static AtomicInteger onlineCount = new AtomicInteger(0);
    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
    private static CopyOnWriteArraySet<TrafficLampInfoWebSocketServer> webSocketSet = new CopyOnWriteArraySet<>();

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session) throws Exception {
        log.info("TrafficLampInfoWebSocketServer服务新增用户连接；{}",session.getId());
        this.session=session;
        webSocketSet.add(this);
        log.info("当前用户数为:{}",onlineCount.incrementAndGet());
//        session.getBasicRemote().sendText("连接成功");

    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        log.info("用户离线当前用户数为：{}", onlineCount.decrementAndGet());
        webSocketSet.remove(this);
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) throws Exception {
    }

    public static void messageBroadcast(String messages) {
        webSocketSet.parallelStream().forEach(webSocketSession -> {
                try {
                    webSocketSession.session.getBasicRemote().sendText(messages);
                }catch (Exception e){
                    log.error("TrafficLampInfoWebSocketServer服务发送消息失败:",e);
                }
            });

    }

    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error(String.format("会话：%s 发生错误.", session.getId()), error);
    }

}

