package com.ssi.websocket;

import com.alibaba.fastjson.JSON;
import com.ssi.entity.VehicleRoad;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 *
 * 车路协调websocket
 */
@ServerEndpoint("/vehicleRoadCoordinationWebSocketServer")
@Component
public class VehicleRoadCoordinationWebSocketServer {

    private final static Logger log = LoggerFactory.getLogger(VehicleRoadCoordinationWebSocketServer.class);
    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;
    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
    private static CopyOnWriteArraySet<VehicleRoadCoordinationWebSocketServer> webSocketSet = new CopyOnWriteArraySet<VehicleRoadCoordinationWebSocketServer>();

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session) {
        this.session = session;
        webSocketSet.add(this);     //加入set中
        addOnlineCount();           //在线数加1
        log.info("有新窗口开始监听:" + session.getId() + ",当前在线人数为" + getOnlineCount());
        try {
            sendMessage("连接成功");
        } catch (IOException e) {
            log.error("websocket IO异常");
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        webSocketSet.remove(this);  //从set中删除
        subOnlineCount();           //在线数减1
        log.info("有一连接关闭！当前在线人数为" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        log.info("收到来自窗口" + session.getId() + "的信息:" + message);
       // VehicleRoad vehicleRoad = JSON.parseObject(message, VehicleRoad.class);
    }

    public void addIgnoreMessage(VehicleRoad vehicleRoad){
        // ignoreMap.put(key,new Date());
    }

    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("发生错误",error);
    }

    /**
     * 实现服务器主动推送
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        VehicleRoadCoordinationWebSocketServer.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        VehicleRoadCoordinationWebSocketServer.onlineCount--;
    }

    public static void sendInfo(String message) {
        log.info("异常推送websocket收到信息:" + message);
        //群发消息
        for (VehicleRoadCoordinationWebSocketServer item : webSocketSet) {
            try {
                item.sendMessage(message);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void sendMessage(VehicleRoad vehicleRoad) {
        //群发消息
        for (VehicleRoadCoordinationWebSocketServer item : webSocketSet) {
            try {
                item.sendMessage(JSON.toJSONString(vehicleRoad));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}