package com.ssi.websocket;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ssi.model.RedisDataModel;
import com.ssi.utils.ConfigUtils;
import com.ssi.utils.SpringUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.*;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * 测试websocket
 */
/*@ServerEndpoint(value = "/VMSWebsocketTest")
@Component*/
public class WebSocketServiceTest {

    private static Logger log = LoggerFactory.getLogger(WebSocketServiceTest.class);

    private String preLocationFix = "ivccs:vms:vehicle:preLocation";

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session) throws Exception {
        InputStream inputStream = new FileInputStream("C:\\Users\\lh314\\Documents\\Tencent Files\\863825024\\FileRecv\\1020-23数据记录");
        String text = IOUtils.toString(inputStream, "utf8");
        JSONObject jsonObject = JSON.parseObject(text);
        JSONObject hits = jsonObject.getJSONObject("hits");
        JSONArray array = hits.getJSONArray("hits");
        long lastCollectTime = 0;
        for (int i = 0; i < array.size(); i++) {
            JSONObject o = (JSONObject) array.get(i);
            JSONObject source = o.getJSONObject("_source");
            long collectTime = (long) source.get("collectTime");
            if (collectTime == lastCollectTime) {
                continue;
            }
            addLastLocationInfo(source);
            ArrayList<Object> list = new ArrayList<>();
            HashMap<Object, Object> map = new HashMap<>();
            map.put("timeStamp", System.currentTimeMillis());
            map.put("car", source);
            list.add(map);
            Thread.sleep(1000);
            System.out.println(JSON.toJSONString(list));
            session.getBasicRemote().sendText(JSON.toJSONString(list));
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) throws Exception {
    }


    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error(String.format("会话：%s 发生错误.", session.getId()), error);
    }

    private Map<String, Object> preWithSpeedMap;

    private void addLastLocationInfo(Map<String, Object> map) {
        RedisDataModel redisDataModel = SpringUtils.getObject(RedisDataModel.class);
        String location = redisDataModel.get(String.format("%s:%s", preLocationFix, map.get("vin")));
        if (StringUtils.isNotBlank(location)) {
            String[] split = location.split(",");
            Double lastLongitude = Double.parseDouble(split[0]);
            Double lastLatitude = Double.parseDouble(split[1]);
            if (lastLatitude != null && lastLongitude != null) {
                map.put("preLocation", new double[]{lastLongitude, lastLatitude});
            }
        }
        double speed = Double.parseDouble(map.getOrDefault("speed", "0").toString());
        /*Double longitude = ConfigUtils.getAsDoubleWithDefault(map, "longitude", null);
        Double latitude = ConfigUtils.getAsDoubleWithDefault(map, "latitude", null);*/
        JSONArray location1 = (JSONArray) map.get("location");
        BigDecimal longitude = (BigDecimal) location1.get(0);
        BigDecimal latitude = (BigDecimal) location1.get(1);

        if (speed != 0) {
            if (longitude != null && latitude != null) {
                redisDataModel.set(String.format("%s:%s", preLocationFix, map.get("vin"))
                        , String.format("%s,%s", longitude, latitude));
                Map<String, Object> preWithSpeedMapTemp = new HashMap<>();
                preWithSpeedMapTemp.put("longitude", longitude);
                preWithSpeedMapTemp.put("latitude", latitude);
                preWithSpeedMap = preWithSpeedMapTemp;
            }

        } else {
            if (preWithSpeedMap != null) {
                Double longitude1 = ConfigUtils.getAsDoubleWithDefault(preWithSpeedMap, "longitude", null);
                Double latitude1 = ConfigUtils.getAsDoubleWithDefault(preWithSpeedMap, "latitude", null);
                if (longitude1 != null && latitude1 != null) {
                    map.put("location", new double[]{longitude1, latitude1});
                }
            } else {
                Map<String, Object> preWithSpeedMapTemp = new HashMap<>();
                preWithSpeedMapTemp.put("longitude", longitude);
                preWithSpeedMapTemp.put("latitude", latitude);
                preWithSpeedMap = preWithSpeedMapTemp;
            }
        }
    }

}

