package com.ssi.websocket.push.task;

import com.alibaba.fastjson.JSONObject;
import com.ssi.entity.VmsCranePadBind;
import com.ssi.entity.VmsVehicle;
import com.ssi.service.VmsCranePadBindService;
import com.ssi.service.VmsVehicleService;
import com.ssi.service.VmsTosOrdersService;
import com.ssi.websocket.CraneAppWebSocketServer;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 吊具APP任务推送任务
 * @author ZhangLiYao
 * @version 1.0
 * @date 2020/8/6 10:00
 */
@Component
@EnableScheduling
public class CraneAppOrdersTask {

    @Autowired
    private VmsTosOrdersService vmsTosOrdersService;
    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private VmsCranePadBindService vmsCranePadBindService;

    @Scheduled(fixedRateString = "5000")
    private void getOrdersOfTask() {

        CopyOnWriteArraySet<CraneAppWebSocketServer> webSocketSet = CraneAppWebSocketServer.getWebSocketSet();
        if(webSocketSet.isEmpty()){
            return;
        }
        List<VmsVehicle> vehicleList = vmsVehicleService.list();
        Map<String, VmsVehicle> vinMap = vehicleList.stream().collect(Collectors.toMap(VmsVehicle::getVin, Function.identity()));
        List<VmsCranePadBind> vmsCranePadBindList = vmsCranePadBindService.list();
        Map<String, VmsCranePadBind> padMacMap = vmsCranePadBindList.stream().collect(Collectors.toMap(VmsCranePadBind::getPadMac, Function.identity()));

        //群发消息
        for (CraneAppWebSocketServer item : webSocketSet) {
            try {
                if(StringUtils.isBlank(item.getPadMac())){
                    return;
                }

                List<Map<String, Object>> resultList = vmsCranePadBindService.getCraneOrderList(vinMap, padMacMap.get(item.getPadMac()));
                if(!resultList.isEmpty()){
                    item.sendMessage(JSONObject.toJSONString(resultList));
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
