package com.ssi.websocket.push.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ssi.entity.CraneInfo;
import com.ssi.entity.VmsVehicle;
import com.ssi.mapper.VmsVehicleMapper;
import com.ssi.model.RedisDataModel;
import com.ssi.service.CraneInfoService;
import com.ssi.websocket.CraneInfoWebSocketServer;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.stream.Collectors;

/**
 * 推送吊具实时信息定时任务
 */
@Component
@EnableScheduling
public class CraneInfoTask {

    @Autowired
    private CraneInfoService craneInfoService;
    @Autowired
    private RedisDataModel redisDataModel;
    @Autowired
    private VmsVehicleMapper vmsVehicleMapper;

    @Value("${order.latestOrderKeyPrefix:harbor:command:status}")
    private String latestOrderKeyPrefix;
    @Value("${harbor.crane.confirmExpireTime}")
    private int confirmExpireTime;

    @Scheduled(fixedRateString = "1000")
    public void getCraneInfo() throws Exception {
        CopyOnWriteArraySet<CraneInfoWebSocketServer> webSocketSet = CraneInfoWebSocketServer.getWebSocketSet();
        if (CollectionUtils.isNotEmpty(webSocketSet)) {
            for (CraneInfoWebSocketServer craneInfoWebSocketServer : webSocketSet) {
                String craneNo = craneInfoWebSocketServer.getCraneNo();
                CraneInfo craneInfo = null;
                if (StringUtils.isNotBlank(craneNo)) {
                    craneInfo = new CraneInfo();
                    craneInfo.setCraneNo(craneNo);
                }
                List<Object> list = craneInfoService.queryCraneListFromRedis(craneInfo);
                if (CollectionUtils.isNotEmpty(list)) {
                    List<Map> res = list.stream().map(o -> {
                        Map map = (Map) o;
                        String portCode = (String) map.get("portCode");
                        Map craneConfirmInfo = craneInfoService.queryCraneConfirmInfo(portCode);
                        if (craneConfirmInfo != null) {
                            //portCode对应的vin craneStatus确认状态
                            long timestamp = (long)craneConfirmInfo.get("collectTime");
                            if((System.currentTimeMillis() - timestamp) > confirmExpireTime * 1000){
                                map.put("craneStatus", 0);//空闲
                            }else{
                                String vin = (String) craneConfirmInfo.get("vin");
                                int craneStatus = (int) craneConfirmInfo.get("craneStatus");
                                //获取redis任务获取taskType
                                String orderKey = String.format("%s:%s", latestOrderKeyPrefix, vin);
                                Map<String, Object> order = redisDataModel.getJson2Map(orderKey);
                                int taskType = (int) order.get("taskType");
                                //根据vin获取vehicleNum
                                String key="harbor:command:vin:info";
                                String keys = String.format("%s:%s", key, vin);
                                String vehicleMap = redisDataModel.get(keys);
                                if(vehicleMap != null){//判断redis是否存在，不存在则查询数据库
                                    JSONObject jsonObject=JSONObject.parseObject(vehicleMap);
                                    String vehicleNum = null;
                                    Integer vehicleType = 0;
                                    if (jsonObject.containsKey("vehicleNum")){//判断redis是否存在车辆编号
                                       vehicleNum = jsonObject.get("vehicleNum").toString();
                                    }else{
                                        VmsVehicle vmsVehicle = vmsVehicleMapper.selectOne(new LambdaQueryWrapper<VmsVehicle>()
                                                .eq(VmsVehicle::getVin, vin));
                                        if (vmsVehicle!=null){
                                            vehicleNum = vmsVehicle.getVehicleNum();
                                            vehicleType=vmsVehicle.getVehicleType()==null ? 0 : vmsVehicle.getVehicleType();
                                        }

                                    }
                                    if (jsonObject.containsKey("vehicleType")){
                                        vehicleType =(Integer) jsonObject.get("vehicleType");
                                    }
                                    //设置车辆编号、装卸箱、吊具确认状态,车辆类型
                                    map.put("vehicleNum", vehicleNum);
                                    map.put("taskType", taskType);
                                    map.put("craneStatus", craneStatus);
                                    map.put("vehicleType",vehicleType);
                                }else{
                                    VmsVehicle vmsVehicle = vmsVehicleMapper.selectOne(new LambdaQueryWrapper<VmsVehicle>()
                                            .eq(VmsVehicle::getVin, vin));
                                    String vehicleNum = "";
                                    Integer vehicleType = 0;
                                    if (vmsVehicle!=null){
                                        vehicleNum = vmsVehicle.getVehicleNum();
                                        vehicleType=vmsVehicle.getVehicleType()==null ? 0 : vmsVehicle.getVehicleType();
                                    }

                                    //设置车辆编号、装卸箱、吊具确认状态,车辆类型
                                    map.put("vehicleNum", vehicleNum);
                                    map.put("taskType", taskType);
                                    map.put("craneStatus", craneStatus);
                                    map.put("vehicleType",vehicleType);
                                }
                            }
                            //未来时间设置成现在
                            long craneTime = (long)map.get("collectTime");
                            if(craneTime > System.currentTimeMillis()){
                                map.put("collectTime",System.currentTimeMillis());
                            }
                            return map;
                        } else {
                            return null;
                        }
                    }).filter(map -> map != null).collect(Collectors.toList());
                    craneInfoWebSocketServer.sendMessage(JSON.toJSONString(res));
                }
            }
        }
    }
}
