package com.ssi.websocket.push.task;

import static com.ssi.constant.RedisKey.CRANE_LOCATION;

import com.alibaba.fastjson.JSONObject;
import com.ssi.model.CraneInfoRedis;
import com.ssi.model.RedisDataModel;
import com.ssi.service.CraneInfoService;
import com.ssi.service.VmsShipsDrawingService;
import com.ssi.service.VmsVehicleService;
import com.ssi.websocket.LocationWebSocketService;
import com.ssi.websocket.vo.CraneLocation;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Map;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import com.ssi.service.VmsAlertThresholdService;

/**
 * @author 成东
 * @since 2022-02-23 11:25
 */
@Component
public class PushPortMachineryLocationTask {

  @Autowired
  private RedisDataModel redisDataModel;

  @Autowired
  private CraneInfoService craneInfoService;

  @Autowired
  private VmsShipsDrawingService vmsShipsDrawingService;

  @Autowired
  private VmsVehicleService vmsVehicleService;

  @Value("${vehicle.latestData.redis.protCode:ivccs:vms:portCode}")
  private String portCodeKey;

  @Value("${vehicle.latestData.redis.cranePos:ivccs:vms:cranePos}")
  private String cranePosKey;
  //websocket获取桥吊数据
  @Scheduled(initialDelay = 2 * 1000, fixedDelay = 2000)
  public void getOrdersOfTask() {
      if (LocationWebSocketService.isPush()) {
      val locationMap = redisDataModel.mgetForMap(
          redisDataModel.keys(CRANE_LOCATION.getKeyPrefix() + "*")
      );
      long time = System.currentTimeMillis();
      Map<String, CraneLocation> map1 = new HashMap<>();
      locationMap.forEach((key, value) -> {
        if (value != null) {
          String no = StringUtils.removeStart(key, CRANE_LOCATION.getKeyPrefix());
          val p = new CraneLocation();
          p.setPortCode(no);
          val sp = ((String) value).split(",");
          String regex="^2\\d*$";
          if (no.matches(regex)){//判断no是否以2开头
            //String portCode="ivccs:vms:portCode";
            String keys = String.format("%s:%s",portCodeKey,no);
            Map<String, Object> map = redisDataModel.getJson2Map(keys);
            if (map==null){//先从redis中获取数据,不存在在请求数据库
              map=vmsShipsDrawingService.getPileInfo(no);//查询龙门吊的视图
              JSONObject jsonObject=new JSONObject();
              jsonObject.put("CY_AREA_NO",String.valueOf(map.get("CY_AREA_NO").toString()));
              redisDataModel.set(keys, String.valueOf(jsonObject));
            }
            p.setLatitude(new BigDecimal(sp[0]));
            p.setLongitude(new BigDecimal(sp[1]));
            String name=String.valueOf(map.get("CY_AREA_NO").toString());
            Map map2=getLocaltion(Double.parseDouble(sp[1]),Double.parseDouble(sp[0]),name);
            p.setLatitude(new BigDecimal(map2.get("lat").toString()));
            p.setLongitude(new BigDecimal(map2.get("lng").toString()));
            p.setPortType(2);
            p.setTimestamp(time);
            map1.put(no, p);
          }else {
              Map map2=getLocaltion(Double.parseDouble(sp[1]),Double.parseDouble(sp[0]),"Bridge0001");
              p.setLatitude(new BigDecimal(map2.get("lat").toString()));
              p.setLongitude(new BigDecimal(map2.get("lng").toString()));
              p.setTimestamp(time);
              p.setPortType(1);
              map1.put(no, p);
          }
        }
      });
      LocationWebSocketService.messageBroadcast(map1.values());
  }
  }

    //重新映射场吊GPS位置
    private Map getLocaltion(double lng3,double lat3,String name) {
        Map map = new HashMap();
       // String cranePos="ivccs:vms:cranePos";
        String keys = String.format("%s:%s",cranePosKey,name);
        Map<String, Object> cranePosMap = redisDataModel.getJson2Map(keys);
        if (cranePosMap==null){//先从redis中获取数据,不存在在请求数据库
            cranePosMap=vmsVehicleService.getVmsCranePosMapInfo(name);
            JSONObject jsonObject=new JSONObject();
            jsonObject.put("startLng",cranePosMap.get("startLng").toString());//起始位置经度
            jsonObject.put("startLat",cranePosMap.get("startLat").toString());//起始位置纬度
            jsonObject.put("endLng",cranePosMap.get("endLng").toString());//终点位置经度
            jsonObject.put("endLat",cranePosMap.get("endLat").toString());//终点位置纬度
            redisDataModel.set(keys, String.valueOf(jsonObject));
        }
        double lng1=Double.parseDouble(cranePosMap.get("startLng").toString());//起始位置经度
        double lat1=Double.parseDouble(cranePosMap.get("startLat").toString());//起始位置纬度
        double lng2=Double.parseDouble(cranePosMap.get("endLng").toString());//终点位置经度
        double lat2=Double.parseDouble(cranePosMap.get("endLat").toString());//终点位置纬度
        double dLng = lng1 - lng2;//相差距离经度
        double dLat = lat1 - lat2;//想差距离纬度

        double u = (lng3 - lng1) * dLng + (lat3 - lat1) * dLat;
        u /= dLng * dLng + dLat * dLat;
        double lng = new BigDecimal(lng1 + u * dLng).setScale(8, RoundingMode.HALF_UP).doubleValue();
        map.put("lng",lng);
        //foot.setLng(new BigDecimal(p1.getLng() + u * dLng).setScale(8, RoundingMode.HALF_UP).doubleValue());
        double lat = new BigDecimal(lat1 + u * dLat).setScale(8, RoundingMode.HALF_UP).doubleValue();
        map.put("lat",lat);
        double d = Math.pow(lng1 - lng2, 2) + Math.pow(lat1 - lat2, 2);
        double d1 = Math.pow(lng1 - lng, 2) + Math.pow(lat1 - lat, 2);
        double d2 = Math.pow(lng2 - lng, 2) + Math.pow(lat2 - lat, 2);
        if (d1 > d || d2 > d) {
            if (d1 > d2) {
                map.put("lng",Math.pow(lng2 - lng, 2));
                map.put("lat",Math.pow(lat2 - lat, 2));
                return map;
            } else {
                map.put("lng",Math.pow(lng1 - lng, 2));
                map.put("lat",Math.pow(lat1 - lat, 2));
                return map;
            }
        }
        return map;
    }
}
