package com.ssi.websocket.push.task;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ssi.constant.RedisKey;
import com.ssi.entity.VmsAlertThreshold;
import com.ssi.entity.VmsVehicle;
import com.ssi.kafka.model.KafkaRecordModel;
import com.ssi.model.RedisDataModel;
import com.ssi.model.VehicleSocketDataCacheQueueModel;
import com.ssi.service.VmsAlertThresholdService;
import com.ssi.service.VmsVehicleService;
import com.ssi.service.platform.WebSocketDataService;
import com.ssi.utils.SpringUtils;
import com.ssi.utils.VehicleDataUtil;
import com.ssi.websocket.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import sun.misc.Unsafe;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * 目前策略是读取所有数据
 * 后期车辆数多， 考虑从CismWebSocketService中获取所有的vin
 */
@Component
@EnableScheduling
@Slf4j
public class WebSocketScheduledSendTask {

    @Autowired
    private WebSocketDataService webSocketDataService;
    @Autowired
    private VmsAlertThresholdService vmsAlertThresholdService;

    @Autowired
    private VmsVehicleService vmsVehicleService;
    @Autowired
    private RedisDataModel redisDataModel;
    @Autowired
    private RedisDataModel singleNodeModel;
    @Value("${vehicle.cache.key:ivccs:vms:vehicle:cache}")
    private String vehicleCacheKey;

    @Autowired(required = false)
    private VehicleSocketDataCacheQueueModel vehicleSocketDataCacheQueueModel;

    @Autowired(required = false)
    private KafkaRecordModel kafkaRecordModel;

    @Autowired
    private VehicleDataUtil vehicleDataUtil;

    @Value("${spring.websocket.pushKafka.enable:false}")
    private boolean openPushKafka;

    @Value("${spring.websocket.pushInPool.enable:false}")
    private boolean pushInPool;
    @Value("${harbor.command.info_key:harbor:command:vin:info}")
    private String infoKey;
    @Value("${vehicle.latestData.redis.vehicleTypeSwitch:ivccs:vms:vehicle_type_switch}")
    private String typeSwitch;


    private Queue<Map<String, Map<String, Object>>> mapQueue = Lists.newLinkedList();
    private boolean first=true;

    /**
     * 缓存车辆信息
     */
    @Scheduled(fixedRateString = "${spring.websocket.scheduled.time:60000}")
    private void cacheVehicleInfo() {
        List<VmsVehicle> list = vmsVehicleService.list();
        list.stream().forEach(vehicle -> {
            redisDataModel.hset(vehicleCacheKey,vehicle.getVin(), JSONObject.toJSONString(vehicle));
        });
    }

    /**
     * 定时读取所有车辆的最新数据（无人集卡）
     */
    @Scheduled(fixedRateString = "${spring.websocket.scheduled.time:2000}")
    private void scheduledSendInfoMageNoCrad() {
        Map<String, Map<String, Object>> messages;//全部
        Map<String, Map<String, Object>> messages1 = Maps.newTreeMap();//无人集卡
        if (WebSocketService.hasWebSocketSession() || WebSocketServiceForScreenList.hasWebSocketSession() || openPushKafka) {
            if (pushInPool) {
                messages = vehicleSocketDataCacheQueueModel.getAllVehicleData();
            } else {
                messages = webSocketDataService.realTimeDataAll();
            }
            if (!messages.isEmpty()) {
            //推送到kafka
            try {
                mapQueue.offer(messages);
            } catch (Exception e) {
                log.error("添加推送队列失败。", e);
            }
           }

            if (WebSocketService.hasWebSocketSession()){
                if (!messages.isEmpty()) {
                    //推送至客户端
                    messages.forEach((vin, record) -> {
                        WebSocketService.messageBroadcast(vin, record);
                    });
                }
            }
            if (WebSocketServiceForScreenList.hasWebSocketSession()){
                if (!messages.isEmpty()) {
                    //大屏列表推送
                    //log.info(String.format("进入无人集卡判断: %s", JSON.toJSONString(messages)));
                    Collection<Map<String, Object>> values = null;
                   for (Map.Entry<String, Map<String, Object>> entry:messages.entrySet()){
                       //String key="harbor:command:vin:info";
                       String keys = String.format("%s:%s", infoKey, entry.getKey());
                       String vehicleMap = redisDataModel.get(keys);
                       if(vehicleMap != null) {//判断redis是否存在，不存在则查询数据库
                           JSONObject jsonObject = JSONObject.parseObject(vehicleMap);
                           Integer vehicleType =(Integer) jsonObject.get("vehicleType");//1:X69B ,2:X69C,3:有人集卡两秒
                           if (vehicleType != 3){
                               String key = Objects.isNull(entry.getValue().get("vehicleNum"))?entry.getKey():(String) entry.getValue().get("vehicleNum");
                               //处理异常信息
                               processVehcileErrorMsg(entry.getValue());
                               messages1.put(key,entry.getValue());
                           }
                       }
                     }
                    values=messages1.values();
                    WebSocketServiceForScreenList.messageBroadcast(values);
                }else{
                    WebSocketServiceForScreenList.messageBroadcast(Lists.newArrayList());
                }
            }
            //debug app车辆实时状态推送
            if (WebSocketServiceForDebugApp.hasWebSocketSession()){
                messages = webSocketDataService.getVehicleRunData();
                if (!messages.isEmpty()) {
                    //推送至客户端
                    messages.forEach((vin, record) -> {
                        WebSocketServiceForDebugApp.messageBroadcast(vin, record);
                    });
                }
            }
        }
    }

    /**
     * 定时读取所有车辆的最新数据(有人集卡)
     */
    @Scheduled(fixedRateString = "${spring.websocket.scheduled.time:2000}")
    private void scheduledSendInfoMageYesCrad() {
        Map<String, Map<String, Object>> messages;//全部
        Map<String, Map<String, Object>> messages1 = Maps.newHashMap();//用于存放有人集卡的数据
        String vmsAlertThresholdInfo=redisDataModel.get(typeSwitch);
        //用于判断是否开启给有人集卡推送数据的字段
       /* String vehicleTypeSwitch="1";
        if (vmsAlertThresholdInfo!=null){//redis缓存存在就用redis缓存否则就查询数据库
            JSONObject jsonObject = JSONObject.parseObject(vmsAlertThresholdInfo);
            vehicleTypeSwitch = (String) jsonObject.get("paramValue");
        }else{
            VmsAlertThreshold info=new VmsAlertThreshold();
            info.setParamKey("vehicle_type_switch");
            VmsAlertThreshold current = vmsAlertThresholdService.getOne(new LambdaQueryWrapper<>(info));
            //无人集卡开关0关闭,1开启
            vehicleTypeSwitch= current.getParamValue();
            redisDataModel.set(typeSwitch,JSONObject.toJSONString(current));
        }*/
        if ((WebSocketService.hasWebSocketSession() || WebSocketServiceForScreenSomeoneCardList.hasWebSocketSession() || openPushKafka)) {
            if (pushInPool) {
                messages = vehicleSocketDataCacheQueueModel.getAllVehicleData();
            } else {
                messages = webSocketDataService.realTimeDataAll();
            }
            /*if (!messages.isEmpty()) {
                //推送到kafka
                try {
                    mapQueue.offer(messages);
                } catch (Exception e) {
                    log.error("添加推送队列失败。", e);
                }
            }

            if (WebSocketService.hasWebSocketSession()){
                if (!messages.isEmpty()) {
                    //推送至客户端
                    messages.forEach((vin, record) -> {
                        WebSocketService.messageBroadcast(vin, record);
                    });
                }
            }*/
            if (WebSocketServiceForScreenSomeoneCardList.hasWebSocketSession()){
                if (!messages.isEmpty()) {
                    //前端列表推送
                    Collection<Map<String, Object>> values = null;
//                    if(vehicleTypeSwitch.equals("0")){//无人集卡开关0关闭,1开启
                        for (Map.Entry<String, Map<String, Object>> entry:messages.entrySet()){
                            String keys = String.format("%s:%s", infoKey, entry.getKey());
                            String vehicleMap = singleNodeModel.get(keys);
                            if(vehicleMap != null) {//判断redis是否存在，不存在则查询数据库
                                JSONObject jsonObject = JSONObject.parseObject(vehicleMap);
                                Integer vehicleType =jsonObject.getInteger("vehicleType");//1:X69B ,2:X69C,3:有人集卡两秒
                                if (vehicleType == 3){
                                    messages1.put(entry.getKey(),entry.getValue());
                                }
                            }
                        }
                        values=messages1.values();
                        WebSocketServiceForScreenSomeoneCardList.messageBroadcast(values);
//                    }else{//
//                        WebSocketServiceForScreenSomeoneCardList.messageBroadcast(Lists.newArrayList());
//                    }
                }else{
                    WebSocketServiceForScreenSomeoneCardList.messageBroadcast(Lists.newArrayList());
                }
            }
            //debug app车辆实时状态推送
            if (WebSocketServiceForDebugApp.hasWebSocketSession()){
                messages = webSocketDataService.getVehicleRunData();
                if (!messages.isEmpty()) {
                    //推送至客户端
                    messages.forEach((vin, record) -> {
                        WebSocketServiceForDebugApp.messageBroadcast(vin, record);
                    });
                }
            }
        }else{
            WebSocketServiceForScreenSomeoneCardList.messageBroadcast(Lists.newArrayList());
        }
    }

    /**
     * 将队列中的数据推送到kafka
     */
    @Scheduled(fixedRateString = "${spring.websocket.scheduled.pushKafka.time:500}")
    private void scheduledSendInfoMageToKafka() {

        List<Map<String, Map<String, Object>>> res = Lists.newLinkedList();

        int n = 0;
        Map<String, Map<String, Object>> poll = Maps.newHashMap();
        int size = 0;
        while (n <= 100 && poll != null) {
            poll = mapQueue.poll();
            if (poll != null && !poll.isEmpty()) {
                size += poll.size();
                res.add(poll);
            }
        }

        //批量推kafka, 避免积压
        if (!res.isEmpty()) {
            long l = System.currentTimeMillis();
            kafkaRecordModel.messageBroadcast(res);
            log.debug(String.format("推送数据到kafka完成，推送数据记录：%s条, 积压数据批次：%s, 耗时：%s",
                    size, mapQueue.size(), (System.currentTimeMillis() - l)));
        }

    }

    private void processVehcileErrorMsg(Map<String, Object> record) {
        try {
            JSONObject jsonObject = new JSONObject();
//            Double speed = (Double) record.get("speed");
            //V2X异常
            String v2xStr = redisDataModel.get(RedisKey.ERROR_V2X_INFO.getKeyPrefix().concat((String) record.get("vin")));
            if (StringUtils.isNotBlank(v2xStr)) {
                JSONObject v2xObj = JSON.parseObject(v2xStr);
                if (System.currentTimeMillis() - v2xObj.getLong("time") < 30000) {
                    record.put("vinFlag", 1);
                    jsonObject.put("v2xError", v2xObj.getString("description"));
                }
       /*   if(StringUtils.isNotBlank(v2xObj.getString("description"))){
            jsonObject.put("v2xError",v2xObj.getString("description"));
          }*/
            }
            //三级告警
            String tosStr = redisDataModel.get(RedisKey.ERROR_TOS_INFO.getKeyPrefix().concat((String) record.get("vin")));
            if (StringUtils.isNotBlank(tosStr)) {
                JSONObject tosObj = JSON.parseObject(tosStr);
                jsonObject.put("v2xError", tosObj.getString("faultName"));
            }

            //平台异常
            StringBuilder sb = new StringBuilder("");
            Map<String, Object> vehicleEmergencyRecord = redisDataModel.getJson2Map(RedisKey.EMERGENCY_PARKING.getKeyPrefix().concat((String) record.get("vin")));
            if (MapUtils.isNotEmpty(vehicleEmergencyRecord)) {
                Integer emergencyStatus = (Integer) vehicleEmergencyRecord.get("emergencyStatus");
                if (1 == emergencyStatus) {
                    record.put("vinFlag", 1);
                    sb.append("车辆紧急急停 ");
                }
            }
            Object stopExceed = record.get("stopExceed");
            if (Objects.nonNull(stopExceed) && 1 == (Integer) stopExceed) {
                record.put("vinFlag", 1);
                sb.append("车辆停止超时 ");
            }

            Object dynamicFenceAlert = SpringUtils.getValue(RedisKey.DYNAMIC_FENCE_ALERT.getKeyPrefix(), (String) record.get("vin"));
            if (Objects.nonNull(dynamicFenceAlert)) {
                record.put("vinFlag", 1);
                sb.append("封闭区触发任务暂停 ");
            }
            Object fenceOrderError = SpringUtils.getValue(RedisKey.DYNAMIC_FENCE_ORDER_ERROR.getKeyPrefix(), (String) record.get("vin"));
            if (Objects.nonNull(dynamicFenceAlert)) {
                sb.append(fenceOrderError+" ");
            }
            Object chargingAction = SpringUtils.getValue(RedisKey.VMS_CHARGING_ACTION.getKeyPrefix(), (String) record.get("vin"));
            if (Objects.nonNull(chargingAction)&&chargingAction instanceof JSONObject) {
                JSONObject actionObj = (JSONObject) chargingAction;
                if(actionObj.getInteger("chargingAction")==2 && actionObj.getLong("reciveTime")<(System.currentTimeMillis()-300000)){
                    record.put("vinFlag", 1);
                    sb.append("请检查车辆充电接口 ");
                }
            }

            jsonObject.put("platError", sb.toString());
            //TOS异常
            Map<String, Object> json2Map = redisDataModel.getJson2Map(RedisKey.VMS_TOS_ORDER.getKeyPrefix().concat((String) record.get("vin")));
            if (MapUtils.isNotEmpty(json2Map)) {
                JSONObject lastedOrder = new JSONObject();
                lastedOrder.put("startTime", json2Map.get("collectTime"));
                lastedOrder.put("lockLabel", json2Map.get("lockLabel"));
                lastedOrder.put("portCode", json2Map.get("portCode"));
                lastedOrder.put("containerSize", json2Map.get("containerSize"));
                lastedOrder.put("containerPosition", json2Map.get("containerPosition"));
                Integer subTaskType = Objects.isNull(json2Map.get("subTaskType")) ? null : (Integer) json2Map.get("subTaskType");
                String description = vehicleDataUtil.generateOrderDescription(json2Map);
                //判断subTaskType：0去缓冲区1，2 去扭锁站 其他用查询出来的
                if (Objects.nonNull(subTaskType) && subTaskType == 0) {
                    lastedOrder.put("orderDescription", "去缓冲区");
                } else if (Objects.nonNull(subTaskType) && (subTaskType == 1 || subTaskType == 2)) {
                    lastedOrder.put("orderDescription", "去扭锁站");
                } else {
                    lastedOrder.put("orderDescription", description);
                }
                jsonObject.put("tosError", lastedOrder);
            }
            record.put("errorMsg", jsonObject);
        }catch (Exception e){
            log.error("告警信息出现异常",e);
        }
    }
}
